#ifndef __NM6408_H__
#define __NM6408_H__

#include "nm6408_map.h"
#include "nm6408_ioctl.h"

#define NM6408_BRD_MINOR	0
#define NM6408_BRD_NUM		32

#define NM6408_XFER_TIMEOUT	100

#define NM6408_BAR_CTRL		0
#define NM6408_BAR_MEM		1
#define NM6408_BAR_MMAP		2

#define NM6408_BAR_NMB		3

struct nm6408_brd {
	struct pci_dev		*pdev;
	struct cdev		cdev;
	struct semaphore	semap;

	atomic_t		count;

	char			*name;
	int			indx;

	unsigned int		bus_nmb;
	unsigned int		dev_nmb;
	unsigned int		func_nmb;

	void __iomem		*region_virt[NM6408_BAR_NMB];
	phys_addr_t		region_phys[NM6408_BAR_NMB];
	resource_size_t		region_len[NM6408_BAR_NMB];
	unsigned long		region_flags[NM6408_BAR_NMB];
	int			region_indx[NM6408_BAR_NMB];
	int			region_count;

	unsigned long		bar_ctrl;

	struct timer_list	timer;
	spinlock_t		lock;
	int			irq;

	wait_queue_head_t	wait_queue;
	struct workqueue_struct	*work_queue;

	dma_addr_t		desc_bus_addr;
	struct dma_desc		*desc_cpu_addr;
	int			desc_num;

	struct nm6408_xfer	*xfer_cur;
	int			need_wait;

	int			is_alive;
};

enum {
	XFER_COMPLETE_ERROR	= 0x1,
	XFER_COMPLETE_SUCCESS	= 0x2,
	XFER_COMPLETE_TIMEOUT	= 0x4
};

struct nm6408_xfer {
	struct nm6408_brd	*brd;
	struct work_struct	work;
	int			stat;

	u32			cluster_base[NM6408_INBOUND_AXI_NMB];
	u32			cluster_offset[NM6408_INBOUND_AXI_NMB];
	u32			cluster_limit[NM6408_INBOUND_AXI_NMB];
	int			cluster_indx;

	struct sg_table		sc_table;
	struct scatterlist	*sc_entry;
	unsigned int		sc_nents;

	int			prev_dma_desc;
	int			cur_dma_desc;
	int			cnt_dma_desc;
	int 			wait_dma_done;

	size_t			dma_offset;
	size_t			str_pos;
	size_t			cur_pos; // can overflow

	int			rw;
};

struct dma_desc {
	u32	axi_data_addr;
	u32	axi_data_attr;
	u64	pcie_data_addr;
	u32	pcie_data_attr;
	u32	xfer_prm;
	u32	xfer_stat;
	u32	next_desc_addr;
} __attribute__((packed, aligned(32)));

#define AXI_ATTR_AXPROT(x)		(((x) & 0x7) << 0)
#define AXI_ATTR_AXCACHE(x)		(((x) & 0xF) << 3)
#define AXI_ATTR_AXLOCK			(1 << 7)

#define PCIE_HDR_NO_SNOOP		(1 << 0)
#define PCIE_HDR_RELAX_ORDER		(1 << 1)
#define PCIE_HDR_ID_ORDERING		(1 << 2)
#define PCIE_HDR_XFER_CLASS(x)		(((x) & 0x7) << 3)
#define PCIE_HDR_USE_REQ_ID		(1 << 9)
#define PCIE_HDR_REQUEST_ID(x)		(((x) & 0xFFFF) << 10)

#define XFER_PRM_SIZE_BYTE(x)		(((x) & 0xFFFFFF) << 0)
#define XFER_PRM_ALLOW_INTR		(1 << 24)
#define XFER_PRM_DESC_TYPE(x)		(((x) & 0x3) << 25)
#define XFER_PRM_ALLOW_CHAIN		(1 << 29)

#define XFER_STAT_AXI_RESP(x)		(((x) >> 0) & 0x3)
#define XFER_STAT_AXI_ERR		(1 << 2)
#define XFER_STAT_PCIE_TLP_ERR(x)	(((x) >> 8) & 0xF)
#define XFER_STAT_DMA_XFER_DONE		(1 << 16)
#define XFER_STAT_DMA_PCIE_ERR		(1 << 17)
#define XFER_STAT_DMA_AXI_ERR		(1 << 18)
#define XFER_STAT_DMA_DATA_ERR		(1 << 19)
#define XFER_STAT_DMA_DESC_ERR		(1 << 20)
#define XFER_STAT_DMA_OVERFLOW		(1 << 21)
#define XFER_STAT_DMA_UNDERFLOW		(1 << 22)
#define XFER_STAT_DMA_NOT_EMPTY		(1 << 23)

#endif // __NM6408_H__
