#!/bin/bash -e

DEV_DIR="/sys/bus/pci/devices/$DEV_NUMBER"
BRG_DIR="/sys/bus/pci/devices/$BRG_NUMBER"

function load_module()
{
	if lsmod | grep -i "$DRIVER" 1> /dev/null 2>&1; then
		return 1
	else
		/sbin/insmod "${DRIVER}.ko"
	fi

	return 0
}

function unload_module()
{
	if lsmod | grep -i "$DRIVER" 1> /dev/null 2>&1; then
		/sbin/rmmod "$DRIVER"
	else
		return 1
	fi

	return 0
}

function reload_device()
{
	if [ -f "${DEV_DIR}/remove" ]
	then
		echo 1 > "${DEV_DIR}/remove"
	fi

	if [ -f "${BRG_DIR}/rescan" ]
	then
		echo 1 > "${BRG_DIR}/rescan"
	else
		return 1
	fi

	if [ ! -d "$DEV_DIR" ]
	then
		return 2
	fi

	if [ -f "${DEV_DIR}/reset" ]
	then
		echo 1 > "${DEV_DIR}/reset"
	else
		return 3
	fi

	return 0
}

case "$1" in
	load)
		load_module

		if [ $? -ne 0 ]
		then
			exit 1
		fi

		;;
	unload)
		unload_module

		if [ $? -ne 0 ]
		then
			exit 2
		fi

		;;
	reload)
		if [ -z "$DEV_NUMBER" ]
		then
			echo "ERROR: set PCIe device number - domain:bus:slot.func"
			exit 3
		fi

		if [ -z "$BRG_NUMBER" ]
		then
			echo "ERROR: set PCIe bridge number - domain:bus:slot.func"
			exit 4
		fi

		reload_device

		if [ $? -ne 0 ]
		then
			exit 5
		fi

		;;
	*)
		echo "Usage: $FILE_NAME load|unload|reload [PCIE_DEVICE_NUMBER] [PCIE_BRIDGE_NUMBER]"
		echo "       NUMBER format - domain:bus:slot.func"
		exit -1
esac

if ls /dev | grep -i "$DEVICE" 1> /dev/null 2>&1; then
	sudo sh -c "chmod a+rw /dev/${DEVICE}_*"
fi

exit 0
