#define _MULTI_THREADED

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <pthread.h>
#include <sched.h>

#include <sys/types.h>
#include <sys/time.h>

#if defined(MC12705)
#include "mc12705load.h"
#elif defined(NM_CARD)
#include "nm_card_load.h"
#elif defined(NM_MEZZO)
#include "nm_mezzo_load.h"
#elif defined(NM_QUAD)
#include "nm_quad_load.h"
#elif defined(NEURO_MEZZANINE)
#include "neuro_mezzanine_load.h"
#else
#error "Unknown board!"
#endif

#include "io_host.h"

void *calc_fact_test(void *arg);

pthread_mutex_t work_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t  wait_cond = PTHREAD_COND_INITIALIZER;

int		pipe_fd[2];

PL_CoreNo	all_core[21];
int		test_ret[21];

PL_Board 	*board;

int 		in_progress = 0;
int		do_wait = 1;

const char *NM0_PART_FILE_NAME = "./nmc/nmc0_part.abs";
const char *NM1_PART_FILE_NAME = "./nmc/nmc1_part.abs";
const char *NM2_PART_FILE_NAME = "./nmc/nmc2_part.abs";
const char *NM3_PART_FILE_NAME = "./nmc/nmc3_part.abs";
const char *CCPU_PART_FILE_NAME = "./arm/central_arm_part.elf";
const char *PC_PART_FILE_NAME = "./arm/cluster_arm_part.elf";

int main()
{
	unsigned int		boardCount;
	pthread_t		test_thread[21];
	pthread_t		_test_thread;
	int			*_test_ret;
	int			cluster_id, nm_id;
	int			i, ret;

	ret = pipe(pipe_fd);
	if (ret < 0)
		return  1;

	if (PL_GetBoardCount(&boardCount) != PL_OK) {
		printf("ERROR: Failed get count of boards!\n");
		return  2;
	}

	if (boardCount < 1) {
		printf("ERROR: Failed find board!\n");
		return 3;
	}

	if (PL_GetBoardDesc(0, &board) != PL_OK) {
		printf("ERROR: Failed open board!\n");
		return 4;
	}
#if 1
	if (PL_ResetBoard(board) != PL_OK) {
		printf("ERROR: Failed reset board!\n");
		PL_CloseBoardDesc(board);
		return 5;
	}

	if (PL_LoadInitCode(board) != PL_OK) {
		printf("ERROR: Failed load init code!\n");
		PL_CloseBoardDesc(board);
		return 6;
	}
#endif
	for (i = 0; i < 21; i++) {
		if (i == 0) {
			cluster_id = -1;
			nm_id =  -1;
		}
		else if (i == 1) {
			cluster_id = 0;
			nm_id =  -1;
		}
		else if (i == 6) {
			cluster_id = 1;	
			nm_id =  -1;
		}
		else if (i == 11) {
			cluster_id = 2;
			nm_id =  -1;
		}
		else if (i == 16) {
			cluster_id = 3;
			nm_id =  -1;
		}

		all_core[i].cluster_id = cluster_id;
		all_core[i].nm_id = nm_id;

		test_ret[i] = -1;

		if (pthread_create(&test_thread[i], NULL, calc_fact_test, (void *)((unsigned long)i))) {
			for (; i > 0; i--)
				pthread_cancel(test_thread[i - 1]);

			exit (7);
		}

		nm_id++;
	}

	sleep(1);

	pthread_mutex_lock(&work_mutex);
	do_wait = 0;
	pthread_cond_broadcast(&wait_cond);
	pthread_mutex_unlock(&work_mutex);

	i = 0;

	while (1) {
		ret = read(pipe_fd[0], &_test_thread, sizeof(pthread_t));
		if (ret != sizeof(pthread_t))
			exit (8);

		pthread_join(_test_thread, (void **)&_test_ret);

		if (*_test_ret == 0)
			 i++;

		pthread_mutex_lock(&work_mutex);
		in_progress--;

		if (!in_progress) {
			pthread_mutex_unlock(&work_mutex);
			break;
		}

		pthread_mutex_unlock(&work_mutex);
	}

	PL_CloseBoardDesc(board);

	if (i != 21)
		return -1;

	printf("OK: test successful!\n");

	return 0;
}

void *calc_fact_test(void *arg)
{
	IO_Service	*service;
	PL_Access	*access;
	int		index = (unsigned long)arg;
	pthread_t	self_thread;
	int		rep_count;
	int		ret;

	pthread_mutex_lock(&work_mutex);

	in_progress++;

	if (do_wait)
		pthread_cond_wait(&wait_cond, &work_mutex);

	pthread_mutex_unlock(&work_mutex);

	print_mutex_lock();
	printf("Thread[%d] - Load and run program!\n", index);
	print_mutex_unlock();

	service = NULL;
	access = NULL;

	if (PL_GetAccess(board, &all_core[index], &access) != PL_OK) {
		print_mutex_lock();
		printf("ERROR[%d]: Failed get access!\n", index);
		print_mutex_unlock();

		goto err_calc_exit;
	}

	if (all_core[index].nm_id < 0)
		if (all_core[index].cluster_id == -1)
			ret = PL_LoadProgramFile(access, CCPU_PART_FILE_NAME);
		else
			ret = PL_LoadProgramFile(access, PC_PART_FILE_NAME);
	else {
		if (all_core[index].nm_id == 0)
			ret = PL_LoadProgramFile(access, NM0_PART_FILE_NAME);
		else if (all_core[index].nm_id == 1)
			ret = PL_LoadProgramFile(access, NM1_PART_FILE_NAME);
		else if (all_core[index].nm_id == 2)
			ret = PL_LoadProgramFile(access, NM2_PART_FILE_NAME);
		else
			ret = PL_LoadProgramFile(access, NM3_PART_FILE_NAME);
	}

	if (ret != PL_OK) {
		print_mutex_lock();
		printf("ERROR[%d]: Failed load program\n", index);
		print_mutex_unlock();

		goto err_calc_exit;
	}

	service = IO_ServiceStart(access, NULL, NULL,NULL);
	rep_count = 100000;

	while (1) {
		PL_Word status;
		PL_Word result;

		if (PL_GetStatus(access, &status) == PL_OK) {
			if (status == PROGRAM_PROGRESS) {
				rep_count--;
				
				if (!rep_count) {
					print_mutex_lock();
					printf("ERROR[%d]: Failed wait program done!\n", index);
					print_mutex_unlock();

					goto err_calc_exit;
				}

				continue;
			}

			if (status != PROGRAM_FINISHED) {
				print_mutex_lock();
				if (status == PROGRAM_EXEPTION)
					printf("ERROR[%d]: Abort program execution!\n", index);
				else
					printf("ERROR[%d]: Failed program status!\n", index);
				print_mutex_unlock();

				goto err_calc_exit;
			}
			else if (PL_GetResult(access, &result) == PL_OK) {
				print_mutex_lock();
				printf("TEST[%d] result = %d!\n", index, result);
				print_mutex_unlock();

				break;
			}
			else {
				print_mutex_lock();
				printf("ERROR[%d]: Failed get result!\n", index);
				print_mutex_unlock();

				goto err_calc_exit;
			}
		}
		else {
			print_mutex_lock();
			printf("ERROR[%d]: Failed get status!\n", index);
			print_mutex_unlock();

			goto err_calc_exit;
		}
	}

	test_ret[index] = 0;

err_calc_exit:

	if (service)
		IO_ServiceStop(&service,NULL);

	if (access)
		PL_CloseAccess(access);

	self_thread = pthread_self();

	ret = write(pipe_fd[1], &self_thread, sizeof(pthread_t));
	if (ret != sizeof(pthread_t))
		exit (8);

	sleep(1);

	pthread_exit(&test_ret[index]);
}
