#include <iostream>
#include <stdlib.h>

#if defined(MC12705)
#include "mc12705load.h"
#elif defined(NM_CARD)
#include "nm_card_load.h"
#elif defined(NM_MEZZO)
#include "nm_mezzo_load.h"
#elif defined(NM_QUAD)
#include "nm_quad_load.h"
#elif defined(NEURO_MEZZANINE)
#include "neuro_mezzanine_load.h"
#else
#error "Unknown board!"
#endif

#include "io_host.h"

using namespace std;

const char *NM0_PART_FILE_NAME = "./nmc/nmc0_part.abs";
const char *NM1_PART_FILE_NAME = "./nmc/nmc1_part.abs";
const char *NM2_PART_FILE_NAME = "./nmc/nmc2_part.abs";
const char *NM3_PART_FILE_NAME = "./nmc/nmc3_part.abs";
const char *CCPU_PART_FILE_NAME = "./arm/central_arm_part.elf";
const char *PC_PART_FILE_NAME = "./arm/cluster_arm_part.elf";

int main()
{
	PL_Board	*board;
	const char	*arm_name;
	const char	*nmc_name;
	IO_Service	*service[5][5];
	PL_Access	*access[5][5];
	int		is_done[5][5];
	PL_CoreNo	core;
	PL_Word		status;
	PL_Word		result;
	unsigned int	boardCount;
	int		i, j;
	int		wait_count, done_count;

	if (PL_GetBoardCount(&boardCount) != PL_OK) {
		cout << "ERROR: Failed get count of boards!\n";
		exit(1);
	}

	if (boardCount < 1) {
		cout << "ERROR: Failed find board!\n";
		exit(2);
	}

	if (PL_GetBoardDesc(0, &board) != PL_OK) {
		cout << "ERROR: Failed open board!\n";
		exit(3);
	}
#if 1
	if (PL_ResetBoard(board) != PL_OK) {
		cout << "ERROR: Failed reset board!\n";
		PL_CloseBoardDesc(board);
		exit(4);
	}

	if (PL_LoadInitCode(board) != PL_OK) {
		cout << "ERROR: Failed load init code!\n";
		PL_CloseBoardDesc(board);
		exit(5);
	}
#endif
	core.cluster_id = -1;
	arm_name = CCPU_PART_FILE_NAME;

	for (i = 0; i < 5; i++) {
		j = 0;
		core.nm_id = -1;

		access[i][j] = NULL;
		service[i][j] = NULL;
		is_done[i][j] = 0;

		if ((PL_GetAccess(board, &core, &access[i][j]) != PL_OK) ||
		    (PL_LoadProgramFile(access[i][j], arm_name) != PL_OK)) {
			cout << "ERROR: Failed access or load arm processor!\n";

			if (access[i][j] != NULL)
				PL_CloseAccess(access[i][j]);

			for (; i > 0; i--) {
				for (j = 5; j > 0; j--) {
					if (i == 1) {
						PL_CloseAccess(access[0][0]);
						break;
					}
					else
						PL_CloseAccess(access[i - 1][j - 1]);
				}
			}

			PL_CloseBoardDesc(board);
			exit(6);
		}

		if (core.cluster_id == -1) {
			arm_name = PC_PART_FILE_NAME;
			core.cluster_id += 1;
			continue;
		}


		core.nm_id = 0;

		for (j = 1; j < 5; j++) {
			service[i][j] = NULL;
			access[i][j] = NULL;
			is_done[i][j] = 0;

			if (core.nm_id == 0)
				nmc_name = NM0_PART_FILE_NAME;
			else if (core.nm_id == 1)
				nmc_name = NM1_PART_FILE_NAME;
			else if (core.nm_id == 2)
				nmc_name = NM2_PART_FILE_NAME;
			else
				nmc_name = NM3_PART_FILE_NAME;
				

			if ((PL_GetAccess(board, &core, &access[i][j]) != PL_OK) ||
			    (PL_LoadProgramFile(access[i][j], nmc_name) != PL_OK)) {
				cout << "ERROR: Failed access or load nmc processor!\n";

				if (access[i][j] != NULL)
					PL_CloseAccess(access[i][j]);

				for (; j > 0; j--)
					PL_CloseAccess(access[i][j - 1]);

				for (; i > 0; i--) {
					for (j = 5; j > 0; j--) {
						if (i == 1) {
							PL_CloseAccess(access[0][0]);
							break;
						}
						else
							PL_CloseAccess(access[i - 1][j - 1]);
					}
				}

				PL_CloseBoardDesc(board);
				exit(7);
			}

			core.nm_id += 1;
		}

		core.cluster_id += 1;
	}

	for (i = 0; i < 5; i++) {
		for (j = 0; j < 5; j++) {
			service[i][j] = IO_ServiceStart(access[i][j], NULL, NULL,NULL);

			if (i == 0)
				break;
		}
	}

	wait_count = 100000;
	done_count = 0;

	do {
		for (i = 0; i < 5; i++) {
			for (j = 0; j < 5; j++) {
				if ((i != 0) && is_done[i][j])
					continue;
				else if (is_done[i][j])
					break;

				if (PL_GetStatus(access[i][j], &status) == PL_OK) {
					if (status == PROGRAM_PROGRESS) {
						if (i != 0)
							continue;
						else
							break;
					}

					if (status != PROGRAM_FINISHED) {
						print_mutex_lock();
						if (status == PROGRAM_EXEPTION)
							cout << "ERROR: Abort program execution!\n";
						else
							cout << "ERROR: Failed program status!\n";
						print_mutex_unlock();

						goto err_test_end;
					}
					else if (PL_GetResult(access[i][j], &result) == PL_OK) {
						print_mutex_lock();
						cout << "TEST[" << i <<"][" << j << "] =  " << result << "!\n";
						print_mutex_unlock();

						is_done[i][j] = 1;

						done_count++;

						if (i == 0)
							break;
					}
					else {
						print_mutex_lock();
						cout << "ERROR: Failed get result!\n";
						print_mutex_unlock();

						goto err_test_end;
					}
				}
				else {
					print_mutex_lock();
					cout << "ERROR: Failed get status!\n";
					print_mutex_unlock();

					goto err_test_end;
				}
			}
		}

		if (!wait_count) {
			print_mutex_lock();
			cout << "ERROR: Failed wait program done!\n";
			print_mutex_unlock();

			goto err_test_end;
		}

		wait_count--;

	} while (done_count != 21);

err_test_end:

	for (i = 0; i < 5; i++) {
		for (j = 0 ; j < 5; j++) {
			if (service[i][j] != NULL)
				IO_ServiceStop(&service[i][j],NULL);

			PL_CloseAccess(access[i][j]);

			if (i == 0)
				break;
		}
	}

	PL_CloseBoardDesc(board);

	if (done_count != 21)
		exit(8);

	cout << "OK: test successful!\n";

	return 0;
}
