#define _MULTI_THREADED

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <pthread.h>
#include <sched.h>

#include <sys/types.h>
#include <sys/time.h>

#if defined(MC12705)
#include "mc12705load.h"
#elif defined(NM_CARD)
#include "nm_card_load.h"
#elif defined(NM_MEZZO)
#include "nm_mezzo_load.h"
#elif defined(NM_QUAD)
#include "nm_quad_load.h"
#elif defined(NEURO_MEZZANINE)
#include "neuro_mezzanine_load.h"
#else
#error "Unknown board!"
#endif

void *barrier_sync_test(void *arg);

pthread_mutex_t work_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t print_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t  wait_cond = PTHREAD_COND_INITIALIZER;

int		pipe_fd[2];

PL_CoreNo	all_core[21];
int		test_ret[21];

PL_Board 	*board;

int 		in_progress = 0;
int		do_wait = 1;

const char *NM_PART_FILE_NAME = "./nmc/nmc_part.abs";
const char *CCPU_PART_FILE_NAME = "./arm/central_arm_part.elf";
const char *PC_PART_FILE_NAME = "./arm/cluster_arm_part.elf";

#undef USE_CHAN_EDCL

#ifdef USE_CHAN_EDCL

unsigned char HOST_MAC_ADDRESS[] = {0x00, 0x14, 0xD1, 0x16, 0x69, 0xD0};
unsigned char BOARD_MAC_ADDRESS[] = {0xEC, 0x17, 0x66, 0x64, 0x08, 0x00};

#endif

int main()
{
	unsigned int		boardCount;
	pthread_t		test_thread[21];
	pthread_t		_test_thread;
	int			*_test_ret;
	int			cluster_id, nm_id;
	int			i, ret;

	ret = pipe(pipe_fd);
	if (ret < 0)
		return  1;

#ifdef USE_CHAN_EDCL

	if (PL_SetChannelEDCL(HOST_MAC_ADDRESS, BOARD_MAC_ADDRESS) != PL_OK) {
		printf("ERROR: Failed set channel EDCL!\n");
		return 2;
	}

	boardCount = 1;

#else

	if (PL_GetBoardCount(&boardCount) != PL_OK) {
		printf("ERROR: Failed get count of boards!\n");
		return  2;
	}

#endif

	if (boardCount < 1) {
		printf("ERROR: Failed find board!\n");
		return 3;
	}

	if (PL_GetBoardDesc(0, &board) != PL_OK) {
		printf("ERROR: Failed open board!\n");
		return 4;
	}
#if 1
	if (PL_ResetBoard(board) != PL_OK) {
		printf("ERROR: Failed reset board!\n");
		PL_CloseBoardDesc(board);
		return 5;
	}

	if (PL_LoadInitCode(board) != PL_OK) {
		printf("ERROR: Failed load init code!\n");
		PL_CloseBoardDesc(board);
		return 6;
	}
#endif
	for (i = 0; i < 21; i++) {
		if (i == 0) {
			cluster_id = -1;
			nm_id =  -1;
		}
		else if (i == 1) {
			cluster_id = 0;
			nm_id =  -1;
		}
		else if (i == 6) {
			cluster_id = 1;	
			nm_id =  -1;
		}
		else if (i == 11) {
			cluster_id = 2;
			nm_id =  -1;
		}
		else if (i == 16) {
			cluster_id = 3;
			nm_id =  -1;
		}

		all_core[i].cluster_id = cluster_id;
		all_core[i].nm_id = nm_id;

		test_ret[i] = -1;

		if (pthread_create(&test_thread[i], NULL, barrier_sync_test, (void *)((unsigned long)i))) {
			for (; i > 0; i--)
				pthread_cancel(test_thread[i - 1]);

			exit (7);
		}

		nm_id++;
	}

	sleep(1);

	pthread_mutex_lock(&work_mutex);
	do_wait = 0;
	pthread_cond_broadcast(&wait_cond);
	pthread_mutex_unlock(&work_mutex);

	i = 0;

	while (1) {
		ret = read(pipe_fd[0], &_test_thread, sizeof(pthread_t));
		if (ret != sizeof(pthread_t))
			exit (8);

		pthread_join(_test_thread, (void **)&_test_ret);

		if (*_test_ret == 0)
			 i++;

		pthread_mutex_lock(&work_mutex);
		in_progress--;

		if (!in_progress) {
			pthread_mutex_unlock(&work_mutex);
			break;
		}

		pthread_mutex_unlock(&work_mutex);
	}

	PL_CloseBoardDesc(board);

	if (i != 21)
		return -1;

	printf("OK: test successful!\n");

	return 0;
}

void *barrier_sync_test(void *arg)
{
	PL_Access	*access;
	int		index = (unsigned long)arg;
	pthread_t	self_thread;
	int		k, l;
	int		ret;

	pthread_mutex_lock(&work_mutex);
	in_progress++;
	pthread_mutex_unlock(&work_mutex);

	pthread_mutex_lock(&print_mutex);
	printf("Thread[%d] - Load program!\n", index);
	pthread_mutex_unlock(&print_mutex);

	access = NULL;

	if (PL_GetAccess(board, &all_core[index], &access) != PL_OK) {
		pthread_mutex_lock(&print_mutex);
		printf("ERROR[%d]: Failed get access!\n", index);
		pthread_mutex_unlock(&print_mutex);

		goto err_sync_exit;
	}

	if (all_core[index].nm_id < 0)
		if (all_core[index].cluster_id == -1)
			ret = PL_LoadProgramFile(access, CCPU_PART_FILE_NAME);
		else
			ret = PL_LoadProgramFile(access, PC_PART_FILE_NAME);
	else
		ret = PL_LoadProgramFile(access, NM_PART_FILE_NAME);

	if (ret != PL_OK) {
		pthread_mutex_lock(&print_mutex);
		printf("ERROR[%d]: Failed load program\n", index);
		pthread_mutex_unlock(&print_mutex);

		goto err_sync_exit;
	}

	pthread_mutex_lock(&work_mutex);
	if (do_wait)
		pthread_cond_wait(&wait_cond, &work_mutex);
	pthread_mutex_unlock(&work_mutex);

	pthread_mutex_lock(&print_mutex);
	printf("Thread[%d] - Start program!\n", index);
	pthread_mutex_unlock(&print_mutex);

	{
		DWORD Count = 0;
		PL_Word *Buffer = NULL;
		PL_Addr AddrLoad = 0;
		PL_Addr AddrStore = 0;
		PL_Word syncValue = 0;

		if (PL_SyncArray(access, 0, 0, 0, &syncValue, &AddrLoad, &Count) != PL_OK) {
			pthread_mutex_lock(&print_mutex);
			printf("ERROR[%d]: ERROR in sync 1!\n", index);
			pthread_mutex_unlock(&print_mutex);

			goto err_sync_exit;
		}

		if (syncValue != 1) {
			pthread_mutex_lock(&print_mutex);
			printf("ERROR[%d]: Wrong value in sync 1!\n", index);
			pthread_mutex_unlock(&print_mutex);

			goto err_sync_exit;
		}

		Buffer = malloc(sizeof(PL_Word) * Count);
		if (!Buffer) {
			pthread_mutex_lock(&print_mutex);
			printf("ERROR[%d]: Insufficient memory!\n", index);
			pthread_mutex_unlock(&print_mutex);

			goto err_sync_exit;
		}

		for (k = 1; k <= 10; k++) {
			if (PL_SyncArray(access, 1, 0, 0, &syncValue, &AddrStore, NULL) != PL_OK) {
				pthread_mutex_lock(&print_mutex);
				printf("ERROR[%d]: ERROR in sync 2!\n", index);
				pthread_mutex_unlock(&print_mutex);

				free(Buffer);
				goto err_sync_exit;
			}

			if (syncValue != 2) {
				pthread_mutex_lock(&print_mutex);
				printf("ERROR[%d]: Wrong value in sync 2!\n", index);
				pthread_mutex_unlock(&print_mutex);

				free(Buffer);
				goto err_sync_exit;
			}

			if (PL_ReadMemBlock(access, Buffer, AddrLoad, Count) != PL_OK) {
				pthread_mutex_lock(&print_mutex);
				printf("ERROR[%d]: Failed receive array!\n", index);
				pthread_mutex_unlock(&print_mutex);

				free(Buffer);
				goto err_sync_exit;
			}

			pthread_mutex_lock(&print_mutex);
			printf("Verify[%d] number %d ...\n", index, k);
			pthread_mutex_unlock(&print_mutex);

			for (l = 0; l < Count; l++) {
				if (Buffer[l] != k) {
					pthread_mutex_lock(&print_mutex);
					printf("ERROR[%d]: Bad buffer[%d] on step %d", index, l, k);
					pthread_mutex_unlock(&print_mutex);

					free(Buffer);
					goto err_sync_exit;
				}

				Buffer[l] = k + 1;
			}

			if (PL_WriteMemBlock(access, Buffer, AddrStore, Count) != PL_OK) {
				pthread_mutex_lock(&print_mutex);
				printf("ERROR[%d]: Failed send array!\n", index);
				pthread_mutex_unlock(&print_mutex);

				free(Buffer);
				goto err_sync_exit;
			}

			if (PL_Sync(access, 2, &syncValue) != PL_OK) {
				pthread_mutex_lock(&print_mutex);
				printf("ERROR[%d]: ERROR in sync 3!\n", index);
				pthread_mutex_unlock(&print_mutex);

				free(Buffer);
				goto err_sync_exit;
			}

			if (syncValue != 3) {
				pthread_mutex_lock(&print_mutex);
				printf("ERROR[%d]: Wrong value in sync 3!\n", index);
				pthread_mutex_unlock(&print_mutex);

				free(Buffer);
				goto err_sync_exit;
			}
		}

		free(Buffer);

		if (PL_Sync(access, 17, NULL) != PL_OK) {
			pthread_mutex_lock(&print_mutex);
			printf("ERROR[%d]: ERROR in sync 4!\n", index);
			pthread_mutex_unlock(&print_mutex);

			goto err_sync_exit;
		}

		pthread_mutex_lock(&print_mutex);
		printf("Test[%d] - done!\n", index);
		pthread_mutex_unlock(&print_mutex);
	}

	test_ret[index] = 0;

err_sync_exit:

	if (access)
		PL_CloseAccess(access);

	self_thread = pthread_self();

	ret = write(pipe_fd[1], &self_thread, sizeof(pthread_t));
	if (ret != sizeof(pthread_t))
		exit (8);

	sleep(1);

	pthread_exit(&test_ret[index]);
}
