#include <iostream>
#include <stdlib.h>

#if defined(MC12705)
#include "mc12705load.h"
#elif defined(NM_CARD)
#include "nm_card_load.h"
#elif defined(NM_MEZZO)
#include "nm_mezzo_load.h"
#elif defined(NM_QUAD)
#include "nm_quad_load.h"
#elif defined(NEURO_MEZZANINE)
#include "neuro_mezzanine_load.h"
#else
#error "Unknown board!"
#endif

using namespace std;

const char *NM_PART_FILE_NAME = "./nmc/nmc_part.abs";
const char *CCPU_PART_FILE_NAME = "./arm/central_arm_part.elf";
const char *PC_PART_FILE_NAME = "./arm/cluster_arm_part.elf";

#undef USE_CHAN_EDCL

#ifdef USE_CHAN_EDCL

unsigned char HOST_MAC_ADDRESS[] = {0x00, 0x14, 0xD1, 0x16, 0x69, 0xD0};
unsigned char BOARD_MAC_ADDRESS[] = {0xEC, 0x17, 0x66, 0x64, 0x08, 0x00};

#endif

int main()
{
	PL_Board	*board;
	const char	*arm_name;
	PL_Access	*access[5][5];
	PL_CoreNo	core;
	unsigned int	boardCount, doneCount;
	int		i, j;

#ifdef USE_CHAN_EDCL

	if (PL_SetChannelEDCL(HOST_MAC_ADDRESS, BOARD_MAC_ADDRESS) != PL_OK) {
		cout << "ERROR: Failed set channel EDCL!\n";
		exit(1);
	}

	boardCount = 1;

#else

	if (PL_GetBoardCount(&boardCount) != PL_OK) {
		cout << "ERROR: Failed get count of boards!\n";
		exit(1);
	}

#endif

	if (boardCount < 1) {
		cout << "ERROR: Failed find board!\n";
		exit(2);
	}

	if (PL_GetBoardDesc(0, &board) != PL_OK) {
		cout << "ERROR: Failed open board!\n";
		exit(3);
	}
#if 1
	if (PL_ResetBoard(board) != PL_OK) {
		cout << "ERROR: Failed reset board!\n";
		PL_CloseBoardDesc(board);
		exit(4);
	}

	if (PL_LoadInitCode(board) != PL_OK) {
		cout << "ERROR: Failed load init code!\n";
		PL_CloseBoardDesc(board);
		exit(5);
	}
#endif
	core.cluster_id = -1;
	arm_name = CCPU_PART_FILE_NAME;

	for (i = 0; i < 5; i++) {
		j = 0;
		core.nm_id = -1;

		access[i][j] = NULL;

		if ((PL_GetAccess(board, &core, &access[i][j]) != PL_OK) ||
		    (PL_LoadProgramFile(access[i][j], arm_name) != PL_OK)) {
			cout << "ERROR: Failed access or load arm processor!\n";

			if (access[i][j] != NULL)
				PL_CloseAccess(access[i][j]);

			for (; i > 0; i--) {
				for (j = 5; j > 0; j--) {
					if (i == 1) {
						PL_CloseAccess(access[0][0]);
						break;
					}
					else
						PL_CloseAccess(access[i - 1][j - 1]);
				}
			}

			PL_CloseBoardDesc(board);
			exit(6);
		}

		if (core.cluster_id == -1) {
			arm_name = PC_PART_FILE_NAME;
			core.cluster_id += 1;
			continue;
		}


		core.nm_id = 0;

		for (j = 1; j < 5; j++) {
			access[i][j] = NULL;

			if ((PL_GetAccess(board, &core, &access[i][j]) != PL_OK) ||
			    (PL_LoadProgramFile(access[i][j], NM_PART_FILE_NAME) != PL_OK)) {
				cout << "ERROR: Failed access or load nmc processor!\n";

				if (access[i][j] != NULL)
					PL_CloseAccess(access[i][j]);

				for (; j > 0; j--)
					PL_CloseAccess(access[i][j - 1]);

				for (; i > 0; i--) {
					for (j = 5; j > 0; j--) {
						if (i == 1) {
							PL_CloseAccess(access[0][0]);
							break;
						}
						else
							PL_CloseAccess(access[i - 1][j - 1]);
					}
				}

				PL_CloseBoardDesc(board);
				exit(7);
			}

			core.nm_id += 1;
		}

		core.cluster_id += 1;
	}

	doneCount = 0;

	for (i = 0; i < 5; i++) {
		for (j = 0; j < 5; j++) {
			if ((i == 0) && (j != 0))
				break;

			DWORD Count = 0;
			PL_Word *Buffer = NULL;
			PL_Addr AddrLoad = 0;
			PL_Addr AddrStore = 0;
			PL_Word syncValue = 0;

			if (PL_SyncArray(access[i][j], 0, 0, 0, &syncValue, &AddrLoad, &Count) != PL_OK) {
				cout << "ERROR: ERROR in sync 1!\n";
				goto err_test_end;
			}

			if (syncValue != 1) {
				cout << "ERROR: Wrong value in sync 1!\n";
				goto err_test_end;
			}

			try {
				Buffer = new PL_Word[Count];
			}
			catch (...) {
				cout << "ERROR: Insufficient memory!\n";
				goto err_test_end;
			}

			for (int k = 1; k <= 10; k++) {
				if (PL_SyncArray(access[i][j], 1, 0, 0, &syncValue, &AddrStore, NULL) != PL_OK) {
					cout << "ERROR: ERROR in sync 2!\n";
					delete[] Buffer;
					goto err_test_end;
				}

				if (syncValue != 2) {
					cout << "ERROR: Wrong value in sync 2!\n";
					delete[] Buffer;
					goto err_test_end;
				}

				if (PL_ReadMemBlock(access[i][j], Buffer, AddrLoad, Count) != PL_OK) {
					cout << "ERROR: Failed receive array!\n";
					delete[] Buffer;
					goto err_test_end;
				}

				cout << "Verify number " << k << "...\n";

				for (int l = 0; l < Count; l++) {
					if (Buffer[l] != k) {
						cout << "ERROR: Bad buffer[" << l << "] on step " << k << "!\n";
						delete[] Buffer;
						goto err_test_end;
					}

					Buffer[l] = k + 1;
				}

				if (PL_WriteMemBlock(access[i][j], Buffer, AddrStore, Count) != PL_OK) {
					cout << "ERROR: Failed send array!\n";
					delete[] Buffer;
					goto err_test_end;
				}

				if (PL_Sync(access[i][j], 2, &syncValue) != PL_OK) {
					cout << "ERROR: ERROR in sync 3!\n";
					delete[] Buffer;
					goto err_test_end;
				}

				if (syncValue != 3) {
					cout << "ERROR: Wrong value in sync 3!\n";
					delete[] Buffer;
					goto err_test_end;
				}
			}

			delete[] Buffer;

			if (PL_Sync(access[i][j], 17, NULL) != PL_OK) {
				cout << "ERROR: ERROR in sync 4!\n";
				goto err_test_end;
			}

			cout << "TEST[" << i <<"][" << j << "] - done!\n";
			doneCount++;
		}
	}

err_test_end:

	for (i = 0; i < 5; i++) {
		for (j = 0 ; j < 5; j++) {
			PL_CloseAccess(access[i][j]);

			if (i == 0)
				break;
		}
	}

	PL_CloseBoardDesc(board);

	if (doneCount != 21)
		exit(8);

	cout << "OK: test successful!\n";

	return 0;
}
