#include "init_code.h"
#include "libload_def.h"

extern void EMIC_init(unsigned int base);

extern int __stack_irq;
extern int __stack_svc;

#define container_of(ptr, type, member) ({			\
	const typeof(((type *)0)->member) * __mptr = (ptr);	\
	(type *)((char *)__mptr - offsetof(type, member)); })

volatile struct libload_table ll_tbl __attribute__((section (".ll_table")));
static void __attribute__((naked, optimize("-O0"))) wait_loop(int cl_id, void *arg);

void cli_loop(void *arg)
{
	volatile struct libload_table *tbl = (struct libload_table *)arg;
	volatile struct libload_top *ll_top = &tbl->top;
	void (*run_prog)(void);

#ifdef MC12705
	*((volatile unsigned int *)0x000CC010) |= 0x9;
	*((volatile unsigned int *)0x000CC000) |= 0x9;

	*((volatile unsigned int *)0x000CD000) = 0x0BEBC200;
	*((volatile unsigned int *)0x000CD018) = 0x0BEBC200;
	*((volatile unsigned int *)0x000CD008) = 0x83;

	asm("DMB":::"memory");
	*((volatile unsigned int *)0x000CD018) = 0x00000000;
	asm("DMB":::"memory");

	*((volatile unsigned int *)0x000CC000) |= 0x1;
	*((volatile unsigned int *)0x000CC000) &= ~0x8;
#endif

	if (*((volatile unsigned int *)tbl->ptr) != 0x5A5A5A5A) {

#ifndef USE_EDCL

		if (*((volatile unsigned int *)0x000D8090) & (1 << 16)) {
			unsigned int tmp;

			tmp = *((volatile unsigned int *)0x000D8094);
			*((volatile unsigned int *)0x000DC000) = ((tmp & ~0xFF) | 0x7);

			tmp = *((volatile unsigned int *)0x000D8098);
			*((volatile unsigned int *)0x000DC004) = tmp;

			*((volatile unsigned int *)0x000DC008) = 0x00000002;

			asm("DMB":::"memory");
			*((volatile unsigned int *)0x000DC018) = 0x40000007;
			asm("DMB":::"memory");
		}

#endif

		ll_top->arm.wait_loop = (unsigned int)wait_loop;

		ll_top->cluster[0].arm.wait_loop = ((unsigned int)wait_loop | CCPU_INBOUND_AXI_BASE);
		ll_top->cluster[1].arm.wait_loop = ((unsigned int)wait_loop | CCPU_INBOUND_AXI_BASE);
		ll_top->cluster[2].arm.wait_loop = ((unsigned int)wait_loop | CCPU_INBOUND_AXI_BASE);
		ll_top->cluster[3].arm.wait_loop = ((unsigned int)wait_loop | CCPU_INBOUND_AXI_BASE);

		asm("DMB":::"memory");
		*((volatile unsigned int *)tbl->ptr) = 0xA5A5A5A5;
		asm("DMB":::"memory");

		while (*((volatile unsigned int *)tbl->ptr) != 0x5A5A5A5A) { asm("NOP":::"memory"); };
	}

	while (1) {

#ifndef USE_EDCL

		if (*((volatile unsigned int *)0x000DC824) != 0x00000000) {
			*((volatile unsigned int *)0x000DC824) = 0x00000000;

			asm("DMB":::"memory");

			while (((*((volatile unsigned int *)0x000CB018) >> 16) & 0x3F) != 0x07) {
#ifdef MC12705
				if (*((volatile unsigned int *)0x000CD004) == 0x00000000) {
					*((volatile unsigned int *)0x000CC000) ^= 0x1;
					*((volatile unsigned int *)0x000CD000) = 0x0BEBC200;
				}
#endif
			};
		}

#endif

		switch (ll_top->arm.command) {
			case LIBLOAD_NONE_COMMAND:
#ifdef MC12705
				*((volatile unsigned int *)0x000CC000) |= 0x1;
#endif
				break;
			case LIBLOAD_RUN_PROGRAM:
#ifdef MC12705
				*((volatile unsigned int *)0x000CC000) |= 0xF;
				*((volatile unsigned int *)0x000CC010) &= ~0xF;

				*((volatile unsigned int *)0x000CD008) = 0x00;
				*((volatile unsigned int *)0x000CD00C) = 0x1;

				asm("DMB":::"memory");
#endif
				run_prog = (void *)ll_top->arm.entry;
				ll_top->arm.command = LIBLOAD_NONE_COMMAND;

				run_prog(); // Should newer return;

				// WARN RESET ALL CORES!!!
				*((volatile unsigned int *)0x000CA014) = 0x6; // SW_RESET TOP
				asm("DMB":::"memory");
				while (1) { asm("NOP":::"memory"); };
				
				break;
			default:
#ifdef MC12705
				*((volatile unsigned int *)0x000CC000) &= ~0x1;
#endif
				break;
		}
#ifdef MC12705
		if (*((volatile unsigned int *)0x000CD004) == 0x00000000) {
			*((volatile unsigned int *)0x000CC000) ^= 0x8;
			*((volatile unsigned int *)0x000CD000) = 0x0BEBC200;
		}
#endif
	}
}

static void wait_loop(int id, void *arg)
{
	volatile struct libload_table *tbl = (struct libload_table *)arg;
	volatile struct libload_top *ll_top = &tbl->top;
	unsigned int *stack;
	unsigned int state;

	if (id == NM6408_CCPU) {
		unsigned int *vaddr;

		asm volatile ("MRS %0, CPSR":"=r" (state)::"memory");
		state &= ~0x1DF;
		state |= 0x00D2;
		asm volatile ("MSR CPSR_fxc, %0"::"r" (state):"memory");
		stack = &__stack_irq;
		asm volatile ("MOV SP, %0"::"r" (stack):"memory");

		asm volatile ("MRS %0, CPSR":"=r" (state)::"memory");
		state &= ~0x1DF;
		state |= 0x0053;
		asm volatile ("MSR CPSR_fxc, %0"::"r" (state):"memory");
		stack = &__stack_svc;
		asm volatile ("MOV SP, %0"::"r" (stack):"memory");

		*((volatile unsigned int *)0x000B0000) = 0x00000000;
		*((volatile unsigned int *)0x000B1000) = 0x00000000;
		asm("DMB":::"memory");
		*((volatile unsigned int *)0x000B0184) = 0xFFFFFFFF;
		*((volatile unsigned int *)0x000B0188) = 0xFFFFFFFF;
		asm("DMB":::"memory");
		*((volatile unsigned int *)0x000B0104) = 0x00000040;
		*((volatile unsigned int *)0x000B0108) = 0x00003F00;
		asm("DMB":::"memory");
		*((volatile unsigned int *)0x000B1004) = 0x000000FF;
		*((volatile unsigned int *)0x000B1000) = 0x00000001;
		asm("DMB":::"memory");
		*((volatile unsigned int *)0x000B1008) = 0x00000002;
		*((volatile unsigned int *)0x000B0000) = 0x00000001;
		
		ll_top->arm.status = PROGRAM_STOPPED;

		vaddr = ((void *)&tbl->irq_table[0] - CCPU_INBOUND_AXI_BASE);
		asm volatile ("MOV R9, %0"::"r" (vaddr):"memory");
		cli_loop((void *)tbl);
	}


	asm volatile ("MRS %0, CPSR":"=r" (state)::"memory");
	state &= ~0x1DF;
	state |= 0x01D3;
	asm volatile ("MSR CPSR_fxc, %0"::"r" (state):"memory");

	*((volatile unsigned int *)0x000B0000) = 0x00000000;
	*((volatile unsigned int *)0x000B1000) = 0x00000000;

	ll_top->cluster[id].arm.status = PROGRAM_STOPPED;

	while (1) {
		register void (*call)(void);

		if (ll_top->cluster[id].arm.command != LIBLOAD_RUN_PROGRAM)
			continue;

		call = (void *)ll_top->cluster[id].arm.entry;
		ll_top->cluster[id].arm.command = LIBLOAD_NONE_COMMAND;

		call();
	}
}

static void isr_default_wdt(int id, void *args)
{
	volatile struct libload_top *top = (struct libload_top *)args;

	if (id < 0) {
		unsigned int tmp;

		tmp = *((volatile unsigned int *)0x000CE014) & 0x1;

		if (tmp)
			id = 38;
	}

	if (id == 38) {
		unsigned int tmp;

		tmp = *((volatile unsigned int *)0x000CE014) & 0x1;

		while (((*((volatile unsigned int *)0x000CB018) >> 16) & 0x3F) != 0x00) {
			if (*((volatile unsigned int *)0x000CE008) & 0x2)
				while (*((volatile unsigned int *)0x000CE010) & 0x1) {};

			tmp = *((volatile unsigned int *)0x000CE014) & 0x1;
			if (!tmp)
				break;
		}

		if (tmp) {
			asm("DMB":::"memory");
			*((volatile unsigned int *)0x000CA014) = 0x00000006;
			asm("DMB":::"memory");

			while (1) { asm("NOP":::"memory"); };
		}
	}

	return;
}

static void isr_default_pcie(int id, void *args)
{
	volatile struct libload_top *top = (struct libload_top *)args;

	if (id < 0) {
		unsigned int tmp1, tmp2;

		tmp1 = *((volatile unsigned int *)0x000DE0A0) & 0x101;
		tmp2 = *((volatile unsigned int *)0x000DE0A4) & 0x101;

		if (tmp1 & ~tmp2)
			id = 72;
	}

	if (id == 72) {
		unsigned int tmp;

		if (*((volatile unsigned int *)0x000D8090) & (1 << 16)) {
			unsigned int msi_data;
			unsigned int msi_addr;
			unsigned int msi_irq;

			tmp = *((volatile unsigned int *)0x000D8090);
			tmp = (1 << ((tmp >> 20) & 0x7));

			msi_irq = 0;

			msi_data = *((volatile unsigned int *)0x000D809C);
			msi_data = (msi_data & ~(tmp - 1)) | (msi_irq & (tmp - 1));

			msi_addr = *((volatile unsigned int *)0x000D8094) & 0xFF;
			msi_addr |= 0x40000000;

			// TODO: get MSI masks and set MSI pending bits
			
			asm("DMB":::"memory");
			*((volatile unsigned int *)msi_addr) = msi_data;
			asm("DMB":::"memory");
		}
		else {
			tmp = *((volatile unsigned int *)0x000CB018);
			tmp |= 0x100;
			*((volatile unsigned int *)0x000CB018) = tmp;

			asm("DMB":::"memory");

			*((volatile unsigned int *)0x000CB020) = 0x1;

			while (!(*((volatile unsigned int *)0x000CB020) & 0x4))
				asm("NOP":::"memory");
		}

		tmp = *((volatile unsigned int *)0x000DE0A0);

		asm("DMB":::"memory");
		*((volatile unsigned int *)0x000DE0A0) = tmp;
		asm("DMB":::"memory");

		top->arm.dma_status = tmp;

		while (top->arm.dma_status)
			asm("NOP":::"memory");
	}

	return;
}

static void isr_default_swi0_end(void)
{
	volatile struct libload_table	*tbl;
	volatile struct libload_top	*top;
	struct irq_entry		*irq_entry;
	unsigned long			table;

	asm volatile ("MOV %0, R9":"=r" (table)::"memory");
	irq_entry = (struct irq_entry *)table;
	tbl = container_of(irq_entry, struct libload_table, irq_table[0]);
	top = &tbl->top;

	if (top->arm.swi_command == SWI_BOARD_RESET) {
		top->arm.swi_command = 0;
		asm("DMB":::"memory");

#ifdef MC12705
		*((volatile unsigned int *)0x000CC010) |= 0xF;
		*((volatile unsigned int *)0x000CC000) &= ~0xF;
#endif

		while (*((volatile unsigned int *)0x000DC824) == 0) {};

		asm("DMB":::"memory");
		*((volatile unsigned int *)0x000DC824) = 0;
		asm("DMB":::"memory");

		while (((*((volatile unsigned int *)0x000CB018) >> 16) & 0x3F) != 0x00) {};

		asm("DMB":::"memory");
		*((volatile unsigned int *)0x000CA014) = 0x00000006;
		asm("DMB":::"memory");

		while (1) { asm("NOP":::"memory"); };
	}

	*((volatile unsigned int *)0x000C6014) = 0x00000006;
	while (!(*((volatile unsigned int *)0x000C601C) & 0x00000002)) {};

	*((volatile unsigned int *)0x000C7014) = 0x00000006;
	while (!(*((volatile unsigned int *)0x000C701C) & 0x00000002)) {};

	*((volatile unsigned int *)0x000C8014) = 0x00000006;
	while (!(*((volatile unsigned int *)0x000C801C) & 0x00000002)) {};

	*((volatile unsigned int *)0x000C9014) = 0x00000006;
	while (!(*((volatile unsigned int *)0x000C901C) & 0x00000002)) {};

	asm("NOP":::"memory");

	if (top->arm.swi_command == SWI_BOARD_SUSPEND) {
		top->arm.swi_command = SWI_BOARD_RESUME;
		asm("DMB":::"memory");

		if (!(*((volatile unsigned int *)0x000CB01C) & 0x4)) {
			while (!(*((volatile unsigned int *)0x000CB01C) & 0x1)) {};

			*((volatile unsigned int *)0x000D904C) = 0x0000;
			*((volatile unsigned int *)0x000D9040) = 0x00000000;

			asm("DMB":::"memory");

			*((volatile unsigned int *)0x000CB01C) = 0x2;
		}

		while ((*((volatile unsigned int *)0x000D8084) & 0x3) == 0x0) {};

		*((volatile unsigned int *)0x000CB018) |= (1 << 7); // TODO:

		while ((*((volatile unsigned int *)0x000D8084) & 0x3) != 0x0) {};

		if (!(*((volatile unsigned int *)0x000CB01C) & 0x4)) {
			while (*((volatile unsigned int *)0x000CB01C) & 0x2) {};

			*((volatile unsigned int *)0x000D9040) = 0x0000FFFF;
			*((volatile unsigned int *)0x000D904C) = 0x00FF;

			asm("DMB":::"memory");

			*((volatile unsigned int *)0x000CB01C) = 0x4;
		}
	}

	asm("NOP":::"memory");

	*((volatile unsigned int *)0x000C9014) = 0x00000004;
	while (*((volatile unsigned int *)0x000C901C) & 0x00000002) {};

	EMIC_init(PC3_INBOUND_AXI_BASE);

	*((volatile unsigned int *)0x000C8014) = 0x00000004;
	while (*((volatile unsigned int *)0x000C801C) & 0x00000002) {};

	EMIC_init(PC2_INBOUND_AXI_BASE);

	*((volatile unsigned int *)0x000C7014) = 0x00000004;
	while (*((volatile unsigned int *)0x000C701C) & 0x00000002) {};

	EMIC_init(PC1_INBOUND_AXI_BASE);

	*((volatile unsigned int *)0x000C6014) = 0x00000004;
	while (*((volatile unsigned int *)0x000C601C) & 0x00000002) {};

	EMIC_init(PC0_INBOUND_AXI_BASE);

	if (top->arm.swi_command != SWI_BOARD_RESUME) {
		*((volatile unsigned int *)tbl->ptr) = 0xFFFFFFFF;

		asm("DMB":::"memory");
		top->arm.status = PROGRAM_IS_RESET;
		asm("DMB":::"memory");
#if 0
		*((volatile unsigned int *)0x000CEC00) = 0x1ACCE551;
		*((volatile unsigned int *)0x000CE008) = 0x0;
		*((volatile unsigned int *)0x000CE00C) = 0x1;
		*((volatile unsigned int *)0x000CEC00) = ~0x1ACCE551;
#endif
		while (top->arm.status)
			asm("NOP":::"memory");
	}

	do {
		struct libload_ext	*ext = (struct libload_ext *)tbl->ptr;
		unsigned int		swi_command;
		unsigned int		post_status;
		unsigned int		version[2];
		unsigned int		serial[2];
		unsigned long		ptr;
		int			i;
		
		swi_command = top->arm.swi_command;

		version[0] = top->arm.version[0];
		version[1] = top->arm.version[1];

		serial[0] = top->arm.serial[0];
		serial[1] = top->arm.serial[1];

		post_status = top->arm.post_status;

		ptr = tbl->ptr;
		asm("NOP":::"memory");

		for (int i = 0; i < sizeof(*tbl) / 4; i++)
			*((__u32 *)tbl + i) = 0;

		asm("NOP":::"memory");
		tbl->ptr = ptr;

		top->arm.post_status = post_status;

		top->arm.version[0] = version[0];
		top->arm.version[1] = version[1];

		top->arm.serial[0] = serial[0];
		top->arm.serial[1] = serial[1];

		top->arm.swi_command = swi_command;

		ext->info.invalid = ~0;
#ifndef USE_EDCL
		ext->info.board_addr = *((volatile unsigned int *)0x000D922C);
#else
		ext->info.board_addr = *((volatile unsigned int *)0x000D001C);
#endif
		ext->info.board_indx = -1;

		for (i = 0; i < NM6408_CCPU; i++)
			ext->info.cluster_rdy[i] = 0;

		ext->info.board_cnt = 0;
	} while (0);

	top->arm.wait_loop = (unsigned int)wait_loop;

	top->cluster[0].arm.wait_loop = ((unsigned int)wait_loop | CCPU_INBOUND_AXI_BASE);
	top->cluster[1].arm.wait_loop = ((unsigned int)wait_loop | CCPU_INBOUND_AXI_BASE);
	top->cluster[2].arm.wait_loop = ((unsigned int)wait_loop | CCPU_INBOUND_AXI_BASE);
	top->cluster[3].arm.wait_loop = ((unsigned int)wait_loop | CCPU_INBOUND_AXI_BASE);

	if (top->arm.swi_command == SWI_BOARD_RESUME) {
		top->arm.swi_command = 0xFFFFFFFF;

		while (top->arm.swi_command)
			asm("NOP":::"memory");
	}

	*((volatile unsigned int *)0x000B0184) = 0xFFFFFFFF;
	*((volatile unsigned int *)0x000B0188) = 0xFFFFFFFF;
	asm("DMB":::"memory");
	*((volatile unsigned int *)0x000B0104) = 0x00000040;
	*((volatile unsigned int *)0x000B0108) = 0x00003F00;
	asm("DMB":::"memory");
	*((volatile unsigned int *)0x000B1004) = 0x000000FF;
	*((volatile unsigned int *)0x000B1000) = 0x00000001;
	asm("DMB":::"memory");
	*((volatile unsigned int *)0x000B1008) = 0x00000002;
	*((volatile unsigned int *)0x000B0000) = 0x00000001;

	asm("DMB":::"memory");
	*((volatile unsigned int *)tbl->ptr) = 0x5A5A5A5A;
	asm("DMB":::"memory");

	cli_loop((void *)tbl);

	while (1) {
		asm("NOP":::"memory");
	}
}

static void __attribute__((naked, optimize("-O0"))) isr_default_swi0_start(void);
static void isr_default_swi0_start(void)
{
	register void (*call)(void) = isr_default_swi0_end;
	register unsigned int *stack = &__stack_svc;

	asm volatile ("MOV SP, %0"::"r" (stack):"memory");
	asm volatile ("ISB":::"memory");
	asm volatile ("BX %0"::"r" (call):"memory");
}

// irq from 0 to 95
void irq_handler(int irq) asm("_irq_handler");
void irq_handler(int irq)
{
	struct libload_table	*tbl;
	struct libload_top	*top;
	struct irq_entry	*irq_entry;
	unsigned long		vectors;

	if ((irq < 0) || (irq > 95))
		return;

	asm("MRC p15, 0, %0, c12, c0, 0":"=r" (vectors)::"memory");

	irq_entry = (struct irq_entry *)((unsigned int *)vectors)[5];
	tbl = container_of(irq_entry, struct libload_table, irq_table[0]);
	top = &tbl->top;
	irq_entry = irq_entry + irq;

	if (irq == 0) {
		unsigned long table;
		unsigned int state;
		unsigned int *stack;
		void (*call)(void);

		table = (unsigned long)((unsigned int *)vectors)[5];
		asm("MOV R9, %0"::"r" (table):"memory");

		*((volatile unsigned int *)0x000B1010) = irq;
		asm("DMB":::"memory");
		*((volatile unsigned int *)0x000B0000) = 0x00000000;
		asm("DMB":::"memory");
		*((volatile unsigned int *)0x000B1000) = 0x00000000;
		asm("DMB":::"memory");

		call = isr_default_swi0_start;
		stack = &__stack_irq;

		asm volatile ("MRS %0, CPSR":"=r" (state)::"memory");
		state &= ~0x1DF;
		state |= 0x0053;
		asm volatile ("MSR SPSR_fxc, %0"::"r" (state):"memory");

		asm volatile ("MOV LR, %0"::"r" (call):"memory");
		asm volatile ("MOV SP, %0"::"r" (stack):"memory");
		asm volatile ("MOVS PC, LR":::"memory");

		while (1) {
			asm("NOP":::"memory");
		}
	}

	if (irq == 38) {
		isr_default_wdt(irq, top);
		return;
	}

	if ((irq >= 72) && (irq <= 77)) {
		isr_default_pcie(irq, top);
		return;
	}

	if (irq_entry->handler) {
		irq_entry->count++;
		irq_entry->handler(irq, irq_entry->args);
	}
	else {
		volatile struct libload_format	*format;
		format = &top->arm;

		if ((irq > 0) && (irq < 8))
			return;

		format->result = irq;
		asm volatile ("DMB":::"memory");
		format->status = PROGRAM_ABORTED;

		while (1) {
			if (irq == 0)
				irq_handler(irq);

			isr_default_pcie(-1, top);
			isr_default_wdt(-1, top);

			*((volatile unsigned int*)0x000B1010) = irq;
			asm volatile ("DMB":::"memory");
			irq = *((volatile unsigned int*)0x000B100C);
		}
	}

	return;
}

static void svc_default(int id, unsigned int *regs)
{
	int (*call)(void *arg);
	void *arg;
	int *ret;

	if ((id != 0) || (regs[3] != 0x64086408))
		return;

	call = (void *)regs[0];

	if (call) {
		int tmp;

		arg = (void *)regs[1];
		ret = (int *)regs[2];

		tmp = call(arg);

		if (ret)
			*ret = tmp;
	}

	return;
}

void svc_handler(int id, unsigned int *regs) asm("_svc_handler");
void svc_handler(int id, unsigned int *regs)
{
	struct irq_entry	*irq_entry;
	unsigned long		vectors;

	asm("MRC p15, 0, %0, c12, c0, 0":"=r" (vectors)::"memory");
	irq_entry = &((struct irq_entry *)((unsigned int *)vectors)[5])[96];

	if (irq_entry->handler)
		irq_entry->handler(id, regs);
	else
		svc_default(id, regs);

	return;
}

static void __attribute__((naked, optimize("-O0"))) isr_reset_wait(void);
static void isr_reset_wait(void)
{
	register unsigned int *stack = &__stack_irq;
	register unsigned int irq;

	asm volatile ("MOV SP, %0"::"r" (stack):"memory");
	asm volatile ("ISB":::"memory");
	asm volatile ("MOV %0, R9":"=r" (irq)::"memory");

	while (1) {
		if ((irq == 0) || (irq == 38) || ((irq >= 72) && (irq <= 77)))
			irq_handler(irq);

		*((volatile unsigned int *)0x000B1010) = irq;
		asm volatile ("DMB":::"memory");
		irq = *((volatile unsigned int *)0x000B100C);
	}
}

void __attribute__((noreturn)) exception_handler(void);
void exception_handler(void)
{
	struct libload_table		*tbl;
	struct libload_top		*top;
	volatile struct libload_format	*format;
	unsigned int			state;
	unsigned long			vectors;

	asm volatile ("MRS %0, CPSR":"=r" (state)::"memory");
	state |= 0x100;
	asm volatile ("MSR CPSR_fx, %0"::"r" (state):"memory");

	asm("MRC p15, 0, %0, c12, c0, 0":"=r" (vectors)::"memory");
	tbl = container_of((struct irq_entry *)((unsigned int *)vectors)[5],
			   struct libload_table, irq_table[0]);
	top = &tbl->top;
	format = &top->arm;

	format->result = 0xFFFFFFFF;
	asm volatile ("DMB":::"memory");
	format->status = PROGRAM_ABORTED;

	while (1) {
		unsigned int irq;
		void (*call)(void);

		irq = *((volatile unsigned int *)0x000B100C);

		if (irq == 0) {
			asm("MOV R9, %0"::"r" (irq):"memory");

			asm volatile ("MRS %0, CPSR":"=r" (state)::"memory");
			state &= ~0x1DF;
			state |= 0x00D2;
			asm volatile ("MSR SPSR_fxc, %0"::"r" (state):"memory");

			call = isr_reset_wait;

			asm volatile ("MOV LR, %0"::"r" (call):"memory");
			asm volatile ("MOVS PC, LR":::"memory");

			while (1) {
				asm("NOP":::"memory");
			}	
		}

		isr_default_pcie(-1, top);
		isr_default_wdt(-1, top);

		*((volatile unsigned int *)0x000B1010) = irq;
	}
}

void __attribute__((alias("exception_handler"))) undefined_handler(void) asm("_undefined_handler");
void __attribute__((alias("exception_handler"))) prefetch_handler(void) asm("_prefetch_handler");
void __attribute__((alias("exception_handler"))) abort_handler(void) asm("_abort_handler");
void __attribute__((alias("exception_handler"))) fiq_handler(void) asm("_fiq_handler");

