.extern __exec_begin
.extern __exec_limit

.extern __stack_mon
.extern __stack_irq
.extern __stack_fiq
.extern __stack_abt
.extern __stack_und
.extern __stack_usr
.extern __stack_svc

.extern __bss_start
.extern __bss_end

#define Mode_USR	0x10
#define Mode_FIQ	0x11
#define Mode_IRQ	0x12
#define Mode_SVC	0x13
#define Mode_MON	0x16
#define Mode_ABT	0x17
#define Mode_UND	0x1B
#define Mode_SYS	0x1F

#define I_Bit		0x80	/* when I bit is set, IRQ is disabled */
#define F_Bit		0x40	/* when F bit is set, FIQ is disabled */
#define A_Bit		0x100	/* when A bit is set, Data Abort Interrupts is disabled */

#define CR_V		0x2000	/* Vector relocate to 0xFFFF0000 */

.extern _undefined_handler
.extern _svc_handler
.extern _prefetch_handler
.extern _abort_handler
.extern _irq_handler
.extern _fiq_handler

.extern _main
.weak _start
.global _reset

.global init_hook
.set init_hook, reset_handler

.code 32

.section .heap.reserved, "aw",%nobits
.align 3
init_ready:
.space 0x1000
local_stack:

.section .text.vectors,"ax",%progbits
.align 5

_reset:
	B jump_reset

#ifdef USE_EDCL

	LDR PC, undefined_addr
	LDR PC, svc_addr
	LDR PC, prefetch_addr
	LDR PC, abort_addr
nop_0:	.word 0
	LDR PC, irq_addr
	LDR PC, fiq_addr

dead_loop:
	B dead_loop

undefined_addr:	.word dead_loop
svc_addr:	.word dead_loop
prefetch_addr:	.word dead_loop
abort_addr:	.word dead_loop
irq_addr:	.word dead_loop
fiq_addr:	.word dead_loop

#else /* USE_EDCL */

	B undefined_wrapper
	B svc_wrapper
	B prefetch_wrapper
	B abort_wrapper
nop_0:	.word 0
	B irq_wrapper
	B fiq_wrapper

dead_loop:
	B dead_loop

undefined_wrapper:
	PUSH {R0-R4, R9, R12, LR}
	LDR R0, undefined_addr
	BLX R0
	POP {R0-R4, R9, R12, LR}
	MOVS PC, LR

svc_wrapper:
	STMFD SP!, {R0-R4, R9, R12, LR}
	LDR R2, svc_addr
	MOV R1, SP
	MRS R0, SPSR
	TST SP, #0x7
	STREQ R1, [SP, #-4]!
	STR R0, [SP, #-4]!
	LDR R0, [LR, #-4]
	BIC R0, R0, #0xFF000000
	BLX R2
	LDR R0, [SP], #4
	TST SP, #0x7
	LDRNE R1, [SP], #4
	MSR SPSR_cf, R0
	LDMFD SP!, {R0-R4, R9, R12, PC}^

prefetch_wrapper:
	PUSH {R0-R4, R9, R12, LR}
	LDR R0, prefetch_addr
	BLX R0
	POP {R0-R4, R9, R12, LR}
	SUBS PC, LR, #4

abort_wrapper:
	PUSH {R0-R4, R9, R12, LR}
	LDR R0, abort_addr
	BLX R0
	POP {R0-R4, R9, R12, LR}
	SUBS PC, LR, #8

irq_wrapper:
	PUSH {R0-R4, R9, R12, LR}
	LDR R2, irq_addr
	MOV R1, #0xB1000
	LDR R0, [R1, #0xC] // IAR
	PUSH {R0, R1}
	BLX R2
	POP {R0, R1}
	STR R0, [R1, #0x10] // EOIR
	POP {R0-R4, R9, R12, LR}
	SUBS PC, LR, #4

fiq_wrapper:
	PUSH {R0-R4, LR}
	LDR R0, fiq_addr
	BLX R0
	POP {R0-R4, LR}
	SUBS PC, LR, #4

undefined_addr:	.word _undefined_handler
svc_addr:	.word _svc_handler
prefetch_addr:	.word _prefetch_handler
abort_addr:	.word _abort_handler
irq_addr:	.word _irq_handler
fiq_addr:	.word _fiq_handler

#endif /* USE_EDCL */

jump_reset:
	MOV R0, PC
	LDR R1, =check_pc
check_pc:
	SUBS R2, R0, R1

	LDR R3, =local_stack
	ADD R3, R3, R2
	MOV SP, R3

	PUSH {R0-R1}

	LDR R3, reset_addr
	ADD R3, R3, R2
	MOV PC, R3

reset_addr: .word _start

.section .text.startup, "ax",%progbits
.align 2

_start:
reset_handler:
	POP {R0-R1}

	SUBS R5, R0, R1
	BEQ copy_done

	LDR R1, =EMIC_init
	ADD R1, R1, R5
	MOV R0, #0

	PUSH {R0-R12}
	BLX R1
	POP {R0-R12}

	LDR R0, =__exec_begin
	ADD R1, R0, R5
	LDR R2, =__exec_limit

copy_loop:
	CMP R0, R2
	BEQ copy_done
	LDR R3, [R1], #4
	STR R3, [R0], #4
	B copy_loop

copy_done:
	LDR R3, =init_ready
	MVN R4, #0
	STR R4, [R3]

	LDR R3, =0x640810ad
	LDR R4, =_reset
	ADD R4, R4, R5
	STR R3, [R4]

	SUB R5, PC, R5
	MOV PC, R5

	MRS R0, CPSR
	BIC R0, R0, #0x1F;
	ORR R0, R0, #(Mode_SVC | I_Bit | F_Bit)
	ORR R0, R0, #A_Bit
	MSR CPSR_fxc, R0

	MRC p15, 0, R0, c1, c0, 0 	/* Read CP15 SCTLR */
	BIC R0, #CR_V			/* V = 0 */
	MCR p15, 0, R0, c1, c0, 0 	/* Write CP15 SCTLR */

	MSR CPSR_c, #(Mode_MON | I_Bit | F_Bit)
	LDR SP, =__stack_mon

	MSR CPSR_c, #(Mode_IRQ | I_Bit | F_Bit)
	LDR SP, =__stack_irq

	MSR CPSR_c, #(Mode_FIQ | I_Bit | F_Bit)
	LDR SP, =__stack_fiq

	MSR CPSR_c, #(Mode_ABT | I_Bit | F_Bit)
	LDR SP, =__stack_abt

	MSR CPSR_c, #(Mode_UND | I_Bit | F_Bit)
	LDR SP, =__stack_und

	MSR CPSR_c, #(Mode_SYS | I_Bit | F_Bit)
	LDR SP, =__stack_usr

	LDR R0, =__bss_start
	LDR R1, =__bss_end
	MOV R2, #0

zero_loop:
	CMP R1, R0
	BEQ call_main
	STR R2, [R0], #4
	B zero_loop

call_main:
	MSR CPSR_c, #(Mode_SVC | I_Bit | F_Bit)
	LDR SP, =__stack_svc

	BL _main

finish_loop:
	NOP
	B finish_loop
