#ifndef __INIT_CODE_H__
#define __INIT_CODE_H__

#include <stdint.h>
#include <stddef.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef uint32_t	__u32;
typedef uint16_t	__u16;
typedef uint8_t		__u8;

struct irq_entry {
	void (*handler)(int id, void *args);
	void		*args;

	__u32		prior_flags;
	__u32		count;
} __attribute__ ((packed, aligned(4)));

struct libload_format {
	__u32		status;
	__u32		result;
	__u32		reserved0[2];
	__u32		command;
	__u32		entry;
	__u32		wait_loop;
	__u32		post_status;
	__u32		sync[8];
	__u32		version[2];
	__u32		serial[2];
	__u32		is_busy;
	__u32		dma_status;
	__u32		swi_command;
	__u32		reserved1[7]; // used in extention
	__u32		rpc_region[2];
} __attribute__ ((packed, aligned(4)));

struct libload_cluster {
	struct libload_format	arm;
	struct libload_format	nmc[4];
} __attribute__ ((packed, aligned(4)));

struct libload_top {
	struct libload_format	arm;
	struct libload_cluster	cluster[4];
} __attribute__ ((packed, aligned(4)));

struct libload_table {
	struct libload_top	top;	// It is fixed position, don't change!!!

	// TODO:

	struct irq_entry	irq_table[100];
	unsigned long		ptr; // Store address of libload extention!!!
} __attribute__ ((packed, aligned(4)));

struct libload_ext {
	// Condition of init code ready (must be 0x5A5A5A5A), it's used from HOST side.
	const __u32	reserved; // It is fixed position, don't use it!!!

	// TODO:

	// Maximum size in byte of memory reverved to 'struct board_info'.
	__u32		info_max_size;

	struct board_info {
		__u32	invalid;

		__u32	board_addr;
		__u32	board_indx;

		int gpio_port[5][8];

		// TODO:

		__u32	cluster_rdy[4];
		__u32	board_cnt;

		int connect[][4];
	} info;
} __attribute__ ((packed, aligned(4)));

#ifdef __cplusplus
};
#endif

#endif // __INIT_CODE_H__
