#ifndef IMAGE_VERSION_MAJOR
#define IMAGE_VERSION_MAJOR	2
#endif

#ifndef IMAGE_VERSION_MINOR
#define IMAGE_VERSION_MINOR	1
#endif

#ifndef BUILD_Y
#define BUILD_Y 0
#endif

#ifndef BUILD_M
#define BUILD_M 0
#endif

#ifndef BUILD_D
#define BUILD_D 0
#endif

#ifndef BUILD_N
#define BUILD_N 0
#endif

#define IMAGE_MAJOR_MINOR	((IMAGE_VERSION_MAJOR << 16) | (IMAGE_VERSION_MINOR & 0xFFFF))
#define IMAGE_DATE_REVID	((((BUILD_Y << 9) | (BUILD_M << 5) | (BUILD_D << 0)) << 16) | (BUILD_N & 0xFFFF))

#define INIT_CODE_READY		0x0006F000 // To previous version compatibility

.extern __rel_start
.extern __rel_end
.extern __rel_base

.extern __abs_desc

.extern __bss_start
.extern __bss_end

.extern __stack_limit
.weak __stack
.extern __stack_default

.extern init_hook
.global _start

.code 32

.section .text.reset, "ax",%progbits
.align 5

_start:
	B jump_reset
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

dead_loop:
	NOP
	B dead_loop

jump_reset:
	MOV R0, PC
	LDR R1, =check_pc
check_pc:
	SUB R9, R0, R1
	LDR R1, reset_addr
	ADD R1, R1, R9

	MOV PC, R1

reset_addr: .word reset_handler

#ifndef USE_EDCL
.section .rodata.magic, "a",%progbits

.set magic_pointer, magic_addr
magic_addr: .word 0x000DC870 // Early: __rel_base == 0x0006F000

magic_code:
#if defined(MC12705)
	.word 0x64080001
#elif defined(NM_CARD)
	.word 0x64080002
#elif defined(NM_MEZZO)
	.word 0x64080003
#elif defined(NM_QUAD)
	.word 0x64080004
#elif defined (NEURO_MEZZANINE)
	.word 0x64080005
#else
	#error "Unknow board type!"
#endif

version_major: .word IMAGE_MAJOR_MINOR
version_minor: .word IMAGE_DATE_REVID

serial_lo: .word -1
serial_hi: .word -1

load_begin: .word __rel_start // 0x00000000
load_limit: .word __rel_base  // 0x00070000

is_rumboot: .word 0
data_point: .word __abs_desc

post_test: .word 0
post_addr: .word -1
post_size: .word 0
.set magic_size, 52
#endif

.section .text.init, "ax",%progbits
.align 2

reset_handler:
	MOV R0, #0x4

	LDR R1, =0x000CA014
	STR R0, [R1, #0x0]

	LDR R1, =0x000C6014
	STR R0, [R1, #0x0]

	LDR R1, =0x800CA014
	STR R0, [R1, #0x0]

	LDR R1, =0x000C7014
	STR R0, [R1, #0x0]

	LDR R1, =0xA00CA014
	STR R0, [R1, #0x0]

	LDR R1, =0x000C8014
	STR R0, [R1, #0x0]

	LDR R1, =0xC00CA014
	STR R0, [R1, #0x0]

	LDR R1, =0x000C9014
	STR R0, [R1, #0x0]

	LDR R1, =0xE00CA014
	STR R0, [R1, #0x0]

	DMB

	LDR R1, =0x000C0000
	LDR R0, [R1, #0x0]
	TST R0, #1
	BNE skip_pll

#if 1 // Normal clock (PLL)
	LDR R1, =0x000CA000
	LDR R0, =0x0810012A
	STR R0, [R1, #0x0]
	DMB

	MOV R0, #4

delay_loop_cc:
	SUBS R0, R0, #1
	BNE delay_loop_cc

setup_loop1_cc:
	LDR R0, [R1, #0x04]
	TST R0, #1
	BEQ setup_loop1_cc

	MOV R0, #0x40
	STR R0, [R1, #0x010]

setup_loop2_cc:
	LDR R0, [R1, #0x010]
	CMP R0, #0x00000040
	BNE setup_loop2_cc
#else // Low clock (PLL): Need make Macros or Function
	LDR R1, =0x000CA000
	LDR R0, =0x081000C4
	STR R0, [R1, #0x0]
	DMB

	MOV R0, #4

delay_loop_cc:
	SUBS R0, R0, #1
	BNE delay_loop_cc

setup_loop1_cc:
	LDR R0, [R1, #0x04]
	TST R0, #1
	BEQ setup_loop1_cc

	MOV R0, #0x30
	STR R0, [R1, #0x010]

setup_loop2_cc:
	LDR R0, [R1, #0x010]
	CMP R0, #0x00000030
	BNE setup_loop2_cc

	LDR R1, =0x000C6000
	LDR R0, =0x081000C4
	STR R0, [R1, #0x0]
	DMB

	MOV R0, #4

delay_loop_c0:
	SUBS R0, R0, #1
	BNE delay_loop_c0

setup_loop1_c0:
	LDR R0, [R1, #0x04]
	TST R0, #1
	BEQ setup_loop1_c0

	MOV R0, #0x30
	STR R0, [R1, #0x010]

setup_loop2_c0:
	LDR R0, [R1, #0x010]
	CMP R0, #0x00000030
	BNE setup_loop2_c0

	LDR R1, =0x800CA000
	LDR R0, =0x0C3000F8
	STR R0, [R1, #0x0]
	DMB

	MOV R0, #4

delay_loop_n0:
	SUBS R0, R0, #1
	BNE delay_loop_n0

setup_loop1_n0:
	LDR R0, [R1, #0x04]
	TST R0, #1
	BEQ setup_loop1_n0

	MOV R0, #0x00
	STR R0, [R1, #0x010]

setup_loop2_n0:
	LDR R0, [R1, #0x010]
	CMP R0, #0x00000000
	BNE setup_loop2_n0

	LDR R1, =0x000C7000
	LDR R0, =0x081000C4
	STR R0, [R1, #0x0]
	DMB

	MOV R0, #4

delay_loop_c1:
	SUBS R0, R0, #1
	BNE delay_loop_c1

setup_loop1_c1:
	LDR R0, [R1, #0x04]
	TST R0, #1
	BEQ setup_loop1_c1

	MOV R0, #0x30
	STR R0, [R1, #0x010]

setup_loop2_c1:
	LDR R0, [R1, #0x010]
	CMP R0, #0x00000030
	BNE setup_loop2_c1

	LDR R1, =0xA00CA000
	LDR R0, =0x0C3000F8
	STR R0, [R1, #0x0]
	DMB

	MOV R0, #4

delay_loop_n1:
	SUBS R0, R0, #1
	BNE delay_loop_n1

setup_loop1_n1:
	LDR R0, [R1, #0x04]
	TST R0, #1
	BEQ setup_loop1_n1

	MOV R0, #0x00
	STR R0, [R1, #0x010]

setup_loop2_n1:
	LDR R0, [R1, #0x010]
	CMP R0, #0x00000000
	BNE setup_loop2_n1

	LDR R1, =0x000C8000
	LDR R0, =0x081000C4
	STR R0, [R1, #0x0]
	DMB

	MOV R0, #4

delay_loop_c2:
	SUBS R0, R0, #1
	BNE delay_loop_c2

setup_loop1_c2:
	LDR R0, [R1, #0x04]
	TST R0, #1
	BEQ setup_loop1_c2

	MOV R0, #0x30
	STR R0, [R1, #0x010]

setup_loop2_c2:
	LDR R0, [R1, #0x010]
	CMP R0, #0x00000030
	BNE setup_loop2_c2

	LDR R1, =0xC00CA000
	LDR R0, =0x0C3000F8
	STR R0, [R1, #0x0]
	DMB

	MOV R0, #4

delay_loop_n2:
	SUBS R0, R0, #1
	BNE delay_loop_n2

setup_loop1_n2:
	LDR R0, [R1, #0x04]
	TST R0, #1
	BEQ setup_loop1_n2

	MOV R0, #0x00
	STR R0, [R1, #0x010]

setup_loop2_n2:
	LDR R0, [R1, #0x010]
	CMP R0, #0x00000000
	BNE setup_loop2_n2

	LDR R1, =0x000C9000
	LDR R0, =0x081000C4
	STR R0, [R1, #0x0]
	DMB

	MOV R0, #4

delay_loop_c3:
	SUBS R0, R0, #1
	BNE delay_loop_c3

setup_loop1_c3:
	LDR R0, [R1, #0x04]
	TST R0, #1
	BEQ setup_loop1_c3

	MOV R0, #0x30
	STR R0, [R1, #0x010]

setup_loop2_c3:
	LDR R0, [R1, #0x010]
	CMP R0, #0x00000030
	BNE setup_loop2_c3

	LDR R1, =0xE00CA000
	LDR R0, =0x0C3000F8
	STR R0, [R1, #0x0]
	DMB

	MOV R0, #4

delay_loop_n3:
	SUBS R0, R0, #1
	BNE delay_loop_n3

setup_loop1_n3:
	LDR R0, [R1, #0x04]
	TST R0, #1
	BEQ setup_loop1_n3

	MOV R0, #0x00
	STR R0, [R1, #0x010]

setup_loop2_n3:
	LDR R0, [R1, #0x010]
	CMP R0, #0x00000000
	BNE setup_loop2_n3
#endif // PLL clock

skip_pll:

#ifdef USE_EDCL

	LDR R2, =0x000D0000
	LDR R3, [R2]

	LDR R0, [R2, #0x2C]
	LDR R1, [R2, #0x1C]
	DMB

	ORR R3, R3, #(1<<6)
	ORR R3, R3, #(1<<12)
	ORR R3, R3, #(1<<14)
	STR R3, [R2]

setup_loop_eth:
	LDR R4, [R2]
	TST R4, #(1<<6)
	BNE setup_loop_eth

	STR R0, [R2, #0x2C]
	STR R1, [R2, #0x1C]
	DMB

	BIC R3, R3, #(1<<14)
	BIC R3, R3, #(1<<6)
	STR R3, [R2]

	LDR R0, =init_hook
	ADD R0, R0, R9

	MOV LR, PC
	MOV PC, R0

#else /* USE_EDCL */

#if 0
	LDR R2, =0x000D0000
	LDR R0, [R2, #0x00]
	AND R0, R0, #0xFFFFBFFF
	STR R0, [R2, #0x00]
#endif

#if defined(MC12705)
	LDR R0, =0x000CC010
	LDR R1, =0x000CC000
	LDR R2, =0x00000006

	LDR R3, [R0]
	ORR R3, R3, R2
	STR R3, [R0]

	LDR R3, [R1]
	LDR R2, =0x00000006
	ORR R3, R3, R2
	LDR R2, =0x00000002
	BIC R3, R3, R2
	STR R3, [R1]
#endif

	LDR R4, =__rel_start
	ADD R4, R4, R9

	LDR R5, =__rel_end
	ADD R5, R5, R9

	LDR R6, =__rel_base
	LDR R7, =__stack_limit

	LDR R8, =desc_pointer
	ADD R8, R8, R9

	SUB R0, R5, R4
	SUB R1, R7, R6

	CMP R0, R1
	BHI finish_loop

	SUBS R10, R6, R4
	BEQ enable_pci_setup

	CMP R5, R6
	BLS fill_desc

	LDR R7, =__stack_default
	CMP R4, R7
	BLO finish_loop

fill_desc:
	LDR R0, =desc_pointer
	LDR R1, =__rel_start
	SUB R0, R0, R1

	LDR R1, =__rel_base
	ADD R1, R1, R0

	LDR R2, =desc_size
	ADD R2, R2, R1

	MOV R8, R1
	MOV R3, #0

zero_desc:
	STR R3, [R1], #4
	CMP R1, R2
	BNE zero_desc

enable_pci_setup:
	LDR R1, =0x000CB018
	LDR R0, [R1, #0x0]
	BIC R0, R0, #0x30
	STR R0, [R1, #0x0]
	DMB
	LDR R1, =0x000CB00C
	LDR R0, [R1, #0x0]
	TST R0, #1
	BNE check_pci_setup

#if 0
	LDR R1, =0x000E0790
	LDR R0, =0x00000360
	STR R0, [R1, #0x0]
	DMB
#endif

	LDR R1, =0x000CB018
#if !defined(NM_QUAD)
	LDR R0, =0x00000009
#else
	LDR R0, =0x00000001
#endif
	STR R0, [R1, #0x0]
	DMB

	LDR R1, =0x000CB00C
	LDR R0, =0x00000001
	STR R0, [R1, #0x0]

setup_loop1_pci:
	LDR R0, [R1, #0x0]
	TST R0, #1
	BEQ setup_loop1_pci

	LDR R1, =0x000D9044
	MOVW R0, #0x17CD
	MOVT R0, #0x17CD
	STR R0, [R1]

	LDR R1, =0x000D8000
	LDR R0, [R1]
#if defined(MC12705)
	MOVT R0, #0x0001
#elif defined(NM_CARD)
	MOVT R0, #0x0002
#elif defined(NM_MEZZO)
	MOVT R0, #0x0003
#elif defined(NM_QUAD)
	MOVT R0, #0x0004
#elif defined(NEURO_MEZZANINE)
	MOVT R0, #0x0005
#else
	#error "Unknow board type!"
#endif
	STR R0, [R1]

	LDR R1, =0x000D8008
	LDR R0, =0xFF000000
	STR R0, [R1, #0x0]

	LDR R1, =0x000D8090
	LDR R0, =0x018AC005
	STR R0, [R1, #0x0]
#if 0
	LDR R1, =0x000D80C8
	LDR R0, =0x00002000
	STR R0, [R1, #0x0]
#endif
	LDR R1, =0x000D9240
	LDR R0, =0x00CF008D
	STR R0, [R1, #0x0]

#if !defined(NM_MEZZO) && !defined(NEURO_MEZZANINE)
	LDR R1, =0x000D9244
	LDR R0, =0x000000F2
	STR R0, [R1, #0x0]
#endif

	LDR R1, =0x000DC840
	LDR R0, =0x60000000
	STR R0, [R1, #0x0]

	LDR R1, =0x000DC850
	LDR R0, =0x00000000
	STR R0, [R1, #0x0]

	LDR R1, =0x000DC860
	LDR R0, =0x00000000
	STR R0, [R1, #0x0]

	LDR R1, =0x000D904C
	LDR R0, =0x000000FF
	STR R0, [R1, #0x0]

	LDR R1, =0x000D9048
	LDR R0, =0x000003FF
	STR R0, [R1, #0x0]

	LDR R1, =0x000D9040
	LDR R0, =0x0000FFFF
	STR R0, [R1, #0x0]

	LDR R1, =0x000CB01C
	LDR R0, =0x00000004
	STR R0, [R1, #0x0]

	LDR R1, =0x000D900C
	LDR R0, =0x020203E0
	STR R0, [R1, #0x0]

	LDR R1, =0x000D9010
	LDR R0, =0x4002C020
	STR R0, [R1, #0x0]

check_pci_setup:
	LDR R1, =INIT_CODE_READY
	MOV R0, #0
	STR R0, [R1, #0x0]
	DMB
	LDR R1, =0x000CB018
	LDR R0, [R1, #0x0]
	ORR R0, R0, #0x30
	STR R0, [R1, #0x0]

#if defined(MC12705)
	LDR R0, =0x000CC000
	LDR R1, =0x00000002
	LDR R2, =0x00000004

	LDR R3, [R0]
	ORR R3, R3, R1
	BIC R3, R3, R2
	STR R3, [R0]
#endif

	LDR R1, =0x000CD008
	LDR R0, =0x00
	STR R0, [R1, #0x0]

	LDR R1, =0x000CD028
	LDR R0, =0x00
	STR R0, [R1, #0x0]

	LDR R1, =0x000CD000
	LDR R0, =0x23c34600
	STR R0, [R1, #0x0]

	LDR R1, =0x000CD018
	LDR R0, =0x23c34600
	STR R0, [R1, #0x0]

	LDR R1, =0x000CD008
	LDR R0, =0x83
	STR R0, [R1, #0x0]

	DMB

	LDR R1, =0x000CD018
	LDR R0, =0x00000000
	STR R0, [R1, #0x0]

	LDR R1, =0x000D9000
	LDR R2, =0x000CD004

setup_loop2_pci:
	LDR R0, [R2, #0x0]
	CMP R0, #0
	BEQ setup_loop2_end

	LDR R0, [R1, #0x0]
	TST R0, #1
	BEQ setup_loop2_pci

setup_loop2_end:
	LDR R1, =0x000CD000
	LDR R0, =0x0BEBC200
	STR R0, [R1, #0x0]

	DMB

	LDR R1, =0x000CB018

lock_loop_pci:
	LDR R0, [R1, #0x0]
	LSR R0, R0, #16
	AND R0, R0, #3

	CMP R0, #3
	BEQ lock_loop_end

	LDR R3, [R2, #0x0]
	CMP R3, #0
	BNE lock_loop_pci

	TST R0, #3
	BNE lock_loop_pci

	@B finish_loop

lock_loop_end:
	LDR R1, =0x000CD008
	LDR R0, =0x00
	STR R0, [R1, #0x0]

	LDR R1, =0x000CD00C
	LDR R0, =0x1
	STR R0, [R1, #0x0]

#if defined(MC12705)
	LDR R1, =0x000CC000
	LDR R2, =0x00000002

	LDR R3, [R1]
	BIC R3, R3, R2
	STR R3, [R1]
#endif

	LDR R1, =0x000D1000
	LDR R0, [R1, #0x0C]
	BIC R0, R0, #1
	STR R0, [R1, #0x0C]
	LDR R0, [R1, #0x04]
	BIC R0, R0, #1
	STR R0, [R1, #0x04]
	LDR R0, [R1, #0x00]
	ORR R0, R0, #1
	STR R0, [R1, #0x00]
	LDR R0, =0x000007CF
	STR R0, [R1, #0x10]
	LDR R0, =0x00000D9F
	STR R0, [R1, #0x14]
	LDR R0, [R1, #0x08]
	BIC R0, R0, #1
	STR R0, [R1, #0x08]
	LDR R0, [R1, #0x00]
	BIC R0, R0, #1
	STR R0, [R1, #0x00]

lock_loop_tsns:
	LDR R0, [R1, #0x1c]
	TST R0, #1
	BNE lock_loop_tsns

	LDR R0, [R1, #0x04]
	ORR R0, R0, #1
	STR R0, [R1, #0x04]

	LDR R7, .Lstack
	CMP R7, #0
	LDREQ R7, =__stack_default

	MOV SP, R7

	LDR R7, =magic_pointer
	ADD R7, R7, R9

	// TODO: POST_TEST

	CMP R10, #0
	BEQ relocate_done

relocate_code:
	LDR R7, [R4], #4
	STR R7, [R6], #4
	CMP R4, R5
	BNE relocate_code

	ADD R5, R10, PC
	MOV PC, R5

relocate_done:
	LDR R0, =__rel_start
	LDR R1, =__bss_start
	LDR R2, =__bss_end
	LDR R3, =__rel_base

	SUB R1, R1, R0
	ADD R1, R1, R3

	SUB R2, R2, R0
	ADD R2, R2, R3

	LDR R4, =__stack_limit
	CMP R2, R4
	MOV R4, #0
	BLS zero_loop

	LDR R4, =__stack_default
	CMP R1, R4
	MOV R4, #0
	BLO finish_loop

zero_loop:
	CMP R2, R1
	BEQ set_ready_flag
	STR R4, [R1], #4
	B zero_loop

set_ready_flag:
	LDR R9, =magic_pointer
	SUB R9, R9, R0
	ADD R9, R9, R3

	MOV R4, #0

	LDR R3, [R9, #0x0]
	STR R4, [R3, #0x0]

	MVN R4, #0

	DMB
	STR R4, [R8, #0x0]
	DMB

	LDR R4, =0x000CB018
	LDR R5, =0x000DC824
	LDR R6, =0x00000001

wait_load_end:
	LDR R7, [R5, #0x0]
	BIC R7, R7, R6
	STR R7, [R5, #0x0]

wait_link_ready:
	LDR R2, [R4, #0x0]
	LSR R2, R2, #16
	AND R2, R2, #7
	CMP R2, #7
	BNE wait_link_ready

	LDR R2, [R3, #0x0]
	CMP R2, #0x0
	BEQ wait_load_end

	LDR R0, [R8, #0x8]
	LDR R1, [R8, #0xC]

	BL CRC32_F

	LDR R1, [R8, #0x1C]
	CMP R1, R0
	BNE finish_loop

	LDR R0, [R8, #0x18]
	BLX R0

#endif /* USE_EDCL */

finish_loop:
	NOP
	B finish_loop

.Lstack:
	.word __stack

#ifndef USE_EDCL
.section .data.desc, "aw",%progbits

.set desc_pointer, is_ready
is_ready: .word 0
command: .word 0
image: .word -1, -1
flash: .word -1, -1
entry: .word -1
crc: .word 0
.set desc_size, 32
#endif
