#define SSP_Base  0x000CF000
#define SSP_CR0   0x000
#define SSP_CR1   0x004
#define SSP_DR    0x008
#define SSP_SR    0x00C
#define SSP_CPSR  0x010

.extern WAIT_FLASH_WRITE_END

.global SIMPLE_WRITE
SIMPLE_WRITE: /*-----------------------------------------*/
/*
 * R0 - offset
 * R1 - number of bytes
 * R2 - ptr from
 * R3 - SSP_Base
 * R4 - work register
 * R5 - number of bytes to write in Page Program command 
 * R6 - number of bytes to receive in Page Program command (R5 + 5 due to WREN & PP)
 * R7 - SSP Status Register (SR)
 *
 */
    PUSH {R1-R8, LR}

    MOV R0, R1 /* shift due to R0 == flash structure */
    MOV R1, R2
    MOV R2, R3

    CMP R1, #0
    BEQ EXIT_MM

    BL  SIMPLE_INIT
    LDR  R3, =SSP_Base

WRITE_PAGE_M:

    CMP   R1, #0
    BLE   EXIT_M  /* end  */
    MOV   R5, #256
    CMP   R5, R1
    MOVGT R5, R1
    SUB   R1, R1, R5
    /* ADD   R6, R5, #5 */
    ADD   R6, R5, #4

    BL  WAIT_FLASH_WRITE_END

    MOV  R4, #0x06          /* write enable command */
    STR  R4, [R3, #SSP_DR]

    BL  WAIT_FLASH_WRITE_END

    MOV  R4, #0x02          /* page program command */
    STR  R4, [R3, #SSP_DR]
    LSR  R4, R0, #16
    STR  R4, [R3, #SSP_DR]  /*  bits [23:16] */
    LSR  R4, R0, #8
    STR  R4, [R3, #SSP_DR]  /*  bits [15:8] */
    STR  R0, [R3, #SSP_DR]  /*  bits [7:0] */

TRANSMIT_M:

    LDR  R7, [R3, #SSP_SR]
    TST  R7, #0x2
    BEQ  RECEIVE_M  /*  Transmit FIFO is full */

    CMP  R5, #0
    BEQ  RECEIVE_M
    LDRB  R4, [R2], #1
    STRB  R4, [R3, #SSP_DR]
    SUB  R5, R5, #1

RECEIVE_M:

    TST  R7, #0x4
    BEQ  TRANSMIT_M  /*  Receive FIFO is empty */
    
    LDR  R4, [R3, #SSP_DR]
    SUBS R6, R6, #1

    BGT  TRANSMIT_M
    ADD  R0, R0, #256
    B    WRITE_PAGE_M

EXIT_M:

    BL  WAIT_FLASH_WRITE_END

EXIT_MM:
    MOV R0, #0
    POP {R1-R8, PC}

