
void PCIe_setting_ ();

typedef struct 
{
    //-----------------------------------------------------
    //      [31:0]    AXI Base Address
    //-----------------------------------------------------
    u32   axi_base_addr       : 32;
    //-----------------------------------------------------
    //  AXI Address Phase (AR or AW) controls
    //      [31:8]   Reserved
    //      [7]      AxLOCK
    //      [6:3]    AxCACHE
    //      [2:0]    AxPROT
    //-----------------------------------------------------
    u32   axi_addr_phase      : 32;
    //-----------------------------------------------------
    //      [63:0] PCIe Base Address
    //-----------------------------------------------------
    u64   pcie_base_addr      : 64;
    //-----------------------------------------------------
    //  PCIe TLP Header Attributes
    //      [31:26]  Reserved
    //      [25:10]  Requester ID
    //      [9]      Use 25:10 as Requester ID
    //      [8:6]    Reserved
    //      [5:3]    PCIe Transfer Class 
    //      [2]      ID-Based Ordering 
    //      [1]      Relaxed Ordering 
    //      [0]      No Snoop 
    //-----------------------------------------------------
    u32   tlp_header_att      : 32;
    //-----------------------------------------------------
    //      [23:0] Length of transfer in bytes (0 indicates
    //         maximum length transfer 2^24 bytes)
    //-----------------------------------------------------
    u32   transfer_length     : 24;
    //-----------------------------------------------------
    //  Control Byte
    //      [7:6]    Reserved
    //      [5]      Continue to execute linked-list
    //      [4:3]    Reserved
    //      [2:1]    Continuity (bulk | scatter/gather)
    //      [0]      Interrupt after execution of descriptor
    //-----------------------------------------------------
    u8    control_byte        :  8;
    //-----------------------------------------------------
    //  AXI Bus Status
    //      [7:3]    Reserved
    //      [2]      Internal data integrity error detected
    //      [1:0]    BRESP[1:0] or RRESP[1:0]
    //-----------------------------------------------------
    u8    axi_bus_status      :  8;
    //-----------------------------------------------------
    //  PCIe Bus Status
    //      [7:4]    Reserved
    //      [3:0]    TLP Completion status or internal error
    //        0x0      Normal completion
    //        !=0      Completion with error (look spec)
    //-----------------------------------------------------
    u8    pcie_bus_status     :  8;
    //-----------------------------------------------------
    //  Channel Status
    //      [7]      Buffer Not Empty
    //      [6]      Buffer Underflow
    //      [5]      Buffer Overflow
    //      [4]      Descriptor Error
    //      [3]      Data Integrity Error
    //      [2]      AXI transfer completed early with incomplete data
    //      [1]      PCIe transfer completed early with incomplete data
    //      [0]      Descriptor action completed
    //-----------------------------------------------------
    u8    channel_status      :  8;
    //-----------------------------------------------------
    //      [7:0] Reserved
    //-----------------------------------------------------
    u8    reserved            :  8;
    //-----------------------------------------------------
    //      [31:0] Pointer to Next Descriptor in linked-list
    //-----------------------------------------------------
    u32   next_desc_pointer   : 32;
} PCIe_DMA_desc;

