/* 
 *  defs.incl
 */ 

/* --- Mode bits in PSRs --- */

#define Mode_USR    0x10
#define Mode_FIQ    0x11
#define Mode_IRQ    0x12
#define Mode_SVC    0x13
#define Mode_MON    0x16
#define Mode_ABT    0x17
#define Mode_UND    0x1B
#define Mode_SYS    0x1F

/* --- Interrupt disable flags in PSRs --- */

#define I_Bit   0x80  /* when I bit is set, IRQ is disabled */
#define F_Bit   0x40  /* when F bit is set, FIQ is disabled */
#define A_Bit   0x100 /* when A bit is set, Imbrecise Data Abort Interrupts disabled */

/* --- System memory locations --- */

#define CMB0_Base    0x00000000 
#define CMB0_Limit   0x00010000
#define CMB1_Base    0x00010000 
#define CMB1_Limit   0x00020000
#define CMB2_Base    0x00020000 
#define CMB2_Limit   0x00030000
#define CMB3_Base    0x00030000 
#define CMB3_Limit   0x00040000

#define CCMB0_Base   0x00000000
#define CCMB1_Base   0x00020000
#define CCMB2_Base   0x00040000
#define CCMB3_Base   0x00060000

#define DDR_Base     0x00300000 /* External DDR SDRAM memory base */
#define DDR_Limit    0x40000000 /* External DDR SDRAM memory last address plus 1 */

#define PCIe_Base    0x40000000

#define CCPU_Base    0x60000000
#define PC0_Base     0x80000000
#define PC1_Base     0xA0000000
#define PC2_Base     0xC0000000
#define PC3_Base     0xE0000000

/* --- Stack at the end of CMB0??? --- */

/* --- APB Peripheral Slots (Maybe Not Needed) --- */

#define PERIPH_Base  0x00080000
#define PERIPH_Size  0x11 /* 256KB   WRONG YET !!! */

/*#define XXXX_Base    0x00000000*/
#define GICD_Base    0x000B0000
#define GICC_Base    0x000B1000

#define LC0_Base     0x000B4000
#define LC1_Base     0x000B5000
#define LC2_Base     0x000B6000
#define LC3_Base     0x000B7000

#define CP0_Base     0x000B8000
#define CP1_Base     0x000B9000
#define CP2_Base     0x000BA000
#define CP3_Base     0x000BB000

#define MDMAC_Base   0x000BC000

#define EMIC_Base    0x000C0000
#define EMI_Base     0x00300000

#define SC_Base      0x000CB000
#define GERA_Base    0x000CC000
#define DIT_Base     0x000CD000
#define CCRG_Base    0x000CA000
#define CRGNM_Base   0x000CA000
#define CRG0_Base    0x000C6000
#define CRG1_Base    0x000C7000
#define CRG2_Base    0x000C8000
#define CRG3_Base    0x000C9000
#define WDOG_Base    0x000CE000
#define SPI_Base     0x000CF000
#define ETH_Base     0x000D0000
#define L2C_Base     0x000D2000
#define MPU_Base     0x000BD000
#define TSNS_Base    0x000D1000

#define TUBE_Base    0x000CD800

#define PCIEC_Base   0x000D8000
#define ELC_Base     0x000D8000
#define ELPHY_Base   0x000E0000

/* === Peripherals Regs === */

#define GICC_IAR     0x00C
#define GICC_EOIR    0x010
#define GICD_ISENABLER0 0x100
#define GICD_ISENABLER1 0x104
#define GICD_ISENABLER2 0x108
#define GICD_ICENABLER0 0x180
#define GICD_ICENABLER1 0x184
#define GICD_ICENABLER2 0x188
#define GICC_CTLR 0x000
#define GICD_CTLR 0x000
#define GICD_SGIR 0xF00
#define GICD_IGROUPR0 0x080
#define GICD_IGROUPR1 0x084
#define GICD_IGROUPR2 0x088
#define GICD_TYPER 0x004
#define GICC_PMR 0x004
#define GICD_ICFGR0 0xC00
#define GICD_ICFGR1 0xC04
#define GICD_ICFGR2 0xC08
#define GICD_ICFGR3 0xC0C
#define GICD_ICFGR4 0xC10
#define GICD_ICFGR5 0xC14

/* --- TIMER (ARM SP804) --- */

#define TIMER_Base    DIT_Base
#define TIMER1_Base    DIT_Base
#define TIMER2_Base    DIT_Base + 0x020

#define TIMER_LOAD     0x000
#define TIMER_VALUE    0x004
#define TIMER_CONTROL  0x008
#define TIMER_CLEAR    0x00C
#define TIMER_RIS      0x010
#define TIMER_MIS      0x014
#define TIMER_BGLOAD   0x018

#define TIMER_ITCR     0xF00
#define TIMER_ITOP     0xF04

#define TIMER_PERIPHID0  0xFE0
#define TIMER_PERIPHID1  0xFE4
#define TIMER_PERIPHID2  0xFE8
#define TIMER_PERIPHID3  0xFEC
#define TIMER_PCELLID0   0xFF0
#define TIMER_PCELLID1   0xFF4
#define TIMER_PCELLID2   0xFF8
#define TIMER_PCELLID3   0xFFC

/* --- WDOG (ARM SP805) --- */

#define WDOG_LOAD    0x000
#define WDOG_VALUE   0x004
#define WDOG_CONTROL 0x008
#define WDOG_INTCLR  0x00C
#define WDOG_RIS     0x010
#define WDOG_MIS     0x014
#define WDOG_LOCK    0xC00

#define WDOG_ITCR  0xF00
#define WDOG_ITOP  0xF04

#define WDOG_PERIPHID0  0xFE0
#define WDOG_PERIPHID1  0xFE4
#define WDOG_PERIPHID2  0xFE8
#define WDOG_PERIPHID3  0xFEC
#define WDOG_PCELLID0   0xFF0
#define WDOG_PCELLID1   0xFF4
#define WDOG_PCELLID2   0xFF8
#define WDOG_PCELLID3   0xFFC

/* --- EXTIRC (Included in GERA) --- */

#define EXTIRC_Base GERA_Base+0x20
#define EXTIRC_EIENB  0x00
#define EXTIRC_EIREQ  0x04
#define EXTIRC_EILVL  0x08


/* --- RA (Included in GERA) --- */

#define RA_Base GERA_Base+0x30
#define RA_CONTR  0x00
#define RA_VAL  0x04
#define REGIME  0x08

/* --- SSP (ARM PL022) --- */


#define SSP_CR0         0x000
#define SSP_CR1         0x004
#define SSP_DR          0x008
#define SSP_SR          0x00C
#define SSP_CPSR        0x010
#define SSP_IMSC        0x014
#define SSP_RIS         0x018
#define SSP_MIS         0x01C
#define SSP_ICR         0x020
#define SSP_DMACR       0x024

#define SSP_DMATXADR    0x100
#define SSP_DMARXADR    0x104
#define SSP_DMACNT      0x108
#define SSP_DMARXDIS    0x10C
#define SSP_DMACTR      0x110

#define SSP_DMASTS      0x11C
#define SSP_DMAIM       0x120
#define SSP_DMAIR       0x124

#define SSP_DMAPT       0x130

#define SSP_DMACS       0x140


/* --- GPIO (Module) --- */
#define GPIO_Base   GERA_Base
#define GPIO_PDR0   0x000 /* Port Data Register */
#define GPIO_DDR0   0x010 /* Data Direction Register */

/* --- GRETH ---- */
#define GRETH_CTRL_OFFSET        0x000
#define GRETH_STTS_OFFSET        0x004
#define GRETH_MACADR1_OFFSET     0x008
#define GRETH_MACADR2_OFFSET     0x00C
#define GRETH_MDIOCTRL_OFFSET    0x010
#define GRETH_TPTR_OFFSET        0x014
#define GRETH_RPTR_OFFSET        0x018
#define GRETH_EDCLIP_OFFSET      0x01C
#define GRETH_HASHT1_OFFSET      0x020
#define GRETH_HASHT2_OFFSET      0x024
#define GRETH_EDCLMAC1_OFFSET    0x028
#define GRETH_EDCLMAC2_OFFSET    0x02C

/* --- TSNS ----*/
#define TSNS_PWDN     0x000
#define TSNS_START    0x004
#define TSNS_IR       0x008
#define TSNS_IM       0x00C
#define TSNS_CLKDIV   0x010
#define TSNS_LEVEL    0x014
#define TSNS_DATA     0x018
#define TSNS_ENZCALIB 0x01C

/* --- TUBE --- */

#define TUBE_PRINTSTR   0x000
#define TUBE_PRINTDEC   0x004
#define TUBE_PRINTHEX   0x008
#define TUBE_PRINTFINISH 0x00c

#define PRINT_BUFFER_LNG   200
/* --- MDMAC --- */
#define MDMAC_MainCounter    0x0000
#define MDMAC_SrcAddress     0x0004
#define MDMAC_SrcBias        0x0008
#define MDMAC_SrcRowCounter  0x000C
#define MDMAC_SrcAddressMode 0x0010
#define MDMAC_Control        0x0014

#define MDMAC_DstAddress     0x0024
#define MDMAC_DstBias        0x0028
#define MDMAC_DstRowCounter  0x002C
#define MDMAC_DstAddressMode 0x0030

#define MDMAC_InterruptMask  0x0038
#define MDMAC_State          0x003C

/* --- MPU --- */
#define MPU_RD_MASK                 0x000 //R/W
#define MPU_RD_MASK_SET             0x008 //W
#define MPU_RD_MASK_CLEAR           0x010 //W
#define MPU_RD_MASK_NULL            0x018 //W
#define MPU_RD_IRQ_STATUS_RAW       0x020 //R
#define MPU_RD_IRQ_STATUS_RAW_SET   0x028 //W
#define MPU_RD_IRQ_STATUS_RAW_CLEAR 0x030 //W
#define MPU_RD_IRQ_STATUS_RAW_NULL  0x038 //W
#define MPU_RD_IRQ_MASK             0x040 //R/W
#define MPU_RD_IRQ_MASK_SET         0x048 //W
#define MPU_RD_IRQ_MASK_CLEAR       0x050 //W
#define MPU_RD_IRQ_MASK_NULL        0x058 //W
#define MPU_RD_IRQ_STATUS           0x060 //R
#define MPU_RD_IRQ_STATUS_NULL      0x078 //W

#define MPU_WR_MASK                 0x100 //R/W
#define MPU_WR_MASK_SET             0x108 //W
#define MPU_WR_MASK_CLEAR           0x110 //W
#define MPU_WR_MASK_NULL            0x118 //W
#define MPU_WR_IRQ_STATUS_RAW       0x120 //R
#define MPU_WR_IRQ_STATUS_RAW_SET   0x128 //W
#define MPU_WR_IRQ_STATUS_RAW_CLEAR 0x130 //W
#define MPU_WR_IRQ_STATUS_RAW_NULL  0x138 //W
#define MPU_WR_IRQ_MASK             0x140 //R/W
#define MPU_WR_IRQ_MASK_SET         0x148 //W
#define MPU_WR_IRQ_MASK_CLEAR       0x150 //W
#define MPU_WR_IRQ_MASK_NULL        0x158 //W
#define MPU_WR_IRQ_STATUS           0x160 //R
#define MPU_WR_IRQ_STATUS_NULL      0x178 //W


/* --- EMIC --- */
#define DENALI_CTL_00 0x000
#define DENALI_CTL_01 0x004
#define DENALI_CTL_02 0x008
#define DENALI_CTL_03 0x00C
#define DENALI_CTL_04 0x010
#define DENALI_CTL_05 0x014
#define DENALI_CTL_06 0x018
#define DENALI_CTL_07 0x01C
#define DENALI_CTL_08 0x020
#define DENALI_CTL_09 0x024
#define DENALI_CTL_10 0x028
#define DENALI_CTL_11 0x02C
#define DENALI_CTL_12 0x030
#define DENALI_CTL_13 0x034
#define DENALI_CTL_14 0x038
#define DENALI_CTL_15 0x03C
#define DENALI_CTL_16 0x040
#define DENALI_CTL_17 0x044
#define DENALI_CTL_18 0x048
#define DENALI_CTL_19 0x04C
#define DENALI_CTL_20 0x050
#define DENALI_CTL_21 0x054
#define DENALI_CTL_22 0x058
#define DENALI_CTL_23 0x05C
#define DENALI_CTL_24 0x060
#define DENALI_CTL_25 0x064
#define DENALI_CTL_26 0x068
#define DENALI_CTL_27 0x06C
#define DENALI_CTL_28 0x070
#define DENALI_CTL_29 0x074
#define DENALI_CTL_30 0x078
#define DENALI_CTL_31 0x07C
#define DENALI_CTL_32 0x080
#define DENALI_CTL_33 0x084
#define DENALI_CTL_34 0x088
#define DENALI_CTL_35 0x08C
#define DENALI_CTL_36 0x090
#define DENALI_CTL_37 0x094
#define DENALI_CTL_38 0x098
#define DENALI_CTL_39 0x09C
#define DENALI_CTL_40 0x0A0
#define DENALI_CTL_41 0x0A4
#define DENALI_CTL_42 0x0A8
#define DENALI_CTL_43 0x0AC
#define DENALI_CTL_44 0x0B0
#define DENALI_CTL_45 0x0B4
#define DENALI_CTL_46 0x0B8
#define DENALI_CTL_47 0x0BC
#define DENALI_CTL_48 0x0C0
#define DENALI_CTL_49 0x0C4
#define DENALI_CTL_50 0x0C8
#define DENALI_CTL_51 0x0CC
#define DENALI_CTL_52 0x0D0
#define DENALI_CTL_53 0x0D4
#define DENALI_CTL_54 0x0D8
#define DENALI_CTL_55 0x0DC
#define DENALI_CTL_56 0x0E0
#define DENALI_CTL_57 0x0E4
#define DENALI_CTL_58 0x0E8
#define DENALI_CTL_59 0x0EC
#define DENALI_CTL_60 0x0F0
#define DENALI_CTL_61 0x0F4
#define DENALI_CTL_62 0x0F8
#define DENALI_CTL_63 0x0FC
#define DENALI_CTL_64 0x100
#define DENALI_CTL_65 0x104
#define DENALI_CTL_66 0x108
#define DENALI_CTL_67 0x10C
#define DENALI_CTL_68 0x110
#define DENALI_CTL_69 0x114
#define DENALI_CTL_70 0x118
#define DENALI_CTL_71 0x11C
#define DENALI_CTL_72 0x120
#define DENALI_CTL_73 0x124
#define DENALI_CTL_74 0x128
#define DENALI_CTL_75 0x12C
#define DENALI_CTL_76 0x130
#define DENALI_CTL_77 0x134
#define DENALI_CTL_78 0x138
#define DENALI_CTL_79 0x13C
#define DENALI_CTL_80 0x140
#define DENALI_CTL_81 0x144
  /*Phy is a part of address spase of EMIC*/
  
#define DENALI_PHY_BASE  0x1000

#define DENALI_PHY_00  0x000
#define DENALI_PHY_01  0x004
#define DENALI_PHY_02  0x008
#define DENALI_PHY_03  0x00C
#define DENALI_PHY_04  0x010
#define DENALI_PHY_05  0x014
#define DENALI_PHY_06  0x018
#define DENALI_PHY_07  0x01C
#define DENALI_PHY_08  0x020
#define DENALI_PHY_09  0x024
#define DENALI_PHY_10  0x028
#define DENALI_PHY_11  0x02C
#define DENALI_PHY_12  0x030
#define DENALI_PHY_13  0x034
#define DENALI_PHY_14  0x038
#define DENALI_PHY_15  0x03C
#define DENALI_PHY_16  0x040
#define DENALI_PHY_17  0x044
#define DENALI_PHY_18  0x048
#define DENALI_PHY_19  0x04C
#define DENALI_PHY_20  0x050
#define DENALI_PHY_21  0x054
#define DENALI_PHY_22  0x058
#define DENALI_PHY_23  0x05C
#define DENALI_PHY_24  0x060
#define DENALI_PHY_25  0x064
#define DENALI_PHY_26  0x068
#define DENALI_PHY_27  0x06C
#define DENALI_PHY_28  0x070
#define DENALI_PHY_29  0x074
#define DENALI_PHY_30  0x078
#define DENALI_PHY_31  0x07C
#define DENALI_PHY_32  0x080
#define DENALI_PHY_33  0x084
#define DENALI_PHY_34  0x088
#define DENALI_PHY_35  0x08C
#define DENALI_PHY_36  0x090
#define DENALI_PHY_37  0x094
#define DENALI_PHY_38  0x098
#define DENALI_PHY_39  0x09C
#define DENALI_PHY_40  0x0A0
#define DENALI_PHY_41  0x0A4
#define DENALI_PHY_42  0x0A8
#define DENALI_PHY_43  0x0AC
#define DENALI_PHY_44  0x0B0
#define DENALI_PHY_45  0x0B4
#define DENALI_PHY_46  0x0B8
#define DENALI_PHY_47  0x0BC
#define DENALI_PHY_48  0x0C0
#define DENALI_PHY_49  0x0C4
#define DENALI_PHY_50  0x0C8
#define DENALI_PHY_51  0x0CC
#define DENALI_PHY_52  0x0D0
#define DENALI_PHY_53  0x0D4
#define DENALI_PHY_54  0x0D8
#define DENALI_PHY_55  0x0DC
#define DENALI_PHY_56  0x0E0
#define DENALI_PHY_57  0x0E4
#define DENALI_PHY_58  0x0E8
#define DENALI_PHY_59  0x0EC
#define DENALI_PHY_60  0x0F0
#define DENALI_PHY_61  0x0F4
#define DENALI_PHY_62  0x0F8
#define DENALI_PHY_63  0x0FC
#define DENALI_PHY_64  0x100
#define DENALI_PHY_65  0x104
#define DENALI_PHY_66  0x108
#define DENALI_PHY_67  0x10C
#define DENALI_PHY_68  0x110
#define DENALI_PHY_69  0x114
#define DENALI_PHY_70  0x118
#define DENALI_PHY_71  0x11C
#define DENALI_PHY_72  0x120
  /*registers 73 - 127 are intentionally blank*/
#define DENALI_PHY_128 0x200
#define DENALI_PHY_129 0x204
#define DENALI_PHY_130 0x208
#define DENALI_PHY_131 0x20C
#define DENALI_PHY_132 0x210
#define DENALI_PHY_133 0x214
#define DENALI_PHY_134 0x218
#define DENALI_PHY_135 0x21C
#define DENALI_PHY_136 0x220
#define DENALI_PHY_137 0x224
#define DENALI_PHY_138 0x228
#define DENALI_PHY_139 0x22C
#define DENALI_PHY_140 0x230
#define DENALI_PHY_141 0x234
#define DENALI_PHY_142 0x238
#define DENALI_PHY_143 0x23C
#define DENALI_PHY_144 0x240
#define DENALI_PHY_145 0x244
#define DENALI_PHY_146 0x248
#define DENALI_PHY_147 0x24C
#define DENALI_PHY_148 0x250
#define DENALI_PHY_149 0x254
#define DENALI_PHY_150 0x258
#define DENALI_PHY_151 0x25C
#define DENALI_PHY_152 0x260
#define DENALI_PHY_153 0x264
#define DENALI_PHY_154 0x268
#define DENALI_PHY_155 0x26C
#define DENALI_PHY_156 0x270
#define DENALI_PHY_157 0x274
#define DENALI_PHY_158 0x278
#define DENALI_PHY_159 0x27C
#define DENALI_PHY_160 0x280
#define DENALI_PHY_161 0x284
#define DENALI_PHY_162 0x288
#define DENALI_PHY_163 0x28C
#define DENALI_PHY_164 0x290
#define DENALI_PHY_165 0x294
#define DENALI_PHY_166 0x298
#define DENALI_PHY_167 0x29C
#define DENALI_PHY_168 0x2A0
#define DENALI_PHY_169 0x2A4
#define DENALI_PHY_170 0x2A8
#define DENALI_PHY_171 0x2AC
#define DENALI_PHY_172 0x2B0
#define DENALI_PHY_173 0x2B4
#define DENALI_PHY_174 0x2B8
#define DENALI_PHY_175 0x2BC
#define DENALI_PHY_176 0x2C0
#define DENALI_PHY_177 0x2C4
#define DENALI_PHY_178 0x2C8
#define DENALI_PHY_179 0x2CC
#define DENALI_PHY_180 0x2D0
#define DENALI_PHY_181 0x2D4
#define DENALI_PHY_182 0x2D8
#define DENALI_PHY_183 0x2DC
#define DENALI_PHY_184 0x2E0
#define DENALI_PHY_185 0x2E4
#define DENALI_PHY_186 0x2E8
#define DENALI_PHY_187 0x2EC
#define DENALI_PHY_188 0x2F0
#define DENALI_PHY_189 0x2F4
#define DENALI_PHY_190 0x2F8
#define DENALI_PHY_191 0x2FC
#define DENALI_PHY_192 0x300
#define DENALI_PHY_193 0x304
#define DENALI_PHY_194 0x308
#define DENALI_PHY_195 0x30C
#define DENALI_PHY_196 0x310
#define DENALI_PHY_197 0x314
#define DENALI_PHY_198 0x318
#define DENALI_PHY_199 0x31C
#define DENALI_PHY_200 0x320
  /*registers 201 - 255 are intentionally blank*/
#define DENALI_PHY_256 0x400
#define DENALI_PHY_257 0x404
#define DENALI_PHY_258 0x408
#define DENALI_PHY_259 0x40C
#define DENALI_PHY_260 0x410
#define DENALI_PHY_261 0x414
#define DENALI_PHY_262 0x418
#define DENALI_PHY_263 0x41C
#define DENALI_PHY_264 0x420
#define DENALI_PHY_265 0x424
#define DENALI_PHY_266 0x428
#define DENALI_PHY_267 0x42C
#define DENALI_PHY_268 0x430
#define DENALI_PHY_269 0x434
#define DENALI_PHY_270 0x438
#define DENALI_PHY_271 0x43C
#define DENALI_PHY_272 0x440
#define DENALI_PHY_273 0x444
#define DENALI_PHY_274 0x448
#define DENALI_PHY_275 0x44C
#define DENALI_PHY_276 0x450
#define DENALI_PHY_277 0x454
#define DENALI_PHY_278 0x458
#define DENALI_PHY_279 0x45C
#define DENALI_PHY_280 0x460
#define DENALI_PHY_281 0x464
#define DENALI_PHY_282 0x468
#define DENALI_PHY_283 0x46C
#define DENALI_PHY_284 0x470
#define DENALI_PHY_285 0x474
#define DENALI_PHY_286 0x478
#define DENALI_PHY_287 0x47C
#define DENALI_PHY_288 0x480
#define DENALI_PHY_289 0x484
#define DENALI_PHY_290 0x488
#define DENALI_PHY_291 0x48C
#define DENALI_PHY_292 0x490
#define DENALI_PHY_293 0x494
#define DENALI_PHY_294 0x498
#define DENALI_PHY_295 0x49C
#define DENALI_PHY_296 0x4A0
#define DENALI_PHY_297 0x4A4
#define DENALI_PHY_298 0x4A8
#define DENALI_PHY_299 0x4AC
#define DENALI_PHY_300 0x4B0
#define DENALI_PHY_301 0x4B4
#define DENALI_PHY_302 0x4B8
#define DENALI_PHY_303 0x4BC
#define DENALI_PHY_304 0x4C0
#define DENALI_PHY_305 0x4C4
#define DENALI_PHY_306 0x4C8
#define DENALI_PHY_307 0x4CC
#define DENALI_PHY_308 0x4D0
#define DENALI_PHY_309 0x4D4
#define DENALI_PHY_310 0x4D8
#define DENALI_PHY_311 0x4DC
#define DENALI_PHY_312 0x4E0
#define DENALI_PHY_313 0x4E4
#define DENALI_PHY_314 0x4E8
#define DENALI_PHY_315 0x4EC
#define DENALI_PHY_316 0x4F0
#define DENALI_PHY_317 0x4F4
#define DENALI_PHY_318 0x4F8
#define DENALI_PHY_319 0x4FC
#define DENALI_PHY_320 0x500
#define DENALI_PHY_321 0x504
#define DENALI_PHY_322 0x508
#define DENALI_PHY_323 0x50C
#define DENALI_PHY_324 0x510
#define DENALI_PHY_325 0x514
#define DENALI_PHY_326 0x518
#define DENALI_PHY_327 0x51C
#define DENALI_PHY_328 0x520
  /*registers 329 - 383 are intentionally blank*/
#define DENALI_PHY_384 0x600
#define DENALI_PHY_385 0x604
#define DENALI_PHY_386 0x608
#define DENALI_PHY_387 0x60C
#define DENALI_PHY_388 0x610
#define DENALI_PHY_389 0x614
#define DENALI_PHY_390 0x618
#define DENALI_PHY_391 0x61C
#define DENALI_PHY_392 0x620
#define DENALI_PHY_393 0x624
#define DENALI_PHY_394 0x628
#define DENALI_PHY_395 0x62C
#define DENALI_PHY_396 0x630
#define DENALI_PHY_397 0x634
#define DENALI_PHY_398 0x638
#define DENALI_PHY_399 0x63C
#define DENALI_PHY_400 0x640
#define DENALI_PHY_401 0x644
#define DENALI_PHY_402 0x648
#define DENALI_PHY_403 0x64C
#define DENALI_PHY_404 0x650
#define DENALI_PHY_405 0x654
#define DENALI_PHY_406 0x658
#define DENALI_PHY_407 0x65C
#define DENALI_PHY_408 0x660
#define DENALI_PHY_409 0x664
#define DENALI_PHY_410 0x668
#define DENALI_PHY_411 0x66C
#define DENALI_PHY_412 0x670
#define DENALI_PHY_413 0x674
#define DENALI_PHY_414 0x678
#define DENALI_PHY_415 0x67C
#define DENALI_PHY_416 0x680
#define DENALI_PHY_417 0x684
#define DENALI_PHY_418 0x688
#define DENALI_PHY_419 0x68C
#define DENALI_PHY_420 0x690
#define DENALI_PHY_421 0x694
#define DENALI_PHY_422 0x698
#define DENALI_PHY_423 0x69C
#define DENALI_PHY_424 0x6A0
#define DENALI_PHY_425 0x6A4
#define DENALI_PHY_426 0x6A8
#define DENALI_PHY_427 0x6AC
#define DENALI_PHY_428 0x6B0
#define DENALI_PHY_429 0x6B4
#define DENALI_PHY_430 0x6B8
#define DENALI_PHY_431 0x6BC
#define DENALI_PHY_432 0x6C0
#define DENALI_PHY_433 0x6C4
#define DENALI_PHY_434 0x6C8
#define DENALI_PHY_435 0x6CC
#define DENALI_PHY_436 0x6D0
#define DENALI_PHY_437 0x6D4
#define DENALI_PHY_438 0x6D8
#define DENALI_PHY_439 0x6DC
#define DENALI_PHY_440 0x6E0
#define DENALI_PHY_441 0x6E4
#define DENALI_PHY_442 0x6E8
#define DENALI_PHY_443 0x6EC
#define DENALI_PHY_444 0x6F0
#define DENALI_PHY_445 0x6F4
#define DENALI_PHY_446 0x6F8
#define DENALI_PHY_447 0x6FC
#define DENALI_PHY_448 0x700
#define DENALI_PHY_449 0x704
#define DENALI_PHY_450 0x708
#define DENALI_PHY_451 0x70C
#define DENALI_PHY_452 0x710
#define DENALI_PHY_453 0x714
#define DENALI_PHY_454 0x718
#define DENALI_PHY_455 0x71C
#define DENALI_PHY_456 0x720
  /*registers 457 - 511 are intentionally blank*/
#define DENALI_PHY_512 0x800
#define DENALI_PHY_513 0x804
#define DENALI_PHY_514 0x808
#define DENALI_PHY_515 0x80C
#define DENALI_PHY_516 0x810
#define DENALI_PHY_517 0x814
#define DENALI_PHY_518 0x818
#define DENALI_PHY_519 0x81C
#define DENALI_PHY_520 0x820
#define DENALI_PHY_521 0x824
#define DENALI_PHY_522 0x828
#define DENALI_PHY_523 0x82C
#define DENALI_PHY_524 0x830
#define DENALI_PHY_525 0x834
#define DENALI_PHY_526 0x838
#define DENALI_PHY_527 0x83C
#define DENALI_PHY_528 0x840
#define DENALI_PHY_529 0x844
#define DENALI_PHY_530 0x848
#define DENALI_PHY_531 0x84C
#define DENALI_PHY_532 0x850
#define DENALI_PHY_533 0x854
#define DENALI_PHY_534 0x858
#define DENALI_PHY_535 0x85C
#define DENALI_PHY_536 0x860
#define DENALI_PHY_537 0x864
#define DENALI_PHY_538 0x868
#define DENALI_PHY_539 0x86C
#define DENALI_PHY_540 0x870
#define DENALI_PHY_541 0x874
#define DENALI_PHY_542 0x878
#define DENALI_PHY_543 0x87C
#define DENALI_PHY_544 0x880
#define DENALI_PHY_545 0x884
#define DENALI_PHY_546 0x888
#define DENALI_PHY_547 0x88C
#define DENALI_PHY_548 0x890
#define DENALI_PHY_549 0x894
#define DENALI_PHY_550 0x898
#define DENALI_PHY_551 0x89C
#define DENALI_PHY_552 0x8A0
#define DENALI_PHY_553 0x8A4
#define DENALI_PHY_554 0x8A8
#define DENALI_PHY_555 0x8AC
#define DENALI_PHY_556 0x8B0
#define DENALI_PHY_557 0x8B4
#define DENALI_PHY_558 0x8B8
#define DENALI_PHY_559 0x8BC
#define DENALI_PHY_560 0x8C0
#define DENALI_PHY_561 0x8C4
#define DENALI_PHY_562 0x8C8
#define DENALI_PHY_563 0x8CC
#define DENALI_PHY_564 0x8D0
#define DENALI_PHY_565 0x8D4
#define DENALI_PHY_566 0x8D8
#define DENALI_PHY_567 0x8DC
#define DENALI_PHY_568 0x8E0
#define DENALI_PHY_569 0x8E4
#define DENALI_PHY_570 0x8E8
#define DENALI_PHY_571 0x8EC
#define DENALI_PHY_572 0x8F0
#define DENALI_PHY_573 0x8F4
#define DENALI_PHY_574 0x8F8
#define DENALI_PHY_575 0x8FC
#define DENALI_PHY_576 0x900
#define DENALI_PHY_577 0x904
#define DENALI_PHY_578 0x908
#define DENALI_PHY_579 0x90C
#define DENALI_PHY_580 0x910

/* --- ELC --- */
#define ELC_CTRL_0     0x000
#define ELC_CTRL_1     0x400
#define ELC_STATE_0    0x404
#define ELC_STATE_1    0x408
#define ELC_PERIOD     0x40C
#define ELC_L0_L1_PHYS 0x410
#define ELC_L2_L3_PHYS 0x414
#define ELC_L0_COEF    0x418
#define ELC_L1_COEF    0x41C
#define ELC_L2_COEF    0x420
#define ELC_L3_COEF    0x424
#define ELC_ERROR      0x428

/* --- LC0 --- */
#define LC0_PRT0_DST    0x000
#define LC0_PRT1_DST    0x004
#define LC0_PRT2_DST    0x008
#define LC0_PRT3_DST    0x00C
#define LC0_PRT0_SRC    0x010
#define LC0_PRT1_SRC    0x014
#define LC0_PRT2_SRC    0x018
#define LC0_PRT3_SRC    0x01C

/* --- LC1 --- */
#define LC1_PRT0_DST    0x000
#define LC1_PRT1_DST    0x004
#define LC1_PRT2_DST    0x008
#define LC1_PRT0_SRC    0x00C
#define LC1_PRT1_SRC    0x010
#define LC1_PRT2_SRC    0x014

/* --- LC2 --- */
#define LC2_PRT0_DST    0x000
#define LC2_PRT1_DST    0x004
#define LC2_PRT2_DST    0x008
#define LC2_PRT0_SRC    0x00C
#define LC2_PRT1_SRC    0x010
#define LC2_PRT2_SRC    0x014

/* --- LC3 --- */
#define LC3_PRT0_DST    0x000
#define LC3_PRT1_DST    0x004
#define LC3_PRT2_DST    0x008
#define LC3_PRT0_SRC    0x00C
#define LC3_PRT1_SRC    0x010
#define LC3_PRT2_SRC    0x014

/* --- CPx --- */
#define CPx_TR_MainCounter    0x00
#define CPx_TR_Address        0x04
#define CPx_TR_Bias           0x08
#define CPx_TR_RowCounter     0x0C
#define CPx_TR_AddressMode    0x10
#define CPx_TR_Control        0x14
#define CPx_TR_InterruptMask  0x18
#define CPx_TR_State          0x1C
#define CPx_RC_MainCounter    0x20
#define CPx_RC_Address        0x24
#define CPx_RC_Bias           0x28
#define CPx_RC_RowCounter     0x2C
#define CPx_RC_AddressMode    0x30
#define CPx_RC_Control        0x34
#define CPx_RC_InterruptMask  0x38
#define CPx_RC_State          0x3C

/* --- ARMSC (Module) --- */
/* Both Definitions For TOP and PC*/
#define NMPURCVINT      0x10
#define NMPUDRVINT      0x14
#define PCRCVINT      0x10
#define PCDRVINT      0x14
#define NMPUNMICTRL   0x0c
#define ARMSC_BOOTMR     0x04
#define PCSTA 0x00

#define ARMSC_PCIE_RST   0x0C
#define ARMSC_PCIE_REG_1 0x18
#define ARMSC_PCIE_REG_2 0x1C
#define ARMSC_PCIE_REG_3 0x20
/* --- ELPHY --- */
    // paddr [11] was excluded, so it paddr forms next way
#define ELPHY_PCS_CMN_CTRL1             0xC000*2 + 0x000*4
#define ELPHY_PCS_CMN_CTRL2             0xC000*2 + 0x001*4
#define ELPHY_PCS_COM_LOCK_CFG1         0xC000*2 + 0x002*4
#define ELPHY_PCS_COM_LOCK_CFG2         0xC000*2 + 0x003*4
#define ELPHY_PCS_GEN3_EIE_LOCK_CFG     0xC000*2 + 0x004*4
#define ELPHY_PCS_LANE_DSBL             0xC000*2 + 0x005*4
#define ELPHY_PCS_RCV_DET_INH           0xC000*2 + 0x006*4
#define ELPHY_PCS_ISO_CMN_CTRL          0xC000*2 + 0x008*4
#define ELPHY_EQ_TRAIN_CTRL             0xC000*2 + 0x008*4
#define ELPHY_ISO_LN_CTRL               0xD000*2 + 0x000*4
#define ELPHY_EQ_COEF_TRAIN_CTRL        0xD000*2 + 0x001*4
#define ELPHY_PCS_ISO_TX_CTRL           0xD000*2 + 0x000*4
#define ELPHY_PCS_ISO_TX_LPC_LO         0xD000*2 + 0x001*4
#define ELPHY_PCS_ISO_TX_LPC_HI         0xD000*2 + 0x002*4
#define ELPHY_PCS_ISO_TX_DMPH_LO        0xD000*2 + 0x003*4
#define ELPHY_PCS_ISO_TX_DMPH_HI        0xD000*2 + 0x004*4
#define ELPHY_PCS_ISO_TX_FSLF           0xD000*2 + 0x005*4
#define ELPHY_PCS_ISO_TX_DATA_LO        0xD000*2 + 0x006*4
#define ELPHY_PCS_ISO_TX_DATA_HI        0xD000*2 + 0x007*4
#define ELPHY_PCS_ISO_RX_CTRL           0xD000*2 + 0x008*4
#define ELPHY_PCS_ISO_RX_EQ_EVAL        0xD000*2 + 0x009*4
#define ELPHY_PCS_ISO_RX_DATA_LO        0xD000*2 + 0x00E*4
#define ELPHY_PCS_ISO_RX_DATA_HI        0xD000*2 + 0x00F*4
#define ELPHY_PMA_CMN_CTRL              0xE000*2 + 0x000*4
#define ELPHY_PMA_SSM_STATE             0xE000*2 + 0x001*4
#define ELPHY_PMA_PSM_STATE_LO          0xE000*2 + 0x002*4
#define ELPHY_PMA_PSM_STATE_HI          0xE000*2 + 0x003*4
#define ELPHY_PMA_ISO_CMN_CTRL          0xE000*2 + 0x004*4
#define ELPHY_PMA_ISO_MODE              0xE000*2 + 0x005*4
#define ELPHY_PMA_ISO_PWRST_CTRL        0xE000*2 + 0x006*4
#define ELPHY_PMA_ISOLATION_CTRL        0xE000*2 + 0x00F*4
#define ELPHY_PMA_PMA_CMN_SSM_DIAG      0x0000*2 + 0x039*4
#define ELPHY_PMA_XCVR_CTRL             0xF000*2 + 0x000*4
#define ELPHY_PMA_XCVR_LPBK             0xF000*2 + 0x001*4
#define ELPHY_PMA_PI_POS                0xF000*2 + 0x002*4
#define ELPHY_PMA_ISO_XCVR_CTRL         0xF000*2 + 0x003*4
#define ELPHY_PMA_ISO_TX_LPC_LO         0xF000*2 + 0x004*4
#define ELPHY_PMA_ISO_TX_LPC_HI         0xF000*2 + 0x005*4
#define ELPHY_PMA_ISO_TX_DMPH_LO        0xF000*2 + 0x006*4
#define ELPHY_PMA_ISO_TX_DMPH_HI        0xF000*2 + 0x007*4
#define ELPHY_PMA_ISO_TX_FSLF           0xF000*2 + 0x008*4
#define ELPHY_PMA_ISO_TX_MGN            0xF000*2 + 0x009*4
#define ELPHY_PMA_ISO_RX_EQ_CTRL        0xF000*2 + 0x00D*4
#define ELPHY_PMA_ISO_DATA_LO           0xF000*2 + 0x00E*4
#define ELPHY_PMA_ISO_DATA_HI           0xF000*2 + 0x00F*4
#define ELPHY_PMA_TX_BIST_CTRL          0x4000*2 + 0x040*4
#define ELPHY_PMA_TX_BIST_UDDWR         0x4000*2 + 0x041*4
#define ELPHY_PMA_RX_BIST_CTRL          0x8000*2 + 0x120*4
#define ELPHY_PMA_RX_BIST_SYNCCNT       0x8000*2 + 0x121*4
#define ELPHY_PMA_RX_BIST_UDDWR         0x8000*2 + 0x122*4
#define ELPHY_PMA_RX_BIST_ERRCNT        0x8000*2 + 0x123*4
#define ELPHY_PMA_CMN_PID_TYPE          0x0000*2 + 0x000*4
#define ELPHY_PMA_CMN_PID_NUM           0x0000*2 + 0x003*4
#define ELPHY_PMA_CMN_PID_REV           0x0000*2 + 0x004*4
#define ELPHY_PMA_CMN_PID_MFG           0x0000*2 + 0x008*4
#define ELPHY_PMA_CMN_PID_NODE          0x0000*2 + 0x009*4
#define ELPHY_PMA_CMN_PID_FLV           0x0000*2 + 0x00A*4
#define ELPHY_PMA_CMN_PID_IOV           0x0000*2 + 0x00C*4
#define ELPHY_PMA_CMN_PID_METAL         0x0000*2 + 0x00D*4
#define ELPHY_PMA_CMN_PID_LANES         0x0000*2 + 0x010*4
#define ELPHY_RX_EYESURF_CTRL           0x8000*2 + 0x0E0*4
#define ELPHY_RX_EYESURF_TMR_DELLOW     0x8000*2 + 0x0E4*4
#define ELPHY_RX_EYESURF_TMR_DELHIGH    0x8000*2 + 0x0E5*4
#define ELPHY_RX_EYESURF_TMR_TESTLOW    0x8000*2 + 0x0E6*4
#define ELPHY_RX_EYESURF_TMR_TESTHIGH   0x8000*2 + 0x0E7*4
#define ELPHY_RX_EYESURF_NS_COORD       0x8000*2 + 0x0E8*4
#define ELPHY_RX_EYESURF_EW_COORD       0x8000*2 + 0x0E9*4
#define ELPHY_RX_EYESURF_ERRCNT         0x8000*2 + 0x0EA*4

/* --- This registers are used for netlist simspeedup --- */
#define ELPHY_MASSWR_TX_RCVDET_CTRL                    0x6000*2 + 0x020*4
#define ELPHY_CMN_SSM_BANDGAP_TMR                      0x0000*2 + 0x020*4
#define ELPHY_CMN_SSM_BIAS_TMR                         0x0000*2 + 0x021*4
#define ELPHY_CMN_SSM_PLLEN_TMR                        0x0000*2 + 0x022*4
#define ELPHY_CMN_SSM_PLLPRE_TMR                       0x0000*2 + 0x023*4
#define ELPHY_CMN_SSM_PLLVREF_TMR                      0x0000*2 + 0x024*4
#define ELPHY_CMN_SSM_PLLLOCK_TMR                      0x0000*2 + 0x025*4
#define ELPHY_CMN_SSM_PLLCLKDIS_TMR                    0x0000*2 + 0x026*4
#define ELPHY_CMN_SSM_LANECAL_TMR                      0x0000*2 + 0x028*4
#define ELPHY_CMN_SSM_A0IN_TMR                         0x0000*2 + 0x029*4
#define ELPHY_CMN_SSM_A1IN_TMR                         0x0000*2 + 0x02A*4
#define ELPHY_CMN_SSM_A2IN_TMR                         0x0000*2 + 0x02B*4
#define ELPHY_CMN_SSM_A3IN_TMR                         0x0000*2 + 0x02C*4
#define ELPHY_CMN_BGCAL_OVRD                           0x0000*2 + 0x061*4
#define ELPHY_CMN_PLL_CPI_OVRD                         0x0000*2 + 0x081*4
#define ELPHY_MASSWR_CMN_SDCAL0_OVRD                   0xA000*2 + 0x181*4
#define ELPHY_MASSWR_CMN_SDCAL1_OVRD                   0xA000*2 + 0x191*4
#define ELPHY_CMN_PLL_VCOCAL_OVRD                      0x0000*2 + 0x0A2*4
#define ELPHY_CMN_PLL_VCOCAL_CNT_START                 0x0000*2 + 0x0A7*4
#define ELPHY_CMN_PLL_LOCK_CNT_START                   0x0000*2 + 0x0A5*4
#define ELPHY_MASSWR_RX_IQPI_ILL_CAL_OVRD              0xA000*2 + 0x022*4
#define ELPHY_MASSWR_RX_IQPI_ILL_LOCK_REFCNT_START     0xA000*2 + 0x025*4
#define ELPHY_MASSWR_RX_IQPI_ILL_LOCK_CALCNT_START0    0xA000*2 + 0x026*4
#define ELPHY_MASSWR_RX_IQPI_ILL_LOCK_CALCNT_START1    0xA000*2 + 0x027*4
#define ELPHY_MASSWR_RX_EPI_ILL_CAL_OVRD               0xA000*2 + 0x042*4
#define ELPHY_MASSWR_RX_EPI_ILL_LOCK_REFCNT_START      0xA000*2 + 0x045*4
#define ELPHY_MASSWR_RX_EPI_ILL_LOCK_CALCNT_START0     0xA000*2 + 0x046*4
#define ELPHY_MASSWR_RX_EPI_ILL_LOCK_CALCNT_START1     0xA000*2 + 0x047*4
/* #define ELPHY_LANE_CAL_RESET_TIME_VALUE            register wasn`t found */
#define ELPHY_CMN_TXPUCAL_OVRD                         0x0000*2 + 0x0C1*4
#define ELPHY_CMN_TXPDCAL_OVRD                         0x0000*2 + 0x0E1*4
#define ELPHY_CMN_RXCAL_OVRD                           0x0000*2 + 0x101*4
#define ELPHY_MASSWR_RX_SLC_INIT_TMR                   0xA000*2 + 0x14D*4
#define ELPHY_MASSWR_RX_SLC_RUN_TMR                    0xA000*2 + 0x14E*4
/* #define ELPHY_CMN_PRPLL_LOCK_CNT_START             register wasn`t found */

/* --- PCIe --- */
#define PCIe_Core_FuncRPConfig  0x00000000
#define PCIe_Core_LocalMgmt     0x00001000
#define PCIe_Core_AXIConfig     0x00004000
#define PCIe_Core_DMAConfig     0x00006000

#define PCIe_EP_i_vendor_id_device_id                    0x0000
#define PCIe_EP_i_command_status                         0x0004
#define PCIe_EP_i_revision_id_class_code                 0x0008
#define PCIe_EP_i_bist_header_latency_cache_line         0x000C
#define PCIe_EP_i_base_addr_0                            0x0010
#define PCIe_EP_i_base_addr_1                            0x0014
#define PCIe_EP_i_base_addr_2                            0x0018
#define PCIe_EP_i_base_addr_3                            0x001C
#define PCIe_EP_i_base_addr_4                            0x0020
#define PCIe_EP_i_base_addr_5                            0x0024
#define PCIe_EP_i_subsystem_vendor_id_subsystem_i        0x002C
#define PCIe_EP_i_capabilities_pointer                   0x0034
#define PCIe_EP_i_intrpt_line_intrpt_pin                 0x003C
#define PCIe_EP_i_pwr_mgmt_cap                           0x0080
#define PCIe_EP_i_pwr_mgmt_ctrl_stat_rep                 0x0084
#define PCIe_EP_i_msi_ctrl_reg                           0x0090
#define PCIe_EP_i_msi_msg_low_addr                       0x0094
#define PCIe_EP_i_msi_msg_hi_addr                        0x0098
#define PCIe_EP_i_msi_msg_data                           0x009C
#define PCIe_EP_i_msi_mask                               0x00A0
#define PCIe_EP_i_msi_pending_bits                       0x00A4
#define PCIe_EP_i_pcie_cap_list                          0x00C0
#define PCIe_EP_i_pcie_dev_cap                           0x00C4
#define PCIe_EP_i_pcie_dev_ctrl_status                   0x00C8
#define PCIe_EP_i_link_cap                               0x00CC
#define PCIe_EP_i_link_ctrl_status                       0x00D0
#define PCIe_EP_i_pcie_dev_cap_2                         0x00E4
#define PCIe_EP_i_pcie_dev_ctrl_status_2                 0x00E8
#define PCIe_EP_i_link_cap_2_reg                         0x00EC
#define PCIe_EP_i_link_ctrl_status_2                     0x00F0
#define PCIe_EP_i_AER_enhanced_cap_hdr                   0x0100
#define PCIe_EP_i_uncorr_err_status                      0x0104
#define PCIe_EP_i_uncorr_err_mask                        0x0108
#define PCIe_EP_i_uncorr_err_severity                    0x010C
#define PCIe_EP_i_corr_err_status                        0x0110
#define PCIe_EP_i_corr_err_mask                          0x0114
#define PCIe_EP_i_advcd_err_cap_ctrl                     0x0118
#define PCIe_EP_i_hdr_log_0                              0x011C
#define PCIe_EP_i_hdr_log_1                              0x0120
#define PCIe_EP_i_hdr_log_2                              0x0124
#define PCIe_EP_i_hdr_log_3                              0x0128

#define PCIe_RP_i_vendor_id_device_id                    0x0000
#define PCIe_RP_i_command_status                         0x0004
#define PCIe_RP_i_revision_id_class_code                 0x0008
#define PCIe_RP_i_bist_header_latency_cache_line         0x000C
#define PCIe_RP_i_RC_BAR_0                               0x0010
#define PCIe_RP_i_RC_BAR_1                               0x0014
#define PCIe_RP_i_pcie_bus_numbers                       0x0018
#define PCIe_RP_i_pcie_io_base_limit                     0x001C
#define PCIe_RP_i_pcie_mem_base_limit                    0x0020
#define PCIe_RP_i_pcie_prefetch_base_limit               0x0024
#define PCIe_RP_i_pcie_prefetch_base_upper               0x0028
#define PCIe_RP_i_pcie_prefetch_limit_upper              0x002C
#define PCIe_RP_i_pcie_io_base_limit_upper               0x0030
#define PCIe_RP_i_capabilities_pointer                   0x0034
#define PCIe_RP_i_intrpt_line_intrpt_pin                 0x003C
#define PCIe_RP_i_pwr_mgmt_cap                           0x0080
#define PCIe_RP_i_pwr_mgmt_ctrl_stat_rep                 0x0084
#define PCIe_RP_i_msi_ctrl_reg                           0x0090
#define PCIe_RP_i_msi_msg_low_addr                       0x0094
#define PCIe_RP_i_msi_msg_hi_addr                        0x0098
#define PCIe_RP_i_msi_msg_data                           0x009C
#define PCIe_RP_i_msi_mask                               0x00A0
#define PCIe_RP_i_msi_pending_bits                       0x00A4
#define PCIe_RP_i_pcie_cap_list                          0x00C0
#define PCIe_RP_i_pcie_cap                               0x00C4
#define PCIe_RP_i_pcie_dev_ctrl_status                   0x00C8
#define PCIe_RP_i_link_cap                               0x00CC
#define PCIe_RP_i_link_ctrl_status                       0x00D0
#define PCIe_RP_i_slot_ctrl_status                       0x00D8
#define PCIe_RP_i_root_ctrl_cap                          0x00DC
#define PCIe_RP_i_root_status                            0x00E0
#define PCIe_RP_i_pcie_cap_2                             0x00E4
#define PCIe_RP_i_pcie_dev_ctrl_status_2                 0x00E8
#define PCIe_RP_i_link_cap_2                             0x00EC
#define PCIe_RP_i_link_ctrl_status_2                     0x00F0
#define PCIe_RP_i_AER_enhncd_cap                         0x0100
#define PCIe_RP_i_uncorr_err_status                      0x0104
#define PCIe_RP_i_uncorr_err_mask                        0x0108
#define PCIe_RP_i_uncorr_err_severity                    0x010C
#define PCIe_RP_i_corr_err_status                        0x0110
#define PCIe_RP_i_corr_err_mask                          0x0114
#define PCIe_RP_i_adv_err_cap_ctl                        0x0118
#define PCIe_RP_i_hdr_log_0                              0x011C
#define PCIe_RP_i_hdr_log_1                              0x0120
#define PCIe_RP_i_hdr_log_2                              0x0124
#define PCIe_RP_i_hdr_log_3                              0x0128
#define PCIe_RP_i_root_err_cmd                           0x012C
#define PCIe_RP_i_root_err_stat                          0x0130
#define PCIe_RP_i_err_src_id                             0x0134

#define PCIe_LocMgmt_i_pl_config_0_reg                         0x0000
#define PCIe_LocMgmt_i_pl_config_1_reg                         0x0004
#define PCIe_LocMgmt_i_dll_tmr_config_reg                      0x0008
#define PCIe_LocMgmt_i_rcv_cred_lim_0_reg                      0x000C
#define PCIe_LocMgmt_i_rcv_cred_lim_1_reg                      0x0010
#define PCIe_LocMgmt_i_transm_cred_lim_0_reg                   0x0014
#define PCIe_LocMgmt_i_transm_cred_lim_1_reg                   0x0018
#define PCIe_LocMgmt_i_transm_cred_update_int_config_0_reg     0x001C
#define PCIe_LocMgmt_i_transm_cred_update_int_config_1_reg     0x0020
#define PCIe_LocMgmt_i_L0S_timeout_limit_reg                   0x0024
#define PCIe_LocMgmt_i_transmit_tlp_count_reg                  0x0028
#define PCIe_LocMgmt_i_transmit_tlp_payload_dword_count_reg    0x002C
#define PCIe_LocMgmt_i_receive_tlp_count_reg                   0x0030
#define PCIe_LocMgmt_i_receive_tlp_payload_dword_count_reg     0x0034
#define PCIe_LocMgmt_i_compln_tmout_lim_0_reg                  0x0038
#define PCIe_LocMgmt_i_compln_tmout_lim_1_reg                  0x003C
#define PCIe_LocMgmt_i_L1_st_reentry_delay_reg                 0x0040
#define PCIe_LocMgmt_i_vendor_id_reg                           0x0044
#define PCIe_LocMgmt_i_aspm_L1_entry_tmout_delay_reg           0x0048
#define PCIe_LocMgmt_i_pme_turnoff_ack_delay_reg               0x004C
#define PCIe_LocMgmt_i_linkwidth_control_reg                   0x0050
#define PCIe_LocMgmt_i_sris_control_reg                        0x0074
#define PCIe_LocMgmt_i_shdw_hdr_log_0_reg                      0x0100
#define PCIe_LocMgmt_i_shdw_hdr_log_1_reg                      0x0104
#define PCIe_LocMgmt_i_shdw_hdr_log_2_reg                      0x0108
#define PCIe_LocMgmt_i_shdw_hdr_log_3_reg                      0x010C
#define PCIe_LocMgmt_i_shdw_func_num_reg                       0x0110
#define PCIe_LocMgmt_i_shdw_ur_err_reg                         0x0114
#define PCIe_LocMgmt_i_debug_dllp_count_gen1_reg               0x0144
#define PCIe_LocMgmt_i_debug_dllp_count_gen2_reg               0x0148
#define PCIe_LocMgmt_i_debug_dllp_count_gen3_reg               0x014C
#define PCIe_LocMgmt_i_negotiated_lane_map_reg                 0x0200
#define PCIe_LocMgmt_i_receive_fts_count_reg                   0x0204
#define PCIe_LocMgmt_i_debug_mux_control_reg                   0x0208
#define PCIe_LocMgmt_i_local_error_status_register             0x020C
#define PCIe_LocMgmt_i_local_intrpt_mask_reg                   0x0210
#define PCIe_LocMgmt_i_lcrc_err_count_reg                      0x0214
#define PCIe_LocMgmt_i_ecc_corr_err_count_reg                  0x0218
#define PCIe_LocMgmt_i_pme_service_timeout_delay_reg           0x0224
#define PCIe_LocMgmt_i_root_port_requestor_id_reg              0x0228
#define PCIe_LocMgmt_i_ep_bus_device_number_reg                0x022C
#define PCIe_LocMgmt_i_pf_0_BAR_config_0_reg                   0x0240
#define PCIe_LocMgmt_i_pf_0_BAR_config_1_reg                   0x0244
#define PCIe_LocMgmt_i_pf_config_reg                           0x02C0
#define PCIe_LocMgmt_i_rc_BAR_config_reg                       0x0300
#define PCIe_LocMgmt_i_ecc_corr_err_count_reg_axi              0x0C80

#define PCIe_AXI_outregion_0_addr_translation_0                0x0000
#define PCIe_AXI_outregion_0_addr_translation_1                0x0004
#define PCIe_AXI_outregion_0_header_descriptor_0               0x0008
#define PCIe_AXI_outregion_0_header_descriptor_1               0x000C
#define PCIe_AXI_outregion_0_region_base_addr                  0x0018
#define PCIe_AXI_outregion_1_addr_translation_0                0x0020
#define PCIe_AXI_outregion_1_addr_translation_1                0x0024
#define PCIe_AXI_outregion_1_header_descriptor_0               0x0028
#define PCIe_AXI_outregion_1_header_descriptor_1               0x002C
#define PCIe_AXI_outregion_1_region_base_addr                  0x0038
#define PCIe_AXI_outregion_2_addr_translation_0                0x0040
#define PCIe_AXI_outregion_2_addr_translation_1                0x0044
#define PCIe_AXI_outregion_2_header_descriptor_0               0x0048
#define PCIe_AXI_outregion_2_header_descriptor_1               0x004C
#define PCIe_AXI_outregion_2_region_base_addr                  0x0058
#define PCIe_AXI_outregion_3_addr_translation_0                0x0060
#define PCIe_AXI_outregion_3_addr_translation_1                0x0064
#define PCIe_AXI_outregion_3_header_descriptor_0               0x0068
#define PCIe_AXI_outregion_3_header_descriptor_1               0x006C
#define PCIe_AXI_outregion_3_region_base_addr                  0x0078
#define PCIe_AXI_outregion_4_addr_translation_0                0x0080
#define PCIe_AXI_outregion_4_addr_translation_1                0x0084
#define PCIe_AXI_outregion_4_header_descriptor_0               0x0088
#define PCIe_AXI_outregion_4_header_descriptor_1               0x008C
#define PCIe_AXI_outregion_4_region_base_addr                  0x0098
#define PCIe_AXI_outregion_5_addr_translation_0                0x00A0
#define PCIe_AXI_outregion_5_addr_translation_1                0x00A4
#define PCIe_AXI_outregion_5_header_descriptor_0               0x00A8
#define PCIe_AXI_outregion_5_header_descriptor_1               0x00AC
#define PCIe_AXI_outregion_5_region_base_addr                  0x00B8
#define PCIe_AXI_outregion_6_addr_translation_0                0x00C0
#define PCIe_AXI_outregion_6_addr_translation_1                0x00C4
#define PCIe_AXI_outregion_6_header_descriptor_0               0x00C8
#define PCIe_AXI_outregion_6_header_descriptor_1               0x00CC
#define PCIe_AXI_outregion_6_region_base_addr                  0x00D8
#define PCIe_AXI_outregion_7_addr_translation_0                0x00E0
#define PCIe_AXI_outregion_7_addr_translation_1                0x00E4
#define PCIe_AXI_outregion_7_header_descriptor_0               0x00E8
#define PCIe_AXI_outregion_7_header_descriptor_1               0x00EC
#define PCIe_AXI_outregion_7_region_base_addr                  0x00F8
#define PCIe_AXI_outregion_8_addr_translation_0                0x0100
#define PCIe_AXI_outregion_8_addr_translation_1                0x0104
#define PCIe_AXI_outregion_8_header_descriptor_0               0x0108
#define PCIe_AXI_outregion_8_header_descriptor_1               0x010C
#define PCIe_AXI_outregion_8_region_base_addr                  0x0118
#define PCIe_AXI_outregion_9_addr_translation_0                0x0120
#define PCIe_AXI_outregion_9_addr_translation_1                0x0124
#define PCIe_AXI_outregion_9_header_descriptor_0               0x0128
#define PCIe_AXI_outregion_9_header_descriptor_1               0x012C
#define PCIe_AXI_outregion_9_region_base_addr                  0x0138
#define PCIe_AXI_outregion_10_addr_translation_0               0x0140
#define PCIe_AXI_outregion_10_addr_translation_1               0x0144
#define PCIe_AXI_outregion_10_header_descriptor_0              0x0148
#define PCIe_AXI_outregion_10_header_descriptor_1              0x014C
#define PCIe_AXI_outregion_10_region_base_addr                 0x0158
#define PCIe_AXI_outregion_11_addr_translation_0               0x0160
#define PCIe_AXI_outregion_11_addr_translation_1               0x0164
#define PCIe_AXI_outregion_11_header_descriptor_0              0x0168
#define PCIe_AXI_outregion_11_header_descriptor_1              0x016C
#define PCIe_AXI_outregion_11_region_base_addr                 0x0178
#define PCIe_AXI_outregion_12_addr_translation_0               0x0180
#define PCIe_AXI_outregion_12_addr_translation_1               0x0184
#define PCIe_AXI_outregion_12_header_descriptor_0              0x0188
#define PCIe_AXI_outregion_12_header_descriptor_1              0x018C
#define PCIe_AXI_outregion_12_region_base_addr                 0x0198
#define PCIe_AXI_outregion_13_addr_translation_0               0x01A0
#define PCIe_AXI_outregion_13_addr_translation_1               0x01A4
#define PCIe_AXI_outregion_13_header_descriptor_0              0x01A8
#define PCIe_AXI_outregion_13_header_descriptor_1              0x01AC
#define PCIe_AXI_outregion_13_region_base_addr                 0x01B8
#define PCIe_AXI_outregion_14_addr_translation_0               0x01C0
#define PCIe_AXI_outregion_14_addr_translation_1               0x01C4
#define PCIe_AXI_outregion_14_header_descriptor_0              0x01C8
#define PCIe_AXI_outregion_14_header_descriptor_1              0x01CC
#define PCIe_AXI_outregion_14_region_base_addr                 0x01D8
#define PCIe_AXI_outregion_15_addr_translation_0               0x01E0
#define PCIe_AXI_outregion_15_addr_translation_1               0x01E4
#define PCIe_AXI_outregion_15_header_descriptor_0              0x01E8
#define PCIe_AXI_outregion_15_header_descriptor_1              0x01EC
#define PCIe_AXI_outregion_15_region_base_addr                 0x01F8
#define PCIe_AXI_outregion_16_addr_translation_0               0x0200
#define PCIe_AXI_outregion_16_addr_translation_1               0x0204
#define PCIe_AXI_outregion_16_header_descriptor_0              0x0208
#define PCIe_AXI_outregion_16_header_descriptor_1              0x020C
#define PCIe_AXI_outregion_16_region_base_addr                 0x0218
#define PCIe_AXI_outregion_17_addr_translation_0               0x0220
#define PCIe_AXI_outregion_17_addr_translation_1               0x0224
#define PCIe_AXI_outregion_17_header_descriptor_0              0x0228
#define PCIe_AXI_outregion_17_header_descriptor_1              0x022C
#define PCIe_AXI_outregion_17_region_base_addr                 0x0238
#define PCIe_AXI_outregion_18_addr_translation_0               0x0240
#define PCIe_AXI_outregion_18_addr_translation_1               0x0244
#define PCIe_AXI_outregion_18_header_descriptor_0              0x0248
#define PCIe_AXI_outregion_18_header_descriptor_1              0x024C
#define PCIe_AXI_outregion_18_region_base_addr                 0x0258
#define PCIe_AXI_outregion_19_addr_translation_0               0x0260
#define PCIe_AXI_outregion_19_addr_translation_1               0x0264
#define PCIe_AXI_outregion_19_header_descriptor_0              0x0268
#define PCIe_AXI_outregion_19_header_descriptor_1              0x026C
#define PCIe_AXI_outregion_19_region_base_addr                 0x0278
#define PCIe_AXI_outregion_20_addr_translation_0               0x0280
#define PCIe_AXI_outregion_20_addr_translation_1               0x0284
#define PCIe_AXI_outregion_20_header_descriptor_0              0x0288
#define PCIe_AXI_outregion_20_header_descriptor_1              0x028C
#define PCIe_AXI_outregion_20_region_base_addr                 0x0298
#define PCIe_AXI_outregion_21_addr_translation_0               0x02A0
#define PCIe_AXI_outregion_21_addr_translation_1               0x02A4
#define PCIe_AXI_outregion_21_header_descriptor_0              0x02A8
#define PCIe_AXI_outregion_21_header_descriptor_1              0x02AC
#define PCIe_AXI_outregion_21_region_base_addr                 0x02B8
#define PCIe_AXI_outregion_22_addr_translation_0               0x02C0
#define PCIe_AXI_outregion_22_addr_translation_1               0x02C4
#define PCIe_AXI_outregion_22_header_descriptor_0              0x02C8
#define PCIe_AXI_outregion_22_header_descriptor_1              0x02CC
#define PCIe_AXI_outregion_22_region_base_addr                 0x02D8
#define PCIe_AXI_outregion_23_addr_translation_0               0x02E0
#define PCIe_AXI_outregion_23_addr_translation_1               0x02E4
#define PCIe_AXI_outregion_23_header_descriptor_0              0x02E8
#define PCIe_AXI_outregion_23_header_descriptor_1              0x02EC
#define PCIe_AXI_outregion_23_region_base_addr                 0x02F8
#define PCIe_AXI_outregion_24_addr_translation_0               0x0300
#define PCIe_AXI_outregion_24_addr_translation_1               0x0304
#define PCIe_AXI_outregion_24_header_descriptor_0              0x0308
#define PCIe_AXI_outregion_24_header_descriptor_1              0x030C
#define PCIe_AXI_outregion_24_region_base_addr                 0x0318
#define PCIe_AXI_outregion_25_addr_translation_0               0x0320
#define PCIe_AXI_outregion_25_addr_translation_1               0x0324
#define PCIe_AXI_outregion_25_header_descriptor_0              0x0328
#define PCIe_AXI_outregion_25_header_descriptor_1              0x032C
#define PCIe_AXI_outregion_25_region_base_addr                 0x0338
#define PCIe_AXI_outregion_26_addr_translation_0               0x0340
#define PCIe_AXI_outregion_26_addr_translation_1               0x0344
#define PCIe_AXI_outregion_26_header_descriptor_0              0x0348
#define PCIe_AXI_outregion_26_header_descriptor_1              0x034C
#define PCIe_AXI_outregion_26_region_base_addr                 0x0358
#define PCIe_AXI_outregion_27_addr_translation_0               0x0360
#define PCIe_AXI_outregion_27_addr_translation_1               0x0364
#define PCIe_AXI_outregion_27_header_descriptor_0              0x0368
#define PCIe_AXI_outregion_27_header_descriptor_1              0x036C
#define PCIe_AXI_outregion_27_region_base_addr                 0x0378
#define PCIe_AXI_outregion_28_addr_translation_0               0x0380
#define PCIe_AXI_outregion_28_addr_translation_1               0x0384
#define PCIe_AXI_outregion_28_header_descriptor_0              0x0388
#define PCIe_AXI_outregion_28_header_descriptor_1              0x038C
#define PCIe_AXI_outregion_28_region_base_addr                 0x0398
#define PCIe_AXI_outregion_29_addr_translation_0               0x03A0
#define PCIe_AXI_outregion_29_addr_translation_1               0x03A4
#define PCIe_AXI_outregion_29_header_descriptor_0              0x03A8
#define PCIe_AXI_outregion_29_header_descriptor_1              0x03AC
#define PCIe_AXI_outregion_29_region_base_addr                 0x03B8
#define PCIe_AXI_outregion_30_addr_translation_0               0x03C0
#define PCIe_AXI_outregion_30_addr_translation_1               0x03C4
#define PCIe_AXI_outregion_30_header_descriptor_0              0x03C8
#define PCIe_AXI_outregion_30_header_descriptor_1              0x03CC
#define PCIe_AXI_outregion_30_region_base_addr                 0x03D8
#define PCIe_AXI_outregion_31_addr_translation_0               0x03E0
#define PCIe_AXI_outregion_31_addr_translation_1               0x03E4
#define PCIe_AXI_outregion_31_header_descriptor_0              0x03E8
#define PCIe_AXI_outregion_31_header_descriptor_1              0x03EC
#define PCIe_AXI_outregion_31_region_base_addr                 0x03F8
#define PCIe_AXI_inregion_rc_bar_0_addr_translation            0x0800
#define PCIe_AXI_inregion_rc_bar_1_addr_translation            0x0808
#define PCIe_AXI_inregion_rc_bar_2_addr_translation            0x0810
#define PCIe_AXI_l0                                            0x0824
#define PCIe_AXI_inregion_ep_bar_0_addr_translation            0x0840
#define PCIe_AXI_inregion_ep_bar_1_addr_translation            0x0848
#define PCIe_AXI_inregion_ep_bar_2_addr_translation            0x0850
#define PCIe_AXI_inregion_ep_bar_3_addr_translation            0x0858
#define PCIe_AXI_inregion_ep_bar_4_addr_translation            0x0860
#define PCIe_AXI_inregion_ep_bar_5_addr_translation            0x0868
#define PCIe_AXI_inregion_ep_bar_6_addr_translation            0x0870

#define PCIe_DMA_channel_0_ctrl                                0x0000
#define PCIe_DMA_channel_0_sp_l                                0x0004
#define PCIe_DMA_channel_0_sp_u                                0x0008
#define PCIe_DMA_channel_0_attr_l                              0x000C
#define PCIe_DMA_channel_0_attr_u                              0x0010
#define PCIe_DMA_channel_1_ctrl                                0x0014
#define PCIe_DMA_channel_1_sp_l                                0x0018
#define PCIe_DMA_channel_1_sp_u                                0x001C
#define PCIe_DMA_channel_1_attr_l                              0x0020
#define PCIe_DMA_channel_1_attr_u                              0x0024
#define PCIe_DMA_common_udma_int                               0x00A0
#define PCIe_DMA_common_udma_int_ena                           0x00A4
#define PCIe_DMA_common_udma_int_dis                           0x00A8
#define PCIe_DMA_common_udma_ib_ecc_uncorrectable_errors       0x00AC
#define PCIe_DMA_common_udma_ib_ecc_correctable_errors         0x00B0
#define PCIe_DMA_common_udma_ob_ecc_uncorrectable_errors       0x00B4
#define PCIe_DMA_common_udma_ob_ecc_correctable_errors         0x00B8
#define PCIe_DMA_common_udma_cap_ver                           0x00F8
#define PCIe_DMA_common_udma_config                            0x00FC

/* --- CCRG --- */
#define CCRG_PLLCR                0x0000
#define CCRG_PLLRDYMR             0x0004
#define CCRG_IMACR                0x0008
#define CCRG_PICR                 0x000C
#define CCRG_CLKDIV               0x0010
#define CCRG_CRGCR                0x0014
#define CCRG_PSCR                 0x0018

/* --- L2 Chache --- */
#define L2C_reg0_cache_id                 0x000
#define L2C_reg0_cache_type               0x004
#define L2C_reg1_control                  0x100
#define L2C_reg1_aux_control              0x104
#define L2C_reg1_tag_ram_control          0x108
#define L2C_reg1_data_ram_control         0x10C
#define L2C_reg2_ev_counter_ctrl          0x200
#define L2C_reg2_ev_counter1_cfg          0x204
#define L2C_reg2_ev_counter0_cfg          0x208
#define L2C_reg2_ev_counter1              0x20C
#define L2C_reg2_ev_counter0              0x210
#define L2C_reg2_int_mask                 0x214
#define L2C_reg2_int_mask_status          0x218
#define L2C_reg2_int_raw_status           0x21C
#define L2C_reg2_int_clear                0x220
#define L2C_reg7_cache_sync               0x730
#define L2C_reg7_inv_pa                   0x770
#define L2C_reg7_inv_way                  0x77C
#define L2C_reg7_clean_pa                 0x7B0
#define L2C_reg7_clean_index              0x7B8
#define L2C_reg7_clean_way                0x7BC
#define L2C_reg7_clean_inv_pa             0x7F0
#define L2C_reg7_clean_inv_index          0x7F8
#define L2C_reg7_clean_inv_way            0x7FC
#define L2C_reg9_d_lockdown0              0x900
#define L2C_reg9_i_lockdown0              0x904
#define L2C_reg9_d_lockdown1              0x908
#define L2C_reg9_i_lockdown1              0x90C
#define L2C_reg9_d_lockdown2              0x910
#define L2C_reg9_i_lockdown2              0x914
#define L2C_reg9_d_lockdown3              0x918
#define L2C_reg9_i_lockdown3              0x91C
#define L2C_reg9_d_lockdown4              0x920
#define L2C_reg9_i_lockdown4              0x924
#define L2C_reg9_d_lockdown5              0x928
#define L2C_reg9_i_lockdown5              0x92C
#define L2C_reg9_d_lockdown6              0x930
#define L2C_reg9_i_lockdown6              0x934
#define L2C_reg9_d_lockdown7              0x938
#define L2C_reg9_i_lockdown7              0x93C
#define L2C_reg9_lock_line_en             0x950
#define L2C_reg9_unlock_way               0x954
#define L2C_reg12_addr_filtering_start    0xC00
#define L2C_reg12_addr_filtering_end      0xC04
#define L2C_reg15_debug_ctrl              0xF40
#define L2C_reg15_prefetch_ctrl           0xF60
#define L2C_reg15_power_ctrl              0xF80


