@    AREA    InitS, CODE, READONLY

#include "defs_s.h"            
#include "macro.incl"
#include "blink_period.h"

.global __main
.global Reset_Handler

.global SVC_Stack
.global FIQ_Stack
.global IRQ_Stack
.global ABT_Stack
.global UND_Stack
.global MON_Stack
.global USR_Stack
.global Heap_Base

.global SVCStackInitial
    
SVCStackInitial:
    .word SVC_Stack
@    .entry
    
Reset_Handler: @-------------------------

    @Start in SVC mode (A,I,F bits=1 all interrupts disabled)
    MOV R0,  #0
    MOV R1,  #0
    MOV R2,  #0
    MOV R3,  #0
    MOV R4,  #0
    MOV R5,  #0
    MOV R6,  #0
    MOV R7,  #0
    MOV R8,  #0
    MOV R9,  #0
    MOV R10, #0
    MOV R11, #0
    MOV R12, #0
    MOV R13, #0
    MOV R14, #0
SP_init_instruction:
    LDR  SP, [PC,#(SVCStackInitial-SP_init_instruction-8)]
                       /* This Is To Prevent Assembler Changing This Instruction*/
    
    
                MOV     R0,#0xf00000               @ Write CPACR (Coprocessor Access Control Register)
                MCR     p15,0,R0,c1,c0,2           @ to enable coprocessors cp10 & cp11
                MRC     p15,0,R0,c1,c0,2           @ Read CPACR
                UBFX    R0, R0, #20, #1          @ if bit 20 is zero then so the bits 23:21 so FPU present
                
                MOV r0,#1<<30
                MCR     p10, #7, r0, c8, c0, 0                 @ enable FPU by writing FPEXC
                MOV r0,#0
                MCR     p10, #7, r0, c1, c0, 0                 @ init FPSR
                LDC p11, c0, [r0], {32}                    @ init registers d0-d15
    @VMOV.F32 S0, R0

@ Enter each mode in turn and initialize the registers specific to it
@ '_c' means CPSR[7:0]
    MSR  CPSR_c, #Mode_FIQ | I_Bit | F_Bit    @@@@@@@@@@@@@@@@@@@@@@@@@@  
    MOV R8,  #0
    MOV R9,  #0
    MOV R10, #0
    MOV R11, #0
    MOV R12, #0
    @MOV R13, #0
    LDR  SP, =FIQ_Stack
    MOV R14, #0
    
    MSR  CPSR_c, #Mode_IRQ | I_Bit | F_Bit       
    @MOV R13, #0
    LDR  SP, =IRQ_Stack
    MOV R14, #0
    
    MSR  CPSR_c, #Mode_ABT | I_Bit | F_Bit       
    @MOV R13, #0
    LDR  SP, =ABT_Stack
    MOV R14, #0
    
    MSR  CPSR_c, #Mode_UND | I_Bit | F_Bit 
    @MOV R13, #0
    LDR  SP, =UND_Stack
    MOV R14, #0

    MSR  CPSR_c, #Mode_MON | I_Bit | F_Bit
    @MOV R13, #0
    LDR  SP, =MON_Stack
    MOV R14, #0

@ System mode shares user mode registers
    MSR  CPSR_c, #Mode_SYS | I_Bit | F_Bit  
    @MOV R13, #0
    LDR  SP, =USR_Stack
    MOV R14, #0
@//-------------------------------------------------------------------------------

@ Initialize ALU paths

    ADD R0, R1, R2

@ --- Set SVC Mode and enable interrupts + enable imprecise abort (it is needed because of many MPUs and ARRs in system)
    @MSR  CPSR_c, #Mode_SVC | I_Bit | F_Bit
    MSR CPSR_xc, #Mode_SVC

    LDR R1, =SC_Base
    LDR R0, [R1, #PCSTA]
    CMP R0, #4
    BNE main_call

    LDR R1, =0x000CA000
    LDR R0, =0x0810012A
    STR R0, [R1, #0x0]
    
    MOV R0, #4
delayloop:
    SUBS R0, R0, #1
    BNE delayloop
    
crgsetuploop:
    LDR R0, [R1, #0x04]
    TST R0, #1
    BEQ crgsetuploop
    
    MOV R0, #0x40
    STR R0, [R1, #0x010]
    
crgsetuploop2:
    LDR R0, [R1, #0x010]
    CMP R0, #0x00000040
    BNE crgsetuploop2
@------------------------------------------------------------------------------
    LDR R1, =0x000C6000
    LDR R0, =0x040000C6
    STR R0, [R1, #0x0]
    
    MOV R0, #4
delayloop_cl0:
    SUBS R0, R0, #1
    BNE delayloop_cl0
    
crgsetuploop_cl0:
    LDR R0, [R1, #0x04]
    TST R0, #1
    BEQ crgsetuploop_cl0
    
    MOV R0, #0x20
    STR R0, [R1, #0x010]
    
crgsetuploop2_cl0:
    LDR R0, [R1, #0x010]
    CMP R0, #0x00000020
    BNE crgsetuploop2_cl0
@------------------------------------------------------------------------------
    LDR R1, =0x000C7000
    LDR R0, =0x040000C6
    STR R0, [R1, #0x0]
    
    MOV R0, #4
delayloop_cl1:
    SUBS R0, R0, #1
    BNE delayloop_cl1
    
crgsetuploop_cl1:
    LDR R0, [R1, #0x04]
    TST R0, #1
    BEQ crgsetuploop_cl1
    
    MOV R0, #0x20
    STR R0, [R1, #0x010]
    
crgsetuploop2_cl1:
    LDR R0, [R1, #0x010]
    CMP R0, #0x00000020
    BNE crgsetuploop2_cl1
@------------------------------------------------------------------------------
    LDR R1, =0x000C8000
    LDR R0, =0x040000C6
    STR R0, [R1, #0x0]
    
    MOV R0, #4
delayloop_cl2:
    SUBS R0, R0, #1
    BNE delayloop_cl2
    
crgsetuploop_cl2:
    LDR R0, [R1, #0x04]
    TST R0, #1
    BEQ crgsetuploop_cl2
    
    MOV R0, #0x20
    STR R0, [R1, #0x010]
    
crgsetuploop2_cl2:
    LDR R0, [R1, #0x010]
    CMP R0, #0x00000020
    BNE crgsetuploop2_cl2
@------------------------------------------------------------------------------
    LDR R1, =0x000C9000
    LDR R0, =0x040000C6
    STR R0, [R1, #0x0]
    
    MOV R0, #4
delayloop_cl3:
    SUBS R0, R0, #1
    BNE delayloop_cl3
    
crgsetuploop_cl3:
    LDR R0, [R1, #0x04]
    TST R0, #1
    BEQ crgsetuploop_cl3
    
    MOV R0, #0x20
    STR R0, [R1, #0x010]
    
crgsetuploop2_cl3:
    LDR R0, [R1, #0x010]
    CMP R0, #0x00000020
    BNE crgsetuploop2_cl3
@----------------------------------------------------------------
    
@//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
@//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
@//-------------------------------------------------------------------------------
@//------------------- Call Test Program -----------------------------------------
@//-------------------------------------------------------------------------------   
@ --- Enter main
main_call:
    BL  __main
    LDR R1, =TUBE_Base
    STR R0, [R1, #TUBE_PRINTFINISH]
    @STR R0, [R1, #TUBE_PRINTHEX]
    
     /* Store Test Result Into Reserved Vector Cell */
    MOV R1, #0x14
    STR R0, [R1]
    
FinishDeadLoop:
    B FinishDeadLoop
    
@ This function have to be used because
@ c-library default heap don't fit

__user_initial_stackheap:
    LDR R0, =Heap_Base @Heap  (grows up)
    LDR R1, =USR_Stack @Stack (grows down)
    MOV PC, LR
    
    
