//#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "tube_print.h"

// PRINT_LEVEL_DEBUG
// PRINT_LEVEL_INFO 
// PRINT_LEVEL_NONE

static void print_dec(unsigned int dd, int n);
static void print_hex(unsigned int dd, int n);
static void print_str(char * ss);

int print_level = PRINT_LEVEL_INFO;

void tube_print(unsigned int fmt, ...){
/*
 *
 * usage: tube_print( <format>, <param1> ...);
 *
 *
 * format: hex unsigned int, each hex digit is format simbol or flag sinbol.
 * each format simbol correspons to one printed value in paraneters.
 * Printed value uses one parameter whith <value> if two parameters flag is unset.
 * Printed value uses two parameters whith <width> and <value> if two parameters flag is set.
 *
 *
 * format simbols:
 *
 * 0 - format end (after non zero digit; optional)
 *     ignored  (before non zero digit)
 * 1 - string
 * 2 - decimal unsigned int
 * 3 - hex unsigned int (without 0x prefix)
 * 4 - hex unsigned int (with 0x prefix)
 *
 * flag simbols:
 *
 * a - set raw tube print flag
 * b - set insert spase after each parameter flag 
 * c - clear insert spase after each parameter flag 
 * d - set use two parameters (<width>, <value>) for one format simbol flag 
 * e - clear use two parameters for one format simbol flag
 * f - no print buffer flag
 *
 * See tube_print_test for examples.
 */

    int ff, ii, hh, nn;
    int no_print_buf_flag = 0;
    int raw_tube_flag = 0;
    int space_after_flag = 0;
    int two_params_flag = 0;
    char * ss;
    va_list ap;
    va_start(ap, fmt);
    
    if (fmt == 0) return;
 
    while((fmt >> 28) == 0){
        fmt <<= 4;
        //printf("%08x\n", fmt);
    }
  
    while(ff = (fmt >> 28)){
        fmt <<= 4;
        //printf("%d\n", ff);
        if (two_params_flag){
              nn = va_arg(ap, int);
        }
        switch(ff){
           case 0:
              goto mmm;
           case 1:
              ss = va_arg(ap, char*);
              if (raw_tube_flag){
                  tubef_str(ss);
              } else {
                  print_str(ss);
              }
              break;
           case 2:
              ii = va_arg(ap, int);
              if (raw_tube_flag){
                  tubef_dec(ii);
              } else if (two_params_flag){
                  print_dec(ii,nn);
              } else {
                 print_dec(ii,1);
              }   
              break;
           case 3:
              hh = va_arg(ap, int);
              if (raw_tube_flag){
                  tubef_hex(hh);
              } else if (two_params_flag){
                  print_hex(hh,nn);
              } else {
                  print_hex(hh,1);
              }   
              break;
           case 4:
              hh = va_arg(ap, int);
              if (raw_tube_flag){
                  tubef_hex(hh);
              } else {
                  print_str("0x");
                  if (two_params_flag){
                      print_hex(hh,nn);
                  } else {
                      print_hex(hh,1);
                  }
              }   
              break;
           case 0xa:
              raw_tube_flag = 1;
              break;
           case 0xb:
              space_after_flag = 1;
              break;
           case 0xc:
              space_after_flag = 0;
              break;
           case 0xd:
              two_params_flag = 1;
              break;
           case 0xe:
              two_params_flag = 0;
              break;
           case 0xf:
              no_print_buf_flag = 1;
              break;
           default:
              tubef_str("tube_print: unknown format digit");
              tubef_hex(ff);
        }
        if (space_after_flag){
            print_str(" ");
        }
    }
mmm:
    va_end(ap);
    if (! no_print_buf_flag){
        buf_new();
    }
}

void ddprintf(char * f,...) {
	
    va_list list;
	int nmb;


    if (print_level == PRINT_LEVEL_DEBUG) {

        va_start(list, f);
        //nmb = vsprintf(print_buffer_ptr, f, list); DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
        print_buffer_ptr += nmb;
    }
}

void iiprintf(char * f,...) {
	
    va_list list;
	int nmb;

    if (print_level != PRINT_LEVEL_NONE) {
        va_start(list, f);
        //nmb = vsprintf(print_buffer_ptr, f, list); DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
        print_buffer_ptr += nmb;
    }
}

static void print_dec(unsigned int dd, int n) {
    int d;
    int i = 0;
    char * pp = print_buffer_ptr;
    char * qq = print_buffer_ptr;
    char cc;
    while (dd || i < n){
        i++;
        d = dd%10;
        dd = dd/10;
        if (pp >= print_buffer + PRINT_BUFFER_LNG - 1){
            break;
        }
        pp[0] = '0'+d;
        pp++;
    }
    print_buffer_ptr = pp;
    pp--;
    while(pp > qq){
        cc = qq[0];
        qq[0] = pp[0];
        pp[0] = cc;
        qq++;
        pp--;
    }
}

void dprint_dec(unsigned int dd, int n) {
    if (print_level == PRINT_LEVEL_DEBUG) {
        print_dec(dd, n);
    }
}

void iprint_dec(unsigned int dd, int n) {
    if (print_level != PRINT_LEVEL_NONE) {
        print_dec(dd, n);
    }
}

static void print_hex(unsigned int dd, int n){
    int d;
    int i = 0;
    char * pp = print_buffer_ptr;
    char * qq = print_buffer_ptr;
    char cc;
    while (dd || i < n){
        i++;
        d = dd & 0xf;
        dd = dd >> 4;
        if (pp >= print_buffer + PRINT_BUFFER_LNG - 1){
            break;
        }
        if (d < 10){
            pp[0] = '0' + d;
        } else {
            pp[0] = 'a' + d - 10;
        }
        pp++;
    }
    print_buffer_ptr = pp;
    pp--;
    while(pp > qq){
        cc = qq[0];
        qq[0] = pp[0];
        pp[0] = cc;
        qq++;
        pp--;
    }
}

void dprint_hex(unsigned int dd, int n) {
    if (print_level == PRINT_LEVEL_DEBUG) {
        print_hex(dd, n);
    }
}

void iprint_hex(unsigned int dd, int n) {
    if (print_level != PRINT_LEVEL_NONE) {
        print_hex(dd, n);
    }
}

static void print_str(char * ss) {
    char cc;
    for (;;){
        cc = ss[0];
        if (!cc){
            break;
        }
        if (print_buffer_ptr >= print_buffer + PRINT_BUFFER_LNG - 1){
            break;
        }
        print_buffer_ptr[0] = cc;
        ss++;
        print_buffer_ptr++;
    }
}

void dprint_str(char * ss) {
    if (print_level == PRINT_LEVEL_DEBUG) {
        print_str(ss);
    }
}

void iprint_str(char * ss) {
    if (print_level != PRINT_LEVEL_NONE) {
        print_str(ss);
    }
}

void buf_new (void) {
    if (print_buffer_ptr > print_buffer){
        print_buffer_ptr[0] = 0;
        tubef_str(print_buffer);
        print_buffer_ptr = print_buffer;
    }
}
    
void dprint_line (void) {
    if (print_level == PRINT_LEVEL_DEBUG) {
        tubef_str("____________________________________________");
    }
}

void iprint_line (void) {
    if (print_level != PRINT_LEVEL_NONE) {
        tubef_str("____________________________________________");
    }
}
            
