#include <err.h>
#include <errno.h>
#include <fcntl.h>
#include <gelf.h>
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <string.h>

int main (int argc, char *argv[])
{
	int		i, fd;
	Elf		*e;
	size_t		n;

	GElf_Ehdr	ehdr;
	GElf_Phdr	phdr;

	char		str_name[PATH_MAX];
	char		*end_ptr;
	long long int	serial;
	Elf32_Addr	addr;

	if (argc != 3)
		errx(EXIT_FAILURE, "usage: %s \"flash image path\"", argv[0]);

	sprintf(str_name, "%s.%s", argv[1], "elf");

	errno = 0;
	serial = strtoll(argv[2], &end_ptr, 10);

	if (errno || (argv[2] == end_ptr) || (*end_ptr != '\0') || (serial < 0))
		errx(EXIT_FAILURE, "serial number: %lld", serial);

	if (elf_version(EV_CURRENT) == EV_NONE)
		errx(EXIT_FAILURE, "ELF library initialization failed: %s" , elf_errmsg( -1));

	if ((fd = open(str_name, O_RDONLY , 0)) < 0)
		errx(EXIT_FAILURE, "open \"%s\" failed" , str_name);

	if ((e = elf_begin(fd, ELF_C_READ, NULL)) == NULL)
		errx(EXIT_FAILURE, "elf_begin() failed: %s.", elf_errmsg(-1));

	if (elf_kind(e) != ELF_K_ELF)
		errx(EXIT_FAILURE, "\"%s\" is not an ELF object.", "TODO");

	if (gelf_getehdr(e, &ehdr) == NULL)
		errx(EXIT_FAILURE, "getehdr() failed: %s.", elf_errmsg(-1));

	if (ehdr.e_type != ET_EXEC || (ehdr.e_machine != EM_ARM))
		errx(EXIT_FAILURE, "type \"%s\" failed" , str_name);

	if (elf_getphdrnum(e, &n) != 0)
		errx(EXIT_FAILURE, "elf_getphdrnum() failed: %s.", elf_errmsg(-1));

	for (i = 0; i < n; i++) {
		if (gelf_getphdr(e, i, &phdr) != &phdr)
			errx(EXIT_FAILURE, "getphdr() failed: %s.", elf_errmsg(-1));

		if (phdr.p_type != PT_LOAD)
			continue;

		if ((phdr.p_vaddr == phdr.p_paddr) && (phdr.p_paddr == 0x00000000)) {
			addr = 0x00000810;
			break;
		}
	}

	if (i == n)
		errx(EXIT_FAILURE, "file \"%s\" failed" , str_name);

	elf_end(e);
	close(fd);

	sprintf(str_name, "%s.%s", argv[1], "bin");

	if ((fd = open(str_name, O_RDWR, 0)) < 0)
		errx(EXIT_FAILURE, "open \"%s\" failed" , str_name);

	if (lseek(fd, addr, SEEK_SET) < 0)
		errx(EXIT_FAILURE, "lseek \"%s\" failed" , str_name);

	if (write(fd, &serial, sizeof(long long int)) != sizeof(long long int))
		errx(EXIT_FAILURE, "write \"%s\" failed" , str_name);

	close(fd);

	exit (EXIT_SUCCESS);
}
