#define _MULTI_THREADED

#include <time.h>
#include <pthread.h>
#include <signal.h>
#include <sched.h>

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/time.h>

#include <linux/types.h>
#include <stdlib.h>

#include "host.h"
#include "nm6408_ioctl.h"

void *load_cli_board(void *args);

pthread_mutex_t work_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t print_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t  wait_cond = PTHREAD_COND_INITIALIZER;

int pipe_fd[2];
int board_ret[NM6408_BRD_NUM];
int in_progress = 0;
int do_wait = 1;

int fd = -1;
unsigned char * file_buf = NULL;
int file_lng;

int nm6408_fd[NM6408_BRD_NUM];
int nm6408_index[NM6408_BRD_NUM];
int nm6408_count = 0;

unsigned char * flash_buf = NULL;
flash_t map_buf[FLASH_MAP_LINE];
int map_size = 0;

flash_t * spl_map = NULL;
int spl_ind = -1;

#define NM6408_MAGIC_DESC	0x00000800
#define NM6408_LIBLOAD_FORMAT	0x00060000

struct libload_format {
	__u32	reserved0[4];
	__u32	command;
	__u32   entry;
	__u32	reserved1[10];
	__u32	version[2];
	__u32	serial[2];
	__u32	reserved2[12];
} __attribute__ ((packed, aligned(4)));

void print_usage(){
    printf (
"USAGE:\n"
"	host-loader-nm6408\n"
"	host-loader-nm6408 -h, --help\n"
"	host-loader-nm6408 [-d, --dev=<device file> | -b, --brd=<board name>] -l, --load=<binary file>\n"
"	host-loader-nm6408 [-d, --dev=<device file> | -b, --brd=<board name>]\n"
"			   -l, --load=<target-loader-nm6408> [-f, --flash=<map file> | -c, --cmd=<input file>]\n\n"
    );
}

void print_help(){
    print_usage();
    printf (
"DESCRIPTION:\n"
"	host-loader-nm6408\n"
"	- print usage\n\n"

"	host-loader-nm6408 -h, --help\n"
"	- print this help\n\n"

"	host-loader-nm6408 [-d, --dev=<device file> | -b, --brd=<board name>] -l, --load=<binary file>\n"
"	- load <binary file> program in internal memory of top-ARM and start it.\n"
"	  Here <device file> is /dev/MC12705_xx for mc12705 board or /dev/NM-Card_xx to nm_card board or\n"
"	  /dev/NM-Mezzo_xx to nm_mezzo board and /dev/NM-Quad_xx to nm_quad, where \'xx\' can take value from 0 to 31 inclusive.\n\n"
"	  The <board name> is \'mc12705\' and \'nm_card\' and \'nm_mezzo\' and \'nm_quad\' respectively.\n"
"	  When specified <board name> operations be applyed to all finded boards.\n\n"

"	host-loader-nm6408 [-d, --dev=<device file> | -b, --brd=<board name>]\n"
"			   -l, --load=<target-loader-nm6408> [-f, --flash=<map file> | -c, --cmd=<input file>]\n"
"	- load \'target-loader-nm6408\' program in internal memory of top-ARM and start it.\n"
"	  If option \'-c\' is omits then be only loaded each <file name> and write it on flash at corresponding\n"
"	  <flash disp> offset, where <flash disp> <file name> is records in <map file>.\n\n"
"	  Else be executed commands specified in <input file> or run inretactive mode without flash programming.\n\n"
    );
}

static int comp_map(const void *p1, const void *p2);
static __u32 crc32_f(unsigned char * bgn, int length);
static int flash_map_prepare(flash_t *buf, int size);

int main(int argc, char *argv[]) //---------------------------------------- main
{
	pthread_t board_thread[NM6408_BRD_NUM];
	pthread_t _board_thread;
	int *_board_ret;

	struct timeval tv;
	unsigned int start_time;
	unsigned int current_time;

	const char *short_opt = "hd:b:l:f:c::";
	const struct option long_opt[] = {
		{"help", no_argument, NULL, 'h'},
		{"dev", required_argument, NULL, 'd'},
		{"brd", required_argument, NULL, 'b'},
		{"load", required_argument, NULL, 'l'},
		{"flash", required_argument, NULL, 'f'},
		{"cmd", optional_argument, NULL, 'c'},
		{NULL, 0, NULL, 0}
	};

	char * file_name = NULL;
	char * flash_map = NULL;
	char * board = NULL;
	char * device = NULL;

	FILE * flash = NULL;
	char read_str[2 * FLASH_NAME_SIZE];
	char *tmp_str;

	int buf_lng, opt_indx = -1;
	int i, index;
	int err, ret;

	if(argc < 2) {
		print_usage();
		return -1;
	}

	while ((ret = getopt_long (argc, argv, short_opt, long_opt, &opt_indx)) != -1) {
		switch (ret) {
		case 'h':
			print_help();
			return 0;
		case 'd':
			device = optarg;
			break;
		case 'b':
			board = optarg;
			break;
		case 'l':
			file_name = optarg;
			break;
		case 'f':
			flash_map = optarg;
			break;
		case 'c':
			printf ("Ooops, option \"-c, --cmd=<input file>\" does not support now!\n");
			return 0;
		case '?':
		default :
			return -1;
		};

		opt_indx = -1;
	}

	for (i = 0; i < NM6408_BRD_NUM; i++) {
		nm6408_fd[i] = -1;
		nm6408_index[i] = -1;
	}

	if (device != NULL) {
		char *str_index, *end_ptr;

		if (strncmp (device, MC12705_DEVICE, strlen(MC12705_DEVICE) - 2) &&
		    strncmp (device, NM_CARD_DEVICE, strlen(NM_CARD_DEVICE) - 2) &&
		    strncmp (device, NM_MEZZO_DEVICE, strlen(NM_MEZZO_DEVICE) - 2) &&
		    strncmp (device, NM_QUAD_DEVICE, strlen(NM_QUAD_DEVICE) - 2) &&
		    strncmp (device, NEURO_MEZZANINE_DEVICE, strlen(NEURO_MEZZANINE_DEVICE) - 2)) {
			printf ("device \"%s\" is unknown.\n", device);

			ret = 1;
			goto exit_host;
		}

		str_index = strchr (device, '_') + 1;
		errno = 0;
		index = (int)strtol (str_index, &end_ptr, 10);

		if (errno || (str_index == end_ptr) || (*end_ptr != '\0') ||
		    (index < 0) || (index >= NM6408_BRD_NUM)) {
			printf ("device \"%s\" has poisoned index.\n", device);

			ret = 1;
			goto exit_host;
		}

		if ((nm6408_fd[index] = open (device, O_RDWR)) < 0) {
			printf ("device \"%s\" opening ERROR: %d\n", device, errno);

			ret = 1;
			goto exit_host;
		}

		nm6408_index[0] = index;
		nm6408_count = 1;
	}
	else if (board != NULL) {
		char *str_index, *end_ptr;
		struct dirent *entry;
		DIR *d;

		if (!strcmp (board, "mc12705"))
			device = MC12705_DEVICE;
		else if (!strcmp(board, "nm_card"))
			device = NM_CARD_DEVICE;
		else if (!strcmp(board, "nm_mezzo"))
			device = NM_MEZZO_DEVICE;
		else if (!strcmp(board, "nm_quad"))
			device = NM_QUAD_DEVICE;
		else if (!strcmp(board, "NeuroMezzanine"))
			device = NEURO_MEZZANINE_DEVICE;
		else {
			printf ("board type \"%s\" is unknown.\n", board);

			ret = 1;
			goto exit_host;
		}

		d = opendir ("/dev/");
		if (!d) {
			printf ("directory \"%s\" openning ERROR: %d\n", "/dev/", errno);

			ret = 1;
			goto exit_host;
		}

		while (entry = readdir (d)) {
			char dev_name[256];

			end_ptr = strrchr (device, '/') + 1;
			if (strncmp (entry->d_name, end_ptr, strlen(device) - 7))
				continue;

			if (nm6408_count == NM6408_BRD_NUM)
				break;

			str_index = strchr (entry->d_name, '_') + 1;
			errno = 0;
			index = (int)strtol (str_index, &end_ptr, 10);

			if (errno || (str_index == end_ptr) || (*end_ptr != '\0') ||
			    (index < 0) || (index >= NM6408_BRD_NUM)) {
				printf ("device \"/dev/%s\" has poisoned index.\n", entry->d_name);
				closedir (d);

				ret = 1;
				goto exit_host;
			}

			strncpy (dev_name, device, strlen(device) + 1);
			end_ptr = strchr (dev_name, '_') + 1;
			strncpy (end_ptr, str_index, 2);

			if ((nm6408_fd[index] = open (dev_name, O_RDWR)) < 0) {
				printf ("device \"%s\" opening ERROR: %d\n", dev_name, errno);
				closedir (d);

				ret = 1;
				goto exit_host;
			}

			nm6408_index[nm6408_count] = index;
			nm6408_count++;
		}

		closedir(d);

		if (!nm6408_count) {
			printf ("instance of board \"%s\" is not found.\n", board);

			ret = 1;
			goto exit_host;
		}
	}
	else {
		printf("device to load is undefined.\n");

		ret = 1;
		goto exit_host;
	}

	if (file_name == NULL) {
		printf("file to load is undefined\n");

		ret = 1;
		goto exit_host;
	}

	if ((fd = open(file_name, O_RDONLY)) < 0) {
		printf ("file \"%s\" opening ERROR: %d\n", file_name, errno);

		ret = 1;
		goto exit_host;
	}

	file_lng = lseek(fd, 0, SEEK_END);

	if (file_lng <= 0) {
		printf("file \"%s\" offsetting ERROR: %d\n", file_name, errno);

		ret = 1;
		goto exit_host;
	}

	buf_lng = file_lng;

	if (buf_lng & 3)
		buf_lng = (buf_lng & ~3) + 4;

	buf_lng += 4;

	if (buf_lng > MAX_FILE_LNG) {
		printf("file \"%s\" too large, overlap - %u\n", file_name, buf_lng - MAX_FILE_LNG);

		ret = 1;
		goto exit_host;
	}

	file_buf = malloc(buf_lng);

	if (file_buf == NULL) {
		printf("malloc error.\n");

		ret = 1;
		goto exit_host;
	}

	if (lseek(fd, 0, SEEK_SET) < 0) {
		printf("file \"%s\" offsetting ERROR: %d\n", file_name, errno);

		ret = 1;
		goto exit_host;
	}

	ret = read(fd, file_buf, file_lng);
	err = errno;

	close(fd);
	fd = -1;

	if (ret < 0) {
		printf("file \"%s\" read ERROR: %d\n", file_name, err);

		ret = 1;
		goto exit_host;
	}
	else if (ret != file_lng) {
		printf("file \"%s\" error: read %u byte of %u\n", file_name, ret, file_lng);

		ret = 1;
		goto exit_host;
	}

	*(__u32 *)&file_buf[buf_lng - 4] = crc32_f(file_buf, buf_lng - 4);
	file_lng = buf_lng;

	if (flash_map != NULL) {
		if ((flash = fopen(flash_map, "r")) == NULL) {
			printf("file \"%s\" opening ERROR: %d\n", flash_map, errno);

			ret = 1;
			goto exit_host;
		}

		printf("flash map:\n");

		while (((tmp_str = fgets(read_str, sizeof(read_str), flash)) != NULL) &&
		       (map_size < FLASH_MAP_LINE)) {
			int i, j;

			for (i = 0; i < strlen(tmp_str); i++) {
				if ((tmp_str[i] == '\n') || (tmp_str[i] == '\r'))
					continue;

				if ((tmp_str[i] != ' ') && (tmp_str[i] != '\t'))
					break;
			}

			if ((i == strlen(tmp_str)) || (tmp_str[i] == '#'))
				continue;

			errno = 0;
			map_buf[map_size].flash_disp = (int)strtol(read_str, &tmp_str, 0);

			if (errno || (read_str == tmp_str)) {
				printf("file \"%s\", bad format\n", flash_map);

				ret = 1;
				goto exit_host;
			}

			for (i = 0,  j = 0; (i < strlen(tmp_str)) && (j < (FLASH_NAME_SIZE - 1)); i++) {
				if ((tmp_str[i] == '\n') || (tmp_str[i] == '\r'))
					break;

				if ((tmp_str[i] != ' ') && (tmp_str[i] != '\t')) {
					map_buf[map_size].flash_name[j] = tmp_str[i];
					j++;
				}
			}

			map_buf[map_size].flash_name[j] = '\0';

			if (!strlen(map_buf[map_size].flash_name)) {
				printf("file \"%s\", bad format\n", flash_map);

				ret = 1;
				goto exit_host;
			}

			if (map_buf[map_size].flash_disp == 0x0) {
				spl_map = &map_buf[map_size];
				spl_ind = map_size;
			}

			printf("\t0X%08X %s\n", map_buf[map_size].flash_disp, map_buf[map_size].flash_name);

			map_size++;
		}

		printf("-------------------------\n");

		if (!map_size || (!feof(flash) && (map_size != FLASH_MAP_LINE))) {
			printf("file \"%s\", read ERROR\n", flash_map);

			ret = 1;
			goto exit_host;
		}

		fclose(flash);
		flash = NULL;

		flash_buf = malloc(MAX_FLASH_SIZE);
		if (flash_buf == NULL) {
			printf("malloc error.\n");

			ret = 1;
			goto exit_host;
		}

		qsort (map_buf, map_size, sizeof(flash_t), comp_map);

		if ((spl_map != NULL) && (spl_ind != 0)) {
			spl_map = map_buf;
			spl_ind = 0;

			if (map_buf[spl_ind].flash_disp != 0x0)
				abort();
		}

		ret = flash_map_prepare(map_buf, map_size);
		if (ret) {
			ret = 1;
			goto exit_host;
		}
	}

	ret = pipe (pipe_fd);
	if (ret < 0) {
		printf ("pipe error.\n");	

		ret = 1;
		goto exit_host;
	}

	for (i = 0; i < nm6408_count; i++) {
		index = nm6408_index[i];
		board_ret[index] = -1;

		if (pthread_create (&board_thread[index], NULL, load_cli_board, (void *)((unsigned long)index))) {
			for (; i > 0; i--) {
				index = nm6408_index[i - 1];

				pthread_cancel (board_thread[index]);
				pthread_join (board_thread[index], 0);
			}

			printf ("pthread error.\n");

			ret = 1;
			goto exit_host;
		}
	}

	gettimeofday(&tv, NULL);
	start_time = (tv.tv_sec * 1000) + (tv.tv_usec / 1000);

	pthread_mutex_lock (&work_mutex);
	while (in_progress != nm6408_count) {
		pthread_mutex_unlock (&work_mutex);

		gettimeofday(&tv, NULL);
		current_time = (tv.tv_sec * 1000) + (tv.tv_usec / 1000);

		if ((current_time - start_time) > 10000) {
			for (i = 0; i < nm6408_count; i++) {
				index = nm6408_index[i];

				if (pthread_kill (board_thread[index], 0) != ESRCH) {
					pthread_cancel (board_thread[index]);
					pthread_join (board_thread[index], 0);
				}
			}

			printf ("Timeout of loader.\n");

			ret = 1;
			goto exit_host;
		}

		sleep (1);

		pthread_mutex_lock (&work_mutex);
	}
	pthread_mutex_unlock (&work_mutex);

	free(file_buf);
	file_buf = NULL;

	pthread_mutex_lock (&work_mutex);
	do_wait = 0;
	pthread_cond_broadcast (&wait_cond);
	pthread_mutex_unlock (&work_mutex);

	i = 0;

	while (1) {
		ret = read (pipe_fd[0], &_board_thread, sizeof (pthread_t));
		if (ret != sizeof (pthread_t)) {
			printf ("process error.\n");
			exit (-1);
		}

		pthread_join (_board_thread, (void **)&_board_ret);

		if (*_board_ret == 0)
			i++;

		pthread_mutex_lock (&work_mutex);
		in_progress--;

		if (!in_progress) {
			pthread_mutex_unlock (&work_mutex);
			break;
		}

		pthread_mutex_unlock (&work_mutex);
	}

	if (flash_map != NULL) {
		free(flash_buf);
		flash_buf = NULL;
	}

	if (i != nm6408_count)
		ret = -1;
	else
		ret = 0;

exit_host:

	if (file_buf != NULL)
		free(file_buf);

	if (fd >= 0)
		close(fd);

	if (flash_buf != NULL)
		free(flash_buf);

	if (flash != NULL)
		fclose(flash);

	for (i = 0; i < nm6408_count; i++) {
		index = nm6408_index[i];
		if (index < 0)
			continue;

		if (nm6408_fd[index] >= 0)
			close (nm6408_fd[index]);
	}

	return ret;
}

void *load_cli_board(void *args)
{
	int index = (unsigned long)args;
	pthread_t  self_thread;
	struct timeval tv;
	unsigned int start_time;
	unsigned int current_time;
	struct nm6408_access access;
	struct nm6408_wind wind;
	struct nm6408_magic magic, *p_magic;
	struct nm6408_desc desc, *p_desc;
	flash_t local_map[FLASH_MAP_LINE];
	unsigned char *spl_buf = NULL;
	int i, ret, err;

	for (i = 0; i < map_size; i++)
		local_map[i] = map_buf[i];

	pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, 0x00);
	pthread_setcancelstate (PTHREAD_CANCEL_ENABLE, 0x00);

	wind.cluster = NM6408_CCPU;
	wind.addr = 0x0;
	wind.size = 0x0;
	wind.cmd = NM6408_WRITE;

	if (ioctl (nm6408_fd[index], NM6408_CFG_WIND, &wind) || (wind.size < 0x80000))  {
		err = errno;

		pthread_mutex_lock (&print_mutex);
		printf ("Thread[%d] :  ioctl ERROR: %d\n", index, err);
		pthread_mutex_unlock (&print_mutex);

		goto dead_loop;
	}

	access.offset = NM6408_MAGIC_DESC;
	access.width = NM6408_WIDTH32;
	access.cmd = NM6408_READ;

	for (i = 0; i < sizeof (struct nm6408_magic) / 4; i++) {
		if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access)) {
			err = errno;

			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}

		*((__u32 *)&magic + i) = access.data.b32;
		access.offset += 4;
	}

	if (spl_map) {
		spl_buf = malloc(spl_map->flash_lng);
		if (!spl_buf) {
			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : malloc error to SPL.\n", index);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}

		for (i = 0; i < spl_map->flash_lng; i++)
			*(spl_buf + i) = *(spl_map->buffer + i);

		local_map[spl_ind].buffer = spl_buf;

		if (spl_map->flash_lng <= (0x8 + NM6408_MAGIC_DESC + sizeof (struct nm6408_magic))) {
			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : SPL does not compatible.\n", index);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}

		p_magic = (struct nm6408_magic *)(spl_buf + 0x8 + NM6408_MAGIC_DESC);
	}
	else
		p_magic = NULL;

	if (magic.magic_code == MC12705_MAGIC) {
		if (p_magic && (p_magic->magic_code != MC12705_MAGIC)) {
			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : SPL does not compatible.\n", index);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}
	}
	else if (magic.magic_code == NM_CARD_MAGIC) {
		if (p_magic && (p_magic->magic_code != NM_CARD_MAGIC)) {
			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : SPL does not compatible.\n", index);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}
	}
	else if (magic.magic_code == NM_MEZZO_MAGIC) {
		if (p_magic && (p_magic->magic_code != NM_MEZZO_MAGIC)) {
			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : SPL does not compatible.\n", index);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}
	}
	else if (magic.magic_code == NM_QUAD_MAGIC) {
		if (p_magic && (p_magic->magic_code != NM_QUAD_MAGIC)) {
			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : SPL does not compatible.\n", index);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}
	}
	else if (magic.magic_code == NEURO_MEZZANINE_MAGIC) {
		if (p_magic && (p_magic->magic_code != NEURO_MEZZANINE_MAGIC)) {
			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : SPL does not compatible.\n", index);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}
	}
	else {
		// Check legacy firmware

		access.width = NM6408_WIDTH32;
		access.cmd = NM6408_READ;

		access.offset = 0x110;

		if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access))
			goto legacy_err;

		if (0xA5A5A5A5 != access.data.b32)
			goto legacy_err;

		access.offset = NM6408_LIBLOAD_FORMAT + offsetof(struct libload_format, version[0]);

		if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access))
			goto legacy_err;

		if (1 != access.data.b32)
			goto legacy_err;

		access.offset = NM6408_LIBLOAD_FORMAT + offsetof(struct libload_format, version[1]);

		if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access))
			goto legacy_err;

		if ((0 != access.data.b32) && (1 != access.data.b32))
			goto legacy_err;

		if (spl_map) {
			access.width = NM6408_WIDTH32;
			access.cmd = NM6408_READ;

			access.offset = NM6408_LIBLOAD_FORMAT + offsetof(struct libload_format, serial[0]);
			if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access))
				goto legacy_err;
			p_magic->serial[0] = access.data.b32;

			access.offset = NM6408_LIBLOAD_FORMAT + offsetof(struct libload_format, serial[1]);
			if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access))
				goto legacy_err;
			p_magic->serial[1] = access.data.b32;

			*((__u32 *)(local_map[spl_ind].buffer + local_map[spl_ind].flash_lng - 4)) =
					crc32_f(local_map[spl_ind].buffer + 8, local_map[spl_ind].flash_lng - 12);
		}

		if (flash_buf) {
			p_desc = (struct nm6408_desc *)(file_buf + NM6408_MAGIC_DESC);

			desc.image[0] = p_desc->image[0];
			desc.image[1] = p_desc->image[1];

			access.width = NM6408_WIDTH32;
			access.cmd = NM6408_WRITE;

			access.offset = desc.image[0] + NM6408_MAGIC_DESC + offsetof(struct nm6408_desc, is_ready);
			access.data.b32 = 0x5A5A5A5A;

			if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access))
				goto legacy_err;
		}

		access.width = NM6408_WIDTH32;
		access.cmd = NM6408_WRITE;

		access.offset = NM6408_LIBLOAD_FORMAT + offsetof(struct libload_format, entry);
		access.data.b32 = 0;

		if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access))
			goto legacy_err;

		access.offset = 0;

		for (i = 0; i < file_lng; i += 4) {
			access.data.b32 = *(__u32 *)(file_buf + i);

			if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access))
				goto legacy_err;

			access.offset += 4;
		}

		access.offset = NM6408_LIBLOAD_FORMAT + offsetof(struct libload_format, command);
		access.data.b32 = 3;

		if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access))
			goto legacy_err;

		goto legacy_skip;

legacy_err:

		pthread_mutex_lock (&print_mutex);
		printf ("Thread[%d] : device does not ready.\n", index);
		pthread_mutex_unlock (&print_mutex);

		goto dead_loop;
	}

	if (p_magic) {
		p_magic->serial[0] = magic.serial[0];
		p_magic->serial[1] = magic.serial[1];

		*((__u32 *)(local_map[spl_ind].buffer + local_map[spl_ind].flash_lng - 4)) =
				crc32_f(local_map[spl_ind].buffer + 8, local_map[spl_ind].flash_lng - 12);
	
	}

	access.offset = magic.data_point;
	access.width = NM6408_WIDTH32;
	access.cmd = NM6408_READ;

	for (i = 0; i < sizeof (struct nm6408_desc) / 4; i++) {
		if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access)) {
			err = errno;

			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}

		*((__u32 *)&desc + i) = access.data.b32;
		access.offset += 4;
	}

	if (flash_buf) {
		if (file_lng <= (NM6408_MAGIC_DESC + sizeof (struct nm6408_desc))) {
			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : target loader does not compatible.\n", index);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}

		p_desc = (struct nm6408_desc *)(file_buf + NM6408_MAGIC_DESC);
	}
	else
		p_desc = NULL;

	if (desc.is_ready) {
		if (p_desc && (p_desc->crc != 0x64086408)) {
			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : target loader does not compatible.\n", index);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}
	}
	else {
		pthread_mutex_lock (&print_mutex);
		printf ("Thread[%d] : device does not ready.\n", index);
		pthread_mutex_unlock (&print_mutex);

		goto dead_loop;
	}

	desc.is_ready = 0;

	if (p_desc) {
		desc.image[0] = p_desc->image[0];
		desc.image[1] = p_desc->image[1];

		desc.entry = p_desc->entry;
	}
	else {
		desc.image[0] = magic.load_begin;
		desc.image[1] = file_lng - 4;

		desc.entry = magic.load_begin;
	}

	if ((desc.image[0] < magic.load_begin) || ((desc.image[0] + desc.image[1]) > magic.load_limit)) {
		pthread_mutex_lock (&print_mutex);
		printf ("Thread[%d] : load file does not compatible.\n", index);
		pthread_mutex_unlock (&print_mutex);

		goto dead_loop;
	}

	desc.crc = crc32_f(file_buf, desc.image[1]);

	if (desc.crc != (*(__u32 *)&file_buf[file_lng - 4])) {
		pthread_mutex_lock (&print_mutex);
		printf ("Thread[%d] : some wrong while load file.\n", index);
		pthread_mutex_unlock (&print_mutex);

		goto dead_loop;
	}

	access.offset = magic.data_point;
	access.width = NM6408_WIDTH32;
	access.cmd = NM6408_WRITE;

	for (i = 0; i < sizeof (struct nm6408_desc) / 4; i++) {
		access.data.b32 = *((__u32 *)&desc + i);

		if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access)) {
			err = errno;

			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}

		access.offset += 4;
	}
	
	access.offset = desc.image[0];
	access.width = NM6408_WIDTH32;
	access.cmd = NM6408_WRITE;

	i = 0;

	while (i < desc.image[1]) {
		access.data.b32 = *(__u32 *)(file_buf + i);

		if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access)) {
			err = errno;

			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}

		access.offset += 4;
		i += 4;		
	}

	access.offset = magic.magic_addr;
	access.width = NM6408_WIDTH32;
	access.cmd = NM6408_WRITE;

	access.data.b32 = ~0;

	if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access)) {
		err = errno;

		pthread_mutex_lock (&print_mutex);
		printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
		pthread_mutex_unlock (&print_mutex);

		goto dead_loop;
	}

legacy_skip:

	if (flash_buf == NULL)
		goto skip_wait;

	access.offset = desc.image[0] + NM6408_MAGIC_DESC + offsetof(struct nm6408_desc, is_ready);
	access.width = NM6408_WIDTH32;
	access.cmd = NM6408_READ;

	while (1) {
		if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access)) {
			err = errno;

			pthread_mutex_lock (&print_mutex);
			printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
			pthread_mutex_unlock (&print_mutex);

			goto dead_loop;
		}

		if (access.data.b32 == 0xA5A5A5A5) {
			access.cmd = NM6408_WRITE;
			access.data.b32 = 0x5A5A5A5A;

			if (ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access)) {
				err = errno;

				pthread_mutex_lock (&print_mutex);
				printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
				pthread_mutex_unlock (&print_mutex);

				goto dead_loop;
			}

			break;
		}
	}

skip_wait:

	ret = 0;

	pthread_mutex_lock (&print_mutex);
	printf ("Thread[%d] : load and run init code!\n", index);
	pthread_mutex_unlock (&print_mutex);

	pthread_mutex_lock (&work_mutex);
	in_progress++;

	while (do_wait)
		pthread_cond_wait (&wait_cond, &work_mutex);
	pthread_mutex_unlock (&work_mutex);

	if (flash_buf == NULL)
		goto exit_load;

	access.width = NM6408_WIDTH32;
	access.cmd = NM6408_READ;

	access.offset = desc.image[0] + NM6408_MAGIC_DESC + offsetof(struct nm6408_desc, flash[0]);

	ret = ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access);
	if (ret) {
		err = errno;

		pthread_mutex_lock (&print_mutex);
		printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
		pthread_mutex_unlock (&print_mutex);

		ret = 1;
		goto exit_load;
	}
	desc.flash[0] = access.data.b32;

	access.offset = desc.image[0] + NM6408_MAGIC_DESC + offsetof(struct nm6408_desc, flash[1]);

	ret = ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access);
	if (ret) {
		err = errno;

		pthread_mutex_lock (&print_mutex);
		printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
		pthread_mutex_unlock (&print_mutex);

		ret = 1;
		goto exit_load;
	}
	desc.flash[1] = access.data.b32;

	for (i = 0; i < map_size; i++) {
		unsigned char * buf;
		int leng, disp;

		pthread_mutex_lock (&print_mutex);
		printf ("Thread[%d] : 0x%08x %s\n", index, local_map[i].flash_disp, local_map[i].flash_name);
		pthread_mutex_unlock (&print_mutex);
	
		leng = local_map[i].flash_lng;
		disp = local_map[i].flash_disp;

		buf = local_map[i].buffer;
	
		while (leng) {
			int i, copy;

			if (leng > desc.flash[1])
				copy = desc.flash[1];
			else
				copy = leng;

			access.width = NM6408_WIDTH32;
			access.cmd = NM6408_WRITE;

			access.offset = desc.image[0] + NM6408_MAGIC_DESC + offsetof(struct nm6408_desc, flash[0]);
			access.data.b32 = disp;

			ret = ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access);
			if (ret) {
				err = errno;

				pthread_mutex_lock (&print_mutex);
				printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
				pthread_mutex_unlock (&print_mutex);

				ret = 1;
				goto exit_load;
			}

			access.offset = desc.image[0] + NM6408_MAGIC_DESC + offsetof(struct nm6408_desc, flash[1]);
			access.data.b32 = copy;

			ret = ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access);
			if (ret) {
				err = errno;

				pthread_mutex_lock (&print_mutex);
				printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
				pthread_mutex_unlock (&print_mutex);

				ret = 1;
				goto exit_load;
			}
		
			for (i = 0; i < copy; i += 4) {
				access.offset = desc.flash[0] + i;
				access.data.b32 = *((__u32 *)(buf + i));

				ret = ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access);
				if (ret) {
					err = errno;

					pthread_mutex_lock (&print_mutex);
					printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
					pthread_mutex_unlock (&print_mutex);

					ret = 1;
					goto exit_load;
				}

				disp += 4;
				leng -= 4;
			}

			access.offset = desc.image[0] + NM6408_MAGIC_DESC + offsetof(struct nm6408_desc, crc);
			access.data.b32 = crc32_f(buf, copy);
	
			ret = ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access);
			if (ret) {
				err = errno;

				pthread_mutex_lock (&print_mutex);
				printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
				pthread_mutex_unlock (&print_mutex);

				ret = 1;
				goto exit_load;
			}
		
			access.offset = desc.image[0] + NM6408_MAGIC_DESC + offsetof(struct nm6408_desc, command);
			access.data.b32 = 1;
		
			ret = ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access);
			if (ret) {
				err = errno;

				pthread_mutex_lock (&print_mutex);
				printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
				pthread_mutex_unlock (&print_mutex);

				ret = 1;
				goto exit_load;
			}

			access.width = NM6408_WIDTH32;
			access.cmd = NM6408_READ;
		
			access.offset = desc.image[0] + NM6408_MAGIC_DESC + offsetof(struct nm6408_desc, is_ready);

			gettimeofday (&tv, NULL);
			start_time = (tv.tv_sec * 1000) + (tv.tv_usec / 1000);

			while (1) {
				ret = ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access);
				if (ret) {
					err = errno;

					pthread_mutex_lock (&print_mutex);
					printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
					pthread_mutex_unlock (&print_mutex);

					ret = 1;
					goto exit_load;
				}

				if (access.data.b32 == 0xA5A5A5A5)
					break;

				gettimeofday (&tv, NULL);
				current_time = (tv.tv_sec * 1000) + (tv.tv_usec / 1000);

				if ((current_time - start_time) > 100000) {
					pthread_mutex_lock (&print_mutex);
					printf ("Thread[%d] : timeout of flash programming.\n", index);
					pthread_mutex_unlock (&print_mutex);

					ret = 1;
					goto exit_load;
				}

				sleep (1);
			}
		
			access.width = NM6408_WIDTH32;
			access.cmd = NM6408_WRITE;

			access.offset = desc.image[0] + NM6408_MAGIC_DESC + offsetof(struct nm6408_desc, is_ready);
			access.data.b32 = 0x5A5A5A5A;

			ret = ioctl (nm6408_fd[index], NM6408_DO_ACCESS, &access);
			if (ret) {
				err = errno;

				pthread_mutex_lock (&print_mutex);
				printf ("Thread[%d] : ioctl ERROR: %d\n", index, err);
				pthread_mutex_unlock (&print_mutex);

				ret = 1;
				goto exit_load;
			}

			buf += copy;
		}

        	pthread_mutex_lock (&print_mutex);
        	printf ("Thread[%d] : file[%d] on flash!\n", index, i);
        	pthread_mutex_unlock (&print_mutex);
	}

	ret = 0;

exit_load:

	if (spl_buf) {
		free(spl_buf);
		spl_buf = NULL;
	}

	self_thread = pthread_self ();
	board_ret[index] = ret;

	ret = write (pipe_fd[1], &self_thread, sizeof (pthread_t));
	if (ret != sizeof (pthread_t))
		exit (-1);

	sleep(1);

	pthread_exit (&board_ret[index]);

dead_loop:

	if (spl_buf) {
		free(spl_buf);
		spl_buf = NULL;
	}

	while (1) {};
}

static int comp_map(const void *p1, const void *p2)
{
	flash_t *f1 = (flash_t *)p1;
	flash_t *f2 = (flash_t *)p2;

	return (f1->flash_disp - f2->flash_disp);
}

static __u32 crc32_f(unsigned char * bgn, int length)
{
	int i,j;
	__u32 crc;
	__u32 polynom = 0x04C11DB7;
	__u32 * val;
    
	crc = 0xffffffff;

	for (i=0, val = (__u32 *)bgn; i<length; i+=4, val++) {
		crc ^= *val;

		for (j=0; j<32; j++) {
			if (crc & 0x80000000) {
				crc <<= 1;
				crc ^= polynom;
			} else {   
				crc <<= 1;
			}    
		}
	}

	return (crc^0xffffffff);
}

static int flash_map_prepare(flash_t *buf, int size)
{
	unsigned char * other_buf;
	char * file_other;
	int other_fd, other_lng; 
	int flash_lng, flash_disp;
	int rd_byte, wr_byte = 0;
	int i, ret, err;

	other_buf = flash_buf;

	for (i = 0; i < size; i++) {
		file_other = buf[i].flash_name;
		flash_disp = buf[i].flash_disp;

		if((other_fd = open (file_other, O_RDONLY)) < 0) {
			printf ("file opening ERROR %d : %s\n", errno, file_other);
			return 1;
		}

		other_lng = lseek (other_fd, 0, SEEK_END);
		if (other_lng <= 0) {
			printf ("file offsetting ERROR %d: %s\n", errno, file_other);

			close (other_fd);
			return 1;
		}

		flash_lng = other_lng;

		if (flash_lng & 3)
			flash_lng = (flash_lng & ~3) + 4;

		ret = lseek (other_fd, 0, SEEK_SET);
		if (ret < 0) {
			printf ("file offsetting ERROR %d: %s\n", errno, file_other);

			close (other_fd);
			return 1;
		}

		if ((i != 0) && (flash_disp < (buf[i - 1].flash_disp + buf[i - 1].flash_lng))) {
			printf ("files overlap: %s\n", file_other);

			close (other_fd);
			return 1;
		}

		buf[i].buffer = other_buf;

		if(flash_disp == 0) {
			flash_lng += 12; 
			
			if (flash_lng > MAX_FILE_LNG) {
				printf ("file \"%s\" too large, overlap - %u\n",
					file_other , flash_lng - MAX_FILE_LNG);

				close (other_fd);
				return 1;
			}
			
			other_buf += 8;
		}

		buf[i].flash_lng = flash_lng;
		wr_byte += buf[i].flash_lng;

		if ((wr_byte > MAX_FLASH_SIZE) ||
		((buf[i].flash_disp + buf[i].flash_lng) > MAX_FLASH_SIZE)) {
			printf ("Flash memory overflow.\n");

			close (other_fd);
			return 1;
		}

		rd_byte = read (other_fd, other_buf, other_lng);
		err = errno;

		close (other_fd);
		other_fd = -1;

		if (rd_byte < 0) {
			printf ("file read ERROR %d: %s\n", err, file_other);
			return 1;
		}
		else if (rd_byte != other_lng) {
			printf ("file \"%s\" error: read %u byte of %u\n",
				file_other, rd_byte, other_lng);
			return 1;
		}
	
		other_lng = flash_lng;

		if (flash_disp == 0) {
			other_lng -= 8;
			
			*((unsigned int *)(other_buf - 8)) = 0x12345678;
			*((unsigned int *)(other_buf - 4)) = other_lng & 0xFFFFFF;

			
			*((unsigned int *)(other_buf + other_lng - 4)) = crc32_f(other_buf, other_lng - 4);
		}

		other_buf += other_lng;
	}

	return 0;
}

