extern char __exec_begin;
extern char __exec_size;
extern char __exec_limit;

extern char __entry_start;

extern char __image_start;
extern char __image_end;

extern int do_copy(int, int, char *);

struct nm6408_desc {			// -- 0x800
	unsigned int	is_ready;	// +0x00
	unsigned int	command;	// +0x04
	unsigned int	image[2];	// +0x08
	unsigned int	flash[2];	// +0x10
	unsigned int	entry;		// +0x18
	unsigned int	crc;		// +0x1C
} __attribute__((packed, aligned(4)));

extern int SIMPLE_READ(int, long, long, char *);

static char *start_image = &__image_start;
static char *end_image = &__image_end;

static unsigned int crc32_f(char * bgn, int len);

volatile struct nm6408_desc desc __attribute__((used, section(".desc"))) = {
	.is_ready	=	0,
	.image		= {
			(unsigned int)&__exec_begin,
			(unsigned int)&__exec_size
	},
	.entry		=	(unsigned int)&__entry_start,
	.crc		=	0x64086408,
};

int main(void) asm("_main");
int main(void)
{
	desc.flash[0] = (unsigned int)start_image;
	desc.flash[1] = (unsigned int)(end_image - start_image);

	desc.command = 0;

	asm("DMB":::"memory");
	desc.is_ready = 0xA5A5A5A5;
	asm("DMB":::"memory");

	while (0x5A5A5A5A != desc.is_ready) { asm("NOP":::"memory"); };
	
	while (1) {
		int i, cmd, crc;

		if (desc.command == 0) {
			asm("NOP":::"memory");
			continue;
		}
		
		cmd = desc.command;
	
		asm("DMB":::"memory");
		desc.command = 0;
		asm("DMB":::"memory");

		switch (cmd) {
		case 1:
			if (do_copy(desc.flash[0], desc.flash[1], start_image))
				return -1;

			for (i = 0; i < (end_image - start_image); i++)
				*(start_image + i) = ~(*(start_image + i));
			
			SIMPLE_READ (0, desc.flash[0], desc.flash[1], start_image);

			crc = crc32_f(start_image, desc.flash[1]);
			if (crc != desc.crc)
				return -1;
		
			break;
		default:
			continue;
		}

		asm("DMB":::"memory");
		desc.is_ready = 0xA5A5A5A5;
		asm("DMB":::"memory");

		while (0x5A5A5A5A != desc.is_ready) { asm("NOP":::"memory"); };
	}

	return 0;
}

static unsigned int crc32_f(char * bgn, int len)
{
	int i,j;
	unsigned int crc;
	unsigned int polynom = 0x04C11DB7;
	unsigned int * val;

	crc = 0xffffffff;

	for (i=0, val = (unsigned int *)bgn; i<len; i+=4, val++) {
		crc ^= *val;

		for (j=0; j<32; j++) {
			if (crc & 0x80000000) {
				crc <<= 1;
				crc ^= polynom;
			} else {
				crc <<= 1;
			}
		}
	}

	return (crc^0xffffffff);
}
