#define SSP_Base  0x000CF000
#define SSP_CR0   0x000
#define SSP_CR1   0x004
#define SSP_DR    0x008
#define SSP_SR    0x00C
#define SSP_CPSR  0x010

.global WAIT_FLASH_WRITE_END
WAIT_FLASH_WRITE_END: /*----------------------------*/

PUSH {R1-R4, LR}
    
    LDR  R1, =SSP_Base

TEST_FLASH_WRITE_STATUS_M:

    LDR  R2, =0x100

DELAY_M0:

    SUBS R2, #1
    BGT  DELAY_M0

SSP_BUSY_M:

    LDR  R2, [R1, #SSP_SR]
    TST  R2, #0x10
    BNE  SSP_BUSY_M

SSP_READ_M:

    /* Clear FIFO */
    LDR  R2, [R1, #SSP_DR]
    LDR  R2, [R1, #SSP_SR]
    TST  R2, #0x4
    BNE  SSP_READ_M /* fifo not empty */

    MOV  R2, #0x05 /* Read Status Register (RDSR) */
    STR  R2, [R1, #SSP_DR]
    STR  R1, [R1, #SSP_DR] /* dummy byte for answer */

SSP_READ_FIFO_EMPTY_M:

    LDR  R2, [R1, #SSP_SR]
    TST  R2, #4
    BEQ  SSP_READ_FIFO_EMPTY_M

    LDR  R2, [R1, #SSP_DR] /* dummy byte due to command */

SSP_READ_FIFO_EMPTY_M2:

    LDR  R2, [R1, #SSP_SR]
    TST  R2, #4
    BEQ  SSP_READ_FIFO_EMPTY_M2

    LDR  R2, [R1, #SSP_DR] /* FLASH Status Register value */

    TST  R2, #1  
    BNE  TEST_FLASH_WRITE_STATUS_M

    LDR  R2, =0x100

DELAY_M1:

    SUBS R2, #1
    BGT  DELAY_M1

    POP {R1-R4, PC}

.global SIMPLE_ERASE
SIMPLE_ERASE: /*-----------------------------------------*/
/*
 * R0 - offset
 * R1 - number of bytes to erase
 * R2 - work register
 * R3 - SSP_Base
 * R4 - work register
 *
 */
    PUSH {R1-R4, LR}

    MOV R0, R1 /* shift due to R0 == flash structure */
    MOV R1, R2

    CMP R1, #0
    BEQ EXIT_M

    BL  SIMPLE_INIT
    LDR  R3, =SSP_Base

    BL   WAIT_FLASH_WRITE_END

    MOV  R4, #0x06   /* write enable command */
    STR  R4, [R3, #SSP_DR]

    BL   WAIT_FLASH_WRITE_END

    MOV  R4, #0x01 /* write status register (WRSR )*/
    MOV  R2, #0    /* unprotect all blocks */
    STR  R4, [R3, #SSP_DR]
    STR  R2, [R3, #SSP_DR]

    BL WAIT_FLASH_WRITE_END

    MOV  R4, #0x06          /* write enable command */
    STR  R4, [R3, #SSP_DR]

    BL  WAIT_FLASH_WRITE_END

NEXT_SECTOR:

    MOV  R4, #0x06          /* write enable command */
    STR  R4, [R3, #SSP_DR]

    BL  WAIT_FLASH_WRITE_END

    MOV  R4, #0xD8          /* block erase command */
    STR  R4, [R3, #SSP_DR]
    LSR  R4, R0, #16
    STR  R4, [R3, #SSP_DR]  /*  bits [23:16] */
    LSR  R4, R0, #8
    STR  R4, [R3, #SSP_DR]  /*  bits [15:8] */
    STR  R0, [R3, #SSP_DR]  /*  bits [7:0] */

    BL  WAIT_FLASH_WRITE_END

    LDR  R2, =0x10000 /* 64K - block size */
    ADD  R0, R2
    SUBS R1, R2
    BGE  NEXT_SECTOR

EXIT_M:
    MOV R0, #0
    POP {R1-R4, PC}

