#define SSP_Base  0x000CF000
#define SSP_CR0   0x000
#define SSP_CR1   0x004
#define SSP_DR    0x008
#define SSP_SR    0x00C
#define SSP_CPSR  0x010

.extern WAIT_FLASH_WRITE_END

.global SIMPLE_READ
SIMPLE_READ: /*-----------------------------------------*/
 /*
 * R0 - offset
 * R1 - number of bytes to read
 * R2 - buf_ptr 
 * R3 - SSP_Base
 * R4 - work register
 * R5 - number of bytes to write   (R1)
 * R6 - number of bytes to receive (R1 + 5 due to FAST_READ command & dummy byte)
 * R7 - SSP Status Register (SR)
 *
 */

    PUSH {R1-R7, LR}

    MOV R0, R1 /* shift due to R0 == flash structure */
    MOV R1, R2
    MOV R2, R3
    
    CMP R1, #0
    BEQ EXIT_M

    BL  SIMPLE_INIT

    LDR  R3, =SSP_Base
    MOV  R5, R1
    // ADD R6, R1, #5
    ADD  R6, R1, #4

    BL  WAIT_FLASH_WRITE_END

    // MOV  R4, #0x0B          /*  fast read command */
    MOV  R4, #0x03          /*  read command */
    STR  R4, [R3, #SSP_DR]
    LSR  R4, R0, #16
    STR  R4, [R3, #SSP_DR]  /*  bits [23:16] */
    LSR  R4, R0, #8
    STR  R4, [R3, #SSP_DR]  /*  bits [15:8] */
    STR  R0, [R3, #SSP_DR]  /*  bits [7:0] */
    //STR  R0, [R3, #SSP_DR]  /*  dummy */

TRANSMIT_M:

    LDR  R7, [R3, #SSP_SR]
    CMP  R5, #0
    BEQ  RECEIVE_M
    TST  R7, #0x2
    BEQ  RECEIVE_M         /*  Transmit FIFO is full */

    STR  R3, [R3, #SSP_DR] /* any value OK */
    SUB  R5, R5, #1

RECEIVE_M:
    
    TST  R7, #0x4
    BEQ  TRANSMIT_M        /*  Receive FIFO is empty */

    LDR   R4, [R3, #SSP_DR]
    CMP   R6, R1
    BGT   SKEEP_M
    STRB  R4, [R2], #1
SKEEP_M:    
    SUBS  R6, R6, #1
    BGT   TRANSMIT_M

EXIT_M:
    MOV R0, #0
    POP {R1-R7, PC}

