//-----------------------------------------------------------------------//
//                                                                                                   //
//       STUDIO       //
//          nm6408                         //
//                          After PL_ResetBoard                                      //
//----------------------------------------------------------------------//
#include <iostream>
#include <stdlib.h>

#include <common_board_tc.h>
#include <io_host.h>

#define	Pass		0
#define	Fail		1
#define	UnTested	2

#define one_sec	1000

int main(int argc, char* argv[]) 
{
    int res;
    #define test_cnt	100000
    unsigned int count = test_cnt;
	
    int test_res = Pass;
    PL_Board *board = (PL_Board *)0;
    PL_Access *access=NULL;
	FILE *F=NULL; 
    const char *trg_prog =  "part";
	char trg_name[16];
	char prog [40];
    struct PL_CoreNo No = {-1,-1};
	PL_Access *servAccess=NULL;
	IO_Service *nms=NULL;
	PL_Word loop=0;
	int err=0;
	int wait_time = one_sec; 
//	DWORD  funct,funct_error;

	printf("Test  case - Working of destructor after PL_ResetBoard\n");

	if(argc <2) {
		printf("Unknown test or target argc=%d\n",argc);
		return -1;
	}
	 // define core
	No.nm_id = strtoul(argv[1], NULL, 0);  
	if (argc == 4){
		No.cluster_id = strtoul(argv[2], NULL, 0);
	}else{
		No.cluster_id = 2;
	}
	//define nmc or arm and target program
	if(No.nm_id < 0) { // ARM prog
		sprintf(trg_name,"%s","arm");
		sprintf_s(prog,"../bin_trg/arm_%s.elf", trg_prog);
	}else{ // NMC prog
		sprintf(trg_name,"%s","nmc");
		sprintf_s(prog,"../bin_trg/nmc_%s_%d.abs", trg_prog,No.nm_id);
	}
	printf("Target {%d,%d}  prog %s\n",No.nm_id,No.cluster_id,prog);

	PL_SetTimeout(wait_time);
	PL_GetBoardCount(&count);
	if((count == 0) || (count == test_cnt)) {
		printf("count = %d\n",count);
		test_res = UnTested;
	} else if ((res = PL_GetBoardDesc(0, &board)) != PL_OK) {
		printf("PL_GetBoardDesc returns %s\n",sp_error_name[res]);
		test_res = Fail;
	}else if ((res =  PL_ResetBoard(board)) != PL_OK) {
		printf("PL_ResetBoard returns %s\n",sp_error_name[res]);
		test_res = Fail;
	}else if ((res = PL_LoadInitCode(board)) != PL_OK) {
		printf("PL_LoadInitCode returns %s\n",sp_error_name[res]);
		test_res = Fail;
	}else if((res = PL_GetAccess(board, &(No), &(access))) != PL_OK) {
			printf("PL_GetAccess returns %s\n",sp_error_name[res]);
			test_res = Fail;
	} else if((res = PL_LoadProgramFile(access, prog)) != PL_OK) {
			printf("PL_LoadProgramFile returns %s prog=%s\n",
				sp_error_name[res],prog);
			test_res = Fail;
	} else if(PL_GetAccess(board, &(No), &(servAccess)) != PL_OK){
			printf("servAccess returns %s\n",sp_error_name[res]);
			test_res = Fail;
	} 

	if(test_res !=Pass){
		goto finish;
	}
//	sleep(1); //1sec
	
	try	{
		if( (nms = IO_ServiceStart(servAccess,F,NULL,NULL)) == NULL ){
			printf("IO_ServiceStart   Fail\n");
			test_res = Fail;
			goto finish;
		}
	}
	catch (...) {
			printf("ERROR: IO_Service \n");
			test_res = Fail;
			goto finish;
	} 

	if(test_res !=Pass){
		goto finish;
	}
	printf("IO_ServiceStart  PASS\n");
	

	PL_ResetBoard(board);
	PL_LoadInitCode(board);
	printf("PL_ResetBoard  PASS\n");

	if((res = PL_Sync(access,1,&loop)) != PL_TIMEOUT) {
			printf("PL_Sync returns %s  FAIL \n",sp_error_name[res]);
			test_res = Fail;
	}else{
		printf("PL_Sync returns PASS\n");
	}
	
	sleep(1); //1sec
	
finish:
	if (nms != NULL) IO_ServiceStop(&nms,NULL); 
	if (nms != NULL) {
		printf("IO_ServiceStop  FAIL. nms!= NULL\n");
		test_res = Fail;

		IO_ServiceStop(&nms, &err); 
		if (nms != NULL) {
			printf("IO_ServiceStop  FAIL. err=%d\n",err);
		}else{
			printf("Second IO_ServiceStop  PASS\n");
		}
	}
	if (nms == NULL) {
		printf("IO_ServiceStop  PASS\n");
		test_res = Pass;
	}
	for(int i=0;i<3;i++){
		if (servAccess != NULL) {
			if((res=PL_CloseAccess(servAccess))!=PL_OK){
				printf("PL_CloseAccess(servAccess)  FAIL. res=%d\n",res);
				test_res = Fail;
			}else{
				servAccess = NULL;
				printf("PL_CloseAccess(servAccess) PASS\n");
				test_res = Pass;
				break;
			}
		}
		usleep(100*1000);
	}
	if (access != NULL) PL_CloseAccess(access);
	if (F != NULL) fclose(F); 
 	
	PL_ResetBoard(board);
	PL_LoadInitCode(board);
    if((res=PL_CloseBoardDesc(board))!=PL_OK){
		printf("PL_CloseBoardDesc  FAIL. res=%d\n",res);
	}
	if(test_res !=Pass) printf("STDIO ATTR TEST FAIL\n");
	else printf("STDIO ATTR TEST PASS\n");
    return(test_res);
}


