/*
      STUDIO         nm6408     
	           -  printf  arm nmc
*/
#include "nm6408wrap.h"
#include "io_host.h"

#include "common_nm6408_tc.h"

int main(int argc, char* argv[]) {

	const char *ASSERTION="Late binding of stdio for nm6408 series boards is working.";	
	const char *TC="Stress test of the printf command for arm nmc";
	int res, i;
	PL_Word result;
	#define test_cnt	100000
	unsigned int count = test_cnt;
	#define cnt_pr	21

	int test_res = Pass;
	PL_Board *board = (PL_Board *)0;
	PL_Access *access[cnt_pr];
	FILE *F[cnt_pr];
	const char *name_prog[2] = {
				"nmc_printf", //.abs",
				"arm_printf.elf"};
	char *prog[cnt_pr];
	char *prg;
	char cope_prog[32];
	struct PL_CoreNo No[cnt_pr];
	PL_Access *servAccess[cnt_pr];
	IO_Service *nms[cnt_pr];
	void* handle=NULL;
	PL_Word syncValue=0, Value_trg=0, cnt_loop=0, loop=0;
	int wait_time = 6000; // 6 sec
	char *f_name[cnt_pr];
	int nm_id, arm_id, trg=0;

	if (argc <= 2){
		printf("ERROR argc=%d  expected >2\n",argc);
		return -1;
	}
	tc_id(TC, ASSERTION);
	if((handle=nm6408_wrap_open(argv[1])) == NULL){
		printf("nm6408_wrap_open return NULL\n");
		printf("FAIL \n");
		return -1;
	}	

	argc -= 1;
	argv += 1;

	syncValue = (PL_Word)strtoul(argv[argc-1], NULL, 0);
	printf("print size = 0x%x\n",syncValue);
	argc--;
	printf("Target list");
	for(i=1; argc>1; argc-=2, i+=2, trg++){ //define nm's and arm's
		nm_id = strtoul(argv[i], NULL, 0);
		arm_id = strtoul(argv[i+1], NULL, 0);
		No[i/2].nm_id = nm_id;
		No[i/2].cluster_id = arm_id;
		printf("{%d,%d} ",nm_id,arm_id);
		f_name[i/2]=(char*)malloc(50);
		memset(f_name[i/2],0,50);
		sprintf(f_name[i/2],"out_%d_%d_prog",nm_id,arm_id);
	}
	printf("\n Out %d Files\n",trg);
	for(i=0;i<trg; i++){
		printf(" %s \n",f_name[i]);
	}
	NM6408_SetTimeout(wait_time);
	NM6408_GetBoardCount(&count);
	if((count == 0) || (count == test_cnt)) {
		printf("count = %d\n",count);
		test_res = UnTested;
	} else if ((res = NM6408_GetBoardDesc(0, &board)) != PL_OK) {
		printf("PL_GetBoardDesc returns %s\n",sp_error_name[res]);
		test_res = Fail;
	}else if ((res = NM6408_ResetBoard(board)) != PL_OK) {
		printf("PL_ResetBoard returns %s\n",sp_error_name[res]);
		test_res = Fail;
	}else if ((res = NM6408_LoadInitCode(board)) != PL_OK) {
		printf("PL_LoadInitCode returns %s\n",sp_error_name[res]);
		test_res = Fail;
	}
	if(test_res !=Pass){
		printf("ERROR:BoardDesc\n");
		if (board) 	NM6408_CloseBoardDesc(board);
		return(test_res);
	}

	for (i=0; i<trg; i++) {
		nms[i]=NULL;
		access[i]=NULL;
		servAccess[i]=NULL;
		F[i] = NULL;
	}
	for(i=0; i<trg; i++){
		prg = (char*)malloc(32);
		//memset(prg,0,32);
		if((No[i]).nm_id < 0) { // ARM prog
			sprintf(cope_prog,"%s", name_prog[1]);
			strcpy(prg,cope_prog);
			prog[i]=prg;
			printf("ARM LoadProgram i=%d %s\n",i,prog[i]);
		}else{
			sprintf(cope_prog,"%s_%d.abs", name_prog[0],(No[i]).nm_id);
			strcpy(prg,cope_prog);
			prog[i]=prg;
			printf("NMC LoadProgram i=%d %s\n",i,prog[i]);
		}
		if((res = NM6408_GetAccess(board, &(No[i]), &(access[i]))) != PL_OK) {
			printf("PL_GetAccess returns %s\n",sp_error_name[res]);
			test_res = Fail;
		} else if((res = NM6408_LoadProgramFile(access[i], prog[i])) != PL_OK) {
			printf("PL_LoadProgramFile returns %s\n",sp_error_name[res]);
			test_res = Fail;
		} else if((res = NM6408_GetAccess(board, &(No[i]), &(servAccess[i]))) != PL_OK) {
			printf("servAccess returns %s\n",sp_error_name[res]);
			test_res = Fail;
		} 
		if (test_res !=Pass){
			printf("ERROR:i=%d GetAccess\n",i);
			break;
		}
	}

	sleep(1); //1sec
	for(i=0; i<trg; i++){
		if (test_res !=Pass){
			printf("ERROR: test i=%d \n",i);
			break;
		}

		try	{
			if( (F[i] = fopen( f_name[i], "w+" )) == NULL ){
				printf("ERROR fopen \n");
				throw "fopen";
			}

			nms[i] = IO_ServiceStart(
					servAccess[i],
					F[i],
					handle,
					NULL );
			if (nms[i] == NULL){
				printf("ERROR IO_Service \n");
				throw "IO_Service";
			}
		}
		catch (...) {
			printf("ERROR: IO_Service i=%d\n",i);
			test_res = Fail;
		} // Try to work without service	
	}

	for(i=0; i<trg; i++){ // start target test
		if (test_res !=Pass)	break;
		if((res = NM6408_Sync(access[i],syncValue,&loop)) != PL_OK) {
			printf("PL_Sync returns %s\n",sp_error_name[res]);
			test_res = Fail;
		}
	}
	printf("LOOP=0x%x\n",loop);
	for(cnt_loop=0; cnt_loop<=loop; cnt_loop++){
		if(test_res !=Pass)	break;
		for(i=0; i<trg; i++){
			if(test_res !=Pass)	break;
			Value_trg=0;
			if((res = NM6408_Sync(access[i],0,&Value_trg)) != PL_OK) {
				printf("PL_Sync returns %s\n",sp_error_name[res]);
				test_res = Fail;
			}
		}
	}
	for(i=0; i<trg; i++){
		if(test_res !=Pass)	break;
		if((res = WaitEndProgram(access[i], one_sec*3)) != PL_OK) {
			printf("nm_prog doesn't finish\n");
			test_res = Fail;
		}else if((res = NM6408_GetResult(access[i], &result)) != PL_OK) {
			printf("PL_GetResult returns %s\n",sp_error_name[res]);
			test_res = Fail;
		}else if(result != 0) {
			printf( "PL_GetResult result %u\n",result);
			test_res = Fail;
		}
	}

	for(i=0; i<trg; i++){
		if (nms[i]) { IO_ServiceStop(&nms[i],NULL); }
		if (servAccess[i]) NM6408_CloseAccess(servAccess[i]);
		if (access[i]) NM6408_CloseAccess(access[i]);
		if (F[i]) fclose(F[i]);
	}

	NM6408_ResetBoard(board);
	NM6408_LoadInitCode(board);
	NM6408_CloseBoardDesc(board);

	nm6408_wrap_close();
	if (test_res !=Pass) printf("FAIL \n"); else printf("PASS \n");
	return(test_res);
}


