/*
   Test NMC ARM stdio
	fopen, fwrite, fread 
		fseek, ftell, stat
*/

#include <stddef.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#if defined __arm__
#include "nm6408load_arm.h"
#define	CORE_TYPE	"ARM"
#define	core_hostSync		acl_hostSync
#elif defined __NM__
#include "nm6408load_nmc.h"
#define	CORE_TYPE	"NMC"
#define	core_hostSync		ncl_hostSync
#else
#error "Unknown type of arch" 
#endif
int main ()
{
	int FAIL=0;
	int res=345;
	long int cur;
	FILE *fd = NULL;
	char mode[8];
	char arr_in[32], arr_out[32];
	size_t sz=0;
	size_t stat_size, write_size;
	struct stat *buf;
	char f_name[32];
	int binary=0;
	
	strcpy(mode,"w+");
#if defined __arm__
	sprintf(f_name,"F_%d",acl_getClusterID());
#else //__nmc__
	sprintf(f_name,"F_%d_%d",ncl_getCoreID(),ncl_getClusterID());
#endif
	
	printf("*** %s **** Test start\n",CORE_TYPE);
	
//	core_hostSync(0x10);
	
	printf("%s STDIO fopen START file=%s mode=%s\n",
			CORE_TYPE,f_name, mode);
	if((fd=fopen(f_name,mode)) == NULL){
		printf("%s STDIO fopen ERROR  mode=%s errno=%d FAIL \n",
					CORE_TYPE,mode,errno);
		FAIL=1;
		goto OUT;
	}else{
		printf("%s STDIO fopen fileno=%d PASS \n",CORE_TYPE,fileno(fd));
	}

	printf("%s STDIO FWRITE START\n",CORE_TYPE);
	strcpy(arr_out,"1234567890123456");
	write_size=9999999;
	if((write_size=fwrite(arr_out,1,9,fd)) != 9){
		printf("%s STDIO FWRITE ERROR write_size=%d errno=%d FAIL\n",CORE_TYPE,
				write_size,errno);
		FAIL=1;
		goto OUT;
	}else{
		printf("%s STDIO FWRITE PASS write_size=%d errno=%d\n",CORE_TYPE,write_size,errno);
	}
	
	printf("%s STDIO FFLUSH START  stream fd\n",CORE_TYPE);
	if((res=fflush(fd)) <0){
		printf("%s STDIO FFLUSH ERROR errno=%d FAIL\n",CORE_TYPE,errno);
		FAIL=1;
		goto OUT;
	}else{
		printf("%s STDIO FFLUSH PASS \n",CORE_TYPE);
	}
	
	buf = (struct stat *)malloc(sizeof(struct stat));
	memset((char *)buf, 0xF,sizeof(struct stat));
	cur=555;
	if((cur=ftell(fd)) != write_size){
		printf("%s STDIO FTELL ERROR pos=%ld expect %d FAIL\n",CORE_TYPE,cur,write_size);
		FAIL=1;
		goto OUT;
	}
	printf("%s STDIO FTELL = %ld PASS\n",CORE_TYPE,cur);
	
	printf("%s STDIO STAT(f_name) START\n",CORE_TYPE);
	if((res = stat(f_name, buf))<0){
		printf("%s STDIO STAT ERROR  errno=%d FAIL\n",CORE_TYPE,errno);
		FAIL=1;
		goto OUT;
	}
	sz=buf->st_size;
#if defined __arm__
	stat_size = sz;
#else //__nmc__
	stat_size = (binary) ? ((sz + 3) / 4)  :  sz;
#endif
	if(stat_size != write_size){
		printf("%s STDIO STAT st_size=%d expect %d FAIL\n",CORE_TYPE, 
				stat_size,write_size);
		FAIL=1;
		goto OUT;
	}	
	printf("%s STDIO STAT PASS st_size=%d = %d PASS\n",CORE_TYPE, 
			stat_size,write_size);

	printf("%s STDIO FSEEK TO 5\n",CORE_TYPE);
	res=1234567;
	if((res=fseek(fd,5,SEEK_SET)) != 0){
		printf("%s STDIO fseek(fd) ERROR  res=%d errno=%d FAIL\n",CORE_TYPE,
				res,errno);
		FAIL=1;
		goto OUT;
	}
	if((cur=ftell(fd)) != 5){
		printf("%s STDIO FTELL ERROR pos=%ld expect 5 FAIL\n",CORE_TYPE,cur);
		FAIL=1;
		goto OUT;
	}
	printf("%s STDIO FSEEK TO 5= %ld PASS \n",CORE_TYPE,cur);
	memset(arr_in,0,32);
	if((res=fread(arr_in,1,write_size,fd)) <=0){
		printf("%s STDIO FREAD ERROR res=%d errno=%d FAIL\n",CORE_TYPE,res,errno);
		FAIL=1;
		goto OUT;
	}
	if((strncmp(arr_in,(char *)(&arr_out[cur]),write_size-cur)) != 0){
		printf("%s STDIO pos of start reading not 5 FAIL\n",CORE_TYPE);
		printf("  arr_in=%s\n arr_out=%s\n",arr_in,(char *)(&arr_out[cur]));
		FAIL=1;
		goto OUT;
	}
	printf("%s STDIO pos of start reading  5 PASS\n",CORE_TYPE);

	printf("%s STDIO fclose(fd) START\n",CORE_TYPE);
	res=1234567;
	if((res=fclose(fd)) != 0){
		printf("%s STDIO fclose(fd) ERROR  res=%d errno=%d \n",CORE_TYPE,res,errno);
		FAIL=1;
	}else{
		printf("%s STDIO fclose(fd) PASS  \n",CORE_TYPE);
	}

OUT:
	if(FAIL){
		fclose(fd);
		printf("%s STDIO TEST FAIL\n",CORE_TYPE);
	}
	else printf("%s STDIO TEST PASS\n",CORE_TYPE);
    return 0;
}
