#ifndef __USR_TEST_common__
#define __USR_TEST_common__

#ifdef _WIN32
// ------------------------------
#include <tchar.h>
#include <Windows.h>

#include <process.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <io.h>
#include <locale.h>

#else	// linux
// ------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <pthread.h>
#include <unistd.h>
#include <locale.h>

#define sprintf_s	sprintf

#endif	// _WIN32
#include "buard_setting.h"
// ------------------------------
#define	Pass		0
#define	Fail		1
#define	UnTested	2

int one_sec = 1000; // mlsec

const char* sp_error_name[] = {
		"PL_OK",		//0
		"PL_ERROR",		//1
		"PL_TIMEOUT",		//2
		"PL_FILE",		//3
		"PL_BADADDRESS",	//4
};

void tc_id(const char* TC_NO, const char* TC_ASSERTION)
{
    setlocale(LC_ALL, "rus");
    printf("Start TC_%s\n", TC_NO);
    printf("ASSERTION: %s\n", TC_ASSERTION);
}

void err_msg( char* found, char* expected)
{
    printf("    ERROR MESSAGE\n"); 
    printf("FOUND\n%s\n", found);
    printf("EXPECTED\n%s\n", expected);
}

void result_msg(const char* TC_NO, int test_res)
{
    char msg [100];

    sprintf(msg,"%s", "No test result");
    if(test_res == Pass) 
	sprintf_s(msg,"%s", "PASS");

    else if(test_res == Fail)
	sprintf_s(msg,"%s", "FAIL");

    else if(test_res == UnTested)
	sprintf_s(msg,"%s", "UNTESTED");
    
    printf("\nTC_%s %s\n", TC_NO, msg);
}

void wait_mlsec(int mls)
{
#ifdef _WIN32
	Sleep(mls);
#else	// linux
    usleep(mls*1000);
#endif // _WIN32
}

#ifdef _WIN32
#define bzero(a, sz) memset(a,0,sz)

#endif	// _WIN32

//
//	wait, when status will be PROGRAM_FINISHED
//
int WaitEndProgram(PL_Access *access, int time) {
	char fnd [256];
	char exp [256];
	int res, w, wait_programm = PL_TIMEOUT;
	PL_Word status;
	for(w=0; w<3; w++) {
		wait_mlsec(time/3);
		if ((res = PL_GetStatus(access, &status)) != PL_OK) {
		    sprintf_s(fnd, "PL_GetStatus returns %s %d",
    				sp_error_name[res], res);
    		    sprintf_s(exp, "PL_GetStatus returns PL_OK %d", PL_OK);
		    err_msg(fnd, exp);
		    wait_programm = PL_ERROR;
		    break;
		}
		if (status == PROGRAM_FINISHED) { 
			wait_programm = PL_OK; //OK
			break;
		}
	}
	return(wait_programm);
}

#endif	//__USR_TEST_common__
