#ifndef __LIBLOAD_LIN_H__
#define __LIBLOAD_LIN_H__

#include <linux/types.h>

#ifdef __cplusplus
extern "C"
{
#endif

typedef __u32			DWORD;

typedef __u32			PL_Word;
typedef __u32			PL_Addr;

typedef struct PL_Board		PL_Board;
typedef struct PL_Access	PL_Access;

typedef struct PL_CoreNo	PL_CoreNo;

// status nm-programs
#define PROGRAM_UNSTARTED	0
#define PROGRAM_RUNNING		1
enum RetValue {
	PL_OK	= 0, // OK
	PL_ERROR		= 1, // Error
	PL_TIMEOUT		= 2, // Time-out of wait for operation
	PL_FILE			= 3, // Cannot find file for load
	PL_BADADDRESS		= 4, // Bad address ranges
	PL_AGAIN		= 5, // Try again
	PL_NOT_IMPLEMENTED	= -1 // Not implemented
};

struct ll_operation {
	int (*get_version)(unsigned int*, unsigned int*);
	int (*get_board_count)(unsigned int*);
	int (*set_channel_EDCL)(const unsigned char[], const unsigned char[]);
	int (*get_board_desc)(unsigned int, PL_Board**);
	int (*close_board_desc)(PL_Board*);
	int (*get_serial_number)(PL_Board*, unsigned long*);
	int (*get_firmware_version)(PL_Board*, unsigned int*, unsigned int*);
	int (*reset_board)(PL_Board*);
	int (*load_init_code)(PL_Board*);
	int (*get_access)(PL_Board*, PL_CoreNo*, PL_Access**);
	int (*type_access)(PL_Access*, DWORD*);
	int (*close_access)(PL_Access*);
	int (*set_program_args)(PL_Access*, const char*);
	int (*load_program)(PL_Access*, const void*, unsigned int);
	int (*load_program_file)(PL_Access*, const char*);
	int (*read_mem_block)(PL_Access*, PL_Word*, PL_Addr, DWORD);
	int (*write_mem_block)(PL_Access*, PL_Word*, PL_Addr, DWORD);
	int (*read_register)(PL_Access*, PL_Word*, PL_Addr);
	int (*write_register)(PL_Access*, PL_Word, PL_Addr);
	int (*sync)(PL_Access*, PL_Word, PL_Word*);
	int (*sync_array)(PL_Access*, PL_Word, PL_Addr, DWORD, PL_Word*, PL_Addr*, DWORD*);
	int (*set_timeout)(DWORD);
	int (*get_region_RPC)(PL_Access*, PL_Addr*, PL_Addr*);
	int (*get_status)(PL_Access*, PL_Word*);
	int (*get_result)(PL_Access*, PL_Word*);
}; // __attribute__((packed, aligned(8)));

extern struct ll_operation PL_FunctionExport __attribute__((weak));

#ifdef __cplusplus
};
#endif

#endif // __LIBLOAD_LIN_H__

