////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                          //
// service_win.cpp -                                                                              //
//                                                                                                           //
//      Tool for NM and ARM programs to access host devices             //
//                                                                                                            //
//      Note: For NM, the implementation takes into account the sparse  //
//            representation of characters (see charToSparse ).                  //
//            Response part to targets - io_stub.c                                        //
//     ops->write_mem_block,   ops->read_mem_block,                       //
//     ops->type_access,     get_region_RPC                                         //
/////////////////////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>

#include "libload_lin.h"
#include "io_host_lin.h"
#include "io_host.h"

#include <signal.h>
#include <setjmp.h>

static sigjmp_buf sj_env;

void io_service_handler (int signum)
{
//	printf("Start io_service_handler. Found SIGSEGV\n");
	siglongjmp (sj_env, 1);
}

#ifdef __cplusplus
extern "C"
{
#endif
	
IO_Service *IO_ServiceStart(
	PL_Access *access,	// of the core
	FILE *file,	// out print file
	void *ops,
	int * err)
{
	IO_Service *newService;
#if	STDIO_TRACE
	printf("IO_ServiceStart Start\n");
#endif

////////////////Set Handler Segmentation Fault
	struct sigaction act;
	act.sa_handler = io_service_handler;
    act.sa_flags = SA_RESETHAND;
	sigaction(SIGSEGV, &act, NULL);
	// place for long jump
	if( sigsetjmp(sj_env, 1) ){
#if	STDIO_TRACE
	printf("IO_ServiceStart Found SIGSEGV \n");
#endif
		if(err != NULL)	*err = IO_SERV_SIGSEGV;
		return NULL;
	}

///////////////////////////////////////////////////////////

	try {
		newService = new IO_Service(access, file, ops,err);
	}
	catch(...) {
#if	STDIO_TRACE
	printf("IO_ServiceStart ERROR \n");
#endif
		return NULL;
	}
#if	STDIO_TRACE
	printf("IO_ServiceStart End\n");
#endif
	return newService;
}

//////////////////////////////////////////////////////////////////////
int IO_ServiceStop(IO_Service **service, int *err)
{
	int res = 0;
#if	STDIO_TRACE
	printf("IO_ServiceStop Start\n");
#endif
////////////////Set Handler Segmentation Fault
	struct sigaction act;
	act.sa_handler = io_service_handler;
    act.sa_flags = SA_RESETHAND;
	sigaction(SIGSEGV, &act, NULL);
	// place for long jump
	if( sigsetjmp(sj_env, 1) ){
#if	STDIO_TRACE
	printf("IO_ServiceStop Found SIGSEGV \n");
#endif
		if(err != NULL)	*err = IO_SERV_SIGSEGV;
		return IO_SERV_SIGSEGV;
	}
///////////////////////////////////////////////////////////
		if ((res = IOService_finish(*service)) == 0) {
			if (*service) {
#if	STDIO_TRACE
	printf("IO_ServiceStop delete *service\n");
#endif
				delete *service;
			}
			*service = NULL;
		}else if(res >0){
			if(err != NULL)	*err = res;
#if	STDIO_TRACE
		printf("IOService_finish ERROR res=%d\n",res);
#endif
		}

#if	STDIO_TRACE
	printf("IO_ServiceStop End\n");
#endif
	return res;
}

int Set_IO_Serv_attribute(
	IO_Service *service,
	DWORD number_attribute,
	DWORD value_attribute
)
{
#if	STDIO_TRACE
	printf("Set_IO_Serv_attribute Start\n");
#endif

////////////////Set Handler Segmentation Fault
	struct sigaction act;
	act.sa_handler = io_service_handler;
    act.sa_flags = SA_RESETHAND;
	sigaction(SIGSEGV, &act, NULL);
	// place for long jump
    if( sigsetjmp(sj_env, 1) ){
//#if	STDIO_TRACE
	printf("Set_IO_Serv_attribute Found SIGSEGV \n");
//#endif
		return -1;
	}
///////////////////////////////////////////////////////////
	switch (number_attribute)
	{case Att_Forse_Destruct:
		service->destructor_force = value_attribute > 0;
		break;
	case Att_Time_Destruct:
		service->destructor_time =value_attribute;
		break;
	default:
		printf("ERROR in Set_IO_Serv_attribute arguments\n");
		return (-1);
		break;
	}
#if	STDIO_TRACE
	printf("Set_IO_Serv_attribute End\n");
#endif
	return 0;
}

int Get_IO_Serv_attribute(
	IO_Service *service,
	DWORD number_attribute,
	DWORD *value_attribute
)
{
#if	STDIO_TRACE
	printf("Get_IO_Serv_attribute Start\n");
#endif
////////////////Set Handler Segmentation Fault
	struct sigaction act;
	act.sa_handler = io_service_handler;
    act.sa_flags = SA_RESETHAND;
	sigaction(SIGSEGV, &act, NULL);
	// place for long jump
     if( sigsetjmp(sj_env, 1) ){
//#if	STDIO_TRACE
	printf("Get_IO_Serv_attribute Found SIGSEGV \n");
//#endif
		return -1;
	}
///////////////////////////////////////////////////////////
	switch (number_attribute)
	{
	case Att_Forse_Destruct:
		if (value_attribute == NULL)
			return(service->destructor_force);
		*value_attribute = service->destructor_force;
		break;
	case Att_Time_Destruct:
		if (value_attribute == NULL)
			return(service->destructor_time);
		*value_attribute=service->destructor_time;
		break;
	default:
		printf("ERROR in Get_IO_Serv_attribute arguments\n");
		return (-1);
		break;
	}
#if	STDIO_TRACE
	printf("Get_IO_Serv_attribute End\n");
#endif
	return *value_attribute;
}

int Get_IO_Serv_Error(
	IO_Service *service,
	DWORD *function,
	DWORD *function_error)
{
#if	STDIO_TRACE
	printf("Get_IO_Serv_Error Start\n");
#endif
fflush(stdout);
////////////////Set Handler Segmentation Fault
	struct sigaction act;
	act.sa_handler = io_service_handler;
    act.sa_flags = SA_RESETHAND;
	sigaction(SIGSEGV, &act, NULL);
	// place for long jump
    if( sigsetjmp(sj_env, 1) ){
//#if	STDIO_TRACE
	printf("Get_IO_Serv_Error Found SIGSEGV \n");
//#endif
		return -1;
	}
///////////////////////////////////////////////////////////
	*function = service->io_service_fail.function;
	*function_error=service->io_service_fail.err;
#if	STDIO_TRACE
	printf("Get_IO_Serv_Error End\n");
#endif
	return *function_error;
};
#ifdef __cplusplus
}; // extern "C"
#endif

