#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <libload_lin.h>
#include <nm6408wrap_inside.h>

#ifdef __cplusplus
extern "C"
{
#endif

static const char *PL_NAME[] = {
	"PL_GetVersion",
	"PL_GetBoardCount",
	"PL_SetChannelEDCL",
	"PL_GetBoardDesc",
	"PL_CloseBoardDesc",
	"PL_GetSerialNumber",
	"PL_GetFirmwareVersion",
	"PL_ResetBoard",
	"PL_LoadInitCode",
	"PL_GetAccess",
	"PL_CloseAccess",
//	"PL_TypeAccess",
//	"PL_SetProgramArgs",
	"PL_LoadProgram",
	"PL_LoadProgramFile",
	"PL_LoadProgramFileWithoutRun",
	"PL_RunProgram",
	"PL_ReadMemBlock",
	"PL_WriteMemBlock",
	"PL_ReadRegister",
	"PL_WriteRegister",
	"PL_Sync",
	"PL_SyncArray",
	"PL_SetTimeout",
//	"PL_GetRegionRPC",
	"PL_GetStatus",
	"PL_GetResult",
	NULL
};

static void	*handle = NULL;

struct libload_ops {
	int (*get_version)(unsigned int*, unsigned int*);
	int (*get_board_count)(unsigned int*);
	int (*set_channel_EDCL)(const unsigned char[], const unsigned char[]);
	int (*get_board_desc)(unsigned int, PL_Board**);
	int (*close_board_desc)(PL_Board*);
	int (*get_serial_number)(PL_Board*, unsigned long*);
	int (*get_firmware_version)(PL_Board*, unsigned int*, unsigned int*);
	int (*reset_board)(PL_Board*);
	int (*load_init_code)(PL_Board*);
	int (*get_access)(PL_Board*, PL_CoreNo*, PL_Access**);
	int (*close_access)(PL_Access*);
//	int (*type_access)(PL_Access*, DWORD*);
//	int (*set_program_args)(PL_Access*, const char*);
	int (*load_program)(PL_Access*, const void*, unsigned int);
	int (*load_program_file)(PL_Access*, const char*);
	int (*load_program_file_without_run)(PL_Access*, const char*, PL_Addr*);
	int (*run_program)(PL_Access*, PL_Addr);
	int (*read_mem_block)(PL_Access*, PL_Word*, PL_Addr, DWORD);
	int (*write_mem_block)(PL_Access*, PL_Word*, PL_Addr, DWORD);
	int (*read_register)(PL_Access*, PL_Word*, PL_Addr);
	int (*write_register)(PL_Access*, PL_Word, PL_Addr);
	int (*sync)(PL_Access*, PL_Word, PL_Word*);
	int (*sync_array)(PL_Access*, PL_Word, PL_Addr, DWORD, PL_Word*, PL_Addr*, DWORD*);
	int (*set_timeout)(DWORD);
//	int (*get_region_RPC)(PL_Access*, PL_Addr*, PL_Addr*);
	int (*get_status)(PL_Access*, PL_Word*);
	int (*get_result)(PL_Access*, PL_Word*);
}; // __attribute__((packed, aligned(8)));

static struct libload_ops nm6408_ops;

void *PL_Export;
//void *NM6408_Export;

void * nm6408_wrap_open(const char *boardName)
{
	const char	*libname;
	void		**tmp_ptr;
	char		*error;

	int		i = 0;

	handle = NULL;

	if (strcmp(boardName, "mc12705") == 0)
		libname = "libmc12705load.so";
	else if (strcmp(boardName, "nm_card") == 0)
		libname = "libnm_card_load.so";
	else if (strcmp(boardName, "nm_mezzo") == 0)
		libname = "libnm_mezzo_load.so";
	else if (strcmp(boardName, "nm_quad") == 0)
		libname = "libnm_quad_load.so";
	else if (strcmp(boardName, "neuro_mezzanine") == 0)
		libname = "libneuro_mezzanine_load.so";
	else{
		printf("6408wrap error unknown boardName: %s\n", boardName);
		return NULL;
	}
	handle = dlopen(libname, RTLD_LAZY | RTLD_LOCAL);
	if (handle == NULL) {
		printf("6408wrap error dlopen: %s\n", dlerror());
		return NULL;
	}

	tmp_ptr = (void **)&nm6408_ops;

	while (PL_NAME[i] != NULL) {
		tmp_ptr[i] = dlsym(handle, PL_NAME[i]);

		if ((error = dlerror()) != NULL) {
			dlclose(handle);
			handle = NULL;
			printf("6408wrap error dlsym: PL_NAME[%d]=%s\n", i, PL_NAME[i]);
			return NULL;
		}

		i++;
	}
	// PL_Export - for io_service inside
	PL_Export = dlsym(handle, "PL_FunctionExport");
	if ((error = dlerror()) != NULL) {
		dlclose(handle);
		handle = NULL;

		printf("PL_Export error %s\n", error);
		return NULL;
	}
	// NM6408_Export - for outside of io_service 
/*	NM6408_Export= dlsym(handle, "PL_FunctionExport");
	if ((error = dlerror()) != NULL) {
		dlclose(handle);
		handle = NULL;

		printf("NM6408_Export error %s\n", error);
		return NULL;
	}*/

	return PL_Export;
}

void nm6408_wrap_close(void)
{
	char *error;

	dlclose(handle);
	handle = NULL;

	if ((error = dlerror()) != NULL)
		printf("%s\n", error);

	return;	
}
int NM6408_GetVersion(unsigned int *version_major, unsigned int *version_minor){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.get_version(version_major, version_minor);
}

int NM6408_SetChannelEDCL(const unsigned char host_mac_addr[],const unsigned char board_mac_addr[]){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.set_channel_EDCL( host_mac_addr,  board_mac_addr);
}

int NM6408_GetBoardCount(unsigned int *count){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.get_board_count(count);
}

int NM6408_GetBoardDesc(unsigned int index, PL_Board **board){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.get_board_desc(index, board);
}

int NM6408_CloseBoardDesc(PL_Board *board){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.close_board_desc(board);
}

int NM6408_GetSerialNumber(PL_Board *board,unsigned long *serial_number){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.get_serial_number(board, serial_number);
}

int NM6408_GetFirmwareVersion(PL_Board *board,unsigned int *version_major,unsigned int *version_minor){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.get_firmware_version(board,version_major,version_minor);
}

int NM6408_ResetBoard(PL_Board *board){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.reset_board(board);
}

int NM6408_LoadInitCode(PL_Board *board){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.load_init_code(board);
}

int NM6408_GetAccess(PL_Board *board, PL_CoreNo *coreNo, PL_Access **access){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.get_access(board, coreNo, access);
}

int NM6408_CloseAccess(PL_Access *access){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.close_access(access);
}

int NM6408_LoadProgram(PL_Access *access,const void *addrProgram,unsigned int sizeProgram){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.load_program(access, addrProgram, sizeProgram);
}

int NM6408_LoadProgramFile(PL_Access *access, const char *filename){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.load_program_file(access, filename);
}

int NM6408_LoadProgramFileWithoutRun(PL_Access *access,const char *filename,PL_Addr *startAddr){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.load_program_file_without_run(access,  filename, startAddr);
}

int NM6408_RunProgram(PL_Access *access,PL_Addr startAddr){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.run_program(access, startAddr);
}

int NM6408_ReadMemBlock(PL_Access *access, PL_Word *block, PL_Addr address, DWORD len){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.read_mem_block(access, block, address, len);
}

int NM6408_WriteMemBlock(PL_Access *access, PL_Word *block, PL_Addr address, DWORD len){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.write_mem_block(access, block, address, len);
}

int NM6408_ReadRegister(PL_Access *access,PL_Word *returnValue,PL_Addr address){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.read_register(access, returnValue, address);
}

int NM6408_WriteRegister(PL_Access *access,PL_Word value,PL_Addr address){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.write_register(access, value, address);
}

int NM6408_Sync(PL_Access *access, PL_Word value, PL_Word *returnValue){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.sync(access, value, returnValue);
}

int NM6408_SyncArray(PL_Access *access, PL_Word value, PL_Addr outAddress, DWORD outLen,
		     PL_Word *returnValue, PL_Addr *inAddress, DWORD *inLen)
{
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.sync_array(access, value, outAddress, outLen, returnValue, inAddress, inLen);
}

int NM6408_SetTimeout(DWORD timeout){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.set_timeout(timeout);
}

int NM6408_GetStatus(PL_Access *access, PL_Word *status){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.get_status(access, status);
}

int NM6408_GetResult(PL_Access *access, PL_Word *result){
	if (handle == NULL)		return W_PL_ERROR;
	return nm6408_ops.get_result(access, result);
}

#ifdef __cplusplus
} ;// extern "C"
#endif

