#ifndef __HOST_DUTIES_H__
#define __HOST_DUTIES_H__

//////////////////////////////////////////////////////////////////////////
//									//
//		io_service library					//
//	Tool for NMC and ARM programs to access host devices		//
//									//
//	The internal implementation interface description.		//
//	Used for building host and target libraries			//
//									//
//	Copyright (c) 2004-2022 RC Module				//
//									//
//	Author: A. A. Biryukov						//
//	Remake: A. A. Subbotin						//
//	Review: V. V. Dementev						//
//									//
//////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
extern "C" {
#endif

enum IO_ServiceID {
//	INDEX_PUTCHAR		= 0x00000001,	// see INDEX_WRITE
//	INDEX_GETCHAR		= 0x00000002,	// see INDEX_READ
	INDEX_OPEN		= 0x00000003,	// fopen, open
	INDEX_CLOSE		= 0x00000004,	// fclose, close
	INDEX_READ		= 0x00000005,	// fread, read, fgets, fgetc, getc, getchar, gets
	INDEX_WRITE		= 0x00000006,	// fwrite, write, fputs, fputc, putc, putchar, puts
	INDEX_FLUSH		= 0x00000007,	// fflush
//	INDEX_GETPOS		= 0x00000008,	// see INDEX_LSEEK
//	INDEX_SETPOS		= 0x00000009,	// see INDEX_LSEEK
//	INDEX_RENAME		= 0x00000010,	// see INDEX_LINK and INDEX_UNLINK
//	INDEX_REMOVE		= 0x00000011,	// see INDEX_UNLINK
	INDEX_CLOCK		= 0x00000012,
	INDEX_TIME		= 0x00000013,
	INDEX_GETENV		= 0x00000014,
	INDEX_SYSTEM		= 0x00000015,
	INDEX_GMTIME		= 0x00000016,
	INDEX_FILESIZE		= 0x00000017,
	INDEX_FSTAT		= 0x00000018,	// fstat
	INDEX_SYS_INFO		= 0x00000019,
	INDEX_LSEEK		= 0x00000020,	// fseek, fsetpos, rewind, fgetpos, ftell, lseek
	INDEX_UNLINK		= 0x00000021,	// rename, remove, unlink
	INDEX_LINK		= 0x00000022,	// rename, link
	INDEX_STAT		= 0x00000023,	// stat
	INDEX_ISATTY		= 0x00000024,	// isatty

	NMSYNC_SERVICE_TRACE	= 0xff000001,
	NMSYNC_SERVICE_PRINTF
};

#define IO_ServiceVer		2

#define packetLimit		(0x100 * 2)
#define b2hBufSize		(packetLimit * 6)
#define b2hBufLimit		(b2hBufSize - packetLimit)
#define backBufferSize		(packetLimit * 2)
#define backBufferLimit		(backBufferSize - packetLimit)

#define armIntegerSize		4
#define nmcIntegerSize		1

#pragma pack (push, 8)
struct RPC_send_region {
	// changing on board only
	uint32_t	boardOutServBuf[b2hBufSize];
	// changing on host only
	uint32_t	backBuffer[backBufferSize];
}; // __attribute__((packed, aligned(8)));
#pragma pack (pop)

/*
 * содержание буферной области io_services
 * 0: сколько пакетов отправила плата
 * 1: текущий указатель в буфере на плате
 * 2: самый старший записанный адрес в буфере
 * 3: (зарезервировано)для отладки - значение от платы хосту
 * 4: сколько пакетов принял хост
 * 5: место в буфере на плате до которого считал хост
 * 6: количество слов посланных с хоста
 * 7: (зарезервировано)
 */
enum RPC_control_regionID {
	trg_boardSend		= 0,
	trg_sended		= 1,
	trg_sendMostDistant	= 2,
	trg_io_debug		= 3,
	trg_hostReceive		= 4,
	trg_sendConfirmed	= 5,
	trg_backBufferReady	= 6,
	trg_io_debug_back	= 7,
	trg_hostWantMutex	= 8,
	trg_boardWantMutex	= 9,
	trg_whoIsWaitMutex	= 10,
	trg_rpcSendAddr		= 11,
	trg_rpcSendSize		= 12,
	trg_rpc_busy		= 13,
	trg_size		= 14
};

#pragma pack (push, 8)
struct RPC_control_region {
	// changing on board only
	uint32_t	boardSend;		//  .rpc_services +0
	uint32_t	sended;			//  .rpc_services +1
	uint32_t	sendMostDistant;	//  .rpc_services +2
	uint32_t	io_debug;		//  .rpc_services +3

	// changing on host only
	uint32_t	hostReceive;		//  .rpc_services +4
	uint32_t	sendConfirmed;		//  .rpc_services +5

	// changing on host and board
	uint32_t	backBufferReady;	//  .rpc_services +6
	uint32_t	io_debug_back;		//  .rpc_services +7

	uint32_t	hostWantMutex;		//  .rpc_services +8
	uint32_t	boardWantMutex;		//  .rpc_services +9
	uint32_t	whoIsWaitMutex;		//  .rpc_services +10

	uint32_t	rpcSendAddr;		//  .rpc_services +11
	uint32_t	rpcSendSize;		//  .rpc_services +12

	uint32_t	rpc_busy;		//  .rpc_services +13
}; // __attribute__((packed, aligned(8)));
#pragma pack (pop)

#define RPC_BUSY_BIT	0x00000001

#define MUTEX_FALSE	0x00000000
#define MUTEX_TRUE	0xFFFFFFFF

#define HOST_THREAD	1
#define BOARD_THREAD	2

#ifdef __cplusplus
};
#endif

#endif // __HOST_DUTIES_H__

