#ifndef __NM6408WRAP_H__
#define __NM6408WRAP_H__

#include <linux/types.h>

#ifdef __cplusplus
extern "C"
{
#endif

typedef __u32			DWORD;
typedef __u64			QWORD;

typedef __u32			PL_Word;
typedef __u32			PL_Addr;

typedef struct PL_Board		PL_Board;
typedef struct PL_Access	PL_Access;

struct PL_CoreNo {
	int	nm_id;		// NM number (for any ARM: -1)
	int	cluster_id;	// Cluster number (for central ARM: -1)
};

typedef struct PL_CoreNo	PL_CoreNo;

enum RetValue {
	PL_OK			= 0, // OK
	PL_ERROR		= 1, // Error
	PL_TIMEOUT		= 2, // Time-out of wait for operation
	PL_FILE			= 3, // Cannot find file for load
	PL_BADADDRESS		= 4, // Bad address ranges
	PL_AGAIN		= 5, // Try again
	PL_NOT_IMPLEMENTED	= -1 // Not implemented
};

#define PROGRAM_NO		0x0
#define PROGRAM_PROGRESS	0x1
#define PROGRAM_FINISHED	0x2
#define PROGRAM_EXEPTION	0x4

#define W_PL_ERROR		 1
//    Common functions
extern int NM6408_GetVersion(unsigned int *version_major, unsigned int *version_minor);
extern int  NM6408_GetBoardCount(unsigned int *count) ;
extern int NM6408_SetChannelEDCL(const unsigned char host_mac_addr[],const unsigned char board_mac_addr[]);
extern int NM6408_GetBoardCount(unsigned int *count);

extern int NM6408_GetBoardDesc( unsigned int index, PL_Board **board);
extern int NM6408_CloseBoardDesc(PL_Board *board);

//    Board functions
extern int NM6408_GetSerialNumber(PL_Board *board,unsigned long *serial_number);
extern int NM6408_GetFirmwareVersion(PL_Board *board,unsigned int *version_major,unsigned int *version_minor);

extern int NM6408_ResetBoard(PL_Board *board);
extern int NM6408_LoadInitCode(PL_Board *board);

extern int NM6408_GetAccess(PL_Board *board, PL_CoreNo *coreNo, PL_Access **access);
extern int NM6408_CloseAccess(PL_Access *access);

//    Processor functions
extern int NM6408_LoadProgram(PL_Access *access,const void *addrProgram,unsigned int sizeProgram);
extern int NM6408_LoadProgramFile(PL_Access *access, const char *filename);
//extern int NM6408_LoadProgramFileWithoutRun(PL_Access *access,const char *filename,PL_Addr *startAddr);
//extern int NM6408_RunProgram(PL_Access *access,PL_Addr startAddr);

extern int NM6408_ReadMemBlock(PL_Access *access, PL_Word *block, PL_Addr address, DWORD len);
extern int NM6408_WriteMemBlock(PL_Access *access, PL_Word *block, PL_Addr address, DWORD len);

extern int NM6408_ReadRegister(PL_Access *access,PL_Word *returnValue,PL_Addr address);
extern int NM6408_WriteRegister(PL_Access *access,PL_Word value,PL_Addr address);

extern int NM6408_Sync(PL_Access *access, PL_Word value, PL_Word *returnValue);
extern int NM6408_SyncArray(PL_Access *access, PL_Word value, PL_Addr outAddress, DWORD outLen,
			    PL_Word *returnValue, PL_Addr *inAddress, DWORD *inLen);

extern int NM6408_SetTimeout(DWORD timeout);

extern int NM6408_GetStatus(PL_Access *access, PL_Word *status);
extern int NM6408_GetResult(PL_Access *access, PL_Word *result);


//extern void *NM6408_Export;

void * nm6408_wrap_open(const char *boardName);
void nm6408_wrap_close(void);

#ifdef __cplusplus
} // extern "C"
#endif

#endif // __NM6408WRAP_H__

