#ifndef __ARM_PRINTF_H__
#define __ARM_PRINTF_H__

#ifdef __cplusplus
extern "C" {
#endif

#ifdef printf
#undef printf
#endif

#define printf(format, ...)	arm_printf(format, ##__VA_ARGS__)

int arm_printf(const char *format, ...);

static inline unsigned int trg_irqDisable_io(void)
{
	unsigned int state;

	asm("MRS %0, CPSR\n":"=r" (state)::);
	asm("CPSID i\n":::"memory");

	return state;
}

static inline unsigned int trg_irqSetState_io(unsigned int new_state)
{
	unsigned int old_state;

	asm("MRS %0, CPSR\n":"=r" (old_state)::);
	asm("MSR CPSR, %0\n"::"r" (new_state):"memory");

	return old_state;
}

#define TRG_ATOMIC_BLOCK_IO() \
	for(unsigned int state = trg_irqDisable_io(), flag = 1; \
		flag; \
		trg_irqSetState_io(state), flag = 0)

#ifdef __cplusplus
};
#endif

#endif // __ARM_PRINTF_H__

