//////////////////////////////////////////////////
//		NMC/ARM C runtime library	//
//						//
//	io_stub.c				//
//	Implementation, resident part.		//
//						//
//	Author: A. A. Biryukov			//
//	Remake: A. A. Subbotin			//
//	Review: V. V. Dementev			//
//////////////////////////////////////////////////

#include <errno.h>

#include <sys/stat.h>
#include <sys/times.h>
#include <sys/unistd.h>
#include <sys/reent.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stddef.h>
#include <stdint.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>

#include "host_duties.h"

#if defined __arm__
#include "arm_printf.h"
#elif defined __NM__
#include "nmc_printf.h"
#else
#error "Unknown type of arch"
#endif

#define TRG_STDIO_DBG	1

#ifdef __cplusplus
extern "C" {
#endif

#undef errno
extern int errno;

char *__env[1] __attribute__ ((used)) = { 0 };
char **environ __attribute__ ((used)) = __env;

#if defined __arm__
#define bINw	(armIntegerSize)
#elif defined __NM__
#define bINw	(nmcIntegerSize)
#else
#error "Unknown type of arch" 
#endif

#define byte2word(x)	(((x) + bINw - 1) / bINw)

volatile static int delay = 0;

static struct RPC_send_region rpcBuff_ __attribute__((section(".rpc_services.bss")));

volatile static struct RPC_control_region rpcr_ __attribute__((used, section(".rpc_services"))) = {
	0,			//  .rpc_services +0
	0,			//  .rpc_services +1
	0,			//  .rpc_services +2
	0,			//  .rpc_services +3
	0,			//  .rpc_services +4
	0,			//  .rpc_services +5
	0,			//  .rpc_services +6
	0,			//  .rpc_services +7
	MUTEX_FALSE,		//  .rpc_services +8
	MUTEX_FALSE,		//  .rpc_services +9
	0,			//  .rpc_services +10
	(uint32_t)&rpcBuff_,	//  .rpc_services +11
	sizeof(rpcBuff_)/bINw,	//  .rpc_services +12
	0			//  .rpc_services +13
};

static void wstat2lstat(uint32_t *stat, struct stat *st, int ch);

static void __attribute__((optimize("O0"))) delay_long(void);
static void delay_long(void)
{
	volatile static int i;
	volatile static int a;

	for(i = 0, a = 0; i < 500; i++) {
		asm("":::"memory");
		a++;
		asm("":::"memory");
		--a;
		asm("":::"memory");
	}

	return;
}

static void __attribute__((optimize("O0"))) mutexLock(void);
static void mutexLock(void)
{
	volatile uint32_t barrier;

	rpcr_.boardWantMutex = MUTEX_TRUE;

	asm("":::"memory");
	barrier = rpcr_.boardWantMutex;
	asm("":::"memory");

	rpcr_.whoIsWaitMutex = BOARD_THREAD;

	while ((rpcr_.whoIsWaitMutex == BOARD_THREAD) && (rpcr_.hostWantMutex == MUTEX_TRUE)) {
		asm("":::"memory");
		delay++;
		asm("":::"memory");
		--delay;
		asm("":::"memory");
	}

	return;
}

static void __attribute__((optimize("O0"))) mutexUnlock(void);
static void mutexUnlock(void)
{
	volatile uint32_t barrier;

	rpcr_.boardWantMutex = MUTEX_FALSE;

	asm("":::"memory");
	barrier = rpcr_.boardWantMutex;
	asm("":::"memory");

	return;
}

static bool __attribute__((optimize("O0"))) fullBuffer(void);
static bool fullBuffer(void)
{
	bool full = false;

	if (rpcr_.sendConfirmed > rpcr_.sended)
		full = ((rpcr_.sended + packetLimit) >= rpcr_.sendConfirmed);

	return full;
}

static bool __attribute__((optimize("O0"))) needWait(void);
static bool needWait(void)
{
	bool need = false;

	if (rpcr_.sendConfirmed <= packetLimit)
		need = true;

	return need;
}

static inline void makeMsgHeader(uint32_t *ptr, int id, int size)
{
	*ptr++ = (uint32_t)size;
	*ptr++ = (uint32_t)id;
}

volatile static bool out_incomplete = false;

uint32_t *beginMessage()
{
	uint32_t *buf = rpcBuff_.boardOutServBuf;
	volatile uint32_t barrier;
	int error = 0;

	TRG_ATOMIC_BLOCK_IO() {
		if (out_incomplete)
			error = 1;
		else {
			// TODO: Mask IRQ
			out_incomplete = true;
		}
	}

	if (error)
		return NULL;

	mutexLock();
	if (rpcr_.sended >= b2hBufLimit) {
		//overflow limit. Wait host
		while (needWait()) {
			mutexUnlock();
			asm("":::"memory");
			delay++;
			asm("":::"memory");
			--delay;
			asm("":::"memory");
			mutexLock();
		}

		rpcr_.sendMostDistant = rpcr_.sended;

		asm("":::"memory");
		barrier = rpcr_.sendMostDistant;
		asm("":::"memory");

		rpcr_.sended = 0;

		asm("":::"memory");
		barrier = rpcr_.sended;
		asm("":::"memory");
	}
	else {
		while(fullBuffer()) {
			mutexUnlock();
			asm("":::"memory");
			delay++;
			asm("":::"memory");
			--delay;
			asm("":::"memory");
			mutexLock();
		}

		buf += rpcr_.sended;
	}
	mutexUnlock();

	return (buf + 2);
}

int completeMessage(int id, int size)
{
	uint32_t *buf = rpcBuff_.boardOutServBuf;
	volatile uint32_t barrier;
	int error = 0;

	if ((size < 0 ) || (size > ((packetLimit - 2) * bINw))) {
		TRG_ATOMIC_BLOCK_IO() {
			// TODO: unmask IRQ
			out_incomplete = false;
		}

		return -1;
	}

	mutexLock();
	do {
		if (rpcr_.backBufferReady > backBufferLimit) {
			error = 1;
			break;
		}

		buf += rpcr_.sended;
		makeMsgHeader(buf, id, size);
		rpcr_.sended += (byte2word(size) + 2);

		asm("":::"memory");
		barrier = rpcr_.sended;
		asm("":::"memory");

		rpcr_.boardSend++;

		asm("":::"memory");
		barrier = rpcr_.boardSend;
		asm("":::"memory");
	} while (0);
	mutexUnlock();

	TRG_ATOMIC_BLOCK_IO() {
		// TODO: unmask IRQ
		out_incomplete = false;
	}

	return error;
}

void killMessage(void)
{
	TRG_ATOMIC_BLOCK_IO() {
		// TODO: unmask IRQ
		out_incomplete = false;
	}

	return;
}

volatile static bool in_incomplete = false;

int waitResult(uint32_t **buf)
{
	uint32_t count;
	volatile uint32_t barrier;
	int error = 0;

	TRG_ATOMIC_BLOCK_IO() {
		if (in_incomplete)
			error = 1;
		else {
			// TODO: mask IRQ
			in_incomplete = true;
		}
	}

	if (error)
		return 0;

	mutexLock();
	while (rpcr_.boardSend != rpcr_.hostReceive) {
		if (rpcr_.backBufferReady > backBufferLimit) {
			error = 1;

			rpcr_.boardSend--;
			asm("":::"memory");
			barrier = rpcr_.boardSend;
			asm("":::"memory");

			break;
		}

		mutexUnlock();
		asm("":::"memory");
		delay++;
		asm("":::"memory");
		--delay;
		asm("":::"memory");
		mutexLock();
	}
	count = rpcr_.backBufferReady;
	mutexUnlock();

	if (!error) {
		int id, sz;

		sz = rpcBuff_.backBuffer[count - 1];
		id = rpcBuff_.backBuffer[count - 2];

		if (count < (sz + 2))
			return ~0;

		*buf = &rpcBuff_.backBuffer[count - (sz + 2)];
		return id;
	}
	else {
		TRG_ATOMIC_BLOCK_IO() {
			// TODO: unmask IRQ
			in_incomplete = false;
		}

		return 0;
	}
}

int clearResult(int indx)
{
	uint32_t count;
	volatile uint32_t barrier;
	int error = 0;

	mutexLock();
	count = rpcr_.backBufferReady;
	do {
		int id, sz;

		sz = rpcBuff_.backBuffer[count - 1];
		id = rpcBuff_.backBuffer[count - 2];

		if (id != indx) {
			error = 1;
			break;
		}

		rpcr_.backBufferReady = count - (sz + 2);
		asm("":::"memory");
		barrier = rpcr_.backBufferReady;
		asm("":::"memory");
	} while (0);
	mutexUnlock();

	TRG_ATOMIC_BLOCK_IO() {
		// TODO: unmask IRQ
		in_incomplete = false;
	}

	return error;
}

#if TRG_STDIO_DBG

#if defined __arm__
int arm_printf(const char* format, ...)
#elif defined __NM__
int nmc_printf(const char* format, ...)
#else
#error "Unknown type of arch"
#endif 
{
	uint32_t *buf = NULL;
	va_list argptr;
	int sz, rv;

	buf = beginMessage();
	if (buf == NULL)
		return -2;

	va_start(argptr, format);
	sz = vsnprintf((char *)buf, (packetLimit - 2) * bINw, format, argptr);
	va_end(argptr);

	if (sz < (packetLimit - 2) * bINw)
		sz = sz + 1;
	else
		*((char *)buf + sz - 1) = '\0';

	rv = completeMessage(NMSYNC_SERVICE_PRINTF, sz);
	if (rv != 0)
		return -1;

	return sz;
}

#endif // TRG_STDIO_DBG

#ifndef O_BINARY
#define O_BINARY	0x10000
#endif

#ifndef O_TEXT
#define O_TEXT		0x20000
#endif

int _open(char *file, int flags, ...) __attribute__((used, section(".stub._open")));
int _open(char *file, int flags, ...)
{
	#define FLAGS_CNT	11

	char *flags_str[FLAGS_CNT] = {
			"O_EXCL", "O_NOCTTY", "O_NONBLOCK", "O_CREAT",
			"O_TRUNC", "O_APPEND", "O_BINARY", "O_TEXT",
			"O_WRONLY", "O_RDONLY", "O_RDWR"
	};

	unsigned flags_vlu[FLAGS_CNT] = {
			O_EXCL, O_NOCTTY, O_NONBLOCK, O_CREAT,
			O_TRUNC, O_APPEND, O_BINARY, O_TEXT,
			O_WRONLY, O_RDONLY, O_RDWR
	};

	va_list		argptr;
	mode_t		mode = 0;
	uint32_t	*tmp, *buf = NULL;
	int		word_cnt, flag_cnt = 0;

	int rv, indx, isText = 0;

	if (flags & O_CREAT) {
		va_start(argptr, flags);
		mode = va_arg(argptr, mode_t);
		va_end(argptr);
	}

//#if defined __arm__
//	arm_printf("IO_STUB _open   mode=%o \n",mode);
//#endif
	buf = beginMessage();
	if (buf == NULL) {
		errno = EFAULT;
		return -1;
	}
	tmp = buf;
	word_cnt = 2;

	*tmp++ = strlen(file) + 1;
	word_cnt++;

	if (word_cnt > (packetLimit - 1)) {
		killMessage();

		errno = ENOMEM;
		return -1;
	}

	strncpy((char *)tmp, file, (packetLimit - word_cnt) * bINw);
	tmp += byte2word(strlen(file) + 1) + 1; // add (+1) for flag_cnt
	word_cnt += byte2word(strlen(file) + 1) + 1;

	if (word_cnt > (packetLimit - 1)) {
		killMessage();

		errno = ENOMEM;
		return -1;
	}

	if (!(flags & O_BINARY))
		flags |= O_TEXT;

	for (int i = 0; i < FLAGS_CNT; i++) {
		if (flags & flags_vlu[i]) {
			strncpy((char *)tmp, flags_str[i], (packetLimit - word_cnt) * bINw);
			tmp += byte2word(strlen(flags_str[i]) + 1);
			word_cnt += byte2word(strlen(flags_str[i]) + 1);

			flag_cnt++;
		}

		if (word_cnt > (packetLimit - 1)) {
			killMessage();

			errno = ENOMEM;
			return -1;
		}
	}

	if ((flags & O_ACCMODE) == O_RDONLY) {
		strncpy((char *)tmp, "O_RDONLY", (packetLimit - word_cnt) * bINw);
		tmp += byte2word(strlen("O_RDONLY") + 1);
		word_cnt += byte2word(strlen("O_RDONLY") + 1);

		flag_cnt++;
	}

	if (word_cnt > (packetLimit - 2)) {
		killMessage();

		errno = ENOMEM;
		return -1;
	}

	buf[byte2word(strlen(file) + 1) + 1] = flag_cnt;

	*tmp++ = mode;
	word_cnt++;

	rv = completeMessage(INDEX_OPEN, (word_cnt - 2) * bINw);
	if (rv != 0) {
		errno = EFAULT;
		return -1;
	}

	indx = waitResult(&buf);
	if (indx != INDEX_OPEN) {
		if (indx) clearResult(indx);

		errno = ENOSPC;
		return -1;
	}

	rv = buf[0];
	errno = (rv < 0) ? buf[1] : errno;
	isText = buf[2];

	if (clearResult(INDEX_OPEN)) {
		errno = EFAULT;
		return -1;
	}

	if (!isText && (flags & O_TEXT)) {
		errno = -1;
		return -1;
	}

	return rv;
}

int _close(int fd) __attribute__((used, section(".stub._close")));
int _close(int fd)
{
	uint32_t *buf = NULL;
	int rv, indx;

	if (fd < 3) {
		errno = EBADF;
		return -1;
	}

	buf = beginMessage();
	if (buf == NULL) {
		errno = EIO;
		return -1;
	}

	buf[0] = fd;

	rv = completeMessage(INDEX_CLOSE, 1 * bINw);
	if (rv != 0) {
		errno = EIO;
		return -1;
	}

	indx = waitResult(&buf);
	if (indx != INDEX_CLOSE) {
		if (indx) clearResult(indx);

		errno = EIO;
		return -1;
	}

	rv = buf[0];
	errno = (rv < 0) ? buf[1] : errno;

	if (clearResult(INDEX_CLOSE)) {
		errno = EIO;
		return -1;
	}

	return rv;
}

ssize_t _write(int fd, char *ptr, size_t len) __attribute__((used, section(".stub._write")));
ssize_t _write(int fd, char *ptr, size_t len)
{
	uint32_t *buf = NULL;
	char hbuf[bINw], tbuf[bINw];
	int i, head = 0, tail = 0;
	int rv, indx;
	int errno1;

	if (fd == 0) {
		errno = EBADF;
		return -1;
	}

	buf = beginMessage();
	if (buf == NULL) {
		errno = EFAULT;
		return -1;
	}

	if ((fd == 1) || (fd == 2)) {
		for (i = 0; (i < len) && (i < (packetLimit - 3) * bINw); i++)
			*((char *)buf + i) = *(ptr + i);

		*((char *)buf + i) = '\0';

		rv = completeMessage(NMSYNC_SERVICE_PRINTF, i + 1);
		if (rv != 0) {
			errno = EFAULT;
			return -1;
		}

		return i;
	}

	if (len > 4 * packetLimit * bINw)
		len = 4 * packetLimit * bINw;

#if 0

	if (((uint32_t)ptr % bINw) || ((uint32_t)(ptr + len) % bINw)) {
		killMessage();

		errno = EINVAL;
		return -1;
	}

#else

	if ((uint32_t)ptr & (bINw - 1)) {
		head = bINw - ((uint32_t)ptr & (bINw - 1));
		if (head > len)
			head = len;

		for (i = 0; i < head; i++)
			hbuf[bINw - head + i] = *(ptr + i);

		ptr += head;
		len -= head;
	}

	if (len && ((uint32_t)(ptr + len) & (bINw - 1))) {
		tail = (uint32_t)(ptr + len) & (bINw - 1);

		for (i = 0; i < tail; i++)
			tbuf[i] = *(ptr + len - tail + i);

		len -= tail;
	}

#endif

	if ((head + len + tail) == 0) {
		killMessage();

		errno = EINVAL;
		return -1;
	}

	buf[0] = (uint32_t)fd;
	buf[1] = (uint32_t)head;
	buf[2] = *(uint32_t *)hbuf;
	buf[3] = (uint32_t)tail;
	buf[4] = *(uint32_t *)tbuf;
	buf[5] = (uint32_t)len;
	buf[6] = (uint32_t)ptr;

	rv = completeMessage(INDEX_WRITE, 7 * bINw);
	if (rv != 0) {
		errno = EFAULT;
		return -1;
	}

	indx = waitResult(&buf);
	if (indx != INDEX_WRITE) {
		if (indx) clearResult(indx);

		errno = ENOSPC;
		return -1;
	}
//#if defined __arm__
//	arm_printf("IO_STUB _write buf[0]=%d buf[1]=%d\n",buf[0],buf[1]);
//#endif

	rv = buf[0];
	errno1 = buf[1];

	if (clearResult(INDEX_WRITE)) {
		errno = EIO;
		return -1;
	}

	errno = errno1;
	return rv;
}

ssize_t _read(int fd, char *ptr, size_t len) __attribute__((used, section(".stub._read")));
ssize_t _read(int fd, char *ptr, size_t len)
{
	uint32_t *buf = NULL;
	char hbuf[bINw], tbuf[bINw];
	int i, head = 0, tail = 0;
	int rv, indx;

	if ( (fd == 1) || (fd == 2)) {
		errno = EBADF;
		return -1;
	}

	buf = beginMessage();
	if (buf == NULL) {
		errno = EFAULT;
		return -1;
	}

	if (len > 4 * packetLimit * bINw)
		len = 4 * packetLimit * bINw;

#if 0

	if (((uint32_t)ptr % bINw) || ((uint32_t)(ptr + len) % bINw)) {
		killMessage();

		errno = EINVAL;
		return -1;
	}

#else

	if ((uint32_t)ptr & (bINw - 1)) {
		head = bINw - ((uint32_t)ptr & (bINw - 1));
		if (head > len)
			head = len;

		ptr += head;
		len -= head;
	}

	if (len && ((uint32_t)(ptr + len) & (bINw - 1))) {
		tail = (uint32_t)(ptr + len) & (bINw - 1);
		len -= tail;
	}

#endif

	if ((head + len + tail) == 0) {
		killMessage();

		errno = EINVAL;
		return -1;
	}

	buf[0] = (uint32_t)fd;
	buf[1] = (uint32_t)head;
	buf[2] = (uint32_t)tail;
	buf[3] = (uint32_t)len;
	buf[4] = (uint32_t)ptr;

	rv = completeMessage(INDEX_READ, 5 * bINw);
	if (rv != 0) {
		errno = EFAULT;
		return -1;
	}

	indx = waitResult(&buf);
	if (indx != INDEX_READ) {
		if (indx) clearResult(indx);

		errno = EIO;
		return -1;
	}

	rv = buf[0];
	errno = (rv < 0) ? buf[1] : errno;
	*(uint32_t *)hbuf = buf[2];
	*(uint32_t *)tbuf = buf[3];

	if (clearResult(INDEX_READ)) {
		errno = EIO;
		return -1;
	}

	if (rv > 0) {
		for (i = 0; (i < head) && (i < rv); i++)
			*(ptr - head + i) = hbuf[bINw - head + i];

		for (i = 0; (head + len + i) < rv; i++)
			*(ptr + len + i) = tbuf[i];
	}

	return rv;
}

off_t _lseek(int fd, off_t offset, int whence) __attribute__((used, section(".stub._lseek")));
off_t _lseek(int fd, off_t offset, int whence)
{
	uint32_t *buf = NULL;
	int rv, indx;

	if (fd < 3) {
		errno = EBADF;
		return -1;
	}

	buf = beginMessage();
	if (buf == NULL) {
		errno = EFAULT;
		return -1;
	}

	buf[0] = (uint32_t)fd;
	buf[1] = (uint32_t)offset;
	buf[2] = (uint32_t)whence;

	rv = completeMessage(INDEX_LSEEK, 3 * bINw);
	if (rv != 0) {
		errno = EFAULT;
		return -1;
	}

	indx = waitResult(&buf);
	if (indx != INDEX_LSEEK) {
		if (indx) clearResult(indx);

		errno = EIO;
		return -1;
	}

	rv = buf[0];
	errno = (rv < 0) ? buf[1] : errno;

	if (clearResult(INDEX_LSEEK)) {
		errno = EIO;
		return -1;
	}

	return rv;
}

int _unlink(char *name) __attribute__((used, section(".stub._unlink")));
int _unlink(char *name)
{
	uint32_t *buf = NULL;
	int rv, indx;

	buf = beginMessage();
	if (buf == NULL) {
		errno = EFAULT;
		return -1;
	}

	strncpy((char *)buf, name, (packetLimit - 2) * bINw);
	if ((strlen(name) + 1) > ((packetLimit - 2) * bINw)) {
		killMessage();

		errno = ENOMEM;
		return -1;
	}

	rv = completeMessage(INDEX_UNLINK, strlen(name) + 1);
	if (rv != 0) {
		errno = EFAULT;
		return -1;
	}

	indx = waitResult(&buf);
	if (indx != INDEX_UNLINK) {
		if (indx) clearResult(indx);

		errno = EIO;
		return -1;
	}

	rv = buf[0];
	errno = (rv < 0) ? buf[1] : errno;

	if (clearResult(INDEX_UNLINK)) {
		errno = EIO;
		return -1;
	}

	return rv;
}

int _link(char *old, char *new) __attribute__((used, section(".stub._link")));
int _link(char *old, char *new)
{
	uint32_t *tmp, *buf = NULL;
	int word_cnt, rv, indx;

	buf = beginMessage();
	if (buf == NULL) {
		errno = EFAULT;
		return -1;
	}

	tmp = buf;
	word_cnt = 2;

	*tmp++ = strlen(old) + 1;
	word_cnt++;

	if (word_cnt > (packetLimit - 1)) {
		killMessage();

		errno = ENOMEM;
		return -1;
	}

	strncpy((char *)tmp, old, (packetLimit - word_cnt) * bINw);
	tmp += byte2word(strlen(old) + 1);
	word_cnt += byte2word(strlen(old) + 1);

	if (word_cnt > (packetLimit - 2)) {
		killMessage();

		errno = ENOMEM;
		return -1;
	}

	*tmp++ = strlen(new) + 1;
	word_cnt++;

	strncpy((char *)tmp, new, (packetLimit - word_cnt) * bINw);
	tmp += byte2word(strlen(new) + 1);
	word_cnt += byte2word(strlen(new) + 1);

	if (word_cnt > (packetLimit - 1)) {
		killMessage();

		errno = ENOMEM;
		return -1;
	}

	rv = completeMessage(INDEX_LINK, (word_cnt - 2) * bINw);
	if (rv != 0) {
		errno = EFAULT;
		return -1;
	}

	indx = waitResult(&buf);
	if (indx != INDEX_LINK) {
		if (indx) clearResult(indx);

		errno = EIO;
		return -1;
	}

	rv = buf[0];
	errno = (rv < 0) ? buf[1] : errno;

	if (clearResult(INDEX_LINK)) {
		errno = EIO;
		return -1;
	}

	return rv;
}

int _fstat(int fd, struct stat *st) __attribute__((used, section(".stub._fstat")));
int _fstat(int fd, struct stat *st)
{
	uint32_t *buf = NULL;
	int ch, rv, indx;

	if (fd < 3) {
		st->st_mode = S_IFCHR;
		return 0;
	}

	buf = beginMessage();
	if (buf == NULL) {
		errno = EFAULT;
		return -1;
	}

	buf[0] = fd;

	rv = completeMessage(INDEX_FSTAT, 1 * bINw);
	if (rv != 0) {
		errno = EFAULT;
		return -1;
	}

	indx = waitResult(&buf);
	if (indx != INDEX_FSTAT) {
		if (indx) clearResult(indx);

		errno = EIO;
		return -1;
	}

	rv = buf[0];
	errno = (rv < 0) ? buf[1] : errno;
	ch = buf[2];

	if (rv == 0)
		wstat2lstat(&buf[3], st, ch);

	if (clearResult(INDEX_FSTAT)) {
		errno = EIO;
		return -1;
	}

	return rv;
}

int _stat(const char *filepath, struct stat *st) __attribute__((used, section(".stub._stat")));
int _stat(const char *filepath, struct stat *st)
{
	uint32_t *buf = NULL;
	int ch, rv, indx;

	buf = beginMessage();
	if (buf == NULL) {
		errno = EFAULT;
		return -1;
	}

	strncpy((char *)buf, filepath, (packetLimit - 2) * bINw);
	if ((strlen(filepath) + 1) > ((packetLimit - 2) * bINw)) {
		killMessage();

		errno = ENOMEM;
		return -1;
	}

	rv = completeMessage(INDEX_STAT, strlen(filepath) + 1);
	if (rv != 0) {
		errno = EFAULT;
		return -1;
	}

	indx = waitResult(&buf);
	if (indx != INDEX_STAT) {
		if (indx) clearResult(indx);

		errno = EIO;
		return -1;
	}

	rv = buf[0];
	errno = (rv < 0) ? buf[1] : errno;
	ch = buf[2];

	if (rv == 0)
		wstat2lstat(&buf[3], st, ch);

	if (clearResult(INDEX_STAT)) {
		errno = EIO;
		return -1;
	}

	return rv;
}

int _isatty(int file) __attribute__((used, section(".stub._isatty")));
int _isatty(int file)
{
	uint32_t *buf = NULL;
	int rv, indx;

	if (file < 3)
		return 1;

	buf = beginMessage();
	if (buf == NULL) {
		errno = EFAULT;
		return 0;
	}

	buf[0] = file;

	rv = completeMessage(INDEX_ISATTY, 1 * bINw);
	if (rv != 0) {
		errno = EFAULT;
		return 0;
	}

	indx = waitResult(&buf);
	if (indx != INDEX_ISATTY) {
		if (indx) clearResult(indx);

		errno = EIO;
		return 0;
	}

	rv = buf[0];
	errno = (rv == 0) ? buf[1] : errno;

	if (clearResult(INDEX_ISATTY)) {
		errno = EIO;
		return 0;
	}

	return rv;
}

static void wstat2lstat(uint32_t* stat, struct stat *st, int ch)
{
	if(ch == 'w') { // Windows struct stat
		st->st_dev = stat[0];
		st->st_mode = stat[1];
		st->st_nlink = stat[2];
		st->st_rdev = stat[3];
		st->st_size = stat[4];
		st->st_atime = stat[5];
		st->st_mtime = stat[6];
		st->st_ctime = stat[7];

		return;
	}

	if(ch == 'l') { // Linux struct stat
		st->st_dev = stat[0];
		st->st_ino = stat[1];
		st->st_mode = stat[2];
		st->st_nlink = stat[3];
		st->st_uid = stat[4];
		st->st_gid = stat[5];
//		int __pad0; //???????????????????????
		st->st_rdev = stat[6];		// тип устройства 
		st->st_size = stat[7];		// общий размер в байтах 
		st->st_blksize = stat[8];	// размер блока ввода-вывода  
		st->st_blocks = stat[9];	// количество выделенных блоков 
		st->st_atime = stat[10]; 	// время последнего доступа 
		st->st_mtime = stat[11];	// время последней модификации 
		st->st_ctime = stat[12];	// время последнего изменения 

		return;
	}

	return;
}

#ifdef __cplusplus
};
#endif

