#include "init_code.h"
#include "libload_def.h"
#include "nm6408load_arm.h"

#ifdef __cplusplus
extern "C" {
#endif

const char * const BoardName = "MC127.05";

const char *acl_getBoardName(void)
{
	return BoardName;
}

int acl_getClusterID(void)
{
	unsigned int	cid;

	cid = *((volatile unsigned int *)0x000CB000);
	cid = cid & 0x07;

	return cid;
}

void __attribute__((weak, interrupt("IRQ"))) arm_irq_handler(void) asm("_arm_irq_handler");
void arm_irq_handler(void)
{
	struct libload_top		*ll_top = (void *)LIBLOAD_DATA_ADDR;
	volatile struct libload_format	*ll_format;
	int 				cid, id;

	cid = acl_getClusterID();

	if (cid == NM6408_CCPU)
		ll_format = &ll_top->arm;
	else
		ll_format = &ll_top->cluster[cid].arm;

	id = *((volatile unsigned int*)0x000B100C);
	*((volatile unsigned int*)0x000B1004) = 0x00;
	*((volatile unsigned int*)0x000B1010) = id;

	if ((id >= 0) && (id <= 7)) {
		*((volatile unsigned int*)0x000B1004) = 0xFF;
		return;
	}

	ll_format->result = id;
	asm("DMB":::"memory");

	ll_format->status = PROGRAM_ABORTED;
	asm("DMB":::"memory");

	while (1) {};
}

void __attribute__((weak, interrupt("SWI"))) arm_svc_handler(void) asm("_arm_svc_handler");
void arm_svc_handler(void)
{
	return;
}

void __attribute__((weak, interrupt("FIQ"))) arm_fiq_handler(void) asm("_arm_fiq_handler");
void arm_fiq_handler(void)
{
	struct libload_top		*ll_top = (void *)LIBLOAD_DATA_ADDR;
	volatile struct libload_format	*ll_format;
	int				cid;
	
	cid = acl_getClusterID();

	if (cid == NM6408_CCPU)
		ll_format = &ll_top->arm;
	else
		ll_format = &ll_top->cluster[cid].arm;

	ll_format->result = 0xFFFFFFFF;
	asm("DMB":::"memory");
	ll_format->status = PROGRAM_ABORTED;

	while (1) {};
}

void __attribute__((weak, interrupt("UNDEF"))) arm_und_handler(void) asm("_arm_und_handler");
void arm_und_handler(void)
{
	struct libload_top		*ll_top = (void *)LIBLOAD_DATA_ADDR;
	volatile struct libload_format	*ll_format;
	int				cid;

	cid = acl_getClusterID();

	if (cid == NM6408_CCPU)
		ll_format = &ll_top->arm;
	else
		ll_format = &ll_top->cluster[cid].arm;

	ll_format->result = 0xFFFFFFFE;
	asm("DMB":::"memory");
	ll_format->status = PROGRAM_ABORTED;

	while (1) {};
}

void __attribute__((weak, interrupt("ABORT"))) arm_abort_handler(void);
void arm_abort_handler(void)
{
	struct libload_top		*ll_top = (void *)LIBLOAD_DATA_ADDR;
	volatile struct libload_format	*ll_format;
	int				cid;

	cid = acl_getClusterID();

	if (cid == NM6408_CCPU)
		ll_format = &ll_top->arm;
	else
		ll_format = &ll_top->cluster[cid].arm;

	ll_format->result = 0xFFFFFFFD;
	asm("DMB":::"memory");
	ll_format->status = PROGRAM_ABORTED;

	while (1) {};
}

void __attribute__((weak, alias("arm_abort_handler"))) arm_prf_handler(void) asm("_arm_prf_handler");
void __attribute__((weak, alias("arm_abort_handler"))) arm_abt_handler(void) asm("_arm_abt_handler");

#ifdef __cplusplus
};
#endif
