#define Mode_USR	0x10
#define Mode_FIQ	0x11
#define Mode_IRQ	0x12
#define Mode_SVC	0x13
#define Mode_MON	0x16
#define Mode_ABT	0x17
#define Mode_UND	0x1B
#define Mode_SYS	0x1F

#define I_Bit		0x80	/* when I bit is set, IRQ is disabled */
#define F_Bit		0x40	/* when F bit is set, FIQ is disabled */
#define A_Bit		0x100	/* when A bit is set, Data Abort Interrupts is disabled */

#define CR_V		0x2000	/* Vector relocate to 0xFFFF0000 */

.extern main
.extern exit

.extern _arm_und_handler
.extern _arm_svc_handler
.extern _arm_prf_handler
.extern _arm_abt_handler
.extern _arm_irq_handler
.extern _arm_fiq_handler

.extern __bss_start__
.extern __bss_end__

.extern __stack_end
.extern __stack_start

.global _reset

.global hardware_init_hook
.set hardware_init_hook, reset_handler

.global _arm_irq_table

.weak __executable_start
.weak _start

.weak __vectors_base
.weak __vectors_limit

.weak __stack_mon
.weak __stack_irq
.weak __stack_fiq
.weak __stack_abt
.weak __stack_und
.weak __stack_usr
.weak __stack_svc

.code 32

.section .init.stub_start,"ax",%progbits
.align 2
.weak _init
1:	.word 2f
_init:
	MOV IP, SP
	STMDB SP!, {R3, R4, R5, R6, R7, R8, R9, SL, FP, IP, LR, PC}
	SUB FP, IP, #4
	LDR LR, 1b

.section .init.stub_end,"ax",%progbits
2:
	SUB SP, FP, #40
	LDMFD SP, {R4, R5, R6, R7, R8, R9, SL, FP, SP, LR}
	MOV PC, LR
	
.section .fini.stub_start,"ax",%progbits
.align 2
.weak _fini
1:	.word 2f
_fini:
	MOV IP, SP
	STMDB SP!, {R3, R4, R5, R6, R7, R8, R9, SL, FP, IP, LR, PC}
	SUB FP, IP, #4
	LDR LR, 1b

.section .fini.stub_end,"ax",%progbits
2:
	SUB SP, FP, #40
	LDMFD SP, {R4, R5, R6, R7, R8, R9, SL, FP, SP, LR}
	MOV PC, LR

.section .text.vectors_reset,"ax",%progbits
.align 5
_reset:
	LDR PC, rst_addr
	LDR PC, und_addr
	LDR PC, svc_addr
	LDR PC, prf_addr
	LDR PC, abt_addr
nop_0:	.word 0
	LDR PC, irq_addr
	LDR PC, fiq_addr

dead_loop:
	B dead_loop

rst_addr:	.word _start
und_addr:	.word _arm_und_handler
svc_addr:	.word _arm_svc_handler
prf_addr:	.word _arm_prf_handler
abt_addr:	.word _arm_abt_handler
irq_addr:	.word _arm_irq_handler
fiq_addr:	.word _arm_fiq_handler

irq_table_start:
.rept 100
		.word 0
		.word 0
		.word 0
		.word 0
.endr

.section .text.init_start,"ax",%progbits
.align 2
_start:
reset_handler:
	STR LR, cpsr_return_addr

	MRS R0, CPSR
	ORR R0, R0, #(A_Bit | I_Bit | F_Bit)
	STR R0, cpsr_reset_state

	MOV R0, #0x000CB000
	LDR R0, [R0]
	AND R0, R0, #0x7

	CMP R0, #0x4
	BNE check_vectors

	MOVW R0, #0xB004
	MOVT R0, #0x000C
	LDR R0, [R0]

	TST R0, #0x3
	BNE check_vectors

	MRC p15, 0, R0, c12, c0, 0
	ADD R0, R0, #0x14
	LDR R0, [R0]

	CMP R0, R9
	STREQ R0, _arm_irq_table
	BEQ hw_init

check_vectors:
	MOV R9, #0
	STR R9, _arm_irq_table

	LDR R0, link_origin_start
	LDR R1, link_vectors_begin
	LDR R2, link_vectors_end
	LDR R3, link_execut_start

	CMP R1, R2
	BEQ hw_init

	CMP R1, R3
	BEQ set_vbar

	CMP R0, R3
	BEQ set_vbar

	CMP R3, #0
	BNE err_loop

set_vbar:
	MRC p15, 0, R0, c1, c0, 0
	BIC R0, #CR_V
	MCR p15, 0, R0, c1, c0, 0

	MCR p15, 0, R1, c12, c0, 0
#if 0
	ADD R3, R1, #0x20
	STR R3, [R1, #0x24]
#endif
	ADD R2, R1, #0x40
	STR R2, _arm_irq_table

hw_init:
	MOV R0, #0xF00000
	MCR p15, 0, R0, c1, c0, 2
	MRC p15, 0, R0, c1, c0, 2
	UBFX R0, R0, #20, #1

	MOV R0, #(1 << 30)
	MCR p10, 7, R0, c8, c0, 0
	MOV R0, #0
	MCR p10, 7, R0, c1, c0, 0
	LDC p11, c0, [R0], {32}

	@VMOV.F32 S0, R0

	ADDS R0, R1, R2

	MRC p15, 0, R0, c1, c0, 0
	ORR R0, R0, #0x800
	MCR p15, 0, R0, c1, c0, 0
	BL call_1
call_1:
	BL call_2
call_2:
	BL call_3
call_3:
	BIC R0, R0, #0x800
	MCR p15, 0, R0, c1, c0, 0

	LDR R0, link_origin_start
	LDR R1, =reset_handler

	CMP R0, R1
	MOVEQ R9, #1
	MOVNE R9, #0
	BNE call_main

	LDR R2, =_stack_set_call
	BLX R2

	CMP R0, #0
	BEQ check_stack

	CMP R0, #0x7F
	BNE err_loop
	B   clean_bss

check_stack:
	LDR R0, =__stack_start
	LDR R1, =__stack_end
	MOV R2, #0x10000

	ADD R1, R1, R2
	CMP R0, R1
	BNE err_loop

clean_bss:
	LDR R0, =__bss_start__
	LDR R1, =__bss_end__
	MOV R2, #0

zero_loop:
	CMP R1, R0
	BEQ call_main
	STR R2, [R0], #4
	B zero_loop

call_main:
	LDR R1, _arm_irq_table
	CMP R1, #0

	LDR R0, cpsr_reset_state
	BICNE R0, R0, #(A_Bit | I_Bit)
	MSR CPSR_fxc, R0

	CMP R9, #0
	LDREQ PC, cpsr_return_addr

	LDR R2, =__stack_end
	MOV R3, #0x00000000

	LDR R1, [R2], #4
	LDR R0, [R2], #4
	STR R3, [R2, #-8]
	STR R3, [R2, #-4]

	MOVW R3, #0xCCDD
	MOVT R3, #0xAABB

	CMP R0, R3

	MOVEQ R3, R1
	MOVNE R3, R2

	LDR R0, [R3, #-4]!
	LDR R1, [R3, #-4]!

	BL main
	BL exit

err_loop:
	NOP
	B err_loop

cpsr_reset_state:	.word 0
cpsr_return_addr:	.word 0

link_origin_start:	.word _start
link_execut_start:	.word __executable_start

link_vectors_begin:	.word __vectors_base
link_vectors_end:	.word __vectors_limit

_arm_irq_table:		.word 0

.section .text.set_stack,"ax",%progbits
.align 2
.weak _stack_set_call
_stack_set_call:
	MRS R3, CPSR

	MOV R0, #0
	LDR R1, =__stack_start

	MSR CPSR_c, #(Mode_MON | I_Bit | F_Bit)
	LDR R2, upload_stack_mon
	CMP R2, #0

	MOVEQ SP, R1
	SUBEQ R1, R1, #0x800
	MOVNE SP, R2
	ORRNE R0, R0, #0x1

	MSR CPSR_c, #(Mode_IRQ | I_Bit | F_Bit)
	LDR R2, upload_stack_irq
	CMP R2, #0

	MOVEQ SP, R1
	SUBEQ R1, R1, #0x2000
	MOVNE SP, R2
	ORRNE R0, R0, #0x2

	MSR CPSR_c, #(Mode_FIQ | I_Bit | F_Bit)
	LDR R2, upload_stack_fiq
	CMP R2, #0

	MOVEQ SP, R1
	SUBEQ R1, R1, #0x800
	MOVEQ SL, R1
	MOVNE SP, R2
	LDRNE SL, upload_stack_abt
	ORRNE R0, R0, #0x4

	MSR CPSR_c, #(Mode_ABT | I_Bit | F_Bit)
	LDR R2, upload_stack_abt
	CMP R2, #0

	MOVEQ SP, R1
	SUBEQ R1, R1, #0x800
	MOVNE SP, R2
	ORRNE R0, R0, #0x8

	MSR CPSR_c, #(Mode_UND | I_Bit | F_Bit)
	LDR R2, upload_stack_und
	CMP R2, #0

	MOVEQ SP, R1
	SUBEQ R1, R1, #0x800
	MOVNE SP, R2
	ORRNE R0, R0, #0x10

	MSR CPSR_c, #(Mode_SVC | I_Bit | F_Bit)
	LDR R2, upload_stack_svc
	CMP R2, #0

	MOVEQ SP, R1
	SUBEQ R1, R1, #0x8000
	MOVNE SP, R2
	ORRNE R0, R0, #0x20

	MSR CPSR_c, #(Mode_SYS | I_Bit | F_Bit)
	LDR R2, upload_stack_usr
	CMP R2, #0

	MOVEQ SP, R1
	MOVNE SP, R2
	ORRNE R0, R0, #0x40

	LDR SL, =__stack_end

	MSR CPSR_c, R3
	BX LR

upload_stack_mon:	.word __stack_mon
upload_stack_irq:	.word __stack_irq
upload_stack_fiq:	.word __stack_fiq
upload_stack_abt:	.word __stack_abt
upload_stack_und:	.word __stack_und
upload_stack_usr:	.word __stack_usr
upload_stack_svc:	.word __stack_svc

