#ifndef __LIBLOAD_MUTEX_H__
#define __LIBLOAD_MUTEX_H__

#include <pthread.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define LIBLOAD_CLUSTER_NUMBER		4
#define LIBLOAD_CORE_NUMBER		4
#define LIBLOAD_PROCESSOR_NUMBER	(((LIBLOAD_CORE_NUMBER + 1) * LIBLOAD_CLUSTER_NUMBER) + 1)

struct libload_mutex {
	pthread_mutex_t	b_lock;

	pthread_mutex_t	n_lock[LIBLOAD_CLUSTER_NUMBER][LIBLOAD_CORE_NUMBER];
	pthread_mutex_t	a_lock[LIBLOAD_CLUSTER_NUMBER];
	pthread_mutex_t	c_lock;

	int		rpc_pid[LIBLOAD_PROCESSOR_NUMBER];
	int		rpc_tid[LIBLOAD_PROCESSOR_NUMBER];
	int		rpc_rdy[LIBLOAD_PROCESSOR_NUMBER];

	unsigned int	reset_count;
	unsigned int	seqnc_count;

	int		is_valid;	
};

typedef struct libload_mutex	libload_mutex_t;

int libload_mutex_open(char *name, libload_mutex_t **mutex_map);
int libload_mutex_close(libload_mutex_t **mutex_map);

int libload_mutex_lock(libload_mutex_t *mutex_map, int cluster, int core);
int libload_mutex_unlock(libload_mutex_t *mutex_map, int cluster, int core);

int libload_rpc_set(libload_mutex_t *mutex_map, int cluster, int core);
int libload_rpc_clr(libload_mutex_t *mutex_map, int cluster, int core);
int libload_rpc_get(libload_mutex_t *mutex_map, int cluster, int core);

void libload_reset_count(libload_mutex_t *mutex_map, int incr, unsigned int *count);
void libload_seqnc_count(libload_mutex_t *mutex_map, int rw, unsigned int *count);

#ifdef __cplusplus
} // extern "C"
#endif

#endif // __LIBLOAD_MUTEX_H__
