#include <arpa/inet.h>
#include <sys/socket.h>
#include <linux/if_packet.h>
#include <net/ethernet.h>
#include <netdb.h>
#include <ifaddrs.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <fcntl.h>
#include <unistd.h>
#include <limits.h>
#include <errno.h>
#include <linux/if_link.h>

#include "init_code.h"
#include "libload_nm6408.h"

#ifdef __cplusplus
extern "C" {
#endif

extern unsigned char local_mac_addr[EDCL_MAC_ADDR_LEN];
extern unsigned char remote_mac_addr[EDCL_MAC_ADDR_LEN];
extern int use_edcl;

#ifdef __cplusplus
} // extern "C"
#endif

static void convert_xfer(PL_CoreNo *core, DWORD *addr, DWORD *size, int set_global);

static int Check_mem_ranges(PL_CoreNo *core, QWORD entry, QWORD addr,
			    QWORD len, int RWX, int is_self)
{
	QWORD	_addr;

	if (!len)
		return 0;

	_addr = addr + len;

	if (_addr > ((QWORD)NM6408_INBOUND_AXI_SIZE))
		return 1;

	if ((addr & 0x3) || (_addr & 0x3))
		return 2;

	if (RWX == 2) {
		if (entry & 0x3)
			return 3;

		if ((entry >= ((QWORD)NM6408_INBOUND_AXI_SIZE)) &&
		    (entry <= ((QWORD)PCIE_OUTBOUND_AXI_LIMIT)))
			return 4;

		if (entry > ((QWORD)ACCESS_ENTRY_LIMIT))
			return 5;
	}

	if (is_self) {
		if ((core->cluster_id != NM6408_CCPU) && ((RWX == 2) && (entry != 0x0)) &&
		    ((addr < ((QWORD)(INTR_VECTORS_AXI_ADDR + INTR_VECTORS_AXI_SIZE))) &&
		    (_addr > ((QWORD)INTR_VECTORS_AXI_ADDR))))
			return 6;
	}

	if (core->nm_id != -1) {
		if ((addr < ((QWORD)(NMPU_REG_AXI_ADDR + NMPU_REG_AXI_SIZE))) &&
		    (_addr > ((QWORD)NMPU_REG_AXI_ADDR)))
			return 7;
	}
	else if (core->cluster_id != NM6408_CCPU) {
		if ((addr < ((QWORD)(PC_RESERVE_AXI_ADDR + PC_RESERVE_AXI_SIZE))) &&
		    (_addr > ((QWORD)PC_RESERVE_AXI_ADDR)))
			return 8;

		if ((addr < ((QWORD)(PC_REG_AXI_ADDR + PC_REG_AXI_SIZE))) &&
		    (_addr > ((QWORD)PC_REG_AXI_ADDR)))
			return 9;
	}
	else {
		if ((addr < ((QWORD)(CCPU_REG_AXI_ADDR + CCPU_REG_AXI_SIZE))) &&
		    (_addr > ((QWORD)CCPU_REG_AXI_ADDR)))
			return 10;

		if ((RWX != 0) && ((addr < ((QWORD)(LIBLOAD_MAP_ADDR + LIBLOAD_MAP_SIZE))) &&
		    (_addr > ((QWORD)LIBLOAD_MAP_ADDR))))
			return 11;
	}

	return 0;
}

static int Check_is_Valid(void *point)
{
	int fd = open("/dev/zero", O_RDONLY);
	int is_valid;

	is_valid = read(fd, point, 1) == 1;

	close(fd);

	return is_valid;
}

int Check_board(PL_Board *board)
{
	if (!board)
		return 1;

	if (!Check_is_Valid(&board->Check_byte))
		return 2;

	if (board->Magic_code != BOARD_MAGIC_CODE)
		return 3;

	return 0;
}

int Check_access(PL_Access *access)
{
	if (!access)
		return 1;

	if (!Check_is_Valid(&access->Check_byte))
		return 2;

	if (access->Magic_code != PROCESSOR_MAGIC_CODE)
		return 3;

	return 0;
}

PL_Board::PL_Board(int indx):
	Count_PL_Access(0),
	Count_In_Progress(0)
{
	char			dev_name[PATH_MAX];
	int			ret;

	if (use_edcl) {
		struct ifaddrs *ifaddr, *ifa;

		if (getifaddrs(&ifaddr) == -1) {
			PL_DBG("ERROR %d (%s): Failed getifaddrs!",
				errno, strerror(errno));
			throw 1;
		}

		for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
			int family;	

			if (ifa->ifa_addr == NULL)
				continue;

			family = ifa->ifa_addr->sa_family;
			if (family == AF_PACKET) {
				struct sockaddr_ll *s = (struct sockaddr_ll *)ifa->ifa_addr;
				int i;

				if (s->sll_halen != EDCL_MAC_ADDR_LEN)
					continue;

				for (i = 0; i < s->sll_halen; i++) {
					if (s->sll_addr[i] != local_mac_addr[i])
						break;
				}

				if (i == EDCL_MAC_ADDR_LEN)
					break;
			}
		}

		if (ifa == NULL) {
			PL_DBG("ERROR: Failed find interface!");

			freeifaddrs(ifaddr);
			throw 2;
		}

		strncpy(ifname, ifa->ifa_name, sizeof(ifname));
		freeifaddrs(ifaddr);

		sprintf(dev_name, "/%s_%s_%1.1X_MUTEX", NM6408_BRD_NAME, ifname, remote_mac_addr[5]);
		ret = libload_mutex_open(dev_name, &b_mutex_map);
		if (ret)  {
			PL_DBG("ERROR %d (%s): Failed open Shm-mutex!",
				ret, strerror(ret));
			throw 3;
		}

		memcpy(b_config.remote_mac, remote_mac_addr, EDCL_MAC_ADDR_LEN);

		b_config.maxpayload = 456;
		b_config.local_port = 0x8088;
		b_config.remote_port = 0x9099;
		b_config.endian = ENDIAN_LITTLE;

		libload_mutex_lock(b_mutex_map, -1, -1);

		if ((Socket_desc = edcl_test_init(ifname, &b_config, &seqnc_count)) < 0) {
			libload_mutex_unlock(b_mutex_map, -1, -1);

			PL_DBG("ERROR %d (%s): Failed get raw socket to %s!",
				errno, strerror(errno), ifname);

			libload_mutex_close(&b_mutex_map);
			throw 4;
		}

		libload_seqnc_count(b_mutex_map, 1, &seqnc_count);
		libload_reset_count(b_mutex_map, 0, &reset_count);

		libload_mutex_unlock(b_mutex_map, -1, -1);

		File_desc = -1;
		b_index = -1;

		is_used_edcl = 1;
	}
	else {
		struct nm6408_wind      wind;

		sprintf(dev_name, "/dev/%s_%2.2d", NM6408_BRD_NAME, indx);
		File_desc = open(dev_name, O_RDWR);
		if (File_desc < 0) {
			PL_DBG("ERROR %d (%s): Failed open device %s!",
				errno, strerror(errno), dev_name);

			throw 1;
		}

		wind.cmd = NM6408_READ;

		ret = ioctl(File_desc, NM6408_CFG_WIND, &wind);
		if (ret) {
			PL_DBG("ERROR %d (%s): Failed driver ioctl!",
				errno, strerror(errno));

			close(File_desc);
			File_desc = -1;

			throw 2;
		}

		llwind_size = wind.size;
		b_index = indx;

		sprintf(dev_name, "/%s_%2.2d_MUTEX", NM6408_BRD_NAME, b_index);
		ret = libload_mutex_open(dev_name, &b_mutex_map);
		if (ret) {
			PL_DBG("ERROR %d (%s): Failed open Shm-mutex!",
				ret, strerror(ret));

			b_index = -1;

			close(File_desc);
			File_desc = -1;

			throw 3;
		}

		libload_mutex_lock(b_mutex_map, -1, -1);
		libload_reset_count(b_mutex_map, 0, &reset_count);
		libload_mutex_unlock(b_mutex_map, -1, -1);

		Socket_desc = -1;
		b_config.maxpayload = 0;
		ifname[0] = '\0';

		is_used_edcl = 0;
	}

	Magic_code = BOARD_MAGIC_CODE;
}

PL_Board::~PL_Board()
{
	Magic_code = 0x0;

	libload_mutex_close(&b_mutex_map);

	if (is_used_edcl) {
		close(Socket_desc);
		Socket_desc = -1;
	}
	else {
		b_index = -1;

		close(File_desc);
		File_desc = -1;
	}
}

static int do_access_edcl(PL_Board *b, int cluster, DWORD *buf, DWORD addr, DWORD count, int RW)
{
	QWORD		laddr, baddr;
	int		i, ret;

	if (!count)
		return 0;

	laddr = (QWORD)addr + ((QWORD)count << 2);
	baddr = (QWORD)addr & ~((QWORD)NM6408_INBOUND_AXI_SIZE - 1);

	if ((addr & 0x3) || (addr >= NM6408_INBOUND_AXI_SIZE) ||
	    ((laddr - baddr) > ((QWORD)NM6408_INBOUND_AXI_SIZE)))
		return 1;

	if (cluster == NM6408_PC0)
		addr |= PC0_INBOUND_AXI_BASE;
	else if (cluster == NM6408_PC1)
		addr |= PC1_INBOUND_AXI_BASE;
	else if (cluster == NM6408_PC2)
		addr |= PC2_INBOUND_AXI_BASE;
	else if (cluster == NM6408_PC3)
		addr |= PC3_INBOUND_AXI_BASE;
	else
		addr |= CCPU_INBOUND_AXI_BASE;

	while (count) {
		if (count > 114)
			i = 114;
		else
			i = count;

		errno = 0;

		if (RW)
			ret = edcl_write(b->Socket_desc, &b->b_config, &b->seqnc_count,
					 addr, buf, i << 2);
		else
			ret = edcl_read(b->Socket_desc, &b->b_config, &b->seqnc_count,
					addr, buf, i << 2);

		if (ret) {
			PL_DBG("ERROR %d (%s): Failed transfer data!",
				errno, strerror(errno));

			return 2;
		}

		addr += i << 2;

		buf += i;
		count -= i;
	}

	return 0;	
}

static int do_access_pcie(PL_Board *b, int cluster, DWORD *buf, DWORD addr, DWORD count, int RW)
{
	struct nm6408_wind	wind;
	struct nm6408_access	access;
	QWORD			laddr, baddr;
	int			i, ret;

	if (!count)
		return 0;

	laddr = (QWORD)addr + ((QWORD)count << 2);
	baddr = (QWORD)addr & ~((QWORD)NM6408_INBOUND_AXI_SIZE - 1);

	if ((addr & 0x3) || (addr >= NM6408_INBOUND_AXI_SIZE) ||
	    ((laddr - baddr) > ((QWORD)NM6408_INBOUND_AXI_SIZE)))
		return 1;

	while (count) {
		QWORD cnt;

		wind.addr = addr;
		wind.cluster = (enum _nm6408_cluster)cluster;
		wind.cmd = NM6408_WRITE;

		cnt = count;

		ret = ioctl(b->File_desc, NM6408_CFG_WIND, &wind);
		if (ret) {
			PL_DBG("ERROR %d (%s): Failed driver ioctl!",
				errno, strerror(errno));

			return 2;
		}

		access.width = NM6408_WIDTH32;

		if (RW)
			access.cmd = NM6408_WRITE;
		else
			access.cmd = NM6408_READ;

		access.offset = addr - wind.addr;

		cnt <<= 2;
		if (cnt > (b->llwind_size - access.offset))
			cnt = b->llwind_size - access.offset;
		cnt >>= 2; 

		for (i = 0; i < cnt; i++) {
			errno = 0;

			if (RW)
				access.data.b32 = buf[i];

			ret = ioctl(b->File_desc, NM6408_DO_ACCESS, &access);
			if (ret) {
				PL_DBG("ERROR %d (%s): Failed driver ioctl!",
					errno, strerror(errno));

				return 3;
			}

			if (!RW)
				buf[i] = access.data.b32;

			access.offset += 4;
		}

		addr = wind.addr + access.offset;
		buf += cnt;
		count -= cnt;
	}

	return 0;
}

static __u32 crc32_f(__u8 *bgn, size_t length)
{
	int i,j;
	__u32 crc;
	__u32 polynom = 0x04C11DB7;
	__u32 * val;

	crc = 0xffffffff;

	for (i=0, val = (__u32 *)bgn; i<length; i+=4, val++) {
		crc ^= *val;

		for (j=0; j<32; j++) {
			if (crc & 0x80000000) {
				crc <<= 1;
				crc ^= polynom;
			} else {
				crc <<= 1;
			}
		}
	}

	return (crc^0xffffffff);
}

int PL_Board::Command(PL_CoreNo *coreNo, DWORD cmd, DWORD *prms)
{
	int (*do_access)(PL_Board* , int , DWORD* , DWORD , DWORD , int );
	struct libload_top	*ll_top;
	struct libload_format	*ll_format;
	QWORD			paddr, psize;
	DWORD			buf[4], rdata;
	DWORD			laddr, raddr;
	int			cluster;
	int			ret;

	libload_mutex_lock(b_mutex_map, -1, -1);

	ll_top = (struct libload_top *)(LIBLOAD_DATA_ADDR & ~CCPU_INBOUND_AXI_BASE);

	if (coreNo->cluster_id == NM6408_CCPU)
		ll_format = &ll_top->arm;
	else if (coreNo->nm_id == -1)
		ll_format = &ll_top->cluster[coreNo->cluster_id].arm;
	else
		ll_format = &ll_top->cluster[coreNo->cluster_id].nmc[coreNo->nm_id];

	if (is_used_edcl)
		do_access = do_access_edcl;
	else
		do_access = do_access_pcie;

	if ((cmd != SW_RESET_BOARD) && (cmd != LOAD_INIT_CODE)) {
		unsigned int	rst_cnt;
		DWORD		tmp;

		if ((cmd == READ_ARM_REGISTER) || (cmd == WRITE_ARM_REGISTER)
		   || (cmd == READ_MEM_AS_REG) || (cmd == WRITE_MEM_AS_REG))
			goto skip_pl_check;

		libload_reset_count(b_mutex_map, 0, &rst_cnt);
		if (reset_count != rst_cnt) {
			ret = 1;
			goto err_pl_exit;
		}

		if (is_used_edcl)
			libload_seqnc_count(b_mutex_map, 0, &seqnc_count);

		ret = do_access(this, NM6408_CCPU, &tmp, LIBLOAD_DATA_READY, 1, 0);

		if (is_used_edcl)
			libload_seqnc_count(b_mutex_map, 1, &seqnc_count);

		if (ret || (tmp != 0x5A5A5A5A)) {
			ret = 2;
			goto err_pl_exit;
		}
	}

skip_pl_check:

	if (is_used_edcl) {
		libload_seqnc_count(b_mutex_map, 0, &seqnc_count);

		ret = edcl_get_seqnum(Socket_desc, &b_config, &seqnc_count);
		if (ret) {
			ret = 3;
			goto err_pl_exit;
		}
	}

	switch (cmd) {
	case SW_RESET_BOARD:
		do {
			unsigned int rst_cnt;

			libload_reset_count(b_mutex_map, 0, &rst_cnt);
			if (reset_count != rst_cnt)
				goto skip_pl_exit;
		} while (0);

		if (is_used_edcl) {
			if (!seqnc_count) { // Read something to increase seq. counter
				ret = do_access(this, NM6408_CCPU, buf,
						0x000CA014, 1, 0);
				if (ret)
					break;
			}

			buf[0] = 0x00000006;

			ret = do_access(this, NM6408_CCPU, buf,
					0x000CA014, 1, 1);
			if (ret)
				break;

			close(Socket_desc);
			Socket_desc = -1;

			for (int i = 1000 ;; i--) {
				usleep(1000);

				Socket_desc = edcl_test_init(ifname, &b_config, &seqnc_count);
				if (Socket_desc >= 0) {
					ret = 0;
					break;
				}

				if (i < 0) {
					PL_DBG("ERROR %d (%s): Unable get raw socket to %s!",
						errno, strerror(errno), ifname);

					ret = 4;
					goto err_pl_exit;
				}
			}

			ret = do_access(this, NM6408_CCPU, buf,
					0x000CA01C, 1, 0);

			if (ret || (buf[0] & 0x00000002)) {
				ret = 5;
				goto err_pl_exit;
			}
		}
		else {
#if 0
			ret = ioctl(File_desc, NM6408_DO_RESET, NULL);
			if (ret) {
				PL_DBG("ERROR %d (%s): Failed reset board!",
				       errno, strerror(errno));

				ret = 4;
				goto err_pl_exit;
			}

			ret = do_access(this, NM6408_CCPU, buf,
					0x000CA01C, 1, 0);

			if (ret || (buf[0] & 0x00000002)) {
				ret = 5;
				goto err_pl_exit;
			}
#else
			ret = do_access(this, NM6408_CCPU, buf, LIBLOAD_DATA_READY, 1, 0);
			if (!ret && (buf[0] == 0)) {
				ret = 0;
				break;
			}
			else if (ret || (buf[0] != 0x5A5A5A5A)) {
				ret = 4;
				goto err_pl_exit;
			}

			buf[0] = 0x00010000;

			ret = do_access(this, NM6408_CCPU, buf, 0x000B0F00, 1, 1);
			if (!ret) {
				paddr = (QWORD)&ll_format->status;

				for (int i = 1000, j = 0 ;; i--) {
					usleep(1000);

					if (!j) {
						ret = do_access(this, NM6408_CCPU, buf, (DWORD)paddr, 1, 0);

						if (!ret && (buf[0] == PROGRAM_IS_RESET)) {
							buf[0] = 0x0;

							ret = do_access(this, NM6408_CCPU, buf, (DWORD)paddr, 1, 1);
							if (!ret)
								j = 1;
						}
					}
					else {
						ret = do_access(this, NM6408_CCPU, buf, LIBLOAD_DATA_READY, 1, 0);

						if (!ret && (buf[0] == 0x5A5A5A5A))
							break;
					}

					if ((0 == i) || (0 != ret)) {
						ret = -1;
						break;
					}
				}
			}

			if (ret) {
				ret = 5;
				goto err_pl_exit;
			}
#endif
		}

		break;
	case GET_POST_STATUS:
		paddr = (QWORD)&ll_format->post_status;

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 0);

		break;
	case GET_SERIAL_NMB_LO:
		paddr = (QWORD)&ll_format->serial[0];

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 0);

		break;
	case GET_SERIAL_NMB_HI:
		paddr = (QWORD)&ll_format->serial[1];

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 0);

		break;
	case GET_VERSION_MAJOR:
		paddr = (QWORD)&ll_format->version[0];

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 0);

		break;
	case GET_VERSION_MINOR:
		paddr = (QWORD)&ll_format->version[1];

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 0);

		break;
	case GET_LOADER_STATUS:
		paddr = (QWORD)&ll_format->status;

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 0);

		break;
	case SET_LOADER_STATUS:
		paddr = (QWORD)&ll_format->status;

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 1);

		break;
	case GET_ENTRY_POINT:
		paddr = (QWORD)&ll_format->entry;

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 0);

		break;
	case SET_ENTRY_POINT:
		paddr = (QWORD)&ll_format->entry;

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 1);

		break;
	case SET_RPC_REGION:
		ret = libload_rpc_set(b_mutex_map, coreNo->cluster_id, coreNo->nm_id);
		if (ret)
			goto err_pl_exit;

		paddr = (QWORD)ll_format->rpc_region;
		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 2, 1);

		break;
	case CLR_RPC_REGION:
		ret = libload_rpc_clr(b_mutex_map, coreNo->cluster_id, coreNo->nm_id);
		if (ret)
			goto err_pl_exit;

		paddr = (QWORD)ll_format->rpc_region;
		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 2, 1);

		break;
	case GET_RPC_REGION:
		ret = libload_rpc_get(b_mutex_map, coreNo->cluster_id, coreNo->nm_id);
		if (ret)
			goto err_pl_exit;

		paddr = (QWORD)ll_format->rpc_region;
		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 2, 0);

		break;
	case SET_LOADER_COMMAND:
		paddr = (QWORD)&ll_format->command;

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 1);

		break;
	case GET_PROGRAM_RESULT:
		paddr = (QWORD)&ll_format->result;

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 0);

		break;
	case SET_PROCESSOR_BUSY:
		paddr = (QWORD)&ll_format->is_busy;

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 1);

		break;
	case GET_PROCESSOR_BUSY:
		paddr = (QWORD)&ll_format->is_busy;

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 0);

		break;
	case READ_ARM_REGISTER:
	case WRITE_ARM_REGISTER:
		if (coreNo->cluster_id != NM6408_CCPU) {
			if ((prms[0] < PC_REG_AXI_ADDR) ||
			    (prms[0] >= (PC_REG_AXI_ADDR + PC_REG_AXI_SIZE))) {
				ret = -1;
				goto err_pl_exit;
			}
		}
		else {
			if ((prms[0] < CCPU_REG_AXI_ADDR) ||
			    (prms[0] >= (CCPU_REG_AXI_ADDR + CCPU_REG_AXI_SIZE))) {
				ret = -1;
				goto err_pl_exit;
			}
		}

		if (cmd == READ_ARM_REGISTER) {
			ret = do_access(this, coreNo->cluster_id, &prms[1],
					prms[0], 1, 0);
		}
		else {
			ret = do_access(this, coreNo->cluster_id, &prms[1],
					prms[0], 1, 1);
		}

		break;
	case READ_MEM_AS_REG:
	case WRITE_MEM_AS_REG:
		paddr = (QWORD)((DWORD **)prms)[0];
		psize = (QWORD)((DWORD **)prms)[1];

		if (coreNo->nm_id != -1)
			paddr = paddr << 2;

		psize = psize << 2;
		ret = Check_mem_ranges(coreNo, 0, paddr, psize, 0, 0);

		if (ret) {
			ret = -1;
			goto err_pl_exit;
		}

		convert_xfer(coreNo, (DWORD *)&paddr, (DWORD *)&psize, 0);
		psize = psize >> 2;

		if (psize != ((QWORD)((DWORD **)prms)[1])) {
			ret = -1;
			goto err_pl_exit;
		}

		if (cmd == READ_MEM_AS_REG) {
			ret = do_access(this, coreNo->cluster_id, ((DWORD **)prms)[2],
					(DWORD)paddr, (DWORD)psize, 0);
		}
		else {
			ret = do_access(this, coreNo->cluster_id, ((DWORD **)prms)[2],
					(DWORD)paddr, (DWORD)psize, 1);
		}

		break;
	case SET_SYNC_FLAG_TO_PC:
		paddr = (QWORD)&ll_format->sync[SYNC_TO_PC + SYNC_FLAG];

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 1);

		break;
	case GET_SYNC_FLAG_TO_PC:
		paddr = (QWORD)&ll_format->sync[SYNC_TO_PC + SYNC_FLAG];

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 0);

		break;
	case GET_SYNC_ARRAY_TO_PC:
		paddr = (QWORD)&ll_format->sync[SYNC_TO_PC + SYNC_VAL];

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 3, 0);

		break;
	case SET_SYNC_FLAG_FROM_PC:
		paddr = (QWORD)&ll_format->sync[SYNC_FROM_PC + SYNC_FLAG];

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 1);

		break;
	case GET_SYNC_FLAG_FROM_PC:
		paddr = (QWORD)&ll_format->sync[SYNC_FROM_PC + SYNC_FLAG];

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 1, 0);

		break;
	case SET_SYNC_ARRAY_FROM_PC:
		paddr = (QWORD)&ll_format->sync[SYNC_FROM_PC + SYNC_VAL];

		ret = do_access(this, NM6408_CCPU, prms,
				(DWORD)paddr, 3, 1);

		break;
	case LOAD_INIT_CODE:
		do {
			unsigned int rst_cnt;

			libload_reset_count(b_mutex_map, 0, &rst_cnt);
			if (reset_count == rst_cnt)
				goto skip_pl_exit;
		} while (0);

		if (is_used_edcl) {
			psize = (QWORD)((DWORD **)prms)[1];
			if (psize > ((QWORD)0x30000)) {
				ret = -1;
				goto err_pl_exit;
			}

			psize >>= 2;

			ret = do_access(this, NM6408_CCPU, ((DWORD **)prms)[0],
					(DWORD)0, (DWORD)psize, 1);
			if (ret)
				break;

			buf[0] = 0;

			ret = do_access(this, NM6408_CCPU, buf,
					LIBLOAD_DATA_READY, 1, 1);
			if (ret)
				break;

			if (!seqnc_count) { // Read something to increase seq. counter
				ret = do_access(this, NM6408_CCPU,  buf,
						(DWORD)0x30000, 1, 0);
				if (ret)
					break;
			}

			buf[0] = ~0;

			ret = do_access(this, NM6408_CCPU,  buf,
					(DWORD)0x30000, 1, 1);
			if (ret)
				break;

			close(Socket_desc);
			Socket_desc = -1;

			for (int i = 1000 ;; i--) {
				if (i < 0) {
					ret = 6;
					goto err_pl_exit;
				}

				usleep(1000);

				if (Socket_desc < 0) {
					Socket_desc = edcl_test_init(ifname, &b_config, &seqnc_count);
					continue;
				}

				ret = do_access(this, NM6408_CCPU, buf,
						LIBLOAD_DATA_READY, 1, 0);
				if (ret)
					break;

				if (buf[0] == 0xA5A5A5A5) {
					buf[0] = 0x5A5A5A5A;

					ret = do_access(this, NM6408_CCPU, buf,
							LIBLOAD_DATA_READY, 1, 1);
					break;
				}
			}
		}
		else {
			struct nm6408_init_code code;

			code.size = (size_t)((DWORD **)prms)[1];
			code.data = (__u8 *)((DWORD **)prms)[0];
			code.crc32 = crc32_f(code.data, code.size);

                        ret = ioctl(File_desc, NM6408_DO_INIT, &code);
                        if (ret) {
                                PL_DBG("ERROR %d (%s): Failed load init code!",
                                       errno, strerror(errno));

                                ret = 6;
                                goto err_pl_exit;
                        }
		}

		break;
	case SW_GEN_NMI:
	case SW_START_CORE:
		if (coreNo->nm_id != -1) {
			if (coreNo->nm_id == 0) {
				laddr = NMPU0_NMMB_TO_PC(0);
				rdata = 0x1;
			}

			if (coreNo->nm_id == 1) {
				laddr = NMPU1_NMMB_TO_PC(0);
				rdata = 0x2;
			}

			if (coreNo->nm_id == 2) {
				laddr = NMPU2_NMMB_TO_PC(0);
				rdata = 0x4;
			}

			if (coreNo->nm_id == 3) {
				laddr = NMPU3_NMMB_TO_PC(0);
				rdata = 0x8;
			}

			if (cmd == SW_GEN_NMI)
				raddr = 0x000CB00C;
			else
				raddr = 0x000CA018;

			cluster = coreNo->cluster_id;

			ret = do_access(this, cluster, buf, raddr, 1, 0);
			if (ret)
				break;

			buf[0] &= ~rdata;

			ret = do_access(this, cluster, buf, raddr, 1, 1);
			if (ret)
				break;

			buf[0] = 0x48270000;    // goto Start_Addr
			buf[1] = *prms;         // Start_Addr
			buf[2] = 0x50100000;    // nul
			buf[3] = 0x50100000;    // nul
		}
		else if (coreNo->cluster_id != NM6408_CCPU) {
			laddr = 0x0;
			rdata = 0x1;

			if (coreNo->cluster_id == NM6408_PC0)
				raddr = 0x000C6018;

			if (coreNo->cluster_id == NM6408_PC1)
				raddr = 0x000C7018;

			if (coreNo->cluster_id == NM6408_PC2)
				raddr = 0x000C8018;

			if (coreNo->cluster_id == NM6408_PC3)
				raddr = 0x000C9018;

			cluster = NM6408_CCPU;

			buf[0] = 0xE59F0000;    // ldr r0,[pc]
			buf[1] = 0xE12FFF30;    // blx r0
			buf[2] = *prms;         // Start_Addr
			buf[3] = 0xE320F000;    // mov r0,r0
		}
		else {
			ret = 7;
			goto err_pl_exit;
		}

		if (*prms) {
			ret = do_access(this, coreNo->cluster_id,
					buf, laddr, 4, 1);
			if (ret)
				break;
		}

		ret = do_access(this, cluster, buf, raddr, 1, 0);
		if (ret)
			break;

		buf[0] |= rdata;

		ret = do_access(this, cluster, buf, raddr, 1, 1);
		if (ret)
			break;

		ret = do_access(this, cluster, buf, raddr, 1, 0);
		if (ret)
			break;

		if (rdata != (buf[0] & rdata)) {
			ret = 8;
			goto err_pl_exit;
		}

		break;
	default:
		ret = 9;
		goto err_pl_exit;
	}

	if (ret) {
		ret = 10;
		goto err_pl_exit;
	}

	if ((cmd == SW_RESET_BOARD) || (cmd == LOAD_INIT_CODE)) {
		if (cmd == SW_RESET_BOARD)
			libload_reset_count(b_mutex_map, 1, NULL);

		if (cmd == LOAD_INIT_CODE)
			libload_reset_count(b_mutex_map, 0, &reset_count);
	}

skip_pl_exit:

	if (is_used_edcl) {
		ret = edcl_get_seqnum(Socket_desc, &b_config, &seqnc_count);
		if (ret) {
			ret = 11;
			goto err_pl_exit;
		}

		libload_seqnc_count(b_mutex_map, 1, &seqnc_count);
	}

	ret = 0;

err_pl_exit:

	libload_mutex_unlock(b_mutex_map, -1, -1);

	return ret;
}

PL_Access::PL_Access(PL_CoreNo *core, PL_Board *brd):
	Count_In_Progress(0)
{
	coreNo = *core;

	if (!((coreNo.cluster_id >= -1) && (coreNo.cluster_id < LIBLOAD_CLUSTER_NUMBER)) ||
	    !((coreNo.nm_id >= -1) && (coreNo.nm_id < LIBLOAD_CORE_NUMBER)) ||
	    ((coreNo.cluster_id == -1) && (coreNo.nm_id != -1)))
		throw 1;

	if (coreNo.cluster_id == -1)
		coreNo.cluster_id = NM6408_CCPU;

	MainARGS = NULL;
	MainARGC = 0;

	board = brd;
	entry_point = (QWORD)ACCESS_NO_ENTRY;

	Magic_code = PROCESSOR_MAGIC_CODE;
}

PL_Access::~PL_Access()
{
	if (MainARGC) {
		delete[] MainARGS;
		MainARGS = NULL;
	}

	MainARGC = 0;

	Magic_code = 0x0;
	board = NULL;
}

static void convert_xfer(PL_CoreNo *core, DWORD *addr, DWORD *size, int set_global)
{
	if (core->nm_id != -1) {
		if (*addr < (NMPU_NMMB_AXI_ADDR + NMPU_NMMB_AXI_SIZE)) {
			if (core->nm_id == 0)
				*addr = NMPU0_NMMB_TO_PC(*addr);

			if (core->nm_id == 1)
				*addr = NMPU1_NMMB_TO_PC(*addr);

			if (core->nm_id == 2)
				*addr = NMPU2_NMMB_TO_PC(*addr);

			if (core->nm_id == 3)
				*addr = NMPU3_NMMB_TO_PC(*addr);
		}
		else if (*addr < (NMPU_NMPU_AXI_ADDR + NMPU_NMPU_AXI_SIZE)) {
			if ((*addr + *size) > (NMPU_NMPU_AXI_ADDR + NMPU_NMPU_AXI_SIZE))
				*size = (NMPU_NMPU_AXI_ADDR + NMPU_NMPU_AXI_SIZE) - *addr;

			*addr = NMPU_NMPU_TO_PC(*addr);
		}
		else if (*addr < (NMPU_CMB_AXI_ADDR + NMPU_CMB_AXI_SIZE)) {
			if ((*addr + *size) > (NMPU_CMB_AXI_ADDR + NMPU_CMB_AXI_SIZE))
				*size = (NMPU_CMB_AXI_ADDR + NMPU_CMB_AXI_SIZE) - *addr;

			*addr = NMPU_CMB_TO_PC(*addr);
		}
		else
			*addr = NMPU_EMI_TO_PC(*addr);
	}

	if (set_global) {
		if (core->cluster_id == NM6408_PC0)
			*addr |= PC0_INBOUND_AXI_BASE;
		else if (core->cluster_id == NM6408_PC1)
			*addr |= PC1_INBOUND_AXI_BASE;
		else if (core->cluster_id == NM6408_PC2)
			*addr |= PC2_INBOUND_AXI_BASE;
		else if (core->cluster_id == NM6408_PC3)
			*addr |= PC3_INBOUND_AXI_BASE;
		else
			*addr |= CCPU_INBOUND_AXI_BASE;
	}

	return;
}

static int do_xfer_edcl(PL_Board *b, PL_CoreNo *core, DWORD *buf, DWORD addr, DWORD size, int RWX)
{
	DWORD		tmp_size, tmp_addr;
	DWORD		rw_size;
	int		ret;

	if (!size)
		return 0;

	while (size) {
		tmp_addr = addr;
		tmp_size = size;

		convert_xfer(core, &tmp_addr, &tmp_size, 1);

		addr += tmp_size;
		size -= tmp_size;

		while (tmp_size) {
			if (tmp_size > 456)
				rw_size = 456;
			else
				rw_size = tmp_size;

			if (RWX != 0)
				ret = edcl_write(b->Socket_desc, &b->b_config, &b->seqnc_count,
						 tmp_addr, buf, rw_size);
			else
				ret = edcl_read(b->Socket_desc, &b->b_config, &b->seqnc_count,
						tmp_addr, buf, rw_size);

			if (ret) {
				PL_DBG("ERROR %d (%s): Failed edcl read/write!",
					errno, strerror(errno));
				return 1;
			}

			tmp_addr += rw_size;
			buf += (rw_size >> 2);
			tmp_size -= rw_size;
		}
	}

	return 0;
}

static int do_xfer_pcie(PL_Board *b, PL_CoreNo *core, DWORD *buf, DWORD addr, DWORD size, int RWX)
{
	DWORD			tmp_size, tmp_addr;
	ssize_t			rw_size;
	off_t			ret;

	if (!size)
		return 0;

	while (size) {
		tmp_addr = addr;
		tmp_size = size;

		convert_xfer(core, &tmp_addr, &tmp_size, 1);

		errno = 0;
		ret = lseek(b->File_desc, tmp_addr, SEEK_SET);
		if (errno || (ret < 0)) {
			PL_DBG("ERROR %d (%s): Failed driver lseek!",
				errno, strerror(errno));

			return 1;
		}

		if (RWX != 0)
			rw_size = write(b->File_desc, buf, tmp_size);
		else
			rw_size = read(b->File_desc, buf, tmp_size);

		if ((rw_size <= 0) || (rw_size & 0x3)) {
			PL_DBG("ERROR %d (%s): Failed driver read/write!",
				errno, strerror(errno));

			return 2;
		}

		addr += (DWORD)rw_size;
		buf += (rw_size >> 2);
		size -= (DWORD)rw_size;
	}

	return 0;
}

int PL_Access::Xfer(DWORD *buf, QWORD addr, QWORD load_size, QWORD check_size, int RWX)
{
	int (*do_xfer)(PL_Board* , PL_CoreNo* , DWORD* , DWORD , DWORD , int );
	PL_CoreNo		core;
	QWORD			_addr, addr_load, addr_check;
	QWORD			cur_lsize, cur_csize, cur_addr;
	unsigned int		rst_cnt;
	int			is_self, desc;
	int			ret;

	if (!load_size && !check_size)
		return 0;

	if (load_size && !check_size) {
		addr_load = addr + load_size;
		addr_check = 0;

		_addr = addr_load;
	}
	else if (!load_size && check_size) {
		addr_check = addr + check_size;
		addr_load = 0;

		_addr = addr_check;
	}
	else {
		addr_load = addr + load_size;
		addr_check = addr + check_size;

		if (check_size > load_size)
			_addr = addr_check;
		else
			_addr = addr_load;
	}

	if (_addr > ((QWORD)PC3_INBOUND_AXI_LIMIT + 1))
		return -1;

	while (addr < _addr) {
		if (addr < ((QWORD)CCPU_INBOUND_AXI_BASE)) {
			if (addr_check > ((QWORD)CCPU_INBOUND_AXI_BASE))
				cur_csize = (QWORD)CCPU_INBOUND_AXI_BASE - addr;
			else
				cur_csize = check_size;

			if (addr_load > ((QWORD)CCPU_INBOUND_AXI_BASE))
				cur_lsize = (QWORD)CCPU_INBOUND_AXI_BASE - addr;
			else
				cur_lsize = load_size;

			core.cluster_id = coreNo.cluster_id;
			core.nm_id = coreNo.nm_id;

			cur_addr = addr;
			is_self = 1;
		}
		else if (addr < ((QWORD)PC0_INBOUND_AXI_BASE)) {
			if (addr_check > ((QWORD)PC0_INBOUND_AXI_BASE)) 
				cur_csize = (QWORD)PC0_INBOUND_AXI_BASE - addr;
			else
				cur_csize = check_size;

			if (addr_load > ((QWORD)PC0_INBOUND_AXI_BASE))
				cur_lsize = (QWORD)PC0_INBOUND_AXI_BASE - addr;
			else
				cur_lsize = load_size;

			core.cluster_id = NM6408_CCPU;
			core.nm_id = -1;

			cur_addr = addr - (QWORD)CCPU_INBOUND_AXI_BASE;
			is_self = 0;
		}
		else if (addr < ((QWORD)PC1_INBOUND_AXI_BASE)) {
			if (addr_check > ((QWORD)PC1_INBOUND_AXI_BASE)) 
				cur_csize = (QWORD)PC1_INBOUND_AXI_BASE - addr;
			else
				cur_csize = check_size;

			if (addr_load > ((QWORD)PC1_INBOUND_AXI_BASE))
				cur_lsize = (QWORD)PC1_INBOUND_AXI_BASE - addr;
			else
				cur_lsize = load_size;

			core.cluster_id = NM6408_PC0;
			core.nm_id = -1;

			cur_addr = addr - (QWORD)PC0_INBOUND_AXI_BASE;
			is_self = 0;
		}
		else if (addr < ((QWORD)PC2_INBOUND_AXI_BASE)) {
			if (addr_check > ((QWORD)PC2_INBOUND_AXI_BASE)) 
				cur_csize = (QWORD)PC2_INBOUND_AXI_BASE - addr;
			else
				cur_csize = check_size;

			if (addr_load > ((QWORD)PC2_INBOUND_AXI_BASE))
				cur_lsize = (QWORD)PC2_INBOUND_AXI_BASE - addr;
			else
				cur_lsize = load_size;

			core.cluster_id = NM6408_PC1;
			core.nm_id = -1;

			cur_addr = addr - (QWORD)PC1_INBOUND_AXI_BASE;
			is_self = 0;
		}
		else if (addr < ((QWORD)PC3_INBOUND_AXI_BASE)) {
			if (addr_check > ((QWORD)PC3_INBOUND_AXI_BASE)) 
				cur_csize = (QWORD)PC3_INBOUND_AXI_BASE - addr;
			else
				cur_csize = check_size;

			if (addr_load > ((QWORD)PC3_INBOUND_AXI_BASE))
				cur_lsize = (QWORD)PC3_INBOUND_AXI_BASE - addr;
			else
				cur_lsize = load_size;

			core.cluster_id = NM6408_PC2;
			core.nm_id = -1;

			cur_addr = addr - (QWORD)PC2_INBOUND_AXI_BASE;
			is_self = 0;
		}
		else {
			cur_csize = check_size;
			cur_lsize = load_size;

			core.cluster_id = NM6408_PC3;
			core.nm_id = -1;

			cur_addr = addr - (QWORD)PC3_INBOUND_AXI_BASE;
			is_self = 0;
		}

		ret = Check_mem_ranges(&core, entry_point, cur_addr, cur_csize, RWX, is_self);
		if (ret)
			return -1;

		libload_mutex_lock(board->b_mutex_map, -1, -1);

		libload_reset_count(board->b_mutex_map, 0, &rst_cnt);

		if (board->reset_count != rst_cnt) {
			libload_mutex_unlock(board->b_mutex_map, -1, -1);
			return 1;
		}

		if (board->is_used_edcl) {
			libload_seqnc_count(board->b_mutex_map, 0, &board->seqnc_count);

			ret = edcl_get_seqnum(board->Socket_desc, &board->b_config, &board->seqnc_count);
			if (ret) {
				libload_mutex_unlock(board->b_mutex_map, -1, -1);
				return 2;
			}

			do_xfer = do_xfer_edcl;
		}
		else {
			do_xfer = do_xfer_pcie;
		}

		ret = do_xfer(board, &core, buf, (DWORD)cur_addr, (DWORD)cur_lsize, RWX);
		if (ret) {
			libload_mutex_unlock(board->b_mutex_map, -1, -1);
			return 3;
		}

		if (board->is_used_edcl) {
			ret = edcl_get_seqnum(board->Socket_desc, &board->b_config, &board->seqnc_count);
			if (ret) {
				libload_mutex_unlock(board->b_mutex_map, -1, -1);
				return 4;
			}

			libload_seqnc_count(board->b_mutex_map, 1, &board->seqnc_count);
		}

		libload_mutex_unlock(board->b_mutex_map, -1, -1);

		if (cur_csize > cur_lsize)
			addr += cur_csize;
		else
			addr += cur_lsize;

		buf += (cur_lsize >> 2);

		check_size -= cur_csize;
		load_size -= cur_lsize;
	}

	return 0;
}
