#include <errno.h>
#include <sys/stat.h>
#include <sys/times.h>
#include <sys/unistd.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Newlib stubs */

#undef errno
extern int errno;

char *__env[1] __attribute__ ((weak)) = { 0 };
char **environ __attribute__ ((weak)) = __env;

int _open(const char *name, int flags, int mode) __attribute__((weak, section (".stub._open")));
int _open(const char *name, int flags, int mode)
{
	return -1;
}

int _close(int file) __attribute__((weak, section (".stub._close")));
int _close(int file)
{
	return -1;
}

int _execve(char *name, char **argv, char **env) __attribute__((weak, section (".stub._execve")));
int _execve(char *name, char **argv, char **env)
{
	errno = ENOMEM;
	return -1;
}

int _fork(void) __attribute__((weak, section (".stub._fork")));
int _fork(void)
{
	errno = EAGAIN;
	return -1;
}

int _fstat(int file, struct stat *st) __attribute__((weak, section (".stub._fstat")));
int _fstat(int file, struct stat *st)
{
	st->st_mode = S_IFCHR;
	return 0;
}

int _getpid(void) __attribute__((weak, section (".stub._getpid")));
int _getpid(void)
{
	return 1;
}

int _isatty(int file) __attribute__((weak, section (".stub._isatty")));
int _isatty(int file)
{
	switch(file) {
	case STDOUT_FILENO:
	case STDERR_FILENO:
	case STDIN_FILENO:
		return 1;
	default:
		errno = EBADF;
		return 0;
	}
}

int _kill(int pid, int sig) __attribute__((weak, section (".stub._kill")));
int _kill(int pid, int sig)
{
	errno = EINVAL;
	return -1;
}

int _link(char *old, char *new) __attribute__((weak, section (".stub._link")));
int _link(char *old, char *new)
{
	errno = EMLINK;
	return -1;
}

int _lseek(int file, int ptr, int dir) __attribute__((weak, section (".stub._lseek")));
int _lseek(int file, int ptr, int dir)
{
	return 0;
}

caddr_t _sbrk(int incr) __attribute__((weak, section (".stub._sbrk")));
caddr_t _sbrk(int incr)
{
	extern char __heap_start __attribute__((weak));
	extern char __heap_end __attribute__((weak));
	static char *heap_end = &__heap_start;
	static char *heap_limit = &__heap_end;
	char *prev_heap_end;

	prev_heap_end = heap_end;

	if ((heap_end + incr) > heap_limit) {
		errno = ENOMEM;
		/* Heap and stack collision. Normal heap between bss and stack */
		return (caddr_t) -1;
	}

	heap_end += incr;
	return (caddr_t) prev_heap_end;
}

int _read(int file, char *ptr, int len) __attribute__((weak, section (".stub._read")));
int _read(int file, char *ptr, int len)
{
	return 0;
}

int _stat(const char *filepath, struct stat *st) __attribute__((weak, section (".stub._stat")));
int _stat(const char *filepath, struct stat *st)
{
	st->st_mode = S_IFCHR;
	return 0;
}

clock_t _times(struct tms *buf) __attribute__((weak, section (".stub._times")));
clock_t _times(struct tms *buf)
{
	return -1;
}

int _unlink(char *name) __attribute__((weak, section (".stub._unlink")));
int _unlink(char *name)
{
	errno = ENOENT;
	return -1;
}

int _wait(int *status) __attribute__((weak, section (".stub._wait")));
int _wait(int *status)
{
	errno = ECHILD;
	return -1;
}

int _write(int file, char *ptr, int len) __attribute__((weak, section (".stub._write")));
int _write(int file, char *ptr, int len)
{
	return 0;
}

#ifdef __cplusplus
};
#endif
