#pragma once

#include "libload_wrapper.hpp"

enum
{
	TEMPERATURE_REG = 0xd1018
};

#define TS_DATA_VALUE(x) ((x) & 0xFFF)

class TemperatureReader
{
private:

	struct tsns_item 
	{
		int adc_units;
		int degrees;
	};

	typedef struct tsns_item tsns_item_t;

	static tsns_item_t tsns_lut[];

	static int find_lut_index(tsns_item_t* item)
	{
		int is_degrees, mult, units;

		if (item->adc_units == -1) 
		{
			mult = 1;
			units = item->degrees * mult;
			is_degrees = 1;
		}
		else 
		{
			mult = -1;
			units = item->adc_units * mult;
			is_degrees = 0;
		}

		int i = -1;

		do 
		{
			i += 1;

			if (units <= (*((int*)&tsns_lut[i] + is_degrees) * mult))
				break;

		} while (tsns_lut[i + 1].adc_units != -1);

		if ((i == 0) || (units > (*((int*) &tsns_lut[i] + is_degrees) * mult)))
			units = *((int*) &tsns_lut[i] + is_degrees) * mult;

		if (units == (*((int*) &tsns_lut[i] + is_degrees) * mult))
			(is_degrees) ? (item->adc_units = tsns_lut[i].adc_units) :
			(item->degrees = tsns_lut[i].degrees);

		return i;
	}

	static constexpr int tsns_mult = 1000;

	static int adc_to_degrees(const int adc_units)
	{
		tsns_item_t item = { adc_units, -1 };

		const int index = find_lut_index(&item);

		if (item.degrees == tsns_lut[index].degrees)
			return tsns_lut[index].degrees;

		volatile int tmp = (adc_units - tsns_lut[index].adc_units) *
			(tsns_lut[index].degrees - tsns_lut[index - 1].degrees) *
			tsns_mult;
		tmp /= (tsns_lut[index - 1].adc_units - tsns_lut[index].adc_units);
		tmp = ((tsns_lut[index].degrees * tsns_mult - tmp) / tsns_mult);

		return tmp;
	}

public:
	static int ReadTemperature(const LibloadWrapper &wrapper, LibloadWrapper::Access* access)
	{
		std::uint32_t board_temperature = 0;
		if (wrapper.ReadRegister(access, &board_temperature, 0xd1018) != LibloadWrapper::OK)
		{
			throw std::runtime_error("ERROR: Failed to read register");
		}
		return adc_to_degrees(TS_DATA_VALUE(board_temperature));
	}
};

TemperatureReader::tsns_item_t TemperatureReader::tsns_lut[] =
{
    {3795, -40},
    {3787, -35},
    {3779, -30},
    {3769, -25},
    {3761, -20},
    {3751, -15},
    {3743, -10},
    {3733, -5},
    {3723, 0},
    {3713, 5},
    {3703, 10},
    {3693, 15},
    {3683, 20},
    {3673, 25},
    {3663, 30},
    {3651, 35},
    {3641, 40},
    {3629, 45},
    {3617, 50},
    {3605, 55},
    {3593, 60},
    {3581, 65},
    {3569, 70},
    {3555, 75},
    {3543, 80},
    {3529, 85},
    {3515, 90},
    {3501, 95},
    {3487, 100},
    {3471, 105},
    {3457, 110},
    {3441, 115},
    {3425, 120},
    {3409, 125},
    {-1, -1}    // End of lookup table
};

