// Loader for NM6408 based board
// Processing command line parameters

#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <assert.h>

#include "nmrun.h"

using namespace std;

extern unsigned int Board_number;

extern unsigned char host_mac[];
extern unsigned char chip_mac[];
extern bool is_use_edcl;

extern PL_CoreNo CoreNo;

extern int f_Dump_start;
extern int f_Dump_finish;
extern int f_Run_program;
extern int f_Wait_complite;
extern int f_Reset_board;
extern int f_Return_ret_val;
extern int Quiet;

extern int f_Use_print;

extern char *ABS_file_name;
extern char *ABS_file_args;

extern unsigned long Timeout_value;

extern PL_Addr Dump_memory_begin;
extern DWORD Dump_memory_size;
extern unsigned int Dump_field_bits;

extern unsigned char reg_flags;
extern DWORD reg_addr;
extern DWORD reg_value;

extern unsigned char recv_flags;
extern char *recv_file_name;
extern PL_Addr recv_addr;
extern DWORD recv_size;
extern char *recv_sect_name;
extern PL_Addr recv_disp;

extern unsigned char send_flags;
extern char *send_file_name;
extern PL_Addr send_addr;
extern DWORD send_size;
extern char *send_sect_name;
extern PL_Addr send_disp;

extern int f_Sync;
extern unsigned char sync_flags;
extern DWORD Sync_timeout;
extern PL_Word Sync_send;

#ifndef RUN_MAJOR_VERSION
#define RUN_MAJOR_VERSION 2
#endif

#ifndef RUN_MINOR_VERSION
#define RUN_MINOR_VERSION 1
#endif

#ifndef BUILD_DAT
#define BUILD_DAT 0
#endif

#ifndef BUILD_NUM
#define BUILD_NUM 0
#endif

#define _STRINGIFY(X) #X
#define STRINGIFY(X) _STRINGIFY(X)

#define RUN_VERSION STRINGIFY(RUN_MAJOR_VERSION) "." STRINGIFY(RUN_MINOR_VERSION) "." STRINGIFY(BUILD_DAT) "." STRINGIFY(BUILD_NUM)

void Display_version(int argc, char **argv)
{
	for (int i = 1; i < argc; i++)
		if ((argv[i][0] == '-') && (argv[i][1] == 'q'))
			return;

	cout << "Batch loader for "DEVICE_NAME_STR" v";
	cout << RUN_VERSION;
	cout << "; "COPYRIGHT_STR".\n";
}

void Usage(void)
{
	cout << "Usage: "DEVICE_NAME"run FILE.abs [OPTION(S)]\n";

	cout << "Options:\n";
	cout << "-c<num>\tSelect cluster to work (-1; 0; 1; 2; 3)\n";
	cout << " (default: " << dec << Access_core_default.cluster_id << ")\n";
	cout << "-n<num>\tSelect core to work (-1; 0; 1; 2; 3)\n";
	cout << " (default: " << dec << Access_core_default.nm_id << ")\n";
	cout << "-s\tDump shared memory before program start\n";
	cout << "-e\tDump shared memory after program finish\n";
	cout << "-m<num>\tSpecify dump begin address (hex)";
	cout << " (default: 0x" << hex << uppercase << setw(8) << setfill('0') << Dump_memory_begin_default << ")\n";
	cout << "-l<num>\tSpecify dump size in 32 bit words (hex)";
	cout << " (default: 0x" << hex << uppercase << setw(8) << setfill('0') << Dump_memory_size_default << ")\n";
	cout << "-d<num>\tSpecify length of dump words in bits (1; 2; 4; ...; 64) (default: 32)\n";
	cout << "-r\tDo not run program\n";
	cout << "-w\tDo not wait completion\n";
	cout << "-i\tDo not load init code (reset board)\n";
	cout << "-t<num>\tTimeout in milliseconds (0 - " << dec << MAX_TIMEOUT << ") (default: 0 - no timeout)\n";
	cout << "-b<num>\tWhich board to use (default: 0)\n";
	cout << "-a<str>\tUse EDCL rather then PCIe and specify host MAC address (example: -a1A-2B-3C-4D-5E-6F)\n";
	cout << "-z<str>\tSpecify board MAC address (default: ";
	cout << hex << uppercase << setw(2) << setfill('0') << (unsigned int)chip_mac[0] << "-";
	cout << hex << uppercase << setw(2) << setfill('0') << (unsigned int)chip_mac[1] << "-";
	cout << hex << uppercase << setw(2) << setfill('0') << (unsigned int)chip_mac[2] << "-";
	cout << hex << uppercase << setw(2) << setfill('0') << (unsigned int)chip_mac[3] << "-";
	cout << hex << uppercase << setw(2) << setfill('0') << (unsigned int)chip_mac[4] << "-";
	cout << hex << uppercase << setw(2) << setfill('0') << (unsigned int)chip_mac[5] << ")\n";
	cout << "-p\tOutput print from target\n";
	cout << "-v\tReturn user program returned value (if no error occurred)\n";
	cout << "-q\tQuiet mode\n";

	cout << "--args=\'list of args\'\n\tSpecify arguments forwarded to main of user program\n";

	cout << "--read_reg=<addr>\n\tSpecify address of register to read (hex)\n";
	cout << "--write_reg=<addr>\n\tSpecify address of register to write(hex)\n";
	cout << "--write_value=<num>\n\tSpecify value to write in register (hex)\n";
	cout << "--recv_file_name=<File_name>\n\tMake binary dump of shared memory\n";
	cout << "--recv_addr=<num>\n\tSpecify binary dump begin address (hex)";
	cout << " (default: 0x" << hex << uppercase << setw(8) << setfill('0') << Dump_memory_begin_default << ")\n";
	cout << "--recv_size=<num>\n\tSpecify binary dump size in 32 bit words (hex)";
	cout << " (default: 0x" << hex << uppercase << setw(8) << setfill('0') << Dump_memory_size_default << " or section size)\n";
	cout << "--recv_sect=<Section_name>\n\tSpecify binary dump begin address and size via section address/size\n";
	cout << "--recv_disp=<num>\n\tSpecify dump begin address displacement relative begin of section (hex)\n";
	cout << "--send_file_name=<File_name>\n\tCopy binary file to shared memory\n";
	cout << "--send_addr=<num>\n\tSpecify begin address for copying file (hex)";
	cout << " (default: 0x" << hex << uppercase << setw(8) << setfill('0') << Dump_memory_begin_default << ")\n";
	cout << "--send_size=<num>\n\tSpecify size of copying file in 32 bit words (hex)";
	cout << " (default: 0 - copying entire file or section size)\n";
	cout << "--send_sect=<Section_name>\n\tSpecify begin address and size for copying file via section address/size\n";
	cout << "--send_disp=<num>\n\tSpecify begin addr. disp. for copying file rel. begin of sect. (hex)\n";
	cout << "--sync=<num>\n\tCall barrier synchronization function (dec | hex)\n";
	cout << "--sync_tm=<num>\n\tSynchronization timeout in ms (0 - " << dec << MAX_TIMEOUT << ")\n\t"
		" (default: 1000; 0 - no timeout)\n";
}

#define If_this_option(option) \
	if (!(strncmp(ex_opt, option, sizeof(option) - 1)))

int Parse_extended_option(char *ex_opt)
{
	If_this_option("args=")
	{
		if (!ABS_file_args)
		{
			if (strlen(ex_opt) < sizeof("args="))
			{
				cout << "ERROR: No args list specified!\n";
				return 1;
			}

			ABS_file_args = ex_opt + sizeof("args=") - 1;
		}
		else
		{
			cout << "ERROR: Duplicate args to FILE.abs!\n";
			return 1;
		}
	}
	else If_this_option("read_reg=")
	{
		if (reg_flags & 0x1) // 0001b
		{
			cout << "ERROR: Duplicate option 'read_reg'!\n";
			return 1;
		}

		istringstream sin(ex_opt + sizeof("read_reg=") - 1);
		sin >> hex >> reg_addr;
		if ((sin.fail()) || (!(sin.eof())))
		{
			cout << "ERROR: Invalid option '" << ex_opt << "'!\n";
			return 1;
		}

		reg_flags |= 0x1; // 0001b
	}
	else If_this_option("write_reg=")
	{
		if (reg_flags & 0x2) // 0010b
		{
			cout << "ERROR: Duplicate option 'write_reg'!\n";
			return 1;
		}

		istringstream sin(ex_opt + sizeof("write_reg=") - 1);
		sin >> hex >> reg_addr;
		if ((sin.fail()) || (!(sin.eof())))
		{
			cout << "ERROR: Invalid option '" << ex_opt << "'!\n";
			return 1;
		}

		reg_flags |= 0x2; // 0010b
	}
	else If_this_option("write_value=")
	{
		if (reg_flags & 0x4) // 0100b
		{
			cout << "ERROR: Duplicate option 'write_reg'!\n";
			return 1;
		}

		istringstream sin(ex_opt + sizeof("write_value=") - 1);
		sin >> hex >> reg_value;
		if ((sin.fail()) || (!(sin.eof())))
		{
			cout << "ERROR: Invalid option '" << ex_opt << "'!\n";
			return 1;
		}

		reg_flags |= 0x4; // 0100b
	}
	else If_this_option("recv_file_name=")
	{
		if (strlen(ex_opt) < sizeof("recv_file_name="))
		{
			cout << "ERROR: No receive file name specified!\n";
			return 1;
		}

		// If receive file name is already setted
		if (recv_file_name)
		{
			cout << "ERROR: Redefinition receive file name!\n";
			return 1;
		}

		// Setting receive file name
		recv_file_name = ex_opt + sizeof("recv_file_name=") - 1;
	}
	else If_this_option("recv_addr=")
	{
		if (recv_flags & 0x1) // 0001b
		{
			cout << "ERROR: Duplicate option 'recv_addr'!\n";
			return 1;
		}

		istringstream sin(ex_opt + sizeof("recv_addr=") - 1);
		sin >> hex >> recv_addr;
		if ((sin.fail()) || (!(sin.eof())))
		{
			cout << "ERROR: Invalid option '" << ex_opt << "'!\n";
			return 1;
		}

		recv_flags |= 0x1; // 0001b
	}
	else If_this_option("recv_size=")
	{
		if (recv_flags & 0x2) // 0010b
		{
			cout << "ERROR: Duplicate option 'recv_size'!\n";
			return 1;
		}

		istringstream sin(ex_opt + sizeof("recv_size=") - 1);
		sin >> hex >> recv_size;
		if ((sin.fail()) || (!(sin.eof())))
		{
			cout << "ERROR: Invalid option '" << ex_opt << "'!\n";
			return 1;
		}

		recv_flags |= 0x2; // 0010b
	}
	else If_this_option("recv_sect=")
	{
		if (recv_flags & 0x4) // 0100b
		{
			cout << "ERROR: Duplicate option 'recv_sect'!\n";
			return 1;
		}

		recv_sect_name = ex_opt + sizeof("recv_sect=") - 1;

		recv_flags |= 0x4; // 0100b
	}
	else If_this_option("recv_disp=")
	{
		if (recv_flags & 0x8) // 1000b
		{
			cout << "ERROR: Duplicate option 'recv_disp'!\n";
			return 1;
		}

		istringstream sin(ex_opt + sizeof("recv_disp=") - 1);
		sin >> hex >> recv_disp;
		if ((sin.fail()) || (!(sin.eof())))
		{
			cout << "ERROR: Invalid option '" << ex_opt << "'!\n";
			return 1;
		}

		recv_flags |= 0x8; // 1000b
	}
	else If_this_option("send_file_name=")
	{
		if (strlen(ex_opt) < sizeof("send_file_name="))
		{
			cout << "ERROR: No send file name specified!\n";
			return 1;
		}

		// If send file name is already setted
		if (send_file_name)
		{
			cout << "ERROR: Redefinition send file name!\n";
			return 1;
		}

		// Setting send file name
		send_file_name = ex_opt + sizeof("send_file_name=") - 1;
	}
	else If_this_option("send_addr=")
	{
		if (send_flags & 0x1) // 0001b
		{
			cout << "ERROR: Duplicate option 'send_addr'!\n";
			return 1;
		}

		istringstream sin(ex_opt + sizeof("send_addr=") - 1);
		sin >> hex >> send_addr;
		if ((sin.fail()) || (!(sin.eof())))
		{
			cout << "ERROR: Invalid option '" << ex_opt << "'!\n";
			return 1;
		}

		send_flags |= 0x1; // 0001b
	}
	else If_this_option("send_size=")
	{
		if (send_flags & 0x2) // 0010b
		{
			cout << "ERROR: Duplicate option 'send_size'!\n";
			return 1;
		}

		istringstream sin(ex_opt + sizeof("send_size=") - 1);
		sin >> hex >> send_size;
		if ((sin.fail()) || (!(sin.eof())))
		{
			cout << "ERROR: Invalid option '" << ex_opt << "'!\n";
			return 1;
		}

		send_flags |= 0x2; // 0010b
	}
	else If_this_option("send_sect=")
	{
		if (send_flags & 0x4) // 0100b
		{
			cout << "ERROR: Duplicate option 'send_sect'!\n";
			return 1;
		}

		send_sect_name = ex_opt + sizeof("send_sect=") - 1;

		send_flags |= 0x4; // 0100b
	}
	else If_this_option("send_disp=")
	{
		if (send_flags & 0x8) // 1000b
		{
			cout << "ERROR: Duplicate option 'send_disp'!\n";
			return 1;
		}

		istringstream sin(ex_opt + sizeof("send_disp=") - 1);
		sin >> hex >> send_disp;
		if ((sin.fail()) || (!(sin.eof())))
		{
			cout << "ERROR: Invalid option '" << ex_opt << "'!\n";
			return 1;
		}

		send_flags |= 0x8; // 1000b
	}
	else If_this_option("sync=")
	{
		if (f_Sync)
		{
			cout << "ERROR: Duplicate option 'sync'!\n";
			return 1;
		}

		istringstream sin(ex_opt + sizeof("sync=") - 1);

		if ((ex_opt[sizeof("sync=") - 1] == '0') && (ex_opt[sizeof("sync=")] == 'x')) // hex input
		{
			sin >> hex >> Sync_send;
			if ((sin.fail()) || (!(sin.eof())))
			{
				cout << "ERROR: Invalid option '" << ex_opt << "'!\n";
				return 1;
			}
		}
		else // dec input
		{
			sin >> dec >> Sync_send;
			if ((sin.fail()) || (!(sin.eof())))
			{
				cout << "ERROR: Invalid option '" << ex_opt << "'!\n";
				return 1;
			}
		}

		f_Sync = 1;
	}
	else If_this_option("sync_tm=")
	{
		if (sync_flags & 0x1)
		{
			cout << "ERROR: Duplicate option 'sync_tm'!\n";
			return 1;
		}

		istringstream sin(ex_opt + sizeof("sync_tm=") - 1);
		sin >> dec >> Sync_timeout;
		if ((sin.fail()) || (!(sin.eof())))
		{
			cout << "ERROR: Invalid option '" << ex_opt << "'!\n";
			return 1;
		}

		if (Sync_timeout > MAX_TIMEOUT)
		{
			cout << "ERROR: Invalid sync timeout value!\n";
			return 1;
		}

		sync_flags |= 0x1;
	}
	else
	{
		cout << "ERROR: Invalid option '" << ex_opt << "'!\n";
		return 1;
	}

	return 0;
}

#define Check_option_no_param \
	if (argv[i][2] != '\0') \
	{ \
		cout << "ERROR: Invalid option \"" << &argv[i][1] << "\"!\n"; \
		Usage(); \
		exit(1); \
	}

void Parse_command_line(int argc, char **argv)
{
	bool b_key = false, a_key = false, z_key = false;

	// If no input parameters
	if (argc == 1)
	{
		Usage();
		exit(0);
	}

	// Input parameters enumeration cycle
	for (unsigned int i = 1; i < argc; i++)
	{
		// If input parameter is option (beginning with '-')
		if (argv[i][0] == '-')
		{
			// If input parameter is extended option (beginning with '--')
			if (argv[i][1] == '-')
			{
				if (Parse_extended_option(&argv[i][2]))
					exit(1);

				continue;
			}

			// Checking, option not repeated
			for (unsigned int j = 1; j < i; j++)
				if ((argv[j][0] == '-') && (argv[j][1] == argv[i][1]))
				{
					cout << "ERROR: Duplicate option \"" << argv[i][1] << "\"!\n";
					exit(1);
				}

			switch (argv[i][1])
			{
			case 'c':
			{
				istringstream sin(&(argv[i][2]));
				sin >> dec >> CoreNo.cluster_id;
				if ((sin.fail()) || (!(sin.eof())))
				{
					cout << "ERROR: Invalid option \"" << &argv[i][1] << "\"!\n";
					exit(1);
				}

				if ((CoreNo.cluster_id < -1) || (CoreNo.cluster_id > 3))
				{
					cout << "ERROR: Invalid option \"" << &argv[i][1] << "\"!\n";
					exit(1);
				}

				break;
			}
			case 'n':
			{
				istringstream sin(&(argv[i][2]));
				sin >> dec >> CoreNo.nm_id;
				if ((sin.fail()) || (!(sin.eof())))
				{
					cout << "ERROR: Invalid option \"" << &argv[i][1] << "\"!\n";
					exit(1);
				}

				if ((CoreNo.nm_id < -1) || (CoreNo.nm_id > 3))
				{
					cout << "ERROR: Invalid option \"" << &argv[i][1] << "\"!\n";
					exit(1);
				}

				break;
			}
			case 's':
				Check_option_no_param
				f_Dump_start = 1;
				break;
			case 'e':
				Check_option_no_param
				f_Dump_finish = 1;
				break;
			case 'm':
			{
				istringstream sin(&(argv[i][2]));
				sin >> hex >> Dump_memory_begin;
				if ((sin.fail()) || (!(sin.eof())))
				{
					cout << "ERROR: Invalid option \"" << &argv[i][1] << "\"!\n";
					exit(1);
				}
				break;
			}
			case 'l':
			{
				istringstream sin(&(argv[i][2]));
				sin >> hex >> Dump_memory_size;
				if ((sin.fail()) || (!(sin.eof())))
				{
					cout << "ERROR: Invalid option \"" << &argv[i][1] << "\"!\n";
					exit(1);
				}
				break;
			}
			case 'd':
			{
				istringstream sin(&(argv[i][2]));
				sin >> dec >> Dump_field_bits;
				if ((sin.fail()) || (!(sin.eof())))
				{
					cout << "ERROR: Invalid option \"" << &argv[i][1] << "\"!\n";
					exit(1);
				}
				if ((Dump_field_bits <= 0) || (Dump_field_bits > 64))
				{
					cout << "ERROR: Invalid length of dump words!\n";
					exit(1);
				}
				unsigned int n_2 = Dump_field_bits;
				while (n_2 > 2)
				{
					if (n_2 % 2)
					{
						cout << "ERROR: Invalid length of dump words!\n";
						exit(1);
					}
					n_2 /= 2;
				}
				break;
			}
			case 'r':
				Check_option_no_param
				f_Run_program = 0;
				break;
			case 'w':
				Check_option_no_param
				f_Wait_complite = 0;
				break;
			case 'i':
				Check_option_no_param
				f_Reset_board = 0;
				break;
			case 't':
			{
				istringstream sin(&(argv[i][2]));
				sin >> dec >> Timeout_value;
				if ((sin.fail()) || (!(sin.eof())))
				{
					cout << "ERROR: Invalid option \"" << &argv[i][1] << "\"!\n";
					exit(1);
				}
				if (Timeout_value > MAX_TIMEOUT)
				{
					cout << "ERROR: Invalid timeout value!\n";
					exit(1);
				}
				break;
			}
			case 'b':
			{
				istringstream sin(&(argv[i][2]));
				sin >> dec >> Board_number;
				if ((sin.fail()) || (!(sin.eof())))
				{
					cout << "ERROR: Invalid option \"" << &argv[i][1] << "\"!\n";
					exit(1);
				}

				b_key = true;

				break;
			}
			case 'a':
			case 'z':
			{
				if (strlen(&argv[i][2]) != 17) {
					cout << "ERROR: Invalid MAC address '" << &argv[i][2] << "'!\n";
					exit(1);
				}

				char mac_addr[18];
				int mac_count = 0;

				strcpy(mac_addr, &argv[i][2]);

				for (int indx = 0; indx < 17; indx++) {
					if (mac_addr[indx] == '-') {
						if ((indx / 3) != (indx - mac_count))
							break;

						mac_addr[indx] = ' ';
					}
					else
						mac_count++;
				}

				if (mac_count != 12) {
					cout << "ERROR: Invalid MAC address '" << &argv[i][2] << "'!\n";
					exit(1);
				}

				unsigned int mac[6];
				unsigned char *mac_ptr;

				istringstream sin(mac_addr);
				sin >> hex >> mac[0] >> mac[1] >> mac[2] >> mac[3] >> mac[4]>> mac[5];
				if ((sin.fail()) || (!(sin.eof()))) {
					cout << "ERROR: Invalid MAC address '" << &argv[i][2] << "'!\n";
					exit(1);
				}

				if (argv[i][1] == 'a') {
					mac_ptr = host_mac;
					a_key = true;
				}
				else {
					mac_ptr = chip_mac;
					z_key = true;
				}

				for (int indx = 0; indx < 6; indx++)
					mac_ptr[indx] = mac[indx];

				break;
			}
			case 'p':
				Check_option_no_param
				f_Use_print = 1;
				break;
			case 'v':
				Check_option_no_param
				f_Return_ret_val = 1;
				break;
			case 'q':
				Check_option_no_param
				Quiet = 1;
				break;
			default:
				cout << "ERROR: Invalid option \"" << &argv[i][1] << "\"!\n";
				exit(1);
			} // switch
		} // From checking condition first key's character to accordance with '-'
		else // If input parameter is not a option
		{
			if (!ABS_file_name)
				ABS_file_name = argv[i];
			else {
				cout << "ERROR: Duplicate input FILE.abs!\n";
				exit(1);
			}
		}
	} // From input parameters enumeration cycle

	// If executable file name still not setted even for core
	if (!ABS_file_name)
	{
		cout << "ERROR: No input FILE.abs!\n";
		exit(1);
	}

	if (z_key && !a_key)
	{
		cout << "ERROR: With '-z' option, '-a' option must be present!\n";
		exit(1);
	}

	if (a_key)
	{
		if (b_key)
		{
			cout << "ERROR: '-a' option is incompatible with '-b' option!\n";
			exit(1);
		}

		is_use_edcl = true;
	}

	if ((CoreNo.cluster_id == -1) && (CoreNo.nm_id != -1))
	{
		cout << "ERROR: Unknown processor number!\n";
		exit(1);
	}

	if (f_Return_ret_val && (!f_Run_program || !f_Wait_complite))
	{
		cout << "ERROR: Return core return value is not possible without normal run the program!\n";
		exit(1);
	}

	if ((f_Dump_start && !f_Run_program) || (f_Dump_finish && !f_Wait_complite))
	{
		cout << "ERROR: Dump shared memory is not possible without normal run the program!\n";
		exit(1);
	}

	int  div_size;
	int  div_addr;
	int  div_val;

	if (CoreNo.nm_id >= 0) {
		if (Dump_field_bits == 64) {
			div_addr = 2;
			div_size = 2;
		}
		else {
			div_addr = 1;
			div_size = 1;
		}

		div_val = 1;
	}
	else {
		if (Dump_field_bits == 64) {
			div_addr = 8;
			div_size = 2;
		}
		else {
			div_addr = 4;
			div_size = 1;
		}

		div_val = 4;
	}

	if ((f_Dump_start || f_Dump_finish) &&
	    ((Dump_memory_begin % div_addr) || (Dump_memory_size % div_size)))
	{
		cout << "ERROR: With " << Dump_field_bits << "-words dump begin address and\n\t"
			" dump size must be multiple " << div_addr << " and " << div_size << " respectively!\n";
		exit(1);
	}

	// Checking acceptability of extended options

	if ((reg_flags & 0x1) && ((reg_flags & 0x2) || (reg_flags & 0x4)))
	{
		cout << "ERROR: Options 'read_reg' and 'write_reg' or 'write_value' are incompatible!\n";
		exit(1);
	}

	if (((reg_flags & 0x2) && !(reg_flags & 0x4)) || (!(reg_flags & 0x2) && (reg_flags & 0x4)))
	{
		cout << "ERROR: With option 'write_reg' required option 'write_value'!\n";
		exit(1);
	}

	if (((reg_flags & 0x1) || (reg_flags & 0x2)) && ((reg_addr % div_val) || (CoreNo.nm_id != -1)))
	{
		cout << "ERROR: Option 'write_reg' or 'read_reg' required address multiple 4 and 'n' == -1!\n";
		exit(1);
	}

	// recv

	if ((recv_flags) && (!recv_file_name))
	{
		cout << "ERROR: No receive file name specified!\n";
		exit(1);
	}

	if ((recv_flags & 0x1) && (recv_flags & 0x4))
	{
		cout << "ERROR: Options 'recv_addr' and 'recv_sect' are incompatible!\n";
		exit(1);
	}

	if ((recv_flags & 0x8) && (!(recv_flags & 0x4)))
	{
		cout << "ERROR: With option 'recv_disp' required option 'recv_sect'!\n";
		exit(1);
	}

	if (recv_flags & 0x4) // 0100b
	{
		assert(recv_addr == Dump_memory_begin_default);

		if (Get_sect_prm(recv_sect_name, &recv_addr, NULL))
		{
			cout << "ERROR: Failed get address of section '" << recv_sect_name << "'!\n";
			exit(1);
		}

		if (!(recv_flags & 0x2))
		{
			Get_sect_prm(recv_sect_name, NULL, &recv_size);
			recv_size -= (recv_disp / div_val);
		}

		recv_addr += recv_disp;
	}

	if (recv_file_name && (recv_addr % div_val)) {
		cout << "ERROR: Address must be multiple 4!\n";
		exit(1);
	}

	// send

	if ((send_flags) && (!send_file_name))
	{
		cout << "ERROR: No send file name specified!\n";
		exit(1);
	}

	if ((send_flags & 0x1) && (send_flags & 0x4))
	{
		cout << "ERROR: Options 'send_addr' and 'send_sect' are incompatible!\n";
		exit(1);
	}

	if ((send_flags & 0x8) && (!(send_flags & 0x4)))
	{
		cout << "ERROR: With option 'send_disp' required option 'send_sect'!\n";
		exit(1);
	}

	if (send_flags & 0x4) // 0100b
	{
		assert(send_addr == Dump_memory_begin_default);

		if (Get_sect_prm(send_sect_name, &send_addr, NULL))
		{
			cout << "ERROR: Failed get address of section '" << send_sect_name << "'!\n";
			exit(1);
		}

		if (!(send_flags & 0x2))
		{

			Get_sect_prm(send_file_name, NULL, &send_size);
			send_size -= (send_disp / div_val);
		}

		send_addr += send_disp;
	}

	if (send_file_name && (send_addr % div_val)) {
		cout << "ERROR: Address must be multiple 4!\n";
		exit(1);
	}

	if (sync_flags && !f_Sync) {
		cout << "ERROR: No 'sync' option specified!\n";
		exit(1);
	}
}
