// Loader for NM Board

#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>

#include "nmrun.h"

using namespace std;

unsigned int Board_number = 0;

unsigned char host_mac[] = HOST_MAC_ADDRESS_DEFAULT;
unsigned char chip_mac[] = CHIP_MAC_ADDRESS_DEFAULT;
bool is_use_edcl = false;

PL_Board *Board = NULL;
PL_CoreNo CoreNo = Access_core_default;
PL_Access *Access = NULL;

int f_Dump_start = 0;
int f_Dump_finish = 0;
int f_Run_program = 1;
int f_Wait_complite = 1;
int f_Reset_board = 1;
int f_Return_ret_val = 0;
int Quiet = 0; // Suppress output (except errors)

int f_Use_print = 0;

char *ABS_file_name = NULL;
char *ABS_file_args = NULL;

unsigned int Timeout_value = 0; // (mS)

PL_Addr Dump_memory_begin = Dump_memory_begin_default;
DWORD Dump_memory_size = Dump_memory_size_default;
unsigned int Dump_field_bits = 32; // Size of output words in bits (must be power of 2 and not greater than 64)

unsigned char reg_flags = 0x0; // (read, write, value)
DWORD reg_addr;
DWORD reg_value;

unsigned char recv_flags = 0x0; // (disp, sect, size, addr)
char *recv_file_name = NULL;
PL_Addr recv_addr = Dump_memory_begin_default;
DWORD recv_size = Dump_memory_size_default;
char *recv_sect_name = NULL;
PL_Addr recv_disp = 0x0;

unsigned char send_flags = 0x0; // (disp, sect, size, addr)
char *send_file_name = NULL;
PL_Addr send_addr = Dump_memory_begin_default;
DWORD send_size = 0; // 0 - Read entire file
char *send_sect_name = NULL;
PL_Addr send_disp = 0x0;

int f_Sync = 0;
unsigned char sync_flags = 0x0; // (timeout)
DWORD Sync_timeout = 1000; // (mS)
PL_Word Sync_send = 0;
PL_Word Sync_recv;

static int Sync_error = 0;

extern "C" DWORD PL_BuildVersion;
extern "C" DWORD PL_FirmwareVersion;

int main(int argc, char **argv)
{
	Display_version(argc, argv);
	Parse_command_line(argc, argv);

	try
	{
		unsigned int Boards_count;
		unsigned int Firmware_version_major, Firmware_version_minor;
		unsigned int version_major, version_minor;
		unsigned long Serial_number;

		if (!Quiet)
		{
			int ret;

			ret = PL_GetVersion(&version_major, &version_minor);
			if (ret != PL_OK)
			{
				cout << "ERROR: Failed get libload version!\n";
				exit(1);
			}

			cout << "LibLoad v" << version_major << "." << version_minor;
			cout << "." << (((PL_BuildVersion >> 25) & 0x7F)*10000 + ((PL_BuildVersion >> 21) & 0xF)*100 + ((PL_BuildVersion >> 16) & 0x1F));
			cout << "." << hex << (PL_BuildVersion & 0xFFFF) << dec << "\n";
		}

		if (!is_use_edcl)
		{
			if (PL_GetBoardCount(&Boards_count) != PL_OK)
			{
				cout << "ERROR: Failed get count of boards!\n";
				exit(1);
			}

			if (!Boards_count)
			{
				cout << "ERROR: No boards found!\n";
				exit(1);
			}

			if (Board_number >= Boards_count)
			{
				cout << "ERROR: Invalid board number!\n";
				exit(1);
			}
		}
		else
		{
			if (PL_SetChannelEDCL(host_mac, chip_mac) != PL_OK)
			{
				cout << "ERROR: Failed set channel EDCL!\n";
				exit(1);
			}
		}

		if (PL_GetBoardDesc(Board_number, &Board) != PL_OK)
		{
			cout << "ERROR: Failed open board!\n";
			exit(1);
		}

		if (f_Reset_board)
		{
			if (PL_ResetBoard(Board) != PL_OK)
			{
				cout << "ERROR: Failed reset board!\n";
				PL_CloseBoardDesc(Board);
				exit(1);
			}

			if (PL_LoadInitCode(Board) != PL_OK)
			{
				cout << "ERROR: Failed load init code!\n";
				PL_CloseBoardDesc(Board);
				exit(1);
			}
		}

		if (!Quiet && !is_use_edcl)
		{
			if (PL_GetFirmwareVersion(Board, &Firmware_version_major, &Firmware_version_minor) == PL_OK)
			{
				cout << "Firmware v" << Firmware_version_major << "." << Firmware_version_minor;
				cout << "." << (((PL_FirmwareVersion >> 25) & 0x7F)*10000 + ((PL_FirmwareVersion >> 21) & 0xF)*100 + ((PL_FirmwareVersion >> 16) & 0x1F));
				cout << "." << hex << (PL_FirmwareVersion & 0xFFFF) << dec << "\n";
			}
			else
				cout << "WARNING: Failed get firmware version!\n";

			if (PL_GetSerialNumber(Board, &Serial_number) == PL_OK)
				cout << "Serial n" << Serial_number << "\n";
			else
				cout << "WARNING: Failed get serial number!\n";
		}

		if (PL_GetAccess(Board, &CoreNo, &Access) != PL_OK) {
			cout << "ERROR: Failed access core " << "!\n";
			PL_CloseBoardDesc(Board);
			exit(1);
		}

		if (f_Dump_start)
		{
			if (Dump_shared_memory(".dms"))
				cout << "ERROR: Failed create dump of shared memory!\n";
		}
		
		if (send_file_name)
		{
			if (Send_binary_file_to_board())
			{
				cout << "ERROR: Failed copy binary file to board!\n";
				PL_CloseAccess(Access);
				PL_CloseBoardDesc(Board);
				exit(1);
			}
		}

		if (reg_flags & 0x2)
		{
			int ret;

			ret = PL_WriteRegister(Access, reg_value, reg_addr);

			if (ret != PL_OK)
			{
				cout << "ERROR: Failed write register!\n";
				PL_CloseAccess(Access);
				PL_CloseBoardDesc(Board);
				exit(1);
			}
		}

		PL_Word Status, ABS_return_value;

		if (f_Run_program)
		{
			int ret;

			if (ABS_file_args)
			{
				ret = PL_SetProgramArgs(Access, ABS_file_args);

				if (ret != PL_OK)
				{
					cout << "ERROR: Failed set list of main arguments!\n";
					PL_CloseAccess(Access);
					PL_CloseBoardDesc(Board);
					exit(1);
				}
			}

			ret = PL_LoadProgramFile(Access, ABS_file_name);

			if (ret != PL_OK)
			{
				if (ret == PL_FILE)
					cout << ABS_file_name << " :: ERROR: Failed find user program file!\n";
				else if (ret == PL_BADADDRESS)
					cout << ABS_file_name << " :: ERROR: Invalid user program configuration!\n";
				else
					cout << ABS_file_name << " :: ERROR: Failed load program into core " << CoreNo.cluster_id << "." << CoreNo.nm_id << "!\n";

				PL_CloseAccess(Access);
				PL_CloseBoardDesc(Board);
				exit(1);
			}

			if (!Quiet)
			{
				cout << "Start user program on core " << CoreNo.cluster_id << "." << CoreNo.nm_id << "...\n";
				cout.flush();
			}

		}

		IO_Service *print_server = NULL;
		if (f_Use_print)
			print_server = IO_ServiceStart(Access, NULL, NULL, NULL);

		if (f_Sync)
		{
			int ret;

			if (!Quiet) {
				print_mutex_lock();
				cout << "Call barrier synchronization...\n" << flush;
				print_mutex_unlock();
			}


			ret = PL_SetTimeout(Sync_timeout);
			if (ret != PL_OK)
			{
				print_mutex_lock();
				cout << "ERROR: Failed set barrier synchronization timeout!\n";
				print_mutex_unlock();

				if (print_server)
					 IO_ServiceStop(&print_server, NULL);

				PL_CloseAccess(Access);
				PL_CloseBoardDesc(Board);
				exit(1);
			}

			ret = PL_Sync(Access, Sync_send, &Sync_recv);

			if (ret == PL_OK)
			{
				print_mutex_lock();
				cout << ABS_file_name << " :: Barrier synchronization value: ";
				cout << dec << Sync_recv;
				cout << " = 0x" << hex << uppercase << Sync_recv << ".\n";
				print_mutex_unlock();
			}
			else if (ret == PL_TIMEOUT)
			{
				print_mutex_lock();
				cout << "WARN: Timeout barrier synchronization!\n";
				print_mutex_unlock();

				Sync_error = 1;
			}
			else {
				print_mutex_lock();
				cout << "ERROR: Failed call barrier synchronization!\n";
				print_mutex_unlock();

				if (print_server)
					IO_ServiceStop(&print_server, NULL);

				PL_CloseAccess(Access);
				PL_CloseBoardDesc(Board);
				exit(1);
			}
		}

		if (f_Wait_complite)
		{

			struct timeval tv;
			unsigned int start_time;
			unsigned int current_time;

			if (Timeout_value)
			{
				gettimeofday(&tv, NULL);
				start_time = (tv.tv_sec * 1000) + (tv.tv_usec / 1000);
			}

			while (1)
			{
				if (PL_GetStatus(Access, &Status) != PL_OK)
				{
						print_mutex_lock();
						cout << "ERROR: Failed get program status on core!\n";
						print_mutex_unlock();

						if (print_server)
							IO_ServiceStop(&print_server, NULL);

						PL_CloseAccess(Access);
						PL_CloseBoardDesc(Board);
						exit(1);
				}

				if (Status != PROGRAM_PROGRESS)
					break;

				if (Sync_error)
					break;

				if (Timeout_value)
				{
					gettimeofday(&tv, NULL);
					current_time = (tv.tv_sec * 1000) + (tv.tv_usec / 1000);
					if ((current_time - start_time) > Timeout_value)
						break;
				}

				usleep(1000);
			}

			if (Status != PROGRAM_FINISHED)
			{
				if (print_server)
					IO_ServiceStop(&print_server, NULL);

				if (Sync_error)
					cout << ABS_file_name << " :: ERROR: Incorrect sync usage!\n";
				else if (Status == PROGRAM_PROGRESS)
					cout << ABS_file_name << " :: WARN: Timeout for program on core!\n";
				else if (Status == PROGRAM_EXEPTION)
					cout << ABS_file_name << " :: ERROR: Exeption for program on core!\n";
				else if (Status != PROGRAM_NO)
					cout << ABS_file_name << " :: ERROR: Unknow program status on core!\n";

				if (PL_GetResult(Access, &ABS_return_value) == PL_OK) {
					cout << ABS_file_name << " :: error ";
					cout << " = 0x" << hex << uppercase << ABS_return_value << ".\n";
				}

				PL_CloseAccess(Access);
				PL_CloseBoardDesc(Board);
				exit(1);
			}
			else
			{
				if (PL_GetResult(Access, &ABS_return_value) != PL_OK)
				{
					print_mutex_lock();
					cout << "ERROR: Failed get program return value of core!\n";
					print_mutex_unlock();

					if (print_server)
						IO_ServiceStop(&print_server, NULL);

					PL_CloseAccess(Access);
					PL_CloseBoardDesc(Board);
					exit(1);
				}
			}
		}

		if (print_server)
			IO_ServiceStop(&print_server, NULL);

		if (reg_flags & 0x1)
		{
			int ret;

			ret = PL_ReadRegister(Access, &reg_value, reg_addr);

			if (ret != PL_OK)
			{
				cout << "ERROR: Failed read register!\n";
				PL_CloseAccess(Access);
				PL_CloseBoardDesc(Board);
				exit(1);
			}

			
			cout << "*(0x" << hex << uppercase << reg_addr << ") == 0x" << reg_value << ".\n";
		}

		if (f_Dump_finish)
		{
			if (Dump_shared_memory(".dme"))
				cout << "ERROR: Failed create dump of shared memory!\n";
		}
		
		if (recv_file_name)
		{
			if (Receive_binary_dump_from_board())
				cout << "ERROR: Failed create binary dump of shared memory!\n";
		}

		PL_CloseAccess(Access);
		PL_CloseBoardDesc(Board);

		if (f_Wait_complite)
		{
			if (!Quiet)
			{
					cout << ABS_file_name << " :: return ";
					cout << dec << ABS_return_value;
					cout << " = 0x" << hex << uppercase << ABS_return_value << ".\n";
			}

			if (f_Return_ret_val)
				return ABS_return_value;
		}
	}
	catch (...)
	{
		cout << "ERROR: Unhandled exception!\n";
		exit(1);
	}

	return 0;
}
